/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
 
/* Operating System Dependent Information */

/* Define the name of the JAR file that contains the mainline. 
 * If one of the arguments in argVM[] is equal to this name,
 * the resulting argument passed to the VM is the full pathname
 * of that JAR file.
 */
#define STARTUP_JAR_NAME "startup.jar"

/* Define the official name of the program. */
#define getOfficialName() PROGRAM_NAME


/* Global Variables */

extern char   dirSeparator;         /* '/' or '\\' */
extern char   pathSeparator;        /* separator used in PATH variable */
extern char*  consoleVM;            /* name of VM that uses a console  */
extern char*  defaultVM;            /* name of VM that should be used  */
extern char*  argVM[];              /* list of VM command arguments    */


/* OS Specific Functions */

/* Create the Splash Window
 *
 * This function takes the program arguments and the directory 
 * where the corresponding splash bitmap files exist and creates a 
 * borderless window with that pixmap. The window should
 * be centred on the screen.
 * 
 * This function returns a string containing some command info
 * that will cause the event processing loop to exit, thus
 * terminating the splash window program.
 *
 * This string (allocated by malloc) is reformatted and passed
 * to the Java program via the argument "-endsplash". When
 * the Java program is finished its initialization, it will
 * use this argument to start (Runtime.getRuntime().exec( <arg> ))
 * the program responsible for terminating the splash window.
 *
 * The string returned from this function will eventually be passed
 * as the argument to the destroySplash function.
 *
 * Return: end splash argument info string, which must be deallocated.
 */
extern char* createSplash( int* argc, char* argv[], char* homeDir );


/* Destroy the Splash Window
 *
 * This function takes the string returned from createSplash
 * and uses it to bring down the splash window and to terminate
 * the launcher program.
 */
extern void destroySplash( char* splashId );

/* Start Java VM */
extern void startJavaVM( char* args[] );


/* Process Events
 *
 * This function processes events from the OS and must
 * work in conjunction with the splash string argument 
 * sent to the Java program so that it exits the loop
 * when requested.
 */
extern void processEvents();


