package org.eclipse.compare.internal;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
import java.util.ResourceBundle;

import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.Separator;

import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.part.EditorActionBarContributor;

import org.eclipse.compare.CompareConfiguration;
import org.eclipse.compare.CompareEditorInput;


public class CompareEditorContributor extends EditorActionBarContributor {
	
	private IEditorPart fActiveEditorPart= null;

	private IgnoreWhiteSpaceAction fIgnoreWhitespace;
	private NavigationAction fNext;
	private NavigationAction fPrevious;
	//private ShowPseudoConflicts fShowPseudoConflicts;


	public CompareEditorContributor() {
		ResourceBundle bundle= CompareUIPlugin.getResourceBundle();
		fIgnoreWhitespace= new IgnoreWhiteSpaceAction(bundle, null);
		fNext= new NavigationAction(bundle, true);
		fPrevious= new NavigationAction(bundle, false);
		//fShowPseudoConflicts= new ShowPseudoConflicts(bundle, null);
	}
	public void contributeToToolBar(IToolBarManager tbm) {
		tbm.add(new Separator());
		tbm.add(fIgnoreWhitespace);
		tbm.add(fNext);
		tbm.add(fPrevious);
		//tbm.add(fShowPseudoConflicts);
	}
	public void setActiveEditor(IEditorPart targetEditor) {
				
		if (fActiveEditorPart != targetEditor) {
			fActiveEditorPart= targetEditor;
			
			if (fActiveEditorPart != null) {
				IEditorInput input= fActiveEditorPart.getEditorInput();
				if (input instanceof CompareEditorInput) {
					CompareEditorInput compareInput= (CompareEditorInput) input;
					fNext.setCompareEditorInput(compareInput);
					fPrevious.setCompareEditorInput(compareInput);
				}
			}
				
			if (targetEditor instanceof CompareEditor) {
				CompareEditor editor= (CompareEditor) targetEditor;
				editor.setActionBars(getActionBars());
			
				CompareConfiguration cc= editor.getCompareConfiguration();
				fIgnoreWhitespace.setCompareConfiguration(cc);
				//fShowPseudoConflicts.setCompareConfiguration(cc);			
			}
		}
	}
}
