package org.eclipse.jdt.internal.core.refactoring;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.ISourceManipulation;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.core.refactoring.Assert;
import org.eclipse.jdt.internal.core.refactoring.NullChange;
import org.eclipse.jdt.internal.core.refactoring.base.Change;
import org.eclipse.jdt.internal.core.refactoring.base.ChangeContext;
import org.eclipse.jdt.internal.core.refactoring.base.IChange;

public abstract class AbstractRenameChange extends Change {

	private String fNewName;
	private String fOldName;
	private IPath fResourcePath;
	private IChange fUndoChange;
	
	protected AbstractRenameChange(IPath resourcePath, String oldName, String newName){
		Assert.isNotNull(newName, "new name"); //$NON-NLS-1$
		Assert.isNotNull(oldName, "old name"); //$NON-NLS-1$
		
		fResourcePath= resourcePath;
		fOldName= oldName;
		fNewName= newName;
	}
	protected abstract IChange createUndoChange() throws JavaModelException;
	public IJavaElement getCorrespondingJavaElement() {
		return JavaCore.create(getResource());
	}
	/**
	 * Gets the newName.
	 * @return Returns a String
	 */
	protected String getNewName() {
		return fNewName;
	}
	/**
	 * Gets the oldName
	 * @return Returns a String
	 */
	protected String getOldName() {
		return fOldName;
	}
	private IResource getResource(){
		return ResourcesPlugin.getWorkspace().getRoot().findMember(fResourcePath);
	}
	/**
	 * Gets the resourcePath.
	 * @return Returns a IPath
	 */
	protected IPath getResourcePath() {
		return fResourcePath;
	}
	public final IChange getUndoChange() {
		return fUndoChange;
	}
	public void perform(ChangeContext context, IProgressMonitor pm) throws JavaModelException {
		try{
			pm.beginTask(RefactoringCoreMessages.getString("AbstractRenameChange.Renaming"), 1); //$NON-NLS-1$
			if (isActive()){
				fUndoChange= createUndoChange();
				ISourceManipulation element= (ISourceManipulation)getCorrespondingJavaElement();
				element.rename(fNewName, false, pm);
			} else{
				fUndoChange= new NullChange();
			}
		} catch (Exception e) {
			handleException(context, e);
			fUndoChange= new NullChange();
			setActive(false);
		} finally {
			pm.done();
		}
	}
}
