package org.eclipse.jdt.internal.core.refactoring;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
import org.eclipse.core.runtime.IProgressMonitor;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.core.refactoring.base.IChange;
import org.eclipse.jdt.internal.core.refactoring.base.ChangeContext;

public class DeleteCompilationUnitChange extends CompilationUnitChange {

	private String fCUHandle;
	
	public DeleteCompilationUnitChange(ICompilationUnit cu){
		super((IPackageFragment)cu.getParent(), cu.getElementName());
		fCUHandle= cu.getHandleIdentifier();
	}
	public IJavaElement getCorrespondingJavaElement(){
		return JavaCore.create(fCUHandle);
	}
	public String getName(){
		return RefactoringCoreMessages.getFormattedString("DeleteCompilationUnit.delete_cu", new String[] { getCUName(), getPackageName() });	//$NON-NLS-1$
	}
	public IChange getUndoChange() {
		if (!isActive())
			return new NullChange();
		else	
			return new CreateCompilationUnitChange(getPackage(), getSource(), getCUName());
	}
	public void perform(ChangeContext context, IProgressMonitor pm) throws JavaModelException {
		try {
			if (!isActive())
				return;
			String msg= RefactoringCoreMessages.getFormattedString("DeleteCompilationUnit.deleting_resource", getCUName());	//$NON-NLS-1$
			pm.beginTask(msg, 1);
			ICompilationUnit cu= (ICompilationUnit)JavaCore.create(fCUHandle);
			Assert.isNotNull(cu);
			Assert.isTrue(cu.exists());
			Assert.isTrue(!cu.isReadOnly());
			setSource(cu.getSource());
			cu.delete(true, pm);
			pm.done();
		} catch (Exception e) {
			handleException(context, e);
			setActive(false);
		}
	}
}
