package org.eclipse.jdt.internal.debug.pattern;

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.jdt.debug.core.JDIDebugModel;


/**
 * @see org.eclipse.core.resources.IWorkspaceRunnable
 */
class BreakpointCreator implements IWorkspaceRunnable {
	protected IMarker fBreakpoint;
	protected IFile fFile;
	protected int fLine;
	protected String fPattern;

	public BreakpointCreator(IFile file, int line, String pattern) {
		fFile = file;
		fLine = line;
		fPattern = pattern;
	}
	public IMarker getBreakpoint() {
		return fBreakpoint;
	}
	/**
	 * @see IWorkspaceRunnable
	 */
	public void run(IProgressMonitor monitor) throws CoreException {
		// create the marker
		fBreakpoint = fFile.createMarker(PatternDebugModel.PATTERN_BREAKPOINT);
		// configure the standard attributes
		DebugPlugin.getDefault().getBreakpointManager().configureLineBreakpoint(
			fBreakpoint, JDIDebugModel.getPluginIdentifier(), true, fLine, -1, -1);
		fBreakpoint.setAttribute(PatternDebugModel.PATTERN, fPattern);
	}
}
