package org.eclipse.jdt.internal.debug.ui;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
import org.eclipse.debug.ui.IDebugModelPresentation;
import org.eclipse.debug.ui.IDebugViewAdapter;
import org.eclipse.jdt.debug.core.JDIDebugModel;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.custom.BusyIndicator;
import org.eclipse.ui.IViewActionDelegate;
import org.eclipse.ui.IViewPart;

/**
 * An action that toggles the state of its viewer to
 * show/hide hexidecimal values for primitive variables.
 */

public class HexValuesAction extends Action implements IViewActionDelegate {

	private static final String PREFIX= "hex_values_action.";
	private static final String SHOW= PREFIX + "show";
	private static final String HIDE= PREFIX + "hide";

	IDebugViewAdapter fAdapter;

	public HexValuesAction() {
		super(DebugUIUtils.getResourceString(SHOW));
		setToolTipText(DebugUIUtils.getResourceString(SHOW));
	}
	/**
	 * @see IViewActionDelegate
	 */
	public void init(IViewPart view) {
		fAdapter= (IDebugViewAdapter)view.getAdapter(IDebugViewAdapter.class);
	}
	/**
	 * @see IAction
	 */
	public void run() {
	}
	/**
	 * @see IActionDelegate
	 */
	public void run(IAction action) {
		valueChanged(action.isChecked());
		String label= action.isChecked() ? DebugUIUtils.getResourceString(HIDE) : DebugUIUtils.getResourceString(SHOW);
		action.setToolTipText(label);
		action.setText(label);
	}
	/**
	 * @see IViewActionDelegate
	 */
	public void selectionChanged(IAction action, ISelection selection) {
	}
	/**
	 * @see IAction
	 */
	public void setChecked(boolean value) {
		super.setChecked(value);
		valueChanged(value);
	}
	private void valueChanged(boolean on) {
		if (fAdapter.getViewer().getControl().isDisposed()) {
			return;
		}		
		IDebugModelPresentation presentation= fAdapter.getPresentation(JDIDebugModel.getPluginIdentifier());
		presentation.setAttribute(JDIModelPresentation.DISPLAY_HEX_VALUES, new Boolean(on));
		BusyIndicator.showWhile(fAdapter.getViewer().getControl().getDisplay(), new Runnable() {
			public void run() {
				fAdapter.getViewer().refresh();					
			}
		});
		setToolTipText(on ? DebugUIUtils.getResourceString(HIDE) : DebugUIUtils.getResourceString(SHOW));
	}
}
