package org.eclipse.jdt.internal.ui.typehierarchy;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.ISelectionProvider;

import org.eclipse.ui.help.WorkbenchHelp;
import org.eclipse.ui.texteditor.IUpdate;

import org.eclipse.jdt.core.IType;

import org.eclipse.jdt.internal.ui.IJavaHelpContextIds;
import org.eclipse.jdt.internal.ui.util.SelectionUtil;

/**
 * Refocuses the type hierarchy on the currently selection type.
 */
public class FocusOnSelectionAction extends Action implements IUpdate {
		
	private TypeHierarchyViewPart fViewPart;
	private ISelectionProvider fSelectionProvider;
	private String fText;
	
	public FocusOnSelectionAction(TypeHierarchyViewPart part, ISelectionProvider selProvider) {
		super(TypeHierarchyMessages.getString("FocusOnSelectionAction.label")); //$NON-NLS-1$
		setDescription(TypeHierarchyMessages.getString("FocusOnSelectionAction.description")); //$NON-NLS-1$
		setToolTipText(TypeHierarchyMessages.getString("FocusOnSelectionAction.tooltip")); //$NON-NLS-1$
		fViewPart= part;
		fSelectionProvider= selProvider;
		fText= TypeHierarchyMessages.getString("FocusOnSelectionAction.label"); //$NON-NLS-1$
		
		WorkbenchHelp.setHelp(this,	new Object[] { IJavaHelpContextIds.FOCUS_ON_SELECTION_ACTION });
	}
	public boolean canActionBeAdded() {
		Object element= SelectionUtil.getSingleElement(fSelectionProvider.getSelection());
		if (element instanceof IType) {
			setText(fText + " \"" + ((IType)element).getElementName() + "\""); //$NON-NLS-2$ //$NON-NLS-1$
			return true;
		}
		return false;
	}
	/**
	 * Perform the action
	 */
	public void run() {
		Object element= SelectionUtil.getSingleElement(fSelectionProvider.getSelection());
		if (element instanceof IType) {
			fViewPart.setInput((IType)element);
		}
	}
	public void update() {
		setEnabled(canActionBeAdded());
	}
}
