package org.eclipse.jdt.internal.ui.typehierarchy;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.BusyIndicator;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.custom.ViewForm;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.ToolBar;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IAdaptable;

import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.viewers.IInputSelectionProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;

import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.actions.OpenWithMenu;
import org.eclipse.ui.help.ViewContextComputer;
import org.eclipse.ui.help.WorkbenchHelp;
import org.eclipse.ui.part.PageBook;
import org.eclipse.ui.part.ViewPart;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IMember;
import org.eclipse.jdt.core.ISourceReference;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;

import org.eclipse.jdt.ui.IContextMenuConstants;
import org.eclipse.jdt.ui.ITypeHierarchyViewPart;
import org.eclipse.jdt.ui.JavaElementLabelProvider;

import org.eclipse.jdt.internal.ui.IJavaHelpContextIds;
import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jdt.internal.ui.JavaPluginImages;
import org.eclipse.jdt.internal.ui.actions.AddMethodStubAction;
import org.eclipse.jdt.internal.ui.actions.ContextMenuGroup;
import org.eclipse.jdt.internal.ui.compare.JavaReplaceWithEditionAction;
import org.eclipse.jdt.internal.ui.javaeditor.EditorUtility;
import org.eclipse.jdt.internal.ui.packageview.PackageExplorerPart;
import org.eclipse.jdt.internal.ui.refactoring.actions.RefactoringGroup;
import org.eclipse.jdt.internal.ui.util.ExceptionHandler;
import org.eclipse.jdt.internal.ui.util.JavaModelUtility;
import org.eclipse.jdt.internal.ui.util.OpenTypeHierarchyHelper;
import org.eclipse.jdt.internal.ui.viewsupport.SelectionProviderMediator;
import org.eclipse.jdt.internal.ui.viewsupport.StatusBarUpdater;

/**
 * view showing the supertypes/subtypes of its input.
 */
public class TypeHierarchyViewPart extends ViewPart implements ITypeHierarchyLifeCycleListener, ITypeHierarchyViewPart {
	
	public static final int VIEW_ID_TYPE= 2;
	public static final int VIEW_ID_SUPER= 0;
	public static final int VIEW_ID_SUB= 1;
	
	private static final String DIALOGSTORE_HIERARCHYVIEW= "TypeHierarchyViewPart.hierarchyview";	 //$NON-NLS-1$

	private static final String TAG_INPUT= "input"; //$NON-NLS-1$
	private static final String TAG_VIEW= "view"; //$NON-NLS-1$

	private IType fInput;
	
	private IMemento fMemento;
	
	private ArrayList fInputHistory;
	private int fCurrHistoryIndex;
	
	private TypeHierarchyLifeCycle fHierarchyLifeCycle;
		
	private MethodsViewer fMethodsViewer;
			
	private int fCurrentViewerIndex;
	private TypeHierarchyViewer[] fAllViewers;
	
	private boolean fIsEnableMemberFilter;
	
	private SashForm fTypeMethodsSplitter;
	private PageBook fViewerbook;
	private PageBook fPagebook;
	private Label fNoHierarchyShownLabel;
	private Label fEmptyTypesViewer;
	
	private CLabel fMethodViewerPaneLabel;
	private ILabelProvider fPaneLabelProvider;
	
	private IDialogSettings fDialogSettings;
	
	private ToggleViewAction[] fViewActions;
	
	private HistoryAction fForwardAction;
	private HistoryAction fBackwardAction;
	
	private EnableMemberFilterAction fEnableMemberFilterAction;
	private AddMethodStubAction fAddStubAction;
	private FocusOnTypeAction fFocusOnTypeAction;
	private FocusOnSelectionAction fFocusOnSelectionAction;
			
	public TypeHierarchyViewPart() {
		fHierarchyLifeCycle= new TypeHierarchyLifeCycle();
		fHierarchyLifeCycle.addChangedListener(this);
		fIsEnableMemberFilter= false;
		
		fInputHistory= new ArrayList();
		fCurrHistoryIndex= -1;
		
		String title= TypeHierarchyMessages.getString("TypeHierarchyViewPart.toggleaction.supertypes.label"); //$NON-NLS-1$
		ToggleViewAction superViewAction= new ToggleViewAction(this, VIEW_ID_SUPER, title);
		superViewAction.setDescription(TypeHierarchyMessages.getString("TypeHierarchyViewPart.toggleaction.supertypes.description")); //$NON-NLS-1$
		superViewAction.setToolTipText(TypeHierarchyMessages.getString("TypeHierarchyViewPart.toggleaction.supertypes.tooltip")); //$NON-NLS-1$
		JavaPluginImages.setImageDescriptors(superViewAction, "lcl16", "super_co.gif"); //$NON-NLS-2$ //$NON-NLS-1$

		title= TypeHierarchyMessages.getString("TypeHierarchyViewPart.toggleaction.subtypes.label"); //$NON-NLS-1$
		ToggleViewAction subViewAction= new ToggleViewAction(this, VIEW_ID_SUB, title);
		subViewAction.setDescription(TypeHierarchyMessages.getString("TypeHierarchyViewPart.toggleaction.subtypes.description")); //$NON-NLS-1$
		subViewAction.setToolTipText(TypeHierarchyMessages.getString("TypeHierarchyViewPart.toggleaction.subtypes.tooltip")); //$NON-NLS-1$
		JavaPluginImages.setImageDescriptors(subViewAction, "lcl16", "sub_co.gif"); //$NON-NLS-2$ //$NON-NLS-1$

		title= TypeHierarchyMessages.getString("TypeHierarchyViewPart.toggleaction.vajhierarchy.label"); //$NON-NLS-1$
		ToggleViewAction vajViewAction= new ToggleViewAction(this, VIEW_ID_TYPE, title);
		vajViewAction.setDescription(TypeHierarchyMessages.getString("TypeHierarchyViewPart.toggleaction.vajhierarchy.description")); //$NON-NLS-1$
		vajViewAction.setToolTipText(TypeHierarchyMessages.getString("TypeHierarchyViewPart.toggleaction.vajhierarchy.tooltip")); //$NON-NLS-1$
		JavaPluginImages.setImageDescriptors(vajViewAction, "lcl16", "hierarchy_co.gif"); //$NON-NLS-1$ //$NON-NLS-2$
		
		fViewActions= new ToggleViewAction[] { vajViewAction, superViewAction, subViewAction };
		
		superViewAction.setOthers(fViewActions);
		subViewAction.setOthers(fViewActions);
		vajViewAction.setOthers(fViewActions);
		
		fForwardAction= new HistoryAction(this, true);
		fBackwardAction= new HistoryAction(this, false);
		
		fEnableMemberFilterAction= new EnableMemberFilterAction(this, false);
		
		fFocusOnTypeAction= new FocusOnTypeAction(this);
		
		fPaneLabelProvider= new JavaElementLabelProvider(JavaElementLabelProvider.SHOW_BASICS);
		
		fAllViewers= new TypeHierarchyViewer[3];
		
		fDialogSettings= JavaPlugin.getDefault().getDialogSettings();
	}
	private void addHistoryEntry(IType entry) {
		for (int i= fInputHistory.size() - 1; i > fCurrHistoryIndex; i--) {
			fInputHistory.remove(i);
		}
		fInputHistory.add(entry);
		fCurrHistoryIndex++;
		fForwardAction.update();
		fBackwardAction.update();
	}
	private void addOpenPerspectiveItem(IMenuManager menu, IStructuredSelection selection) {
		OpenTypeHierarchyHelper.addToMenu(getSite().getWorkbenchWindow(), menu, selection);
	}
	private void addOpenWithMenu(IMenuManager menu, IStructuredSelection selection) {
		// If one file is selected get it.
		// Otherwise, do not show the "open with" menu.
		if (selection.size() != 1)
			return;

		Object element= selection.getFirstElement();
		if (!(element instanceof IJavaElement))
			return;
		IResource resource= null;
		try {
			resource= ((IJavaElement)element).getUnderlyingResource();	
		} catch(JavaModelException e) {
			// ignore
		}
		if (!(resource instanceof IFile))
			return; 

		// Create a menu flyout.
		MenuManager submenu= new MenuManager(TypeHierarchyMessages.getString("TypeHierarchyViewPart.menu.open")); //$NON-NLS-1$
		submenu.add(new OpenWithMenu(getSite().getPage(), (IFile) resource));

		// Add the submenu.
		menu.appendToGroup(IContextMenuConstants.GROUP_OPEN, submenu);
	}
	private void addRefactoring(IMenuManager menu, IInputSelectionProvider viewer){
		MenuManager refactoring= new MenuManager(TypeHierarchyMessages.getString("TypeHierarchyViewPart.menu.refactor")); //$NON-NLS-1$
		ContextMenuGroup.add(refactoring, new ContextMenuGroup[] { new RefactoringGroup() }, viewer);
		if (!refactoring.isEmpty())
			menu.appendToGroup(IContextMenuConstants.GROUP_REORGANIZE, refactoring);
	}
	private void clearInput() {
		fInput= null;
		fHierarchyLifeCycle.freeHierarchy();
		
		updateTypesViewer();
	}
	private Control createMethodViewerControl(Composite parent) {
	
		fMethodsViewer= new MethodsViewer(parent, this);
		fMethodsViewer.setMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager menu) {
				fillMethodsViewerContextMenu(menu);
			}
		});
		fMethodsViewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				methodSelectionChanged(event.getSelection());
			}
		});
		
		return fMethodsViewer.getTable();
	}
	/**
	 * Returns the inner component in a workbench part.
	 * @see IWorkbenchPart#createPartControl
	 */
	public void createPartControl(Composite container) {
						
		fPagebook= new PageBook(container, SWT.NONE);
						
		// page 1 of pagebook (viewers)

		fTypeMethodsSplitter= new SashForm(fPagebook, SWT.VERTICAL);
		fTypeMethodsSplitter.setVisible(false);

		ViewForm typeViewerViewForm= new ViewForm(fTypeMethodsSplitter, SWT.NONE);
		//setViewFormMargins(typeViewerViewForm);
		
		Control typeViewerControl= createTypeViewerControl(typeViewerViewForm);
		typeViewerViewForm.setContent(typeViewerControl);
				
		ViewForm methodViewerViewForm= new ViewForm(fTypeMethodsSplitter, SWT.NONE);
		//methodViewerViewForm.setTopCenterSeparate(true);
		//setViewFormMargins(methodViewerViewForm);
		fTypeMethodsSplitter.setWeights(new int[] {35, 65});
		
		
		Control methodViewerPart= createMethodViewerControl(methodViewerViewForm);
		methodViewerViewForm.setContent(methodViewerPart);
		
		fMethodViewerPaneLabel= new CLabel(methodViewerViewForm, SWT.NONE);
		methodViewerViewForm.setTopLeft(fMethodViewerPaneLabel);
		
		ToolBar methodViewerToolBar= new ToolBar(methodViewerViewForm, SWT.FLAT | SWT.WRAP);
		methodViewerViewForm.setTopCenter(methodViewerToolBar);
		
		// page 2 of pagebook (no hierarchy label)

		fNoHierarchyShownLabel= new Label(fPagebook, SWT.LEFT + SWT.WRAP);
		fNoHierarchyShownLabel.setText(TypeHierarchyMessages.getString("TypeHierarchyViewPart.empty")); //$NON-NLS-1$
		MenuManager menu= new MenuManager();
		menu.add(fFocusOnTypeAction);
		fNoHierarchyShownLabel.setMenu(menu.createContextMenu(fNoHierarchyShownLabel));
		
		fPagebook.showPage(fNoHierarchyShownLabel);
		
		// toolbar actions	
		IToolBarManager tbmanager= getViewSite().getActionBars().getToolBarManager();
		
		tbmanager.add(fBackwardAction);
		tbmanager.add(fForwardAction);
		
		for (int i= 0; i < fViewActions.length; i++) {
			ToggleViewAction action= fViewActions[i];
			tbmanager.add(action);
			action.setActive(fCurrentViewerIndex == action.getViewerIndex());
		}
		
		ToolBarManager lowertbmanager= new ToolBarManager(methodViewerToolBar);
		lowertbmanager.add(fEnableMemberFilterAction);			
		lowertbmanager.add(new Separator());
		fMethodsViewer.contributeToToolBar(lowertbmanager);
		lowertbmanager.update(true);
						
		fAddStubAction= new AddMethodStubAction(fMethodsViewer);
		updateViewerVisibility(false);
		
		
		// selection provider
		int nHierarchyViewers= fAllViewers.length; 
		Viewer[] trackedViewers= new Viewer[nHierarchyViewers + 1];
		for (int i= 0; i < nHierarchyViewers; i++) {
			trackedViewers[i]= fAllViewers[i];
		}
		trackedViewers[nHierarchyViewers]= fMethodsViewer;
		ISelectionProvider selProvider= new SelectionProviderMediator(trackedViewers);
		IStatusLineManager slManager= getViewSite().getActionBars().getStatusLineManager();
		selProvider.addSelectionChangedListener(new StatusBarUpdater(slManager));
		
		getSite().setSelectionProvider(selProvider);
		fFocusOnSelectionAction= new FocusOnSelectionAction(this, selProvider);		
		
		IType input= determineInputElement();
		if (fMemento != null) {
			restoreState(fMemento, input);
		} else if (input != null) {
			setInput(input);
		}
		
		// fixed for 1GETAYN: ITPJUI:WIN - F1 help does nothing
		WorkbenchHelp.setHelp(fPagebook, new ViewContextComputer(this, IJavaHelpContextIds.TYPE_HIERARCHY_VIEW));
	}
	private Control createTypeViewerControl(Composite parent) {
		
		fViewerbook= new PageBook(parent, SWT.NULL);
		
		ISelectionChangedListener selectionChangedListener= new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				if (!fIsEnableMemberFilter) {
					typeSelectionChanged(event.getSelection());
				}
			}
		};
		
		KeyListener keyListener= new KeyAdapter() {
			public void keyPressed(KeyEvent event) {
				if (event.keyCode == SWT.F3 || event.keyCode == SWT.F4) {
					IStructuredSelection structSel= (IStructuredSelection)getCurrentViewer().getSelection();
					if (structSel.size() == 1) {
						Object firstSelection= structSel.getFirstElement();
						if (event.keyCode == SWT.F4) {
							if (firstSelection instanceof IType) {
								new OpenTypeHierarchyHelper().open(
									new IType[] { (IType) firstSelection }, 
									getSite().getWorkbenchWindow());
								return;
							}
						} else {
							IAdaptable input= getSite().getPage().getInput();
							if (firstSelection instanceof IMember && input.getAdapter(IResource.class) instanceof IContainer) {
								PackageExplorerPart view= PackageExplorerPart.openInActivePerspective();
								if (view != null) {
									view.selectReveal(new StructuredSelection(JavaModelUtility.getOpenable((IMember)firstSelection)));
									return;
								}
							}
						}						
					}
					getCurrentViewer().getControl().getDisplay().beep();
				} else if (event.keyCode == SWT.F5) {
					updateTypesViewer();
				}	
			}
		};
				
		// Create the viewers
		final TypeHierarchyViewer superTypesViewer= new SuperTypeHierarchyViewer(fViewerbook, fHierarchyLifeCycle, this);
		superTypesViewer.getControl().setVisible(false);
		superTypesViewer.setMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager menu) {
				fillTypesViewerContextMenu(superTypesViewer, menu);
			}
		});
		superTypesViewer.addSelectionChangedListener(selectionChangedListener);		
		superTypesViewer.getControl().addKeyListener(keyListener);
		
		final TypeHierarchyViewer subTypesViewer= new SubTypeHierarchyViewer(fViewerbook, fHierarchyLifeCycle, this);
		subTypesViewer.getControl().setVisible(false);
		subTypesViewer.setMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager menu) {
				fillTypesViewerContextMenu(subTypesViewer, menu);
			}
		});
		subTypesViewer.addSelectionChangedListener(selectionChangedListener);
		subTypesViewer.getControl().addKeyListener(keyListener);
		
		final TypeHierarchyViewer vajViewer= new TraditionalHierarchyViewer(fViewerbook, fHierarchyLifeCycle, this);
		vajViewer.getControl().setVisible(false);
		vajViewer.setMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager menu) {
				fillTypesViewerContextMenu(vajViewer, menu);
			}
		});
		vajViewer.addSelectionChangedListener(selectionChangedListener);
		vajViewer.getControl().addKeyListener(keyListener);
		

		fAllViewers= new TypeHierarchyViewer[3];
		fAllViewers[VIEW_ID_SUPER]= superTypesViewer;
		fAllViewers[VIEW_ID_SUB]= subTypesViewer;
		fAllViewers[VIEW_ID_TYPE]= vajViewer;
		
		int currViewerIndex;
		try {
			currViewerIndex= fDialogSettings.getInt(DIALOGSTORE_HIERARCHYVIEW);
			if (currViewerIndex < 0 || currViewerIndex > 2) {
				currViewerIndex= VIEW_ID_TYPE;
			}
		} catch (NumberFormatException e) {
			currViewerIndex= VIEW_ID_TYPE;
		}
			
		fCurrentViewerIndex= currViewerIndex;
		fEmptyTypesViewer= new Label(fViewerbook, SWT.LEFT);
		
		for (int i= 0; i < fAllViewers.length; i++) {
			fAllViewers[i].setInput(fAllViewers[i]);
		}
				
		return fViewerbook;
	}
	/**
	 * Determines the input element to be used initially 
	 */	
	private IType determineInputElement() {
		Object input= getSite().getPage().getInput();
		if (input instanceof IType) { 
			return (IType)input;
		} 
		return null;	
	}
	/**
	 * @see IWorkbenchPart#dispose
	 */	
	public void dispose() {
		fHierarchyLifeCycle.freeHierarchy();
		fHierarchyLifeCycle.removeChangedListener(this);
		fPaneLabelProvider.dispose();
		super.dispose();
	}
	/**
	 * called from EnableMemberFilterAction
	 */	
	public void enableMemberFilter(boolean on) {
		if (on != fIsEnableMemberFilter) {
			fIsEnableMemberFilter= on;
			if (!on) {
				Object methodViewerInput= fMethodsViewer.getInput();
				IType input= getInput();
				setMemberFilter(null);
				if (methodViewerInput != null && getCurrentViewer().containsElement(methodViewerInput)) {
					// avoid that the method view changes content by selecting the previous input
					getCurrentViewer().setSelection(new StructuredSelection(methodViewerInput));
				} else if (input != null) {
					// choose a input that exists
					getCurrentViewer().setSelection(new StructuredSelection(input));
					updateMethodViewer(input);
				}
			} else {
				methodSelectionChanged(fMethodsViewer.getSelection());
			}
		}
	}
	/**
	 * Creates the context menu for the method viewer
	 */	
	private void fillMethodsViewerContextMenu(IMenuManager menu) {
		JavaPlugin.createStandardGroups(menu);
		// viewer entries
		fMethodsViewer.contributeToContextMenu(menu);
		if (fAddStubAction != null) {
			fAddStubAction.setParentType(fInput);
			if (fAddStubAction.canActionBeAdded()) {
				menu.appendToGroup(IContextMenuConstants.GROUP_REORGANIZE, fAddStubAction);
			}
		}
		menu.appendToGroup(IContextMenuConstants.GROUP_REORGANIZE, new JavaReplaceWithEditionAction(fMethodsViewer));	
		addOpenWithMenu(menu, (IStructuredSelection)fMethodsViewer.getSelection());
		addRefactoring(menu, fMethodsViewer);
	}
	/**
	 * Creates the context menu for the hierarchy viewers
	 */
	private void fillTypesViewerContextMenu(TypeHierarchyViewer viewer, IMenuManager menu) {
		JavaPlugin.createStandardGroups(menu);
		// viewer entries
		viewer.contributeToContextMenu(menu);
		IStructuredSelection selection= (IStructuredSelection)viewer.getSelection();
		addOpenPerspectiveItem(menu, selection);
		addOpenWithMenu(menu, selection);
		
		menu.appendToGroup(IContextMenuConstants.GROUP_SHOW, fFocusOnTypeAction);
		if (fFocusOnSelectionAction.canActionBeAdded())
			menu.appendToGroup(IContextMenuConstants.GROUP_SHOW, fFocusOnSelectionAction);
			
		addRefactoring(menu, viewer);
	}
	private TypeHierarchyViewer getCurrentViewer() {
		return fAllViewers[fCurrentViewerIndex];
	}
	private Display getDisplay() {
		if (fPagebook != null && !fPagebook.isDisposed()) {
			return fPagebook.getDisplay();
		}
		return null;
	}
	/**
	 * Gets the next or previous entry from the history
	 * @param forward If true, the next entry is returned, if
	 * if false, the previous
	 * @return The entry from the history or null if no entry available
	 */
	public IType getHistoryEntry(boolean forward) {
		if (forward) {
			int index= fCurrHistoryIndex + 1;
			if (index < fInputHistory.size()) {
				return (IType) fInputHistory.get(index);
			}
		} else {
			int index= fCurrHistoryIndex - 1;
			if (index >= 0) {
				return (IType) fInputHistory.get(index);
			}
		}
		return null;
	}
	/**
	 * Gets the current input (IType)
	 */	
	public IType getInput() {
		return fInput;
	}
	/**
	 * Gets the curret active view index
	 */		
	public int getViewIndex() {
		return fCurrentViewerIndex;
	}
	/**
	 * Goes to the next or previous entry from the history
	 * @param forward If true, the next entry is returned, if
	 * if false, the previous
	 */	
	public void gotoHistoryEntry(boolean forward) {
		IType elem= getHistoryEntry(forward);
		if (elem != null) {
			if (forward) {
				fCurrHistoryIndex++;
			} else {
				fCurrHistoryIndex--;
			}
			updateInput(elem);
			
			fForwardAction.update();
			fBackwardAction.update();
		}
	}
	/* (non-Javadoc)
	 * @see IViewPart#init
	 */
	public void init(IViewSite site, IMemento memento) throws PartInitException {
		super.init(site, memento);
		fMemento= memento;
	}
	private boolean isChildVisible(Composite pb, Control child) {
		Control[] children= pb.getChildren();
		for (int i= 0; i < children.length; i++) {
			if (children[i] == child && children[i].isVisible())
				return true;
		}
		return false;
	}
	private void methodSelectionChanged(ISelection sel) {
		if (sel instanceof IStructuredSelection) {
			List selected= ((IStructuredSelection)sel).toList();
			int nSelected= selected.size();
			if (fIsEnableMemberFilter) {
				IMember[] memberFilter= null;
				if (nSelected > 0) {
					memberFilter= new IMember[nSelected];
					selected.toArray(memberFilter);
				}
				setMemberFilter(memberFilter);
			}
			if (nSelected == 1) {
				revealElementInEditor(selected.get(0));
			}
		}
	}
	/*
	 * Restores the type hierarchy settings from a memento
	 */
	private void restoreState(IMemento memento, IType defaultInput) {
		IType input= defaultInput;
		String elementId= memento.getString(TAG_INPUT);
		if (elementId != null) {
			input= (IType) JavaCore.create(elementId);
		}
		setInput(input);

		String viewerIndex= memento.getString(TAG_VIEW);
		try {
			setView(Integer.parseInt(viewerIndex));
		} catch (NumberFormatException e) {
		}
		
		fMethodsViewer.restoreState(memento);
	}
	private void revealElementInEditor(Object elem) {
		IEditorPart editorPart= EditorUtility.isOpenInEditor(elem);
		if (editorPart != null && (elem instanceof ISourceReference)) {
			try {
				EditorUtility.openInEditor(elem, false);
				EditorUtility.revealInEditor(editorPart, (ISourceReference)elem);
			} catch (PartInitException e) {
				JavaPlugin.log(e);
			} catch (JavaModelException e) {
				JavaPlugin.log(e.getStatus());
			}
		}
	}
	/**
	 * @see ViewPart#saveState(IMemento)
	 */
	public void saveState(IMemento memento) {
		if (fPagebook == null) {
			// part has not been created
			if (fMemento != null) { //Keep the old state;
				memento.putMemento(fMemento);
			}
			return;
		}
		if (fInput != null) {
			memento.putString(TAG_INPUT, fInput.getHandleIdentifier());
		}
		memento.putString(TAG_VIEW, Integer.toString(getViewIndex()));
				
		fMethodsViewer.saveState(memento);
	}
	/**
	 * Selects an member in the methods list
	 */	
	public void selectMember(IMember member) {
		fMethodsViewer.setSelection(new StructuredSelection(member), true);
	}
	/**
	 * @see IWorbenchPart#setFocus
	 */	
	public void setFocus() {
		fPagebook.setFocus();
	}
	/**
	 * Sets the input to a new type
	 */
	public void setInput(IType type) {
		if (type != null) {
			ICompilationUnit cu= type.getCompilationUnit();
			if (cu != null && cu.isWorkingCopy()) {
				type= (IType)cu.getOriginal(type);
			}
		} 
		
		if (type != null && !type.equals(fInput)) {
			addHistoryEntry(type);
		}
			
		updateInput(type);
	}
	private void setMemberFilter(IMember[] memberFilter) {
		for (int i= 0; i < fAllViewers.length; i++) {
			fAllViewers[i].setMemberFilter(memberFilter);
		}
		updateTypesViewer();
		updateTitle();
	}
	/**
	 * Sets the current view (see view id)
	 * called from ToggleViewAction
	 */	
	public void setView(int viewerIndex) {
		if (viewerIndex < fAllViewers.length && viewerIndex != fCurrentViewerIndex) {
			fCurrentViewerIndex= viewerIndex;
			updateTypesViewer();
			if (fInput != null) {
				ISelection currSelection= getCurrentViewer().getSelection();
				if (currSelection == null || currSelection.isEmpty()) {
					getCurrentViewer().setSelection(new StructuredSelection(getInput()));
				}
				if (!fIsEnableMemberFilter) {
					typeSelectionChanged(getCurrentViewer().getSelection());
				}
			}
			updateTitle();
			fDialogSettings.put(DIALOGSTORE_HIERARCHYVIEW, viewerIndex);
		}
	}
	/**
	 * @see ITypeHierarchyLifeCycleListener#typeHierarchyChanged
	 * Can be called from any thread
	 */
	public void typeHierarchyChanged(TypeHierarchyLifeCycle typeHierarchy) {
		Display display= getDisplay();
		if (display != null) {
			display.syncExec(new Runnable() {
				public void run() {
					if (!fHierarchyLifeCycle.getHierarchy().exists()) {
						clearInput();
					} else {
						try {
							fHierarchyLifeCycle.ensureRefreshedTypeHierarchy(fInput);
						} catch (JavaModelException e) {
							JavaPlugin.log(e.getStatus());
							clearInput();
							return;
						}
						updateTypesViewer();
					}
				}
			});
		}
	}
	private void typeSelectionChanged(ISelection sel) {
		if (sel instanceof IStructuredSelection) {
			List selected= ((IStructuredSelection)sel).toList();
			int nSelected= selected.size();
			if (nSelected != 0) {
				List types= new ArrayList(nSelected);
				for (int i= nSelected-1; i >= 0; i--) {
					Object elem= selected.get(i);
					if (elem instanceof IType && !types.contains(elem)) {
						types.add(elem);
					}
				}
				if (types.size() == 1) {
					IType selectedType= (IType)types.get(0);
					if (!selectedType.equals(fMethodsViewer.getInput())) {
						updateMethodViewer(selectedType);
					}
				} else if (types.size() == 0) {
					// method selected, no change
				}
				if (nSelected == 1) {
					revealElementInEditor(selected.get(0));
				}
			} else {
				if (fMethodsViewer.getInput() != null) {
					updateMethodViewer(null);
				}
			}
		}
	}
	/**
	 * Changes the input to a new type
	 */
	public void updateInput(IType type) {
		boolean typeChanged= (type != fInput);
		
		fInput= type;
		if (fInput == null) {	
			clearInput();
		} else {
			// turn off member filtering
			fEnableMemberFilterAction.setChecked(false);
			enableMemberFilter(false);			
			
			try {
				fHierarchyLifeCycle.ensureRefreshedTypeHierarchy(fInput);
			} catch (JavaModelException e) {
				JavaPlugin.log(e.getStatus());
				clearInput();
				return;
			}
				
			fPagebook.showPage(fTypeMethodsSplitter);						
			
			if (typeChanged) {
				updateTypesViewer();
			}
			
			getCurrentViewer().setSelection(new StructuredSelection(fInput));
			
			updateMethodViewer(fInput);
			updateTitle();
		}
	}
	private void updateMethodViewer(IType input) {
		if (input != null) {
			fMethodViewerPaneLabel.setText(fPaneLabelProvider.getText(input));
			fMethodViewerPaneLabel.setImage(fPaneLabelProvider.getImage(input));
		} else {
			fMethodViewerPaneLabel.setText(""); //$NON-NLS-1$
			fMethodViewerPaneLabel.setImage(null);
		}
		fMethodsViewer.setInput(input);
	}
	private void updateTitle() {
		String title= getCurrentViewer().getTitle();
		setTitle(getCurrentViewer().getTitle());
		String tooltip;
		if (fInput != null) {
			String[] args= new String[] { title, JavaModelUtility.getFullyQualifiedName(fInput) };
			tooltip= TypeHierarchyMessages.getFormattedString("TypeHierarchyViewPart.tooltip", args); //$NON-NLS-1$
		} else {
			tooltip= title;
		}
		setTitleToolTip(tooltip);
	}
	/**
	 * When the input changed or the hierarchy pane becomes visible,
	 * <code>updateTypesViewer<code> brings up the correct view and refreshes
	 * the current tree
	 */
	public void updateTypesViewer() {
		if (fInput == null) {
			fPagebook.showPage(fNoHierarchyShownLabel);
		} else {
			if (getCurrentViewer().containsElements()) {
				Runnable runnable= new Runnable() {
					public void run() {
						getCurrentViewer().updateContent();
					}
				};
				Display display= getDisplay();
				if (display != null) {
					BusyIndicator.showWhile(display, runnable);
				} else {
					runnable.run();
				}
				if (!isChildVisible(fViewerbook, getCurrentViewer().getControl())) {
					updateViewerVisibility(false);
				}	
			} else {							
				fEmptyTypesViewer.setText(TypeHierarchyMessages.getFormattedString("TypeHierarchyViewPart.nodecl", fInput.getElementName()));				 //$NON-NLS-1$
				updateViewerVisibility(true);
			}
		}
	}
	/**
	 * Toggles between the empty viewer page and the hierarchy
	 */
	private void updateViewerVisibility(boolean showEmptyViewer) {
		if (!showEmptyViewer) {
			fViewerbook.showPage(getCurrentViewer().getControl());
		} else {
			fViewerbook.showPage(fEmptyTypesViewer);
		}
	}
}
