package org.eclipse.jdt.internal.ui.wizards.buildpaths;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
import java.net.URL;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IPath;

import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.JavaCore;

public class CPListElement {
			
	private int fEntryKind;
	private IPath fPath;
	private IResource fResource;
	
	private boolean fIsMissing;
	
	private IPath fSourceAttachmentPath;
	private IPath fSourceAttachmentPrefix;
	
	private IClasspathEntry fCachedEntry;
	
	public CPListElement(int entryKind, IPath path, IResource res) {
		this(entryKind, path, res, null, null);
	}
	///private URL fJavaDocLocation;
	
	public CPListElement(int entryKind, IPath path, IResource res, IPath attachPath, IPath attachRoot) {
		fEntryKind= entryKind;
		fPath= path;
		fSourceAttachmentPath= attachPath;
		fSourceAttachmentPrefix= attachRoot;	
		fResource= res;
		fIsMissing= false;
		
		fCachedEntry= null;
	}
	/**
	 * @see Object#equals(java.lang.Object)
	 */
	public boolean equals(Object other) {
		if (other.getClass() == getClass()) {
			CPListElement elem= (CPListElement)other;
			return elem.fEntryKind == fEntryKind && elem.fPath == fPath;
		}
		return false;
	}
	public IClasspathEntry getClasspathEntry() {
		if (fCachedEntry == null) {
			fCachedEntry= newClasspathEntry();
		}
		return fCachedEntry;
	}
	public int getEntryKind() {
		return fEntryKind;
	}
	public IPath getPath() {
		return fPath;
	}
	/**
	 * Entries without resource are either non existing or a variable entry
	 * External jars do not have a resource
	 */
	public IResource getResource() {
		return fResource;
	}
	/**
	 * Gets the current path prefix used when accessing the source attachment
	 * @see org.eclipse.jdt.core.IPackageFragmentRoot#getSourceAttachmentPath	 
	 * @return The source attachment prefix
	 */	
	public IPath getSourceAttachmentPath() {
		return fSourceAttachmentPath;
	}
	/**
	 * Returns the root path used for accessing source attchments
	 * @see org.eclipse.jdt.core.IPackageFragmentRoot#getSourceAttachmentRootPath
	 */
	public IPath getSourceAttachmentRootPath() {
		return fSourceAttachmentPrefix;
	}
	/**
	 * returns if entry is missing
	 * @return Returns a boolean
	 */
	public boolean isMissing() {
		return fIsMissing;
	}
	private IClasspathEntry newClasspathEntry() {
		switch (fEntryKind) {
			case IClasspathEntry.CPE_SOURCE:
				return JavaCore.newSourceEntry(fPath);
			case IClasspathEntry.CPE_LIBRARY:
				return JavaCore.newLibraryEntry(fPath, fSourceAttachmentPath, fSourceAttachmentPrefix);
			case IClasspathEntry.CPE_PROJECT:
				return JavaCore.newProjectEntry(fPath);
			default: // IClasspathEntry.CPE_VARIABLE:
				return JavaCore.newVariableEntry(fPath, fSourceAttachmentPath, fSourceAttachmentPrefix);
		}
	}
	/**
	 * Sets to be missing
	 */
	public void setIsMissing(boolean isMissing) {
		fIsMissing= isMissing;
	}
	/**
	 * Sets the paths for source annotation
	 * @see org.eclipse.jdt.core.IPackageFragmentRoot#attachSource	
	 * 
	 */	
	public void setSourceAttachment(IPath path, IPath prefix) {
		fSourceAttachmentPath= path;
		fSourceAttachmentPrefix= prefix;
		
		fCachedEntry= null;
	}
}
