package org.eclipse.scripting;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.io.IOException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.xml.sax.SAXException;

/**
 * This interface is implemented by scripting data models.  The data model
 * is also the root data element.
 * <p>
 * This interface is primarily intended for use 
 * by workbench users writing script code.
 * </p>
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under development and expected to
 * change significantly before reaching stability. It is being made available at this early stage to solicit feedback
 * from pioneering adopters on the understanding that any code that uses this API will almost certainly be broken
 * (repeatedly) as the API evolves.
 * </p>
 *
 * @see IDataElement
 * @see IToolkit#createDataModel
 * @see IToolkitUI#getUIDataModel
 */
public interface IDataModel extends IDataElement {
/**
 * Loads the data model contained in a workbench file.
 *
 * @param workspaceFile data model file to load
 * @exception java.io.IOException 
 * @exception org.xml.sax.SAXException
 * @exception org.eclipse.core.runtime.CoreException
 */
void loadFromFile(IFile workspaceFile)
	throws IOException, SAXException, CoreException;
/**
 * Loads a data model contained in a file system file.
 *
 * @param fileSystemFilename data model file name to load from
 * @exception java.io.IOException 
 * @exception org.xml.sax.SAXException
 */
void loadFromFilename(String fileSystemFilename)
	throws IOException, SAXException;
/**
 * Saves the current data model only if either a workbench file,
 * or a file system file name is associated with this data model through the
 * previous use of one of the following methods.
 * <ul>
 * <li>loadFromFile</li>
 * <li>loadFromFilename</li>
 * <li>saveToFile</li>
 * <li>saveToFilename</li>
 * </ul>
 *
 * @exception java.io.IOException 
 * @exception org.eclipse.core.runtime.CoreException
 */
void save() throws IOException, CoreException;
/**
 * Saves the current data model to the specified workspace file.  If
 * the data model is not currently associated with either a workspace
 * file or a file system file, then the data model is associated with
 * the specified workbench file.
 *
 * @param workspaceFile data model file to save to
 * @exception org.eclipse.core.runtime.CoreException
 */
void saveToFile(IFile workbenchFile) throws CoreException;
/**
 * Saves the current data model to the specified file system file.  If
 * the data model is not currently associated with either a workspace
 * file or a file system file, then the data model is associated with
 * the specified file system file.
 *
 * @param fileSystemFilename data model file name to save to
 * @exception java.io.IOException 
 */
void saveToFilename(String fileSystemFilename) throws IOException;
}
