package org.eclipse.scripting;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.core.runtime.model.ExtensionModel;
import org.eclipse.jface.wizard.Wizard;

/**
 * This interface is the abstract datatype used for scripting
 * preference pages. The scripting preference pages
 * are used to add or modify scripts registered into
 * specific extension points. 
 * <p>
 * This interface is primarily intended for use 
 * by developers who are script-enabling their workbench extensions.
 * </p>
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under development and expected to
 * change significantly before reaching stability. It is being made available at this early stage to solicit feedback
 * from pioneering adopters on the understanding that any code that uses this API will almost certainly be broken
 * (repeatedly) as the API evolves.
 * </p>
 *
 * @see ScriptAbstractPreferencePage
 * @see ScriptRunner
 */
public interface IScriptPreferencePage {
/**
 * Allows the preference page to add script type specific
 * wizard pages to the Add/Modify script wizard when the wizard is
 * being created as the result of an Add button, or Modify button press.
 * The newScript parameter indicates whether a new script is
 * being defined, or an existing script is being modified.
 *
 * @param wizard wizard with type-specific pages
 * @param newScript <code>true</code> if new script is being defined
 */
void addScriptWizardPages(Wizard wizard, boolean newScript);
/**
 * Returns the identification string of the plugin extension adapter.
 *
 * @return extension identifier
 */
String getAdapterID();
/**
 * Returns the extension point which this script will extend.
 *
 * @return extension point identifier
 */
String getExtensionPointID();
/**
 * Allows the preference page to modify a new or existing plugin extension.
 * This method is called when the add/modify wizard is completing.
 *
 * @param extensionModel extension model
 * @param newScript <code>true</code> if new extension is being modified
 */
void modifyPluginExtension(ExtensionModel extensionModel, boolean newScript);
/**
 * Allows the preference page to synchronize the states of
 * its wizard pages.
 */
void syncScriptWizardPages();
}
