package org.eclipse.scripting.internal;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * This class is a script extension. Given a file name that contains
 * a script, and the script language, the script is executed.  The
 * default language is javascript.
 */
import java.util.Iterator;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExecutableExtension;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Composite;

public class Script implements IExecutableExtension
{
	public    ScriptRunnerExtended _scriptRunner           = null;
	public    IResource[]          _resourceSelectionArray = null;
	public    IStructuredSelection _structuredSelection    = null;
/**
 *
 */
public Script() 
{
	super();
//	Trace.functionEntry( this, "Script" );	
//	Trace.functionExit ( this, "Script" );
}
/**
 * Converts the structured selection to an array if IResource.
 */
public IResource[] convertSelectionToResourceArray()
{
	IResource[] resourceSelectionArray = null;
	
	// Convert the "IStructuredSelection" to an array of "IResource"s
	//---------------------------------------------------------------
	if( _structuredSelection != null )
	{
		// Count the number of selected elements
		//--------------------------------------
		Iterator iterSelection = _structuredSelection.iterator();

		int iNumberOfSelected = 0;

		while( iterSelection.hasNext() )
		{
			if( iterSelection.next() instanceof IResource )
			{
				++iNumberOfSelected;
			}
		};
		
		// Create an array for the selected elements
		//------------------------------------------
		if( iNumberOfSelected > 0 )
		{
			resourceSelectionArray = new IResource[ iNumberOfSelected ];

			iterSelection = _structuredSelection.iterator();

			Object objSelected = null;
			int i = 0;
			
			while( iterSelection.hasNext() )
			{
				objSelected = iterSelection.next();
				if( objSelected instanceof IResource )
				{
  					resourceSelectionArray[ i++ ] = (IResource)objSelected;
				}
			}
		}
	}

	// Always have a selection array
	//------------------------------
	if( resourceSelectionArray == null )
	{
		resourceSelectionArray = new IResource[0];
	}
	
	return resourceSelectionArray;
}
/**
 * 
 */
public void run()
{
//	Trace.functionEntry( this, "run" );

	if( _scriptRunner != null )
	{
		// Convert selection object to an array of selections
		//---------------------------------------------------
		_resourceSelectionArray = convertSelectionToResourceArray();
	
		_scriptRunner.initialize( _resourceSelectionArray );
		_scriptRunner.run( (IXGRElement)null  );
	}
	
//	Trace.functionExit( this, "run" );
}
/**
 * Run a script with user interface.
 */
public void run( IXGRElement element )
{
//	Trace.functionEntry( this, "run" );

	if( _scriptRunner != null )
	{
		// Convert selection object to an array of selections
		//---------------------------------------------------
		_resourceSelectionArray = convertSelectionToResourceArray();
		
		_scriptRunner.initialize( _resourceSelectionArray );	
		_scriptRunner.run( element );
	}

//	Trace.functionExit( this, "run" );
}
/**
 * Run a script with user interface.
 */
public void run( Composite parent )
{
//	Trace.functionEntry( this, "run" );
	
	if( _scriptRunner != null )
	{
		// Convert selection object to an array of selections
		//---------------------------------------------------
		_resourceSelectionArray = convertSelectionToResourceArray();
	
		_scriptRunner.initialize( _resourceSelectionArray );	
		_scriptRunner.run( parent );
	}

//	Trace.functionExit( this, "run" );
}
/**
 * 
 */
public void setInitializationData( IConfigurationElement configElement,
								   String                strName,
								   Object                objData )
{
//	Trace.functionEntry( this, "setInitializationData" );

	_scriptRunner = new ScriptRunnerExtended( configElement, objData );	
	
//	Trace.functionExit( this, "setInitializationData" );
}
}
