package org.eclipse.scripting.internal;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * This class contains details about a runnable script, including
 * the filename where the script was obtained from, the line number
 * and column number within the file, the script language, and the 
 * script text itself.
 */
public class ScriptDescriptor 
{
	public String _strFilename       = null;
	public String _strJavaMethod     = null;
	public String _strScriptLanguage = null;
	public String _strScriptText     = null;
	public int    _iColumnNumber     = 0;
	public int    _iLineNumber       = 0;
/**
 * ScriptDescriptor default constructor.
 */
public ScriptDescriptor() {
	super();
}
/**
 * Constructor.
 * @param strScriptText java.lang.String
 * @param strLanguage java.lang.String
 * @param strFilename java.lang.String
 * @param iLineNumber int
 * @param iColumnNumber int
 */
public ScriptDescriptor( String strScriptText,
	                     String strScriptLanguage,
	                     String strFilename,
	                     int    iLineNumber, 
	                     int    iColumnNumber ) 
{
	_strScriptText     = strScriptText;
	_strScriptLanguage = strScriptLanguage;
	_strFilename       = strFilename;
	_iLineNumber       = iLineNumber;
	_iColumnNumber     = iColumnNumber;

	// Determine whether a java method has been specified
	// <script>java:package.method</script>
	//---------------------------------------------------
	if( strScriptText != null )
	{
		if( strScriptText.trim().toLowerCase().indexOf( "java:" ) == 0 )
		{
			_strJavaMethod     = strScriptText.trim().substring( 5 );
			_strScriptLanguage = "java";
			_strScriptText     = null;
		}
	}


	// Eliminate unwanted url style file prepend
	//------------------------------------------
	if( _strFilename != null )
	{
		if( _strFilename.indexOf( "file:/" ) == 0 )
		{
			_strFilename = _strFilename.substring( 6 );
		}
	}
	
//	Trace.string( toString() );
}
/**
 *
 */
public String toString()
{
	return new String( "Language=" + _strScriptLanguage + "\nJavaMethod=" + _strJavaMethod + "\nFile=" + _strFilename + "\nLine=" + Integer.toString( _iLineNumber ) + "\nColumn=" + Integer.toString( _iColumnNumber ) + "\nText=" + _strScriptText );
}
}
