package org.eclipse.scripting.internal;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * This class reads the contents of an ascii file system file.
 */
import java.io.IOException;
import java.io.RandomAccessFile;
import org.eclipse.scripting.internal.IXGRReader;

public class ScriptReaderFileSystemFile implements IXGRReader {
	
	protected RandomAccessFile raf_File = null;
	protected String str_Filename = null;

/**
 * Constructor.
 */
public ScriptReaderFileSystemFile(String strFilename) throws Exception {

	str_Filename = strFilename;

	// Attempt to open the file if not already open
	//---------------------------------------------
	if (raf_File == null) {
	    
		try {
			// Attempt to open the file in read mode
			//--------------------------------------
			raf_File = new RandomAccessFile(str_Filename, "r");
		}

		catch (IOException exception) {
			throw (exception);
		}
	}
}
/**
 * The file is closed.
 */
protected void finalize() {
	
	if (raf_File != null) {
	    
		try {
			raf_File.close();
		}
		
		catch (IOException exception) {
			
			// Ignore the error
			//-----------------
		}

		raf_File = null;
	}
}
/**
 * Obtain the zero based file line
 */
public String getLine(int iLineNumber) {

	String strLine = null;

	try {
		// Read up to, but not including the required line
		//------------------------------------------------
		for (int i = 0; i < iLineNumber; ++i) {
			raf_File.readLine();
		}

		// Read the required line itself
		//------------------------------
		strLine = raf_File.readLine();

		// Replace all tabs with spaces
		//-----------------------------
		if (strLine != null) {
			strLine = strLine.replace('\t', ' ');
		}
	}

	catch (IOException exception) {
		strLine = null;
	}
	
	return strLine;
}
/**
 * Reads a line of ascii text from the file.
 */
public String readLine() throws IOException {
	
	// Read a line
	//------------
	String strLine = null;

	try {

		strLine = raf_File.readLine();

		// Replace all tabs with spaces
		//-----------------------------
		if (strLine != null) {

			strLine = strLine.replace('\t', ' ');
		}
	}

	catch (IOException exception) {

//		Trace.string(exception.toString());
		throw (exception);
	}

	return strLine;
}
}
