package org.eclipse.scripting.internal;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * This class reads the contents of a workbench IFile.
 */
import java.io.*;
import org.eclipse.core.resources.*;
import org.eclipse.scripting.internal.IXGRReader;


public class ScriptReaderWorkbenchFile implements IXGRReader
{
	protected LineNumberReader _reader = null;
/**
 * This constructor creates a new line number reader from an IFile's input stream
 */
public ScriptReaderWorkbenchFile( IFile file ) throws Exception 
{
	if( file != null )
	{
		// Move the contents from the file to an input stream
		// Convert it to an InputSource, and the parse the contents
		//---------------------------------------------------------
		InputStream inputStream = file.getContents(false);
		
		if( inputStream != null )
		{
			_reader = new LineNumberReader( new InputStreamReader( file.getContents(false) ) );
		}
	}
}
/**
 * Obtain the zero based file line
 */
public String getLine( int iLineNumber ) 
{
//	Trace.functionEntry( this, "readLine" );

	String strLine = null;

	try
	{
		// Read up to, but not including the required line
		//------------------------------------------------
		for( int i=0; i<iLineNumber; ++i )
		{
			_reader.readLine();
		}

		// Read the required line itself
		//------------------------------
		strLine = _reader.readLine();

		// Replace all tabs with spaces
		//-----------------------------
		if( strLine != null )
		{
			strLine = strLine.replace( '\t', ' ' );
		}
	}
		
	catch( IOException exception )
	{
		strLine = null;
	}


//	Trace.functionExit( this, "readLine" );
	return strLine;
}
/**
 * Return a line of text
 */
  public String readLine() throws IOException
  {
//	Trace.functionEntry( this, "readLine" );

	// Read a line
	//------------
	String strLine = null;

	try
	{
		strLine = _reader.readLine();

		// Replace all tabs with spaces
		//-----------------------------
		if( strLine != null )
		{
			strLine = strLine.replace( '\t', ' ' );
		}
	}

	catch( IOException exception )
	{
//	  Trace.string( exception.toString() );
	  throw( exception );
	}

//	Trace.functionExit( this, "readLine" );
	return strLine;
  }  
}
