package org.eclipse.scripting.internal;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.swt.widgets.*;
/**
 * This class extends the ScriptTookit class by providing additional
 * toolkit functions that are user interface specific.
 */
import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;

import org.eclipse.scripting.*;
import org.eclipse.scripting.internal.uirenderer.*;
import org.eclipse.scripting.internal.*;

public class ScriptToolkitUI extends ScriptToolkit implements IToolkitUI
{
	protected XGR    _xgr            = null;
	protected Object _objEventWidget = null;
/**
 * ScriptUIToolkit constructor.
 */
public ScriptToolkitUI() 
{
	super( null, null ); 
}
/**
 * ScriptUIToolkit constructor.
 */
public ScriptToolkitUI( Plugin plugin, IResource[] resourceSelectionArray ) 
{
	super( plugin, resourceSelectionArray ); 
}
/**
 * Returns a data identifier given a user interface identifier.
 */
public String getDataSource( String strID )
{
	if( strID != null && _xgr != null )
	{
		return _xgr.getDataSource( strID );
	}
	
	return null;
}
/**
 * Returns the widget that generated the current event.
 */
public Object getEventWidget() {
	return _objEventWidget;
}
/**
 * Returns the user interface element that generates a link event for the given user
 * interface element.
 */
public String getLinkSource( String strID )
{
	if( strID != null && _xgr != null )
	{
		return _xgr.getLinkSource( strID );
	}
	
	return null;
}
/**
 * Returns the array of selected desktop resources.
 */
public IResource[] getResourcesSelected()
{
	return _resourceSelectionArray;
}
/**
 * Returns the data model that is currently associated with the user interface.
 */
public IDataModel getUIDataModel()
{
	return _xgr != null ? _xgr.getDataModel() : null;
}
/**
 * Returns the widget that has the given user interface element ID.
 */
public Object getWidget( String userInterfaceElementID )
{
	if( _xgr != null )
	{
		return _xgr.getWidget( userInterfaceElementID );
	}

	return null;
}
/**
 * Displays a dialog with ok button.
 */
public void promptForOk(String strTitle, String strMessage, String strMessageType) {

	if (_objEventWidget instanceof Control)
		_shell = ((Control) _objEventWidget).getShell();

	super.promptForOk(strTitle, strMessage, strMessageType);
}
/**
 * Displays a dialog with Ok and Cancel buttons.  False is returned if the
 * Cancel button was pressed.  True is returned if the Ok button was pressed.
 */
public boolean promptForOkCancel(String strTitle, String strMessage) {

	if (_objEventWidget instanceof Control)
		_shell = ((Control) _objEventWidget).getShell();

	return super.promptForOkCancel(strTitle, strMessage);
}
/**
 * Displays the system file selection dialog and uses
 * the FileDialog class.
 * The dialog will be displayed as either an open, or saveas dialog.
 * A single
 * IPath is returned, or null if no file was selected.
 */
public IPath promptForSystemFile(boolean bSaveNotOpen) throws Exception {

	if (_objEventWidget instanceof Control)
		_shell = ((Control) _objEventWidget).getShell();

	return super.promptForSystemFile(bSaveNotOpen);
}
/**
 * This method displays the system directory selection dialog and uses
 * the org.eclipse.swt.widgets.DirectoryDialog.
 * The dialog will be displayed as either an open, or saveas dialog.
 * A single IPath is returned, or null if no directory was selected.
 */
public IPath promptForSystemFolder() throws Exception {

	if (_objEventWidget instanceof Control)
		_shell = ((Control) _objEventWidget).getShell();

	return super.promptForSystemFolder();
}
/**
 * Displays the container selection dialog.
 * The dialog will be displayed with the given IFolder as the root.
 * A single
 * IFolder is returned.  If the folderRoot parameter is null, the
 * open solution is used as the root folder.
 */
public IContainer promptForWorkspaceContainer(IContainer containerRoot, String strMessage) throws Exception {

	if (_objEventWidget instanceof Control)
		_shell = ((Control) _objEventWidget).getShell();

	return super.promptForWorkspaceContainer(containerRoot, strMessage);
}
/**
 * Displays the resource selection dialog.  An array
 * of IResources is returned.
 */
public IFile[] promptForWorkspaceFiles(IContainer containerRoot, String strMessage) throws Exception {

	if (_objEventWidget instanceof Control)
		_shell = ((Control) _objEventWidget).getShell();

	return super.promptForWorkspaceFiles(containerRoot, strMessage);
}
/**
 * Assocates a data source identifier with a user interface element.
 */
public boolean setDataSource( String strID, String strDataSource )
{
	if( strID != null && strDataSource != null && _xgr != null )
	{
		return _xgr.setDataSource( strID, strDataSource );
	}
	
	return false;
}
/**
 * Sets the widget that generated the current event.
 */
public void setEventWidget(Object objEventWidget) {
	_objEventWidget = objEventWidget;
}
/**
 * Sets the GUI renderer that renders the user interface.
 */
public void setGuiRenderer( XGR xgr )
{
	_xgr = xgr;
}
/**
 * Associates a user interface element that will provide link events to the specified
 * destination user interface element.
 */
public boolean setLinkSource( String strID, String strLinkSource )
{
	if( strID != null && strLinkSource != null && _xgr != null )
	{
		return _xgr.setLinkSource( strID, strLinkSource );
	}
	
	return false;
}
/**
 * Associates a data source with a user interface element.
 */
public boolean setStringDataSource( String      strID, 
									IStringData stringData )
{
	if( strID != null && stringData != null && _xgr != null )
	{
		return _xgr.setStringDataSource( strID, stringData );
	}
	
	return false;
}
/**
 * Sets the data model that is to be used by the user interface elements.
 */
public void setUIDataModel( IDataModel dataModel ) 
{
	// All data models are of type IXGRDataModel
	//------------------------------------------
	if( _xgr != null && dataModel instanceof IXGRDataModel )
	{
		_xgr.setDataModel( (IXGRDataModel)dataModel );
	}
	
	return;
}
}
