package org.eclipse.scripting.internal.datamodels;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import org.eclipse.scripting.internal.*;

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;

import org.apache.xerces.parsers.SAXParser;
import org.xml.sax.*;
import java.io.*;
import java.util.*;
import java.net.*;

public class XGRXmlDataModel extends XGRXmlDataNode implements IXGRDataModel
{
	protected IFile                _file                    = null;
	protected String               _strDocumentDeclarations = null;
	protected String               _strFilename             = null;
	
	private static final String _strIndentation = "    ";
/**
 */
public XGRXmlDataModel()
{
	// I am the root element with no parent, no value, of type unknown
	//----------------------------------------------------------------
	super( null, "ROOT", null, ELEMENT );

//  Trace.functionEntry( this, "XGRXmlDataModel" );
//	Trace.functionExit( this, "XGRXmlDataModel" );	
}
/**
 * This method returns an array of data nodes given a string path such as
 * "abc.de.fghi".
 */
public IXGRDataNode[] getDataNodes( String strDataPath )
{
//	Trace.functionEntry( this, "getDataNodes" );
//	Trace.string( strDataPath );

	IXGRDataNode[] dataNodes = null;

	if( strDataPath != null )
	{
		// Get the root node from the subclass
		//------------------------------------
		IXGRDataNode nodeRoot = getSearchRootNode();

//		Trace.object( "nodeRoot", nodeRoot );
		
		if( nodeRoot != null )
		{
			dataNodes = nodeRoot.getNodes( strDataPath );
		}
	}

//	Trace.functionExit( this, "getDataNodes" );
	return dataNodes;
}
/**
 * Given a fully qualified data path name, and a starting node,
 * returns all subnodes whose fully qualified name matches the fully qualified data path name.
 */
public IXGRDataNode[] getDataNodes( String strDataPath, IXGRDataNode dataNodeStart ) 
{
//	Trace.functionEntry( this, "getDataNodes( String, IXGRElementDataNode )" );

	IXGRDataNode[] nodeaReturn = null;

	if( dataNodeStart != null && dataNodeStart instanceof XGRXmlDataNode )
	{
		if( ((XGRXmlDataNode)dataNodeStart)._iXmlType == IXGRDataNodeXml.ATTRIBUTE )
		{
			dataNodeStart = ((XGRXmlDataNode)dataNodeStart).getParentNode();
		}
	}
		
	if( strDataPath != null && dataNodeStart != null )
	{
		// Determine end of data path
		// Sample
		//   strDataPath:       a.b.c.d...
		//   strDataPathStart:  a.b
		//   strDataPathSearch: c.d..
		//---------------------------------------------------
		String strDataPathStart = dataNodeStart.getFullyQualifiedName();

//		Trace.string( "strDataPath: " + strDataPath );
//		Trace.string( "strDataNodeStart: " + strDataPathStart );

		if( strDataPathStart != null )
		{
			int iIndex = strDataPath.indexOf( strDataPathStart );
	
			if( iIndex == 0 )
			{
				// If the starting node is the same as the data path
				// return it
				//--------------------------------------------------
				if( strDataPath.equals( strDataPathStart ) == true )
				{
					nodeaReturn = new IXGRDataNode[1];
					nodeaReturn[0] = dataNodeStart;
				}

				// Otherwise, search for the data path
				//------------------------------------
				else
				{
					String strSearch = strDataPath.substring( strDataPathStart.length() + 1 );

//					Trace.string( "strSearch: " + strSearch );
					nodeaReturn = dataNodeStart.getNodes( strSearch );
				}
			}
		}
	}

//	Trace.functionExit( this, "getDataNodes( String, IXGRElementDataNode )" );
	
	return nodeaReturn;
}
/**
 */
protected String getDirectory() 
{
	String strDirectory = null;
	
	if( _file != null )
	{
		// Was getLocalLocation
		//---------------------
		strDirectory = _file.getFullPath().toOSString();

		strDirectory = strDirectory.replace( '/', '\\' );

		strDirectory = strDirectory.substring( 0, strDirectory.lastIndexOf( '\\' ) );
	}

	else if( _strFilename != null )
	{
		int iIndex = _strFilename.lastIndexOf( '\\' );
		if( iIndex > -1 )
		{
			strDirectory = _strFilename.substring( 0, iIndex );
		}
	}

	return strDirectory;
}
/**
 */
public XGRXmlSaxHandler getSaxHandler() 
{
	return new XGRXmlSaxHandler( this, getDirectory() );
}
/**
 */
public IXGRDataNode getSearchRootNode() 
{
	return this;
}
/**
 */
public void loadFromFile( IFile file ) throws IOException, SAXException, CoreException
{
//	Trace.functionEntry( this, "load( IFile )" );

	// Create an org.xml.sax.InputSource, and
	// continue to load from it
	//---------------------------------------
	if( file != null )
	{
		_file = file;

		// Move the contents from the file to an input stream
		// Convert it to an InputSource, and the parse the contents
		//---------------------------------------------------------
		InputStream inputStream = file.getContents(false);

		if( inputStream != null )
		{
			InputSource inputSource = new InputSource( inputStream );
			
			// Parse the data model
			//---------------------
			parseDataModel( inputSource );
		}
	}
	
	else
	{
		ScriptReporter.reportError( ScriptStrings.getString( "S_Data_model_file_has_not_been_specified" ), null );
	}

//	Trace.functionExit( this, "load( IFile )" );
}
/**
 */
public void loadFromFilename(String strFilename) throws IOException, SAXException {
	//	Trace.functionEntry( this, "load( String )" );
	//	Trace.string( strFilename );

	// Create an input source from the file name
	//------------------------------------------
	InputSource inputSource = null;

	// Attempt to create a reader from a URL
	//--------------------------------------
	if (strFilename != null) {
		_strFilename = strFilename;

		URL url = null;

		try {
			url = new URL(strFilename);

			InputStream inputStream = url.openStream();
			inputSource = new InputSource(inputStream);
		}
		catch (MalformedURLException ex) {
			inputSource = null;
		}
		catch (IOException ex) {
	        inputSource = null;
		}
	}

	// Attempt to create a reader from a filename
	//-------------------------------------------
	if (inputSource == null && strFilename != null) {
		// Save the filename for writing
		//------------------------------
		_strFilename = strFilename;

		// Create an org.xml.sax.InputSource, and
		// continue to load from it
		//---------------------------------------
		BufferedReader bufferedReader = null;

		try {
			// Attempt to open the file in read mode
			//--------------------------------------
			FileReader fileReader = new FileReader(strFilename);

			bufferedReader = new BufferedReader(fileReader);
		}

		catch (IOException exception) {
			throw (exception);
		}

		if (bufferedReader != null) {
			// Send in the filename instead of the buffered reader
			// This will give the XGRXmlSaxHandler.resolveEntity()
			// the xml filename to resolve, so that we can find
			// the dtd.  
			//----------------------------------------------------
			inputSource = new InputSource(strFilename);
			//			Trace.object( "inputSource", inputSource );
		}
	}

	// Parse the contents of the input source
	//---------------------------------------
	if (inputSource != null) {
		parseDataModel(inputSource);
	}

	else {
		ScriptReporter.reportError(ScriptStrings.getString("S_Data_model_filename_has_not_been_specified"), null);
	}

	//	Trace.functionExit( this, "load( String )" );
}
/**
 */
protected void parseDataModel(InputSource inputSource)  throws IOException, SAXException
{
//	Trace.functionEntry( this, "load( InputSource )" );
	
	// Create a sax parser and parse the file
	//---------------------------------------
	if( inputSource != null )
	{
		SAXParser saxParser = new SAXParser();

		// Obtain the sax handler
		// Subclasses can supply their own handler
		//----------------------------------------
		XGRXmlSaxHandler saxHandler = getSaxHandler();

		saxParser.setContentHandler( saxHandler );
		saxParser.setErrorHandler   ( saxHandler );
		saxParser.setEntityResolver ( saxHandler );

		// This will throw an exception if the handler determines
		// that the model doesn't match its expected model
		//-------------------------------------------------------
		saxParser.parse( inputSource );

		// Get the text before the first element
		//--------------------------------------
		_strDocumentDeclarations = saxHandler.getDocumentDeclarations();
	}

//	Trace.functionExit( this, "load( InputSource )" );
}
/**
 */
public void save() throws IOException, CoreException
{
	if( _strFilename != null )
	{
		saveToFilename( _strFilename );
	}

	else if( _file != null )
	{
		saveToFile( _file );
	}

	else
	{
		ScriptReporter.reportError( ScriptStrings.getString( "S_Cannot_save_data_model_without_file_specification" ), null );
	}
}
/**
 * This method writes out the data model to the specified workbench file.
 */
public void saveToFile( IFile file ) throws CoreException
{
	if( file != null )
	{
		// Write the data model to a string buffer
		//----------------------------------------
		StringBuffer buffer = new StringBuffer();

		writeDataModel( buffer );

		// Write the string buffer out to the IFile
		//-----------------------------------------
		InputStream inputStream = new ByteArrayInputStream( buffer.toString().getBytes() );
		
		// Set the contents of the file
		//-----------------------------
		file.setContents( inputStream, true, false, null );

		// Set the file to the new file for subsequent saves
		//--------------------------------------------------
		if( _file == null && _strFilename == null )
		{
			_file = file;
		}
	}
	
	else
	{
		ScriptReporter.reportError( ScriptStrings.getString( "S_Cannot_save_to_null_workbench_file" ), null );
	}
}
/**
 * This method writes out the data model to the specified file
 * system file.
 */
public void saveToFilename( String strFilename ) throws IOException
{
//	Trace.functionEntry( this, "saveDataModel" );

	if( strFilename != null )
	{
		// Create a file writer
		// The original file contents will be discarded
		//---------------------------------------------
		FileWriter writer = null;

		try
		{
			writer = new FileWriter( strFilename );
		}
		catch( IOException ex )
		{
			ScriptReporter.reportException( ex );
			throw( ex );
		}

		// Write the XML data model to the file
		//-------------------------------------
		if( writer != null )
		{
			// Write the model to a string buffer
			//-----------------------------------
			StringBuffer buffer = new StringBuffer();

			writeDataModel( buffer );

			// Write out the string buffer to the file
			//----------------------------------------
			try
			{
				writer.write( buffer.toString(), 0, buffer.length() );
				writer.close();

				// Save the filename for subsequent saves only
				// if there was no file name before
				//---------------------------------------
				if( _file == null && _strFilename == null )
				{
					_strFilename = strFilename;
				}
			}
			
			catch( IOException ex )
			{
				ScriptReporter.reportException( ex );
				throw( ex );
			}
		}
	}

	else
	{
		ScriptReporter.reportError( ScriptStrings.getString( "S_Cannot_save_to_null_system_filename" ), null );
	}
	
//	Trace.functionExit( this, "saveDataModel" );
}
/**
 */
public String toString()
{
	StringBuffer strbuf = new StringBuffer();
	
	if( _file != null )
	{
		strbuf.append( _file.getFullPath().toOSString() );
	}

	else if( _strFilename != null )
	{
		strbuf.append( _strFilename );
	}

	else
	{
		strbuf.append( "null" );
	}

	return strbuf.toString();
}
/**
 * This method writes the data model using the given Writer.
 * The original contents of the file are discarded.
 */
protected void writeDataModel( StringBuffer buffer )
{
//	Trace.functionEntry( this, "writeDataModel" );

	// Write out header text such as:
	//  <?xml version="1.0" encoding="UTF-8"?>
	//  <!DOCTYPE company SYSTEM "excalibur.dtd">
	//-------------------------------------------
	if( _strDocumentDeclarations != null )
	{
		// Strip off trailing carriage returns
		//------------------------------------
		do
		{
			if( _strDocumentDeclarations.charAt( _strDocumentDeclarations.length() - 1 ) == '\n' )
			{
				_strDocumentDeclarations = _strDocumentDeclarations.substring( 0, _strDocumentDeclarations.length() - 1 );
			}
			else
			{
				break;
			}
		} while( true );
		
		buffer.append( _strDocumentDeclarations );
	}
	
	// Do not write the root element ROOT
	// Recursively write out its children 
	//-----------------------------------
	Vector vectorNodes = getChildNodes();

	for( int i=0; i<vectorNodes.size(); ++i )
	{
		writeElementPre( (XGRXmlDataNode)vectorNodes.elementAt( i ), buffer, -1 );
	}

//	Trace.functionExit( this, "writeDataModel" );
}
/**
 */
protected void writeElementPost( IXGRDataNode node, StringBuffer buffer, int iIndent )
{
//	Trace.functionEntry( this, "writeElementPost" );

	// If there are no child elements or attributes
	// write the beginning of the next line
	//---------------------------------------------
	if( node.getChildNodes() != null )
	{
		writeIndent( buffer, iIndent );
	}

	// Write the end element tag
	//--------------------------
	buffer.append("</" + node.getName() + ">");
	
//	Trace.functionExit( this, "writeElementPost" );	
}
/**
 */
protected void writeElementPre( XGRXmlDataNode node, StringBuffer buffer, int iIndent )
{
//	Trace.functionEntry( this, "writeElementPre" );

	// Ensure this is a valid element node
	//------------------------------------	
	if( node == null || node.getXmlType() == XGRXmlDataNode.ATTRIBUTE )
	{
//		Trace.functionExit( this, "writeElementPre" );
		return;
	}

	// Write out the indentation spaces
	//---------------------------------
	writeIndent( buffer, ++iIndent );

	// Write out the element begin tag
	//--------------------------------
	buffer.append( "<" + node.getName() );

	// Write the attributes
	//---------------------
	Vector attrList = node.getAttributesVector();
	
	for( int i=0; i<attrList.size(); i++ )
	{
		IXGRDataNode attr = (IXGRDataNode)attrList.elementAt( i );
		buffer.append( " " + attr.getName() + "=\"" + attr.getValue() + "\"" );
	}

	// Write the element's end tag if no child elements or all children are attributes
	//--------------------------------------------------------------------------------
	if( node.getValue() == null && (node.getChildNodes() == null || node.getChildNodes().size() == attrList.size()) )
	{
		buffer.append( "/>" );
	}

	// Otherwise end the begin element tag and
	// write out the value of this element
	//----------------------------------------
	else
	{
		buffer.append(">");

		if( node.getValue() != null )
		{
			// Go to the next line if there are attributes, or child elements
			//---------------------------------------------------------------
			if( node.getChildNodes() != null )
			{
				writeIndent( buffer, iIndent + 1 );
			}

			// Write out the value of this attribute
			//--------------------------------------
			buffer.append( node.getValue() );
		}

		// Recursively write out my child elements
		//----------------------------------------
	    if( node.getChildNodes() != null )
	    {
		    XGRXmlDataNode nodeChild = null;
		    
			for( int i=0; i<node.getChildNodes().size(); ++i )
	 		{
		 		nodeChild = (XGRXmlDataNode)node.getChildNodes().elementAt( i );
		 	 	writeElementPre( nodeChild, buffer, iIndent );
			}
		}

	    writeElementPost( node, buffer, iIndent );		 	  	    
	}

//	Trace.functionExit( this, "writeElementPre" );
}
/**
 * This method writes out a carriage return, and line indentation spaces
 * according to the indentation amount.
 */
protected void writeIndent( StringBuffer buffer, int iIndentation )
{
	// Write out carriage return
	//--------------------------
	buffer.append( "\n" );

	// Write out indentation string
	//-----------------------------
	for( int i=0; i<iIndentation; i++ )
	{
		buffer.append( _strIndentation );
	}
}
}
