package org.eclipse.scripting.internal.datamodels;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import java.util.Vector;
import org.eclipse.scripting.*;
import org.eclipse.scripting.internal.*;

public class XGRXmlDataNode extends XGRBaseDataNode	implements IXGRDataNodeXml,
															   IXGRDataChangedNotifier
{
	protected String _strName    = null;
	protected String _strValue   = null;
	protected int 	 _iXmlType   = IXGRDataNodeXml.UNKNOWN;

	protected Vector _dataChangedListeners = new Vector();

/**
 */
public XGRXmlDataNode( IXGRDataNode nodeParent, 
	 				   String       strName, 
	 				   String       strValue,
	 				   int          iXmlType )
{
	super( nodeParent );

	_strName  = strName;
	_strValue = strValue;
	_iXmlType = iXmlType;
}
/**
 */
public boolean addAttribute( String strAttributeName, String strAttributeValue) 
{
	if( strAttributeName != null && strAttributeValue != null )
	{
		if( getAttribute( strAttributeName ) == null )
		{
			// This constructor adds the attribute to the parent element's list of attributes
			//-------------------------------------------------------------------------------
			XGRXmlDataNode nodeNew = new XGRXmlDataNode( this, strAttributeName, strAttributeValue, ATTRIBUTE );
			return true;
		}
	}
	
	return false;
}
/**
 */
public IDataElement addChildElement( String strElementName )
{
	XGRXmlDataNode nodeChild = null;
	
	if( strElementName != null && _iXmlType == ELEMENT )
	{
		// This constructor will add the new element to its parent
		//--------------------------------------------------------
		nodeChild = new XGRXmlDataNode( this, strElementName, null, ELEMENT );
	}
	
	return nodeChild;
}
/**
 */
public void dumpNodeTree( int iLevel )
{
	StringBuffer strbufDump = new StringBuffer();
	
	for( int i=0; i<iLevel; ++i )
	{
		strbufDump.append( "  " );
	}

	strbufDump.append( _iXmlType == ELEMENT ? "Element " : "Attribute " );
	strbufDump.append( getName() );
	if( _iXmlType == ATTRIBUTE )
	{
	    strbufDump.append( " " + getValue() );
	}

//	Trace.string( strbufDump.toString() );
	
	if( _vectorChildNodes != null )
	{
		XGRXmlDataNode nodeChild = null;
		
	 	for( int i=0; i<_vectorChildNodes.size(); ++i )
	 	{
		 	nodeChild = (XGRXmlDataNode)_vectorChildNodes.elementAt( i );
		 	nodeChild.dumpNodeTree( iLevel + 1 );
	 	} 
	}
}
/**
 */
protected XGRXmlDataNode getAttribute( String strAttributeName )
{
	if( strAttributeName != null && _iXmlType == ELEMENT && _vectorChildNodes != null )
	{
		XGRXmlDataNode nodeChild = null;
		
		for( int i=0; i<_vectorChildNodes.size(); ++i )
		{
			nodeChild = (XGRXmlDataNode)_vectorChildNodes.elementAt( i );
			
			if( nodeChild._iXmlType == ATTRIBUTE && nodeChild._strName.equals( strAttributeName ) == true )
			{
				return nodeChild;
			}
		}
	}
	
	return null;
}
/**
 */
public String[] getAttributeNames() 
{
	// This always returns a non-null vector
	//--------------------------------------
	Vector vectorAttributes = getAttributesVector();

	String[] strNames = new String[ vectorAttributes.size() ];

	for( int i=0; i<vectorAttributes.size(); ++i )
	{
		strNames[i] = ((XGRXmlDataNode)vectorAttributes.elementAt( i ))._strName;
	}
	
	return strNames;
}
/**
 */
public Vector getAttributesVector() 
{
	Vector vectorAttributes = new Vector();

	if( _vectorChildNodes != null )
	{
		for( int i=0; i<_vectorChildNodes.size(); ++i )
		{
			if( ((XGRXmlDataNode)_vectorChildNodes.elementAt( i )).getXmlType() == ATTRIBUTE )
			{
				vectorAttributes.addElement( _vectorChildNodes.elementAt( i ) );
			}
		}
	}
	
	return vectorAttributes;
}
/**
 */
public String getAttributeValue( String strAttributeName ) 
{
	if( strAttributeName != null )
	{
		XGRXmlDataNode nodeAttribute = getAttribute( strAttributeName );
	
		if( nodeAttribute != null )
		{
			return nodeAttribute._strValue;
		}
	}
	
	return null;
}
/**
 */
public IDataElement[] getChildElements() 
{
	// This always returns a non-null vector
	//--------------------------------------
	Vector vectorElements = getElementsVector();

	// Move vector elements to an array
	//---------------------------------
	IDataElement[] elementArray = new IDataElement[ vectorElements.size() ];

	for( int i=0; i<vectorElements.size(); ++i )
	{
		elementArray[i] = (IDataElement)vectorElements.elementAt(i);
	}
	
	return elementArray;
}
/**
 */
public IDataElement[] getChildElements( String strElementName )
{
	IDataElement[] elementArray = null;
	
	if( _vectorChildNodes != null )
	{
		Vector vectorElements = new Vector();
		
		// Search my children for an element with the specified name
		//----------------------------------------------------------
		XGRXmlDataNode nodeElement = null;
		
		for( int i=0; i<_vectorChildNodes.size(); ++i )
		{
			nodeElement = (XGRXmlDataNode)_vectorChildNodes.elementAt( i );
			
			if( nodeElement._iXmlType == ELEMENT &&
				nodeElement._strName.equals( strElementName ) == true )
			{
				vectorElements.addElement( _vectorChildNodes.elementAt( i ) );
			}
		}

		// Transfer the found elements to an array
		//----------------------------------------
		elementArray = new IDataElement[ vectorElements.size() ];

		for( int i=0; i<vectorElements.size(); ++i )
		{
			elementArray[i] = (IDataElement)vectorElements.elementAt( i );
		}
	}

	else
	{
		elementArray = new IDataElement[0];
	}
	
	return elementArray;
}
/**
 */
public Object getElementProperty( Object objKey )
{
	if( objKey != null )
	{
		return getAttribute( objKey.toString() );
	}

	return null;
}
/**
 */
public Vector getElementsVector() 
{
	Vector vectorElements = new Vector();

	if( _vectorChildNodes != null )
	{
		for( int i=0; i<_vectorChildNodes.size(); ++i )
		{
			if( ((XGRXmlDataNode)_vectorChildNodes.elementAt( i )).getXmlType() == ELEMENT )
			{
				vectorElements.addElement( _vectorChildNodes.elementAt( i ) );
			}
		}
	}
	
	return vectorElements;
}
/**
 */
public String getName()
{
	return _strName;
}
/**
 */
public String getValue()
{
	return _strValue;
}
/**
 */
public int getXmlType()
{
	return _iXmlType;
}
/**
 */
public void notifyDataChanged()
{
	for(int idx=0; idx<_dataChangedListeners.size(); idx++)
	{
		IXGRDataChangedListener listener = (IXGRDataChangedListener)_dataChangedListeners.elementAt(idx);
		listener.dataChanged(this);
	} 
}
/**
 */
public void registerDataChangedListener( IXGRDataChangedListener listener )
{
	if( listener != null && !_dataChangedListeners.contains( listener ) )
	{
		_dataChangedListeners.addElement( listener );
	}
}
/**
 */
public boolean removeAttribute( String strAttributeName )
{
	if( strAttributeName != null )
	{
		IXGRDataNode nodeAttribute = getAttribute( strAttributeName );

		if( nodeAttribute != null )
		{
			super.removeChildNode( nodeAttribute );
			return true;
		}
	}

	return false;
}
/**
 */
public boolean removeChildElement( IDataElement elementChild )
{
	if( elementChild != null && elementChild instanceof IXGRDataNode )
	{
		return super.removeChildNode( (IXGRDataNode)elementChild );
	}

	return false;
}
/**
 */
public boolean setAttributeValue( String strAttributeName, String strAttributeValue )
{
	if( strAttributeName != null && strAttributeValue != null )
	{
		XGRXmlDataNode nodeAttribute = getAttribute( strAttributeName );

		if( nodeAttribute != null )
		{
			nodeAttribute.setValue( strAttributeValue );
			return true;
		}
	}

	return false;
}
/**
 */
public void setElementProperty( Object objKey, Object objValue )
{
	if( objKey != null && objValue != null )
	{
		XGRXmlDataNode nodeAttribute = getAttribute( objKey.toString() );
	
		if( nodeAttribute != null )
		{
			nodeAttribute.setValue( objValue.toString() );
		}
	}
}
/**
 */
public void setValue( String strValue ) 
{
	String strValueOld = _strValue;
	
	_strValue = strValue;
		
	if( strValueOld != null && _strValue != null  && strValueOld.equals( _strValue ) == false )
	{
		// Notify UI listeners that the element value has changed
		//-------------------------------------------------------
		notifyDataChanged();
	}

}
/**
 */
public void setXmlType( int type )
{
	_iXmlType = type;
}
/**
 */
public String toString()
{
	return( super.toString() + " XMLType= " + _iXmlType );
}
/**
 */
public void unregisterDataChangedListener( IXGRDataChangedListener listener )
{
	if( listener != null )
	{
	   _dataChangedListeners.removeElement( listener );
	}
}
}
