package org.eclipse.scripting.internal.datamodels;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import org.eclipse.scripting.internal.*;
import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import java.io.*;
import java.util.Vector;
import org.xml.sax.SAXException;

public class XGRXmlElementsAndAttributesDataModel extends XGRXmlDataModel
{
/**
 */
public XGRXmlElementsAndAttributesDataModel()
{
	super();

	_strName  = "itpsea";
	_iXmlType = ELEMENT;
}
/**
 */
public XGRXmlSaxHandler getSaxHandler() 
{
	return new XGRXmlElementsAndAttributesSaxHandler( this, getDirectory() );
}
/**
 */
protected void initializeRootNode()
{
	// The itpsea node is currently my child
	// I become the itpsea node
	//--------------------------------------
	if( _vectorChildNodes != null )
	{
		// This is always true
		//--------------------
		if( _vectorChildNodes.size() == 1 )
		{
			// My child's child nodes are now my child nodes
			//----------------------------------------------
			XGRXmlDataNode nodeItpsea = ((XGRXmlDataNode)_vectorChildNodes.elementAt( 0 ));

			_vectorChildNodes = nodeItpsea.getChildNodes();
			_strName          = nodeItpsea.getName();
			_strValue         = nodeItpsea.getValue();
			_iXmlType         = nodeItpsea.getXmlType();
			_nodeParent       = null;

			// Set their parent to be me
			//--------------------------
			Vector vectorItpseaChildNodes = nodeItpsea.getChildNodes();

			if( vectorItpseaChildNodes != null )
			{
				XGRXmlDataNode childNodeOfItpsea = null;
				
				for( int i=0; i<vectorItpseaChildNodes.size(); ++i )
				{
					childNodeOfItpsea = (XGRXmlDataNode)vectorItpseaChildNodes.elementAt( i );
					childNodeOfItpsea.setParentNode( this );
				}
			}
		}
	}
}
/**
 */
public void loadFromFile( IFile file ) throws IOException, SAXException, CoreException
{
//	Trace.functionEntry( this, "load( IFile )" );

	super.loadFromFile( file );

	initializeRootNode();
	
//	Trace.functionExit( this, "load( IFile )" );
}
/**
 */
public void loadFromFilename( String strFilename ) throws IOException, SAXException
{
//	Trace.functionEntry( this, "load( String )" );
//	Trace.string( strFilename );

	super.loadFromFilename( strFilename );

	initializeRootNode();
	
//	Trace.functionExit( this, "load( String )" );
}
/**
 * This method writes the data model using the given Writer.
 * The original contents of the file are discarded.
 */
protected void writeDataModel( StringBuffer buffer )
{
//	Trace.functionEntry( this, "writeDataModel" );

	// Write out header text
	//  <?xml version="1.0" encoding="UTF-8"?>
	//----------------------------------------
	buffer.append( "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" );
	
	Vector vectorNodes = getChildNodes();

	// Write out the doctype line
	// <!DOCTYPE xxxxx SYSTEM "itpsea.dtd">
	//-------------------------------------
	XGRXmlDataNode nodeChild = (XGRXmlDataNode)vectorNodes.elementAt( 0 );

	StringBuffer strbLine = new StringBuffer();

	strbLine.append( "<!DOCTYPE " );
	strbLine.append( getName() );
	strbLine.append( " SYSTEM \"itpsea.dtd\">\n" );

	buffer.append( strbLine.toString() );

	// Write out the document element start tag
	// <xxxxx type="yyyyy">
	//-----------------------------------------
	strbLine.setLength( 0 );
	strbLine.append( "<" );
	strbLine.append( getName() );
	strbLine.append( ">" );

	buffer.append( strbLine.toString() );
	
	// Write out the other elements
	//-----------------------------
	vectorNodes = getElementsVector();
	
	for( int i=0; i<vectorNodes.size(); ++i )
	{
		writeElement( (XGRXmlDataNode)vectorNodes.elementAt( i ), buffer, 0 );
	}

	// Write out the document element end tag
	// </xxxxx>
	//---------------------------------------
	strbLine.setLength( 0 );
	strbLine.append( "\n</" );
	strbLine.append( getName() );
	strbLine.append( ">\n" );
	
	buffer.append( strbLine.toString() );	

//	Trace.functionExit( this, "writeDataModel" );
}
/**
 * This method writes out an element's begin element tag, its attributes,
 * and its value.
 */
protected void writeElement( XGRXmlDataNode node, StringBuffer buffer, int iIndent )
{
//	Trace.functionEntry( this, "writeElement" );

	// Ensure this is a valid element node
	//------------------------------------	
	if( node == null || node.getXmlType() == XGRXmlDataNode.ATTRIBUTE )
	{
//		Trace.functionExit( this, "writeElement" );
		return;
	}

	// Write out the indentation spaces
	//---------------------------------
	writeIndent( buffer, ++iIndent );

	// Write out the element begin tag
	//--------------------------------
	buffer.append( "<element name=\"" + node.getName() + "\">" );

	// Write the attributes
	//---------------------
	Vector list = node.getAttributesVector();
	XGRXmlDataNode nodeChild = null;
	
	for( int i=0; i<list.size(); i++ )
	{
		writeIndent( buffer, iIndent + 1 );
		nodeChild = (XGRXmlDataNode)list.elementAt( i );
		buffer.append( "<attribute name=\"" + nodeChild.getName() + "\" value=\"" + nodeChild.getValue() + "\" />" );
	}
	
	// Recursively write out my child elements
	//----------------------------------------
	list = node.getElementsVector();
	
	for( int i=0; i<list.size(); i++ )
	{
		nodeChild = (XGRXmlDataNode)list.elementAt( i );
		writeElement( nodeChild, buffer, iIndent );
	}

	// Write the element end tag
	//--------------------------
	writeIndent( buffer, iIndent );
	buffer.append( "</element>" );	 	  	    
	
//	Trace.functionExit( this, "writeElement" );
}
}
