package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * An interface for an contribution node. A node manages a list
 * of subnodes. So contribution nodes can be arranged as a tree.
 */
import java.util.Iterator;

public interface IScriptContributionNode {

/**
 * Adds a subnode to this node
 */
public void add(IScriptContributionNode node);
/**
 * Returns the contribution node under the given id, or <code>null
 * </code> if the node does not exist.
 */
public IScriptContributionNode findSubNode(String id);
/**
 * Returns the contribution item's id. This id is used to identify
 * the contribution inside a contribution manager.
 */
public String getID();
/**
 * Returns a Enumeration of the sub nodes.
 */
public Iterator getSubNodes();
/**
 * Removes the node with the given name from the list of
 * subnodes.
 */
public IScriptContributionNode remove(String node);
/**
 * Removes the given node from the list of subnodes. The methods
 * returns <code>true</code> if the node was present; <code>false
 * </code>otherwise.
 */
public boolean remove(IScriptContributionNode node);
}
