package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Vector;

/**
 * A default implementation of <code>IContributionNode</code>.
 */
public class ScriptDefaultContributionNode implements IScriptContributionNode {

	private String fId;
	private Vector fSubNodes; 
	
	/**
	 * Creates a new contribution node.
	 */
	public ScriptDefaultContributionNode(String id) {
		fId= id;
	}
	/**
	 * Adds a subnode to this node.
	 */
	public void add(IScriptContributionNode node) {
		if (fSubNodes == null)
			fSubNodes= new Vector();
		fSubNodes.addElement(node);
	}
	/**
	 * Returns the contribution node under the given id, or <code>null
	 * </code> if the node does not exist. The empty string represents
	 * <code>this</code>.
	 */
	public IScriptContributionNode findSubNode(String id) {
		if (id.equals(""))
			return this;
		if (fSubNodes == null)
			return null;	
		int size= fSubNodes.size();
		for (int i= 0; i < size; i++) {
			IScriptContributionNode node= (IScriptContributionNode)fSubNodes.elementAt(i);
			if (id.equals(node.getID()))
				return node;
		}
		return null;
	}
	/**
	 * Returns the contribution item's id. This id is used to identify
	 * the contribution inside a contribution manager.
	 */
	public String getID() {
		return fId;
	}
	/**
	 * Returns a Enumeration of the sub nodes.
	 */
	public Iterator getSubNodes() {
		if (fSubNodes == null)
			return new ArrayList(0).iterator();
			
		return fSubNodes.iterator();
	}
	/**
	 * Removes the node with the given name from the list of
	 * subnodes.
	 */
	public IScriptContributionNode remove(String id) {
		IScriptContributionNode node= findSubNode(id);
		remove(node);
		return node;
	}
	/**
	 * Removes the given node from the list of subnodes. The methods
	 * returns <code>true</code> if the node was present; <code>false
	 * </code> otherwise.
	 */
	public boolean remove(IScriptContributionNode node) {
		if (fSubNodes == null)
			return false;
		return fSubNodes.removeElement(node);	
	}
}
