package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/*
 * 
 */
import org.eclipse.jface.preference.*; 
import java.util.*;
import org.eclipse.jface.*;

/**
 * The PreferenceManager maintains top level PreferencePage contributions.
 */
public class ScriptPreferenceManager extends ScriptContributionNodeManager
{

	
	/**
	 * Creates a new preference manager.
	 */
	public ScriptPreferenceManager() {
		super();
	}
	/**
	 * Adds a new preference page. The page is wrapped by an preference node and
	 * then inserted into the preference manager. The <code>id</code> determines
	 * the identifier under which the preference node is inserted into the preference
	 * manager.
	 */
	public boolean addTo(String path, String id, ScriptPreferencePage page) {
		return super.addTo(path, new ScriptPreferenceNode(id, page));
	}
	/**
	 * Adds the given contribution node to the contribution node
	 * determined by the given path. <code>True</code> is returned
	 * if adding was successful. This reimplementation ensures that
	 * node is an instance of type <code>PreferenceNode</code>.
	 */
	public boolean addTo(String path, IScriptContributionNode node) {
		return super.addTo(path, node);
	}
	/**
	 * Adds a new preference page. The page is wrapped by an preference node and
	 * then inserted into the preference manager. The <code>id</code> determines
	 * the identifier under which the preference node is inserted into the preference
	 * manager.
	 */
	public void addToRoot(String id, ScriptPreferencePage page) {
		super.addToRoot(new ScriptPreferenceNode(id, page));
	}
	/**
	 * Adds a new contribution node. This reimplementation ensures
	 * that node is an instance of <code>PreferenceNode</code>.
	 */
	public void addToRoot(IScriptContributionNode node) {
		super.addToRoot(node);
	}
	/**
	 * Finds the page node determined by the given path
	 */
	public IScriptPreferenceNode findPreferenceNode(String path) {
		IScriptContributionNode node= find(path);
		if (node instanceof IScriptPreferenceNode)
			return (IScriptPreferenceNode)node;
		return null;	
	}
	/**
	 * Returns the root node
	 */
	public IScriptContributionNode getRoot() {
		return super.getRoot();
	}
}
