package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.jface.preference.IPreferencePage;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;

/**
 * A proxy for a preference page to avoid creation of preference page
 * just to show a node in the preference dialog tree.
 */
public class ScriptPreferenceNode extends ScriptDefaultContributionNode implements IScriptPreferenceNode {
	private String fLabel;
	private ImageDescriptor imageDescriptor;
	private Image image;
	private String fClassname;
	private ClassLoader fLoader;
	private IPreferencePage fPage;
	/**
	 * Creates a preference node.
	 */
	public ScriptPreferenceNode(String id, String label, String classname, ClassLoader loader) {
		super(id);
		init(label, classname, loader);
	}
/**
 * Creates a preference node.
 */
public ScriptPreferenceNode(
	String id, 
	String label, 
	ImageDescriptor image, 
	String classname, 
	ClassLoader loader) {
	    
	super(id);
	init(label, classname, loader);
	this.imageDescriptor = image;
}
	/**
	 * Creates a preference node.
	 */
	public ScriptPreferenceNode(String id, IPreferencePage page) {
		super(id);
		fPage= page;
	}
	//---- Object creation
		
	/**
	 * Creates a new instance of the given class <code>className</code>.
	 * The class is loaded using the given class loader.
	 */
private Object createObject(Class check, String className, ClassLoader loader) {

				
		if (className == null)
			className= check.getName();
		
		try {
			Class cl= null;
			if (loader != null)
				cl= loader.loadClass(className);
			else
				cl= Class.forName(className);
					
			if (cl != null) {
				Object o= cl.newInstance();
				if (o != null) {
					if (!check.isInstance(o))
						System.out.println("Utilities.createObject: object not instance of " + check.getName());
				} else
					System.out.println("Utilities.createObject: newinstance returns null");
				return o;
			}	
			System.out.println("Utilities.createObject: can't find " + className);
		}
		catch (ClassNotFoundException e) {
			System.out.println("Utilities.createObject: class not found " + className);
		}
		catch (InstantiationException e) {
			System.out.println("Utilities.createObject: got InstantiationException");
		}
		catch (IllegalAccessException e) {
			System.out.println("Utilities.createObject: got IllegalAccessException");
		}
		catch (NoSuchMethodError e) {
			System.out.println("Utilities.createObject: got NoSuchMethodError");
		}
		return null;
	}
/**
 * Returns the image used to present this node in the preference dialog.
 */
public Image getImage() {
	if (image == null && imageDescriptor != null) {
		image = imageDescriptor.createImage();
	}
	return image;
}
	/**
	 * Returns the label used to present this node in the preference dialog.
	 */
	public String getLabel() {
		if (fPage != null)
			return fPage.getTitle();
		return fLabel;	
	}
	/**
	 * Returns the preference page this node stands for.
	 */ 
	public IPreferencePage getPreferencePage() {
		if (fPage != null)
			return fPage;
		
		fPage= (IPreferencePage)createObject(
			IPreferencePage.class, fClassname, fLoader);
		if (getImage() != null)
			fPage.setImageDescriptor(imageDescriptor);
		fPage.setTitle(fLabel);
		return fPage;
	}
	/**
	 * Initialize the preference node.
	 */
	private void init(String label, String classname, ClassLoader loader) {
		// a "null" loader represents the system class loader.
		fLabel= label;
		fClassname= classname;
		fLoader= loader;		
	}
	/**
	 * Returns <code>true</code>, if a preference page has been created
	 * for this node. Otherwise <code>false</code> is returned.
	 */
	public boolean isPageCreated() {
		return fPage != null;
	}
/**
 * Release the page managed by this node.
 */
public void release() {
	fPage = null;
	if (image != null) {
		image.dispose();
		image = null;
	}
}
}
