package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

/**
 * A window represented by a node in a ScriptPreferenceComposite page.
 * When the node is selected in the tree, the current window
 * is shown on the right of the composite page.
 * Extends jface PreferencePage
 */
public class ScriptPreferencePage extends PreferencePage
{
	/**
	 * Creates a new preference page.
	 */
	protected ScriptPreferencePage() {
	}
	/**
	 * Creates a new preference page.
	 */
	protected ScriptPreferencePage(String title) {
		super(title);
	}
	/**
	 * Creates a new preference page.
	 */
	protected ScriptPreferencePage(String title, ImageDescriptor image) {
		super(title, image);
	}
	public Point computeSizePage()
	{
		return computeSize();
	}
	/**
	 * Reimplement this method if a page wants to contribute buttons 
	 * to the page's button box. For each button a subclass contributes
	 * it must increase the parent's grid layout numColumns value by
	 * one (e.g. ((GridLayout)parent.getLayout()).numColumns++);)
	 */
	protected void contributeButtons(Composite parent)
	{
	}
	/**
	 * Hook to create the body of the page.
	 */
	public Control createContents(Composite parent)
	{		
		Composite _parent = new Composite(parent, SWT.NULL);
		//GridLayout
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		_parent.setLayout(layout);

		//GridData
		GridData data = new GridData();
		data.verticalAlignment = GridData.FILL;
		data.horizontalAlignment = GridData.FILL;
		_parent.setLayoutData(data);
/*
		Label label = new Label(_parent, SWT.NONE);
		label.setText("A simple label for " + getTitle() + " page");
//		FontUtil.applyDialogFontTo(label);
*/
		return _parent;
	}
	/**
	 * The apply button has been pressed. The default
	 * implementation is to call <code>okPressed</code>.
	 * @see #okPressed()
	 */
	protected void doApply()
	{
		performOk();
	}
	/**
	 * This method is call whenever this page is selected. Reimplement this
	 * method if, for example, the page needs to read its data from a
	 * preference bundle before it is shown.
	 * @see #leave()
	 */
	public void enterPage()
	{	    		
		
	}
	public boolean leavePage()
	{
		return true;
	}
	/**
	 * Sets the size of the page to the given value.
	 */
	public void setSizePage(Point size)
	{
		setSize(size);
	}
}
