package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.ScrollBar;

public class ScriptViewport extends Composite implements ControlListener, SelectionListener
{
	protected ScrollBar _scrollVert = null;
	protected ScrollBar _scrollHorz = null;
/**
 * ScriptViewport constructor comment.
 */
public ScriptViewport( Composite compositeParent, int iStyle, GridData gridData )
{
	super( compositeParent, iStyle );

	GridLayout layout= new GridLayout();
	layout.numColumns   = 1;
	layout.marginHeight = 0; 
	layout.marginWidth  = 0;
	setLayout( layout );
	setLayoutData( gridData );

	// Get resize events
	//------------------
	addControlListener( this );

	// Get scrollbar events
	//---------------------
	_scrollHorz = getHorizontalBar();
	_scrollVert = getVerticalBar();

	if( _scrollHorz != null )
	{
		_scrollHorz.addSelectionListener( this );
	}

	if( _scrollVert != null )
	{
		_scrollVert.addSelectionListener( this );
	}
}
/**
 * controlMoved method comment.
 */
public void controlMoved(org.eclipse.swt.events.ControlEvent e) {}
/**
 * controlResized method comment.
 */
public void controlResized(org.eclipse.swt.events.ControlEvent e)
{
	updateScrollbars();
}
/**
 */
public void updateScrollbars()
{
	Control[] controls = getChildren();

	if( controls.length > 0 )
	{
		Rectangle rectControl = controls[0].getBounds();
		Rectangle rectContent = getClientArea();
		
		// Horizontal scroll bar
		//----------------------
		if( _scrollHorz != null )
		{
			if( rectControl.width < rectContent.width )
			{
				_scrollHorz.setEnabled( false );
			}

			else
			{
				_scrollHorz.setMinimum( 0 );
				_scrollHorz.setMaximum( rectControl.width );
				_scrollHorz.setThumb( rectContent.width );	
				_scrollHorz.setIncrement( 10 );
				_scrollHorz.setPageIncrement( rectContent.width / 2 );
				_scrollHorz.setSelection( 0 );
				_scrollHorz.setEnabled( true );
			}

			rectControl.y = 0;
			controls[0].setBounds( rectControl );
		}
		
		// Vertical scroll bar
		//--------------------
		if( _scrollVert != null )
		{
			if( rectControl.height < rectContent.height )
			{
				_scrollVert.setEnabled( false );
			}

			else
			{
				_scrollVert.setMinimum( 0 );
				_scrollVert.setMaximum( rectControl.height );
				_scrollVert.setThumb( rectContent.height );	
				_scrollVert.setIncrement( 10 );
				_scrollVert.setPageIncrement( rectContent.height / 2 );
				_scrollVert.setSelection( 0 );
				_scrollVert.setEnabled( true );
			}
		}
	}
}
/**
 * Sent when default selection occurs in the control.
 * <p>
 * For example, on some platforms default selection occurs
 * in a List when the user double-clicks an item.
 * </p>
 */
public void widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent e) {}
/**
 */
public void widgetSelected( SelectionEvent sEvent )
{
//	Trace.functionEntry( this, "widgetSelected" );

	Control[] controls = getChildren();

	if( controls.length > 0 )
	{
		Control control = controls[0];
		
		// Horizontal scroll bar
		//----------------------
		if( _scrollHorz != null && _scrollHorz == sEvent.widget )
		{
			Rectangle rectControl = control.getBounds();
			rectControl.x = -_scrollHorz.getSelection();
			control.setBounds( rectControl );
		}

		// Vertical scroll bar
		//--------------------
		else if( _scrollVert != null && _scrollVert == sEvent.widget )
		{
			Rectangle rectControl = control.getBounds();
			rectControl.y = -_scrollVert.getSelection();
			control.setBounds( rectControl );
		}
	}
	
//	Trace.functionExit( this, "widgetSelected" );
}
}
