package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;

public class ScriptWizardPage extends Composite
{
	public static final String PAGE_COMPLETE = "page_complete";
	public static final String TITLE         = "title";
	public static final String ICON          = "icon";
	public static final String DESCRIPTION   = "description";
	public static final String NEXT_STATE    = "next_state";

	protected boolean              _bPageComplete  = true;
	protected Label                _messageLine    = null;
	protected ScriptWizard         _smartGuide     = null;
	protected ScriptWizardPage     _pagePrevious   = null;
	protected String               _strDescription = null;	
	protected String               _strName        = null;
	protected String               _strTitle       = null;
	protected Control              _windowUI       = null;
	protected Image                _image          = null;
/**
 *
 */
public ScriptWizardPage( Composite compositeParent, int iStyle )
{
	super( compositeParent, iStyle );
}
/**
 * Called when the user has pressed the smart guide's
 * cancel button. If the method returns true, the
 * smart guide will close; otherwise, it will stay up.
 */
protected boolean cancel() 
{
	return true;
}
/**
 * Returns whether or not the next page can be displayed or not. This default 
 * implementation returns the value of <code>isPageComplete && getNextPage 
 * != null</code>. When this methods returns <code>true<code> the smart guide's
 * 'Next' button will be enabled. If this method returns <code> false</code> 
 * the button will be disabled.
 * <p> 
 * Pages can fire a NEXT_STATE event (e.g. <code>fireStateChange(NEXT_STATE)
 * </code>) to inform the smart guide to update the next button's enable state.
 * @see getNextPage
 * @see isPageComplete
 */
protected boolean canFlipToNextPage() 
{
	return isPageComplete() && getNextPage() != null;
}
/**
 * Creates the UI part of the page. Subclasses must reimplement 
 * this method to provide their own graphical page content.
 */
protected Control createContents( Composite parent )
{
	return this;
}
/**
 * Called when the page is supposed to present help.
 */
protected void displayHelp() 
{
}
/**
 * The user has pressed the finish button. If the method returns 
 * true, the wizard will close; otherwise, it will stay up. 
 */
protected boolean done() 
{
	return true;
}
/**
 * Called when the page is entered because the user has selected this
 * page be clicking the back or previous button.
 * @param action Indicates whether the back or the next button has been
 *   pressed. If back as been pressed direction is NEXT. If the back button
 *   has been pressed direction is PREVIOUS. Analogous action can be
 *   FINISH or CANCEL if the user has pressed the finish or cancel button.
 */
protected void enter(int action) 
{
}
	/**
	 * Returns my UI component. If the UI component is null or disposed
	 * create a new one. The UI Component is managed by the page.
	 */
	public Control getControl(Composite parent) 
	{
//		if( !Utilities.okToUse( _windowUI ) )
		{
			_windowUI= createContents( parent );
		}
		return _windowUI;
	}
/**
 * Returns the page's description.
 */
public String getDescription() 
{
	return _strDescription;
}
	/**
	 * Returns the page's image.
	 */
	public Image getImage() {
	return null;
}
	/**
	 * Returns the page that is to be shown if the user 
	 * presses the next button.
	 */
public ScriptWizardPage getNextPage()
{
		return _smartGuide != null ? _smartGuide.getNextPage( this ) : null;
}
/**
 * Returns the page that is to be shown if the user 
 * presses the back button.
 */
protected ScriptWizardPage getPreviousPage() 
{
	if( _pagePrevious != null )
	{
		return _pagePrevious;
	}
	
	else
	{
		return _smartGuide != null ? _smartGuide.getPreviousPage( this ) : null;
	}
}
/**
 * Returns the page's title.
 */
public String getTitle() 
{
	return _strTitle;
}
/**
 * Returns whether the page is complete or not.
 * @see setPageComplete
 */
protected boolean isPageComplete() 
{
	return _bPageComplete;
}
	/**
	 * Called when the user selects a different page. If the methods returns false
	 * the page flipping is aborted and this page remains visible.
	 */
	protected boolean leave(int action) {
		return true;
	}
/**
 * Sets the message line used by the smart guide page to display
 * normal and error messages.
 */
public void setMessageLine(Label messageLine) 
{
	_messageLine= messageLine;
}
	/**
	 * Sets the previous page to the given value.
	 */
	protected void setPreviousPage( ScriptWizardPage page )
	{
		_pagePrevious = page;
	}
/**
 */
public void setSmartGuide( ScriptWizard smartGuide )
{
	_smartGuide = smartGuide;
}
	/**
	 * Sets the page's title to the given value. If the page is
	 * currently shown then the UI is immediately updated.
	 */
	public void setTitle( String strTitle ) 
	{
		_strTitle = strTitle;
//		fireStateChanged(TITLE);
	}
}
