package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import org.eclipse.scripting.internal.*;
import java.util.*;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.scripting.internal.*;
import org.eclipse.core.resources.*;
import org.eclipse.scripting.internal.datamodels.*;
import org.eclipse.scripting.*;

public class XGR
{
	protected boolean                     _bBuildTime              = false;
	protected int                         _iUIType                 = XGR.UNKNOWN;
	protected IFile                       _filePresentation        = null;
	protected IXGRDataModel               _dataModel               = null;
	protected ScriptManager               _scriptManager           = null;
	protected String                      _strPresentationFilename = null;
	protected String                      _strTranslationsFilename = null;
	protected String                      _strPresentation         = null;
	protected Vector                      _vectorObjectDescriptors = null;
	protected XGRPresentationModelElement _presentationModelRoot   = null;
	protected XGRNodePresentation         _nodeRootPresentation    = null;
	protected XGRResources                _resources               = null;


	public static final int UNKNOWN                 =  0;
	public static final int BODY                    =  1;
	public static final int BREAK                   =  2;
	public static final int BUTTON                  =  3;
	public static final int CHECK                   =  4;
	public static final int COMBO                   =  5;
	public static final int COMPOSITE               =  6;
	public static final int COMPOSITEPANEL          =  7;
	public static final int FIELDSET                =  8;
	public static final int FORM                    =  9;
	public static final int GROUPBOX                = 10;
	public static final int HEAD                    = 11;
	public static final int HTML                    = 12;
	public static final int HORZRULE                = 13;
	public static final int IMG                     = 14;
	public static final int LABEL                   = 15;
	public static final int META                    = 16;
	public static final int LIST                    = 17;
	public static final int PANEL                   = 18;
	public static final int PREFERENCES             = 19;
	public static final int PREFERENCEDIALOG        = 20;
	public static final int PREFERENCEDIALOGSTANDIN = 21;
	public static final int PREFERENCEPAGE          = 22;
	public static final int PROPERTYSHEET           = 23;
	public static final int PROPERTYSHEETCOMPOSITE  = 24;
	public static final int RADIO                   = 25;
	public static final int SCRIPT                  = 26;
	public static final int SCRIPTPREFERENCEDIALOG  = 27;
	public static final int SCRIPTPREFERENCEPAGE    = 28;
	public static final int TABLE                   = 29;
	public static final int TABLEDATA               = 30;
	public static final int TABLEHEADING            = 31;
	public static final int TABLELAYOUT             = 32;
	public static final int TABLEROW                = 33;
	public static final int TEXTAREA                = 34;
	public static final int TEXTFIELD               = 35;
	public static final int TEXTFIELDHIDDEN         = 36;
	public static final int WIZARD                  = 37;
	public static final int WIZARDBUILDTIME         = 38;
	public static final int WIZARDPAGEBUILDTIME     = 39;
	public static final int WIZARDPAGE              = 40;

	public static final int CHILDORSIBLING          = 50;
/**
 */
public XGR()
{
}
/**
 */
public void execute(Object parent, boolean bVisible) {
	
	if (parent instanceof Composite)
		executeComposite((Composite) parent, bVisible);
		
	else if (parent instanceof IXGRElement || parent == null)
		executeElement((IXGRElement) parent, bVisible);

	return;
}
/**
 */
public void executeComposite( Composite parent, boolean bVisible )
{
//	Trace.functionEntry( this, "execute" );

	// Create a place to store our resources
	//--------------------------------------
	_resources = new XGRResources();

	// Set the data model
	//-------------------
	_resources.setDataModel( _dataModel );

	// Create a font manager
	//----------------------
	_resources.setFontManager( new XGRFontManager() );

	// Create a widget registery
	//--------------------------
	_resources.setWidgetRegistry( new XGRWidgetRegistry() );

	// Create a NL translation manager
	//--------------------------------
	_resources.setNLTranslationManager( new ScriptNLTranslationManager( _strTranslationsFilename ) );

	// Create a script manager to evaluate scripts
	// A global one may have already been previously
	// set by calling setScriptManager.
	//----------------------------------------------
	if( _scriptManager == null )
	{
//		_scriptManager = new ScriptManager();
	}
	_resources.setScriptManager( _scriptManager );

	// Give the user's java objects to the script manager
	//---------------------------------------------------
	if( _vectorObjectDescriptors != null )
	{
		_scriptManager.setJavaObjectDescriptors( _vectorObjectDescriptors );
	}
	
	// Parse the Presentation ML
	//--------------------------
	_nodeRootPresentation = parsePresentationModel();

	// Trace.object( "_strUIType", _strUIType );
	// Trace.object( "_nodeRootPresentation", _nodeRootPresentation );
	
	if( _nodeRootPresentation != null )
	{
		// Create a UI link manager
		//-------------------------
		_resources.setUILinkManager( new XGRUILinkManager( _nodeRootPresentation ) );

		// Initialize the UI
		//------------------
		_nodeRootPresentation.deduceElementTypes( XGR.UNKNOWN, _bBuildTime, _iUIType );

		// If a parent widget has been created already,
		// tell the presentation to use it, instead of
		// creating a new one
		//--------------------------------------------
		_nodeRootPresentation.setParentElement(parent);		
		_nodeRootPresentation.createElements( null );
		_nodeRootPresentation.initializeElementProperties( _resources );
		_nodeRootPresentation.createWidgets( parent );

		if( bVisible == true )
		{
			_nodeRootPresentation.setVisible( true );
		}

		// Debug only
		//-----------
		// _nodeRootPresentation.dumpDebugInformation();
	}

//	Trace.object( "nodeRootPresentation", _nodeRootPresentation );	
//	Trace.functionExit( this, "execute" );
	return;
}
/**
 */
public void executeElement( IXGRElement element, boolean bVisible )
{
//	Trace.functionEntry( this, "execute" );

	// Create a place to store our resources
	//--------------------------------------
	_resources = new XGRResources();

	// Set the data model
	//-------------------
	_resources.setDataModel( _dataModel );

	// Create a font manager
	//----------------------
	_resources.setFontManager( new XGRFontManager() );

	// Create a widget registery
	//--------------------------
	_resources.setWidgetRegistry( new XGRWidgetRegistry() );

	// Create a NL translation manager
	//--------------------------------
	_resources.setNLTranslationManager( new ScriptNLTranslationManager( _strTranslationsFilename ) );
	
	_resources.setScriptManager( _scriptManager );

	// Give the user's java objects to the script manager
	//---------------------------------------------------
	if( _vectorObjectDescriptors != null )
	{
		_scriptManager.setJavaObjectDescriptors( _vectorObjectDescriptors );
	}

	// Parse the Presentation ML
	//--------------------------
	_nodeRootPresentation = parsePresentationModel();

	// Trace.object( "_strUIType", _strUIType );
	// Trace.object( "_nodeRootPresentation", _nodeRootPresentation );
	
	if( _nodeRootPresentation != null )
	{
		// Create a UI link manager
		//-------------------------
		_resources.setUILinkManager( new XGRUILinkManager( _nodeRootPresentation ) );

		// Initialize the UI
		//------------------
		_nodeRootPresentation.deduceElementTypes( XGR.UNKNOWN, _bBuildTime, _iUIType );

		// If a parent widget has been created already,
		// tell the presentation to use it, instead of
		// creating a new one
		//--------------------------------------------
		if( element != null )
		{
			// Desktop New Smartguide
			//-----------------------
			if( element instanceof XGRElementUISwtWizard )
			{
				// Runtime
				//--------
				_nodeRootPresentation.presetElementInstance( XGR.WIZARD, element );
			}

			else if( element instanceof XGRElementUISwtWizardBuildTime )
			{
				// Buildtime
				//----------
				_nodeRootPresentation.presetElementInstance( XGR.WIZARDBUILDTIME, element );
			}
			
			// Resource Viewer/Editor
			//-----------------------
			else if( element instanceof XGRElementUISwtCompositePanel )
			{
				// Runtime: This goes first since this is also a composite
				//--------------------------------------------------------
				_nodeRootPresentation.presetElementInstance( XGR.COMPOSITEPANEL, element );
			}

			else if( element instanceof XGRElementUISwtComposite )
			{
				// Buildtime
				//----------
				_nodeRootPresentation.presetElementInstance( XGR.COMPOSITE, element );
			}
			
			// Desktop Tools Preference Page
			//------------------------------
			else if( element instanceof XGRElementUISwtPreferencePage )
			{
				// Runtime
				//--------
				_nodeRootPresentation.presetElementInstance( XGR.PREFERENCEPAGE, element );
			}
		}
		
		_nodeRootPresentation.createElements( null );
		_nodeRootPresentation.initializeElementProperties( _resources );
		_nodeRootPresentation.createWidgets( null );

		if( bVisible == true )
		{
			_nodeRootPresentation.setVisible( true );
		}

		// Debug only
		//-----------
		// _nodeRootPresentation.dumpDebugInformation( 0 );
	}
	
//	Trace.functionExit( this, "execute" );
	return;
}
/**
 */
public IXGRDataModel getDataModel()
{
	return _dataModel;
}
/**
 */
public String getDataSource( String strID )
{
	IXGRElementNamed elementNamed = _resources.getWidgetRegistry().getElement( strID );

	if( elementNamed != null && elementNamed instanceof IXGRElementUIDataPresentationElement )
	{
		return ((IXGRElementUIDataPresentationElement)elementNamed).getDataSourceName();
	}
	
	return null ;	
}
/**
 */
public String getLinkSource( String strID )
{
	IXGRElementNamed elementNamed = _resources.getWidgetRegistry().getElement( strID );

	if( elementNamed != null && elementNamed instanceof IXGRElementUIDataPresentationElement )
	{
		return ((IXGRElementUIDataPresentationElement)elementNamed).getUISourceName();
	}
	
	return null ;	
}
/**
 */
public XGRPresentationModelElement getRootModelElement()
{
	return _presentationModelRoot;
}
/**
 */
public XGRNodePresentation getRootPresentationElement()
{
	return _nodeRootPresentation;
}
/**
 */
public Object getWidget() {
	if(_nodeRootPresentation != null)
	  return _nodeRootPresentation.getWidget();
		  
	return null;
}
/**
 */
public Object getWidget( String strName )
{
//	Trace.functionEntry( this, "getWidget" );
	
	Object objWidget = null;

	if( _resources != null )
	{
		XGRWidgetRegistry widgetRegistry = _resources.getWidgetRegistry();
		
		if( widgetRegistry != null )
		{
			// Obtain the element
			//-------------------
			IXGRElementNamed objNamed = widgetRegistry.getElement( strName );

//			Trace.object( strName, objNamed );
//			Trace.string( objNamed.getClass().getName() );
			
			// Obtain the widget
			//------------------
			if( objNamed != null )
			{
				objWidget = objNamed.getWidget();

//				Trace.object( "widget", objWidget );
			}
		}
	}

//	Trace.functionExit( this, "getWidget" );

	return objWidget;
}
/**
 */
public XGRNodePresentation parsePresentationModel()
{
//	Trace.functionEntry( this, "parsePresentationModel" );
	
	XGRNodePresentation xgrNodeRootPresentation = null;	

	// Create an XML tree from the file
	//---------------------------------
	XGRXmlParser parserXml = new XGRXmlParser();
	XGRXmlElement elementXmlRoot = null;
	
	try
	{
		if( _strPresentation != null )
		{
			parserXml.setString( _strPresentation );
		}
		else if( _strPresentationFilename != null )
		{
			parserXml.setFilename( _strPresentationFilename );
		}
		else if( _filePresentation != null )
		{
			parserXml.setFile( _filePresentation );
		}
		
		parserXml.parse();
		elementXmlRoot = parserXml.getRootElement();
	}
	
	catch( Exception ex )
	{
		if( ex instanceof XGRXmlParserException )
		{
			ScriptReporter.reportPresentationParserException( (XGRXmlParserException)ex );
		}
		else
		{
			ScriptReporter.reportException( ex );
		}
	}


	// Create a presentation tree from the XML tree
	//---------------------------------------------
	if( elementXmlRoot != null )
	{
		try
		{
			XGRPresentationModel presentationModel = new XGRPresentationModel();

			presentationModel.parse( elementXmlRoot );
	
			_presentationModelRoot = presentationModel.getRootElement();
		}
		catch( Exception ex )
		{
		}
	}
	
	if( _presentationModelRoot != null )
	{
		XGRPresentationModelElement elementHead = _presentationModelRoot.getChildElement( "head" );

		if( elementHead != null )
		{
			parsePresentationModelHead( elementHead, _resources );
		}

		XGRPresentationModelElement elementBody = _presentationModelRoot.getChildElement( "body" );

		if( elementBody != null )
		{
			xgrNodeRootPresentation = parsePresentationModelBody( elementBody, null, 0 );
		}
	}
	
//	Trace.functionExit( this, "parsePresentationModel" );
	
	return xgrNodeRootPresentation;
}
/**
 */
protected XGRNodePresentation parsePresentationModelBody(XGRPresentationModelElement element, XGRNode xgrNodeParent, int iLevel )
{
//	Trace.functionEntry( this, "parsePresentationModelBody" );

	// Create an XGR node
	//-------------------
	XGRNodePresentation xgrNodePresentation = new XGRNodePresentation( element, xgrNodeParent );

	// If the parent node is non null, add the
	// new node to the parent's child list
	//----------------------------------------
	if( xgrNodeParent != null )
	{
		xgrNodeParent.addChildNode( xgrNodePresentation );
	}

	// Get my children to create their nodes
	//--------------------------------------
	for( int i=0; i<element.getNumberOfChildElements(); ++i )
	{
		parsePresentationModelBody( (XGRPresentationModelElement)element.getChildElementAt(i), xgrNodePresentation, iLevel + 1 );
	}
	
//	Trace.functionExit( this, "parsePresentationModelBody" );
	return xgrNodePresentation;
}
/**
 */
protected void parsePresentationModelHead( XGRPresentationModelElement elementHead, XGRResources resources )
{
//	Trace.functionEntry( this, "parsePresentationModelHead" );

	// Look for script tags, ignore the other tags
	//--------------------------------------------
	ScriptManager scriptManager = resources.getScriptManager();

	if( scriptManager != null )
	{
		XGRPresentationModelElement   element   = null;
		XGRPresentationModelAttribute attribute = null;

		for( int i=0; i<elementHead.getNumberOfChildElements(); ++i )
		{
			element = elementHead.getChildElementAt( i );
		
			// Meta: Default script language
			//       UI Type
			//------------------------------
			if( element._strName != null && element._strName.equals( "meta" ) == true )
			{

				String strHttpEquiv = element.getAttributeValue( "http-equiv" );
				String strContent   = element.getAttributeValue( "content"    );
				String strName      = element.getAttributeValue( "name"       );
		
				// Default script language
				//------------------------
				if( strHttpEquiv != null && strContent != null )
				{
					if( strHttpEquiv.toLowerCase().equals( "content-script-type" ) == true )
					{			 
						scriptManager.setDefaultScriptLanguage( strContent );
					}
				}
			}

			// Script
			//-------
			else if( element._strName != null && element._strName.equals( "script" ) == true )
			{
				// Script type
				//------------
				String strScriptLanguage = null;
					
				attribute = element.getAttribute( "type" );
				
				if( attribute != null )
				{
					strScriptLanguage = attribute._strValue;
				}

				// The file parsers convert #comment elements into text attributes	
				// Script file/line/column/text
				//----------------------------------------------------------------
				attribute = element.getAttribute( "text" );
				if( attribute != null )
				{
					scriptManager.evaluateScript( new ScriptDescriptor( attribute._strValue,
		   																strScriptLanguage,
																		attribute._strFilename,
																		attribute._iLineNumber,
																		attribute._iColumnNumber ), null );
				}
			}
		}
	}
	
//	Trace.functionExit( this, "parsePresentationModelHead" );
	return;
}
/**
 */
public void setBuildTime( boolean bBuildTime )
{
	_bBuildTime = bBuildTime;
}
/**
 */
public void setDataModel( IXGRDataModel dataModel )
{
	_dataModel = dataModel;

	// Update the presentation element tree with the new data model
	// Each presentation element will link itself with the new data model
	//-------------------------------------------------------------------
	if( _nodeRootPresentation != null )
	{
		_nodeRootPresentation.setDataModel( dataModel );
	}
}
/**
 */
public boolean setDataSource( String strID, String strDataSource )
{
	IXGRElementNamed elementNamed = _resources.getWidgetRegistry().getElement( strID );

	if( elementNamed != null && elementNamed instanceof IXGRElementUIDataPresentationElement )
	{
		((IXGRElementUIDataPresentationElement)elementNamed).setDataSourceName( strDataSource );
		((IXGRElementUIDataPresentationElement)elementNamed).linkToData();
		return true;
	}
	
	return false;
}
/**
 */
public void setJavaObjectDescriptors( Vector vectorObjectDescriptors )  
{
	_vectorObjectDescriptors = vectorObjectDescriptors;
}
/**
 */
public boolean setLinkSource( String strID, String strLinkSource )
{
	IXGRElementNamed elementNamed = _resources.getWidgetRegistry().getElement( strID );

	if( elementNamed != null && elementNamed instanceof IXGRElementUIDataPresentationElement )
	{
		((IXGRElementUIDataPresentationElement)elementNamed).setUISourceName( strLinkSource );
		((IXGRElementUIDataPresentationElement)elementNamed).linkToUI();
		return true;
	}
	
	return false;
}
/**
 */
public void setPresentationFile( IFile file )
{
	_filePresentation = file;
}
/**
 */
public void setPresentationFilename( String strFilename )
{
//	Trace.functionEntry( this, "setPresentationFilename" );
	
	_strPresentationFilename = strFilename;

//	Trace.string( _strFilename );
//	Trace.functionExit( this, "setPresentationFilename" );
}
/**
 * The entire user interface definition is in this string instead of in a file.
 * This is used by the user interface definition editor.
 */
public void setPresentationString(String strPresentation) {
	_strPresentation = strPresentation;
}
/**
 */
public void setPresentationType( int iPresentationType )
{
	_iUIType = iPresentationType;
}
/**
 */
public void setScriptManager( ScriptManager scriptManager )
{
	_scriptManager = scriptManager;
}
/**
 */
public boolean setStringDataSource( String strID, IStringData stringData )
{
	IXGRElementNamed elementNamed = _resources.getWidgetRegistry().getElement( strID );

	if( elementNamed != null && elementNamed instanceof IXGRElementUIDataPresentationElement )
	{
		XGRStringDataModel dataModel = new XGRStringDataModel( stringData );
		((IXGRElementUIDataPresentationElement)elementNamed).setDataModel( dataModel );
		((IXGRElementUIDataPresentationElement)elementNamed).setDataSourceName( "stringdata" );
		((IXGRElementUIDataPresentationElement)elementNamed).linkToData();
		return true;
	}
	
	return false;
}
/**
 */
public void setTranslationsFilename(String strFilename) {
	_strTranslationsFilename = strFilename;
}
/**
 */
public void treeChanged( Composite parent, XGRPresentationModelElement element )
{
//	Trace.functionEntry( this, "treeChanged" );
	
	if( _presentationModelRoot != null )
	{
		XGRPresentationModelElement elementRoot = _presentationModelRoot;
		
		XGRPresentationModelElement elementHead = elementRoot.getChildElement( "head" );

		if( elementHead != null )
		{
			parsePresentationModelHead( elementHead, _resources );
		}

		XGRPresentationModelElement elementBody = elementRoot.getChildElement( "body" );

		if( elementBody != null )
		{
			_nodeRootPresentation = parsePresentationModelBody( elementBody, null, 0 );
		}
	}
	
	if( _nodeRootPresentation != null )
	{
		// Create a UI link manager
		//-------------------------
		_resources.setUILinkManager( new XGRUILinkManager( _nodeRootPresentation ) );

		// Initialize the UI
		//------------------
		_nodeRootPresentation.deduceElementTypes( XGR.UNKNOWN, _bBuildTime, _iUIType );

		// If a parent widget has been created already,
		// tell the presentation to use it, instead of
		// creating a new one
		//--------------------------------------------
		_nodeRootPresentation.setParentElement(parent);		
		_nodeRootPresentation.createElements( null );
		_nodeRootPresentation.initializeElementProperties( _resources );
		_nodeRootPresentation.createWidgets( parent );
		_nodeRootPresentation.setVisible( true );

		// Debug only
		//-----------
		// _nodeRootPresentation.dumpDebugInformation( 0 );
	}

//	Trace.object( "nodeRootPresentation", _nodeRootPresentation );	
//	Trace.functionExit( this, "treeChanged" );
}
}
