package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * 
 */
import org.eclipse.scripting.internal.uirenderer.*;
import org.eclipse.scripting.internal.*;
import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.graphics.*;

public class XGRElementUISwtButton implements IXGRElement,
											  IXGRElementCaption,
											  IXGRElementEventGeneratorOnClick,
											  IXGRElementNamed,
											  IXGRElementUI,
											  IXGRElementUISizable,
											  IXGRElementUIGridElement,
											  IXGRElementUITextPresentationElement,
											  SelectionListener
{
	private Point     _dimSize    = null;
	private Point     _dimSpan    = null;
	private Button    _button     = null;
	private Color     _colorBackground = null;
	private Rectangle _rectGrid   = null;
	private String    _strCaption = null;
	private String    _strName    = null;
	private ScriptDescriptor _scriptDescriptorOnClick = null;
	private XGRFontManager   _fontManager   = null;
	private ScriptManager _scriptManager = null;
/**
 */
public XGRElementUISwtButton()
{
//	Trace.functionEntry( this, "XGRElementUISwtButton" );
//	Trace.functionExit( this, "XGRElementUISwtButton" );
}
/**
 */
public Object createWidget( Object objParent )
{
	if( objParent instanceof Composite )
	{
		_button = new Button( (Composite)objParent, SWT.PUSH );

		initialize();
	}

	return _button;
}
/**
 */
public String getName()
{
	return _strName;
}
/**
 */
public Object getWidget() 
{
	return _button;
}
/**
 */
public String getWidgetType()
{
	return "org.eclipse.swt.widgets.Button";
}
/**
 */
public Control getWindow()
{
	return _button;
}
/**
 */
public void highlight(boolean bHighlight, Color colorHighlight )
{
	if( _button != null && _button.isDisposed() == false )
	{
		if( bHighlight == true )
		{
			_colorBackground = _button.getBackground();
			_button.setBackground( colorHighlight );
			_button.setSelection( true );
		}
		else if( _colorBackground != null )
		{
			_button.setBackground( _colorBackground );
			_button.setSelection( false );
		}
	}
}
/**
 */
public void initialize()
{
	if( _button != null )
	{
		// Font
		//-----
//		_button.setFont( _fontManager.getDefaultFont( _button ) );

		// Layout
		//-------
		GridData gridData = new GridData();

		gridData.horizontalAlignment = GridData.FILL;
//		gridData.verticalAlignment   = GridData.FILL;

		// Size
		//-----
		if( _dimSize != null )
		{
			// Width
			//------
			if( _dimSize.x > 0 )
			{
				gridData.widthHint = _dimSize.x;
			}

			// Height
			//-------
			if( _dimSize.y > 0 )
			{
				gridData.heightHint = _dimSize.y;
			}
		}

		// Span
		//-----
		if( _dimSpan != null )
		{
			gridData.horizontalSpan = _dimSpan.x;
			gridData.verticalSpan   = _dimSpan.y;
		}

		_button.setLayoutData( gridData );

		

		// Caption
		//--------	
		if( _strCaption != null )
		{
			_button.setText( _strCaption );
		}

		// Set a minimum width
		//--------------------
		if( gridData.widthHint == SWT.DEFAULT )
		{
			Point ptExtents = _button.computeSize( SWT.DEFAULT, SWT.DEFAULT, false );
			if( ptExtents.x < 60 )
			{
				gridData.widthHint = 60;
				_button.setLayoutData( gridData );
			}
		}
		
		// Add listeners
		//--------------
		if( _scriptDescriptorOnClick != null )
		{
			_button.addSelectionListener( this );
		}

		// Register the widget with the script manager
		//--------------------------------------------
		if( _scriptManager != null && _strName != null && _strName.length() > 0 )
		{
			_scriptManager.declareElement( _strName, _button );
		}
	}
}
/**
 */
public void setCaptionText( String strCaption )
{
	_strCaption = strCaption;
	
}
/**
 */
public void setFontManager( XGRFontManager fontManager) 
{
	_fontManager = fontManager;
}
/**
 */
public void setName( String strName )
{
	_strName = strName;
}
/**
 */
public void setOnClickScriptDescriptor( ScriptDescriptor scriptDescriptor )
{
	_scriptDescriptorOnClick = scriptDescriptor;
}
/**
 */
public void setScriptManager( ScriptManager scriptManager )
{
	_scriptManager = scriptManager;
}
/**
 */
public void setSize( Point dimSize ) 
{
	_dimSize = dimSize;
}
/**
 */
public void setSpan( Point dimSpan )
{
	_dimSpan = dimSpan;
}
/**
 */
public void setVisible(boolean bVisible)
{
	if( _button != null )
	{
		_button.setVisible( bVisible );
	}
}
/**
 * Sent when default selection occurs in the control.
 * <p>
 * For example, on some platforms default selection occurs
 * in a List when the user double-clicks an item.
 * </p>
 */
public void widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent e) {}
/**
 */
public void widgetSelected( SelectionEvent e )
{
//	Trace.functionEntry( this, "widgetSelected" );
	
	if( _scriptManager != null )
	{
//		Trace.string( e.toString() );

		_scriptManager.evaluateScript( _scriptDescriptorOnClick, _button );
	}

//	Trace.functionExit( this, "widgetSelected" );
}
}
