package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * Checkbox that does not support UI linking.
 */
import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.layout.*;
import org.eclipse.scripting.internal.uirenderer.*;
import org.eclipse.scripting.internal.*;
import java.util.Vector;

public class XGRElementUISwtCheckBox implements IXGRElementCaption,
												IXGRElementEventGeneratorOnClick,
												IXGRElementNamed,
												IXGRElementUI,
												IXGRElementUIChecked,
												IXGRElementUIDataPresentationElement,
									 			IXGRElementUIGridElement,
									 			IXGRElementUITextPresentationElement,
									 			SelectionListener,
									 			IXGRDataChangedListener
{
	private boolean          _bChecked                = false;
	private Button           _button                  = null;
	private Color            _colorBackground         = null;
	private Point            _dimSpan                 = null;
	private IXGRDataModel    _dataModel               = null;
	private IXGRDataNode[]   _dataNodes               = null;
	private ScriptDescriptor _scriptDescriptorOnClick = null;
	private ScriptManager    _scriptManager           = null;
	private String           _strCaption              = null;
	private String           _strDataSourceName       = null;
	private String           _strName                 = null;
	private XGRFontManager   _fontManager             = null;
/**
 */
public XGRElementUISwtCheckBox()
{
	super();
	
//	Trace.functionEntry( this, "XGRElementUISwtCheckBox" );
//	Trace.functionExit( this, "XGRElementUISwtCheckBox" );
}
/**
 */
public void addUIListener( IXGRElementUIListener uiListener )
{
}
/**
 */
public Object createWidget( Object objParent )
{
	if( objParent instanceof Composite )
	{
		_button = new Button( (Composite)objParent, SWT.CHECK );
		
		initialize();
	}

	return _button;
}
/**
 */
public void dataChanged(IXGRDataNode dataNode) 
{
	setData( _dataNodes );
}
/**
 */
public String getDataSourceName() 
{
	return _strDataSourceName;
}
/**
 */
public String getName()
{
	return _strName;
}
/**
 */
public String getUISourceName() 
{
	return null;
}
/**
 */
public Object getWidget() 
{
	return _button;
}
/**
 */
public String getWidgetType()
{
	return "org.eclipse.swt.widgets.Button";
}
/**
 */
public Control getWindow()
{
	return _button;
}
/**
 */
public void highlight( boolean bHighlight, Color colorHighlight )
{
	if( _button != null )
	{
		if( bHighlight == true )
		{
			_colorBackground = _button.getBackground();
			_button.setBackground( colorHighlight );
		}
		else if( _colorBackground != null )
		{
			_button.setBackground( _colorBackground );
		}
	}
}
/**
 */
public void initialize()
{
	if( _button != null )
	{
		// Font
		//-----
//		_button.setFont( _fontManager.getDefaultFont( _button ) );
		
		// Layout
		//-------
		if( _dimSpan != null )
		{
			GridData gridData = new GridData();
	
			gridData.horizontalSpan = _dimSpan.x;
			gridData.verticalSpan   = _dimSpan.y;

			_button.setLayoutData( gridData );
		}

		// Caption
		//--------
		if( _strCaption != null )
		{
			_button.setText( _strCaption );
		}

		// Checked
		//--------
		_button.setSelection( _bChecked );

		// Add data from the data model if not linked to a source
		//-------------------------------------------------------
		if( _dataModel != null )
		{
			linkToData();
		}

		// Add listeners
		//--------------
		_button.addSelectionListener( this );

		// Register the widget with the script manager
		//--------------------------------------------
		if( _scriptManager != null && _strName != null && _strName.length() > 0 )
		{
			_scriptManager.declareElement( _strName, _button );
		}
	}
}
/**
 */
public void linkToData()
{
	if( _strDataSourceName != null && _dataModel != null )
	{
		_dataNodes = _dataModel.getDataNodes( _strDataSourceName );
		setData( _dataNodes );
	}
}
/**
 */
public void linkToUI() {}
/**
 */
public void setCaptionText( String strCaption )
{
	_strCaption = strCaption;
}
/**
 */
public void setChecked( boolean bChecked ) 
{
	_bChecked = bChecked;
}
/**
 */
public void setData( IXGRDataNode[] dataNodes )
{
//	Trace.functionEntry( this, "setData" );
	
	// Replace the text
	//-----------------
	if( dataNodes != null && _button != null )
	{
		if( dataNodes.length > 0 )
		{
			if(dataNodes[0] instanceof IXGRDataChangedNotifier)
			{
				((IXGRDataChangedNotifier)dataNodes[0]).registerDataChangedListener( this );
			}
			
			if( dataNodes[0].getValue() != null && dataNodes[0].getValue().equals( "true" ) )
			{	
				_button.setSelection( true );
			}
			else
			{
				_button.setSelection( false );
			}
		}
	}
	
//	Trace.functionExit( this, "setData" );
}
/**
 */
public void setDataModel( IXGRDataModel dataModel) 
{
	_dataModel = dataModel;
}
/**
 */
public void setDataSourceName( String strDataSourceName) 
{
	_strDataSourceName = strDataSourceName;
}
/**
 */
public void setFontManager( XGRFontManager fontManager) 
{
	_fontManager = fontManager;
}
/**
 */
public void setName( String strName )
{
	_strName = strName;
}
/**
 */
public void setOnClickScriptDescriptor( ScriptDescriptor scriptDescriptor )
{
	_scriptDescriptorOnClick = scriptDescriptor;
}
/**
 */
public void setScriptManager( ScriptManager scriptManager )
{
	_scriptManager = scriptManager;
}
/**
 */
public void setSpan( Point dimSpan ) 
{
	_dimSpan = dimSpan;
}
/**
 */
public void setUILinkManager( XGRUILinkManager uiLinkManager )
{
}
/**
 */
public void setUISourceName( String strLinkSourceName ) 
{
}
/**
 */
public void setVisible(boolean bVisible) {}
/**
 * Sent when default selection occurs in the control.
 * <p>
 * For example, on some platforms default selection occurs
 * in a List when the user double-clicks an item.
 * </p>
 */
public void widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent e) {}
/**
 */
public void widgetSelected( SelectionEvent e )
{
//	Trace.functionEntry( this, "widgetSelected" );

	if(_dataNodes != null && _dataNodes.length > 0 && _button != null)
	{
	    IXGRDataNode dataNode = (IXGRDataNode) _dataNodes[0];
	    dataNode.setValue( _button.getSelection() == true ? "true" : "false" );
	}
		
	if( _scriptManager != null )
	{
//		Trace.string( e.toString() );

		_scriptManager.evaluateScript( _scriptDescriptorOnClick, _button );
	}

	
	
//	Trace.functionExit( this, "widgetSelected" );
}
}
