package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import org.eclipse.scripting.internal.uirenderer.*;
import org.eclipse.scripting.internal.*;
import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import java.util.Vector;

public class XGRElementUISwtComboBox implements IXGRElement,
												IXGRElementUIListener,
												IXGRElementEventGeneratorOnChange,  
												IXGRElementNamed,
												IXGRElementUI,
												IXGRElementUIDataPresentationElement,
												IXGRElementUIGridElement,
												IXGRElementUISizable,
												SelectionListener,
												IXGRDataChangedListener
{
	private Combo                      _combo                   = null;
	private Color                      _colorBackground         = null;
	private Point                      _dimSize                 = null;	
	private Point                      _dimSpan                 = null;
	private String                     _strDataSourceName       = null;
	private String                     _strLinkSourceName       = null;	
	private String                     _strName                 = null;
	private ScriptDescriptor           _scriptDescriptorOnChange = null;
	private IXGRDataModel              _dataModel               = null;
	private XGRUILinkManager           _uiLinkManager           = null;
	private IXGRDataNode[]             _dataNodes               = null;
	private XGRFontManager             _fontManager             = null;
	private ScriptManager              _scriptManager           = null;
	private Vector                     _vectorUIListeners       = null;

/**
 */
public XGRElementUISwtComboBox()
{
//	Trace.functionEntry( this, "XGRElementUISwtComboBox" );
//	Trace.functionExit( this, "XGRElementUISwtComboBox" );
}
/**
 */
public void addUIListener( IXGRElementUIListener uiListener) 
{
	if( _vectorUIListeners == null )
	{
		_vectorUIListeners = new Vector();
	}

	_vectorUIListeners.addElement( uiListener );

	// Initialize the listener with data
	//----------------------------------
	fireUIChanged();
}
/**
 */
public Object createWidget( Object objParent )
{
	if( objParent instanceof Composite )
	{
		_combo = new Combo( (Composite)objParent, SWT.DROP_DOWN );

		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.widthHint = 170;
		_combo.setLayoutData( gridData );
		
		initialize();
	}

	return _combo;
}
/**
 */
public void dataChanged( IXGRDataNode dataElement )
{
	if( _dataNodes != null )
	{
		String strNewValue = dataElement.getValue();
		
		int selection = _combo.getSelectionIndex();
			
		for( int i=0; i<_dataNodes.length; ++i )
		{
			if( dataElement == _dataNodes[i] )
			{
				// Synchronize the UI with the dataElement new value
				//--------------------------------------------------
				if( _combo.getItemCount() > i && 
					_combo.getItem( i ).equals( strNewValue ) == false )
				{
					_combo.setItem( i, strNewValue);
				}
			}
		}

		_combo.select( selection );
	}
}
/**
 */
public void fireUIChanged()
{
	if( _dataNodes != null && _combo != null )
	{
		IXGRDataNode nodeSelected = null;
		
		int iSelected = _combo.getSelectionIndex();
			
		if( iSelected >= 0 && iSelected < _dataNodes.length )
		{
			nodeSelected = _dataNodes[ iSelected ];
		}

		IXGRDataNode[] dataNodes = new IXGRDataNode[1];
		dataNodes[0] = nodeSelected;

//		System.out.println(nodeSelected.toString());		
	}
	
	// Tell others that my selection has changed
	//------------------------------------------
	if( _dataNodes != null && _vectorUIListeners != null )
	{
		IXGRDataNode nodeSelected = null;
		
		int iSelected = _combo.getSelectionIndex();
			
		if( iSelected >= 0 && iSelected < _dataNodes.length )
		{
			nodeSelected = _dataNodes[ iSelected ];
		}

		IXGRDataNode[] dataNodes = new IXGRDataNode[1];
		dataNodes[0] = nodeSelected;

//		System.out.println(nodeSelected.toString());
		
		// Notify listeners
		//-----------------
		for( int i=0; i<_vectorUIListeners.size(); ++i )
		{
			((IXGRElementUIListener)(_vectorUIListeners.elementAt(i))).uiChanged( dataNodes );
		}
	}
}
/**
 */
public java.lang.String getDataSourceName()
{
	return _strDataSourceName;
}
/**
 */
public String getName()
{
	return _strName;
}
/**
 */
public java.lang.String getUISourceName()
{
	return _strLinkSourceName;
}
/**
 */
public Object getWidget() 
{
	return _combo;
}
/**
 */
public String getWidgetType()
{
	return "org.eclipse.swt.widgets.Combo";
}
/**
 */
public Control getWindow()
{
	return _combo;
}
/**
 */
public void highlight(boolean bHighlight, Color colorHighlight )
{
	if( _combo != null )
	{
		if( bHighlight == true )
		{
			_colorBackground = _combo.getBackground();
			_combo.setBackground( colorHighlight );
		}
		else if( _colorBackground != null )
		{
			_combo.setBackground( _colorBackground );
		}
	}
}
/**
 */
public void initialize()
{
	if( _combo != null )
	{
		// Font
		//-----
//		_combo.setFont( _fontManager.getDefaultFont( _combo ) );
		
		// Size and span
		//--------------
		if( _dimSize != null || _dimSpan != null )
		{
			GridData gridData = new GridData();
		
			// Size
			//-----
			gridData.widthHint  = 170;

			if( _dimSize != null )
			{
				// Width
				//------
				if( _dimSize.x > 0 )
				{
					gridData.widthHint = _dimSize.x;
				}

				// Height
				//-------
				if( _dimSize.y > 0 )
				{
					gridData.heightHint = _dimSize.y;
				}
			}
		
			// Span
			//-----
			if( _dimSpan != null )
			{
				gridData.horizontalSpan = _dimSpan.x;
				gridData.verticalSpan   = _dimSpan.y;
			}
		
			_combo.setLayoutData( gridData );
		}
		
		// Add data from the data model if not linked to a source
		//-------------------------------------------------------
		if( _dataModel != null )
		{
			if( _strLinkSourceName == null )
			{
				linkToData();
			}

			else if( _uiLinkManager != null )
			{
				_uiLinkManager.addUIListenerToSource( this, _strLinkSourceName );
			}
		}
		
		// Add listeners
		//--------------
		_combo.addSelectionListener( this );

		// Register the widget with the script manager
		//--------------------------------------------
		if( _scriptManager != null && _strName != null && _strName.length() > 0 )
		{
			_scriptManager.declareElement( _strName, _combo );
		}
	}
}
/**
 */
public void linkToData()
{
	if( _strDataSourceName != null && _dataModel != null )
	{
		//remove data element listeners
		if(_dataNodes != null)
		{
			for(int idx=0; idx<_dataNodes.length; idx++)
			   ((IXGRDataChangedNotifier)_dataNodes[idx]).unregisterDataChangedListener(this);
		}
		
		_dataNodes = _dataModel.getDataNodes( _strDataSourceName );
		
		//add data element listeners		
		if(_dataNodes != null)
		{		
			for(int idx=0; idx<_dataNodes.length; idx++)
			   ((IXGRDataChangedNotifier)_dataNodes[idx]).registerDataChangedListener(this);
		}
		
		setData( _dataNodes );
	}
}
/**
 */
public void linkToUI() 
{
	if( _uiLinkManager != null )
	{
		_uiLinkManager.addUIListenerToSource( this, _strLinkSourceName );
	}
}
/**
 */
public void setData( IXGRDataNode[] dataNodes )
{
	// Add the items to the list
	//--------------------------
	if( dataNodes != null && _combo != null )
	{
		if( dataNodes.length > 0 )
		{
			for( int i=0; i< dataNodes.length; ++i )
			{
				if( dataNodes[i] instanceof IXGRDataChangedNotifier )
				{
				  ((IXGRDataChangedNotifier)dataNodes[i]).registerDataChangedListener( this );
				}
								
				_combo.add( dataNodes[i].getValue() );
			}
			
			// Select the first item
			//----------------------
			_combo.select( 0 );
		}
	}
}
/**
 */
public void setDataModel( IXGRDataModel dataModel ) 
{
	_dataModel = dataModel;
}
/**
 */
public void setDataSourceName( String strDataSourceName )
{
	_strDataSourceName = strDataSourceName;
}
/**
 */
public void setFontManager( XGRFontManager fontManager) 
{
	_fontManager = fontManager;
}
/**
 */
public void setName( String strName )
{
	_strName = strName;
}
/**
 */
public void setOnChangeScriptDescriptor( ScriptDescriptor scriptDescriptor )
{
	_scriptDescriptorOnChange = scriptDescriptor;
}
/**
 */
public void setScriptManager( ScriptManager scriptManager )
{
	_scriptManager = scriptManager;
}
/**
 */
public void setSize( Point dimSize ) 
{
	_dimSize = dimSize;
}
/**
 */
public void setSpan( Point dimSpan ) 
{
	_dimSpan = dimSpan;
}
/**
 */
public void setUILinkManager( XGRUILinkManager uiLinkManager )
{
	_uiLinkManager = uiLinkManager;
}
/**
 */
public void setUISourceName( String strLinkSourceName ) 
{
	_strLinkSourceName = strLinkSourceName;
}
/**
 */
public void setVisible(boolean bVisible)
{

}
/**
 */
public void uiChanged( IXGRDataNode[] dataNodes )
{
//	Trace.functionEntry( this, "uiChanged" );

	if( _combo != null )
	{
		// Clear the list
		//---------------
		_combo.removeAll();
		_combo.setText( "" );
		
		// Remove data element listeners
		//------------------------------
		for( int i=0; i<_dataNodes.length; ++i )
		{
			if( _dataNodes[i] instanceof IXGRDataChangedNotifier )
			{
				((IXGRDataChangedNotifier)_dataNodes[i]).unregisterDataChangedListener( this );
			}
		}

		// Get a new list of subnodes with my name
		//----------------------------------------
		if( _dataModel != null && dataNodes != null )
		{
			if( dataNodes.length > 0 )
			{
				_dataNodes = _dataModel.getDataNodes( _strDataSourceName, dataNodes[0] );

				setData( _dataNodes );
			}
		}
	}
	
//	Trace.functionExit( this, "uiChanged" );
}
/**
 * Sent when default selection occurs in the control.
 * <p>
 * For example, on some platforms default selection occurs
 * in a List when the user double-clicks an item.
 * </p>
 */
public void widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent e) {}
/**
 */
public void widgetSelected( SelectionEvent sEvent )
{
	if( _scriptManager != null )
	{
//		Trace.string( sEvent.toString() );

		_scriptManager.evaluateScript( _scriptDescriptorOnChange, _combo );
	}

	fireUIChanged();
}
}
