package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import org.eclipse.swt.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.layout.*;
import org.eclipse.scripting.internal.*;
import java.util.Vector;
import org.eclipse.swt.graphics.*;

public class XGRElementUISwtComposite implements IXGRElement,
												 IXGRElementContainer,
												 IXGRElementNamed, 
												 IXGRElementUI,
												 IXGRElementUIGridElement,
												 IXGRElementUILayoutElement
												 
{
	protected Composite     _composite         = null;
	protected Composite     _compositeParent   = null;
	private Color     _colorBackground = null;
	protected Point         _dimSpan           = null;
	protected GridLayout    _gridLayout        = null;
	protected Vector        _vectorChildren    = new Vector();
	protected ScriptManager _scriptManager     = null;
	protected String        _strName           = null;
	protected int           _iNumberOfColumns  = 1;
	protected int           _iHorizontalIndent = 8;
/**
 */
public XGRElementUISwtComposite()
{
//	Trace.functionEntry( this, "XGRElementUISwtComposite" );
//	Trace.functionExit( this, "XGRElementUISwtComposite" );
}
/**
 * This constructor is used only by a desktop editor/viewer extension
 * The desktop supplies the parent composite.
 */
public XGRElementUISwtComposite( Composite compositeParent ) 
{
//	Trace.functionEntry( this, "XGRElementUISwtComposite" );

	_compositeParent   = compositeParent;
	_iHorizontalIndent = 8;
	
//	Trace.functionExit( this, "XGRElementUISwtComposite" );
}
/**
 */
public void addChildElement( IXGRElement elementChild )
{
	_vectorChildren.addElement( elementChild );
}
/**
 */
public Object createWidget( Object objParent )
{
//	Trace.functionEntry( this, "createWidget" );
//	Trace.object( "objParent", objParent );

	if( objParent != null )
	{
		if( objParent instanceof Composite )
		{
			// Create the component
			//---------------------
			// Trace.string( "Parent is not a Composite" );
			_composite = new Composite( (Composite) objParent, SWT.NULL );
		}
	}

	// If this composite was preconstructed, the parent would have been
	// given to us in the constructor
	//-----------------------------------------------------------------
	else if( _compositeParent != null )
	{
		// Trace.string( "Parent composite was preconstructed" );
		_composite = new Composite( _compositeParent, SWT.NULL );
	}

	// Initialize myself, and create my children
	//------------------------------------------
	if( _composite != null )
	{
		// Initialize myself
		//------------------
		initialize();
		
		// Create and initialize the component's children
		//-----------------------------------------------
		IXGRElementUI elementChild = null;

		for( int i=0; i<_vectorChildren.size(); ++i )
		{
			if( _vectorChildren.elementAt( i ) instanceof IXGRElementUI )
			{
				elementChild = (IXGRElementUI)_vectorChildren.elementAt( i );
				elementChild.createWidget( _composite );
			}
		}
	}

//	Trace.functionExit( this, "createWidget" );

	return _composite;
}
/**
 */
public Vector getChildElements()
{
	return _vectorChildren;
}
/**
 */
public String getName()
{
	return _strName;
}
/**
 */
public int getNumberOfColumns() 
{
	return _iNumberOfColumns;
}
/**
 */
public Object getWidget() 
{
	return _composite;
}
/**
 */
public String getWidgetType()
{
	return "org.eclipse.swt.widgets.Composite";
}
/**
 */
public Control getWindow()
{
	return _composite;
}
/**
 */
public void highlight(boolean bHighlight, Color colorHighlight )
{
	if( _composite != null )
	{
		if( bHighlight == true )
		{
			_colorBackground = _composite.getBackground();
			_composite.setBackground( colorHighlight );
		}
		else if( _colorBackground != null )
		{
			_composite.setBackground( _colorBackground );
		}
	}
}
/**
 */
public void initialize()
{
//	Trace.functionEntry( this, "initialize" );

	if( _composite != null )
	{	
		// Create a grid layout
		// Reset its margin width (default is 5)
		//--------------------------------------
		_gridLayout = new GridLayout();
		_gridLayout.marginWidth = 0;
		_gridLayout.numColumns  = _iNumberOfColumns;

//		Trace.value( "_iNumberOfColumns", _iNumberOfColumns );
	
		_composite.setLayout( _gridLayout );

		// Set the grid data
		//------------------
		GridData gridData = new GridData();

		gridData.horizontalIndent    = _iHorizontalIndent;
		gridData.horizontalAlignment = GridData.FILL;
		
		if( _dimSpan != null )
		{
			gridData.horizontalSpan = _dimSpan.x;
			gridData.verticalSpan   = _dimSpan.y;
		}
				
		_composite.setLayoutData( gridData );

		// Register the widget with the script manager
		//--------------------------------------------
		if( _scriptManager != null && _strName != null && _strName.length() > 0 )
		{
			_scriptManager.declareElement( _strName, _composite );
		}
	}
	
//	Trace.functionExit( this, "initialize" );
}
/**
 */
public void setHorizontalIndent( int iHorizontalIndent )
{
	_iHorizontalIndent = iHorizontalIndent;
}
/**
 */
public void setName( String strName )
{
	_strName = strName;
}
/**
 */
public void setNumberOfColumns( int iNumberOfColumns )
{
//	Trace.functionEntry( this, "setNumberOfColumns" );
	
	_iNumberOfColumns = iNumberOfColumns;
	
//	Trace.value( "_iNumberOfColumns", _iNumberOfColumns );
	
//	Trace.functionExit( this, "setNumberOfColumns" );
}
/**
 */
public void setScriptManager( ScriptManager scriptManager ) 
{
	_scriptManager = scriptManager;
}
/**
 */
public void setSpan( Point dimSpan ) 
{
	_dimSpan = dimSpan;
}
/**
 */
public void setVisible( boolean bVisible )
{
	if( _composite != null )
	{
		_composite.setVisible( bVisible );
	}
}
}
