package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import org.eclipse.scripting.internal.uirenderer.*;
import org.eclipse.swt.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.layout.*;
import org.eclipse.scripting.internal.*;
import java.util.Vector;

public class XGRElementUISwtLabel implements IXGRElementCaption,
											 IXGRElementUIListener,
											 IXGRElementNamed,
								             IXGRElementUI,
								             IXGRElementUIDataPresentationElement,
								             IXGRElementUIGridElement,
								             IXGRElementUITextPresentationElement,
								             IXGRDataChangedListener
{
	private Point                      _dimSpan                 = null;
	private Label                      _label                   = null;
	private Color     _colorBackground = null;	
	private ScriptManager              _scriptManager           = null;
	private String                     _strCaption              = null;
	private String                     _strDataSourceName       = null;
	private String                     _strLinkSourceName       = null;
	private String                     _strName                 = null;
	private IXGRDataModel              _dataModel               = null;
	private XGRUILinkManager           _uiLinkManager           = null;
	private IXGRDataNode[]             _dataNodes               = null;
	private XGRFontManager             _fontManager             = null;
	private Vector                     _vectorUIListeners       = null;

/**
 */
public XGRElementUISwtLabel()
{
	super();
	
//	Trace.functionEntry( this, "XGRElementUISwtLabel" );
//	Trace.functionExit( this, "XGRElementUISwtLabel" );
}
/**
 */
public void addUIListener( IXGRElementUIListener uiListener) 
{
	if( _vectorUIListeners == null )
	{
		_vectorUIListeners = new Vector();
	}

	_vectorUIListeners.addElement( uiListener );

	// Initialize the listener with data
	//----------------------------------
	fireUIChanged();
}
/**
 */
public Object createWidget( Object objParent )
{
//	Trace.functionEntry( this, "createWidget" );
	
	if( objParent instanceof Composite )
	{
		_label = new Label( (Composite)objParent, SWT.LEFT );

		initialize();
	}
	
//	Trace.functionExit( this, "createWidget" );

	return _label;
}
	/**
	*/
	public void dataChanged(IXGRDataNode dataElement)
	{
		//update text value
		setData(_dataNodes);
	}
/**
 */
public void fireUIChanged()
{
	// Tell others that my selection has changed
	//------------------------------------------
	if(  _vectorUIListeners != null && _label != null )
	{
		// Notify listeners
		//-----------------
		for( int i=0; i<_vectorUIListeners.size(); ++i )
		{
			((IXGRElementUIListener)(_vectorUIListeners.elementAt(i))).uiChanged( _dataNodes );
		}
	}
}
/**
 */
public java.lang.String getDataSourceName()
{
	return _strDataSourceName;
}
/**
 */
public String getName()
{
	return _strName;
}
/**
 */
public java.lang.String getUISourceName()
{
	return _strLinkSourceName;
}
/**
 */
public Object getWidget() 
{
	return _label;
}
/**
 */
public String getWidgetType()
{
	return "org.eclipse.swt.widgets.Label";
}
/**
 */
public Control getWindow()
{
	return _label;
}
/**
 */
public void highlight(boolean bHighlight, Color colorHighlight)
{
	if( _label != null )
	{
		if( bHighlight == true )
		{
			_colorBackground = _label.getBackground();
			_label.setBackground( colorHighlight );
		}
		else if( _colorBackground != null )
		{
			_label.setBackground( _colorBackground );
		}
	}
}
/**
 */
public void initialize()
{
//	Trace.functionEntry( this, "initialize" );

	if( _label != null )
	{
		// Font
		//-----
//		_label.setFont( _fontManager.getDefaultFont( _label ) );
		
		// Layout
		//-------	
		GridData gridData = new GridData();
			
		gridData.horizontalAlignment = GridData.FILL;

		if( _dimSpan != null )
		{
			gridData.horizontalSpan = _dimSpan.x;
			gridData.verticalSpan   = _dimSpan.y;	
		}
		
		_label.setLayoutData( gridData );

		// Caption
		//--------	
		if( _strCaption != null )
		{
			_label.setText( _strCaption );
		}
		else
		{
			_label.setText( "               " );
		}

		// Add data from the data model if not linked to a source
		//-------------------------------------------------------
		if( _dataModel != null )
		{
			if( _strLinkSourceName == null )
			{
				linkToData();
			}

			else if( _uiLinkManager != null )
			{
				_uiLinkManager.addUIListenerToSource( this, _strLinkSourceName );
			}
		}

		// Register the widget with the script manager
		//--------------------------------------------
		if( _scriptManager != null && _strName != null && _strName.length() > 0 )
		{
			_scriptManager.declareElement( _strName, _label );
		}
	}
	
//	Trace.functionExit( this, "initialize" );
}
/**
 */
public void linkToData()
{
	if( _strDataSourceName != null && _dataModel != null )
	{
		_dataNodes = _dataModel.getDataNodes( _strDataSourceName );
		setData( _dataNodes );
	}
}
/**
 */
public void linkToUI() 
{
	if( _uiLinkManager != null )
	{
		_uiLinkManager.addUIListenerToSource( this, _strLinkSourceName );
	}
}
/**
 */
public void setCaptionText( String strCaption )
{
//	Trace.functionEntry( this, "setCaptionText" );
	
	_strCaption = strCaption;
	
//	Trace.functionExit( this, "setCaptionText" );
}
/**
 */
public void setData( IXGRDataNode[] dataNodes )
{
//	Trace.functionEntry( this, "setData" );
	
	// Add the items to the list
	//--------------------------
	if( dataNodes != null && _label != null )
	{
		if( dataNodes.length > 0 )
		{
			StringBuffer strbufText = new StringBuffer();

			for( int i=0; i<dataNodes.length; ++i )
			{
				if( dataNodes[i] instanceof IXGRDataChangedNotifier )
				{
				  ((IXGRDataChangedNotifier)dataNodes[i]).registerDataChangedListener( this );
				}

				strbufText.append( dataNodes[i].getValue() );
				strbufText.append( " " );
			}
			
			_label.setText( strbufText.toString() );
		}
	}

	fireUIChanged();

//	Trace.functionExit( this, "setData" );
}
/**
 */
public void setDataModel( IXGRDataModel dataModel ) 
{
	_dataModel = dataModel;
}
/**
 */
public void setDataSourceName( String strDataSourceName )
{
	_strDataSourceName = strDataSourceName;
}
/**
 */
public void setFontManager( XGRFontManager fontManager) 
{
	_fontManager = fontManager;
}
/**
 */
public void setName( String strName )
{
	_strName = strName;
}
/**
 */
public void setScriptManager( ScriptManager scriptManager ) 
{
	_scriptManager = scriptManager;
}
/**
 */
public void setSpan( Point dimSpan ) 
{
	_dimSpan = dimSpan;
}
/**
 */
public void setUILinkManager( XGRUILinkManager uiLinkManager )
{
	_uiLinkManager = uiLinkManager;
}
/**
 */
public void setUISourceName( String strLinkSourceName ) 
{
	_strLinkSourceName = strLinkSourceName;
}
/**
 */
public void setVisible(boolean bVisible) {}
/**
 */
public void uiChanged( IXGRDataNode[] dataNodes )
{
//	Trace.functionEntry( this, "dataChanged" );

	if( _label != null )
	{
		// Clear the label
		//----------------
		_label.setText( "" );

		// Remove data element listeners
		//------------------------------
		if( _dataNodes != null )
		{
			for( int i=0; i<_dataNodes.length; ++i )
			{
				if( _dataNodes[i] instanceof IXGRDataChangedNotifier )
				{
					((IXGRDataChangedNotifier)_dataNodes[i]).unregisterDataChangedListener( this );
				}
			}
		}
		
		// Get a new list of subnodes with my name
		//----------------------------------------
		if( _dataModel != null && dataNodes != null )
		{
			if( dataNodes.length > 0 )
			{
				_dataNodes = _dataModel.getDataNodes( _strDataSourceName, dataNodes[0] );

				// Add data element listeners
				//---------------------------
				if( _dataNodes != null )
				{
					for(int i=0; i<_dataNodes.length; ++i )
					{
						if( _dataNodes[i] instanceof IXGRDataChangedNotifier )
						{
							((IXGRDataChangedNotifier)_dataNodes[i]).registerDataChangedListener( this );
						}
					}
				}
				
				setData( _dataNodes );
			}
		}
	}
	
//	Trace.functionExit( this, "dataChanged" );
}
}
