package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import java.util.Vector;
import org.eclipse.scripting.internal.uirenderer.*;
import org.eclipse.scripting.internal.*;
import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;

public class XGRElementUISwtList implements IXGRElement,
											IXGRElementUIListener,
											IXGRElementEventGeneratorOnChange, 
											IXGRElementNamed,
											IXGRElementUI,
											IXGRElementUISizable,
											IXGRElementUIDataPresentationElement,
											IXGRElementUIGridElement,
											IXGRElementUITextPresentationElement,
											SelectionListener,
											IXGRDataChangedListener
{
	private Color                      _colorBackground         = null;
	private Point                      _dimSize                 = null;	
	private Point                      _dimSpan                 = null;	
	private List                       _list                    = null;
	private String                     _strDataSourceName       = null;
	private String                     _strLinkSourceName       = null;
	private String   	               _strName                 = null;
	private ScriptDescriptor           _scriptDescriptorOnChange = null;
	private Vector      			   _vectorDataNodes         = null;
	private Vector                     _vectorUIListeners       = null;
	private IXGRDataModel              _dataModel               = null;
	private XGRUILinkManager           _uiLinkManager           = null;
	private XGRFontManager             _fontManager             = null;
	private ScriptManager              _scriptManager           = null;

/**
 */
public XGRElementUISwtList()
{
//	Trace.functionEntry( this, "XGRElementUISwtList" );

	_vectorDataNodes = new Vector();
	
//	Trace.functionExit( this, "XGRElementUISwtList" );
}
/**
 */
public void addUIListener( IXGRElementUIListener uiListener) 
{
	if( _vectorUIListeners == null )
	{
		_vectorUIListeners = new Vector();
	}

	_vectorUIListeners.addElement( uiListener );

	// Initialize the listener with data
	//----------------------------------
	fireUIChanged();
}
/**
 */
public Object createWidget( Object objParent )
{
	if( objParent instanceof Composite )
	{
		_list = new List( (Composite)objParent, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL );

		initialize();
	}

	return _list;
}
/**
 */
public void dataChanged( IXGRDataNode dataElement )
{
	if( _vectorDataNodes != null )
	{
		String strNewValue = dataElement.getValue();	
			
		// Synchronize the UI with the dataElement new value
		//--------------------------------------------------	
		for( int i=0; i<_vectorDataNodes.size(); ++i )
		{
			if( dataElement == _vectorDataNodes.elementAt( i ) )
			{
				if( _list.getItemCount() > i &&
					_list.getItem( i ).equals( strNewValue ) == false )
				{
					_list.setItem( i, strNewValue );
				}
			}
		}
	}
}
/**
 */
public void fireUIChanged()
{
//	Trace.functionEntry( this, "fireUIChanged" );
	
	// Tell others that my selection has changed
	//------------------------------------------
	if( _vectorUIListeners != null && _list != null )
	{
		int[] iaSelected = _list.getSelectionIndices();

		if( iaSelected != null )
		{
			IXGRDataNode[] nodesSelected = new IXGRDataNode[ iaSelected.length ];
		
			for( int i=0; i<iaSelected.length; ++i )
			{
				nodesSelected[i] = (IXGRDataNode)_vectorDataNodes.elementAt( iaSelected[i] );
			}

			// Notify listeners
			//-----------------
			for( int i=0; i<_vectorUIListeners.size(); ++i )
			{
				((IXGRElementUIListener)(_vectorUIListeners.elementAt(i))).uiChanged( nodesSelected );
			}
		}
	}
	
//	Trace.functionExit( this, "fireUIChanged" );
}
/**
 */
public java.lang.String getDataSourceName()
{
	return _strDataSourceName;
}
/**
 */
public String getName()
{
	return _strName;
}
/**
 */
public java.lang.String getUISourceName()
{
	return _strLinkSourceName;
}
/**
 */
public Object getWidget() 
{
	return _list;
}
/**
 */
public String getWidgetType()
{
	return "org.eclipse.swt.widgets.List";
}
/**
 */
public Control getWindow()
{
	return _list;
}
/**
 */
public void highlight( boolean bHighlight, Color colorHighlight )
{
	if( _list != null )
	{
		if( bHighlight == true )
		{
			_colorBackground = _list.getBackground();
			_list.setBackground( colorHighlight );
		}
		else if( _colorBackground != null )
		{
			_list.setBackground( _colorBackground );
		}
	}
}
/**
 */
public void initialize()
{
	if( _list != null )
	{
		// Font
		//-----
//		_list.setFont( _fontManager.getDefaultFont( _list ) );
		
		// Layout
		//-------
		GridData gridData = new GridData();
		
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment   = GridData.FILL;
		gridData.widthHint           = 170;
		gridData.heightHint          = 100;

		if( _dimSize != null )
		{
			// Width
			//------
			if( _dimSize.x > 0 )
			{
				gridData.widthHint = _dimSize.x;
			}
			
			// Height
			//-------
			if( _dimSize.y > 0 )
			{
				gridData.heightHint = _dimSize.y;
			}
		}
			
		// Span
		//-----
		if( _dimSpan != null )
		{
			gridData.horizontalSpan = _dimSpan.x;
			gridData.verticalSpan   = _dimSpan.y;
		}

		_list.setLayoutData( gridData );

		// Add data from the data model if not linked to a source
		//-------------------------------------------------------
		if( _dataModel != null )
		{
			if( _strLinkSourceName == null )
			{
				linkToData();
			}

			else
			{
				linkToUI();
			}
		}
	
		// Add listeners
		//--------------
		_list.addSelectionListener( this );

		// Register the widget with the script manager
		//--------------------------------------------
		if( _scriptManager != null && _strName != null && _strName.length() > 0 )
		{
			_scriptManager.declareElement( _strName, _list );
		}
	}
}
/**
 */
public void linkToData()
{
	if( _strDataSourceName != null && _dataModel != null )
	{
		IXGRDataNode[] dataNodes = _dataModel.getDataNodes( _strDataSourceName );
		setData( dataNodes );
	}
}
/**
 */
public void linkToUI() 
{
	if( _uiLinkManager != null )
	{
		_uiLinkManager.addUIListenerToSource( this, _strLinkSourceName );
	}
}
/**
 */
public void setData( IXGRDataNode[] dataNodes )
{
//	Trace.functionEntry( this, "setData" );
	
	// Add the items to the end of the list
	//-------------------------------------
	if( dataNodes != null && _list != null )
	{
		if( dataNodes.length > 0 )
		{
			for( int i=0; i< dataNodes.length; ++i )
			{
				if( dataNodes[i] instanceof IXGRDataChangedNotifier )
				{
					((IXGRDataChangedNotifier)dataNodes[i]).registerDataChangedListener( this );
				}

				_list.add( dataNodes[i].getValue() );
				_vectorDataNodes.addElement( dataNodes[i] );
			}
			
			// Select the first item
			//----------------------
			_list.setSelection( 0 );
		}
	}

	fireUIChanged();

//	Trace.functionExit( this, "setData" );
}
/**
 */
public void setDataModel( IXGRDataModel dataModel ) 
{
	_dataModel = dataModel;
}
/**
 */
public void setDataSourceName( String strDataSourceName )
{
	_strDataSourceName = strDataSourceName;
}
/**
 */
public void setFontManager( XGRFontManager fontManager) 
{
	_fontManager = fontManager;
}
/**
 */
public void setName( String strName )
{
	_strName = strName;
}
/**
 */
public void setOnChangeScriptDescriptor( ScriptDescriptor scriptDescriptor )
{
	_scriptDescriptorOnChange = scriptDescriptor;
}
/**
 */
public void setScriptManager( ScriptManager scriptManager )
{
	_scriptManager = scriptManager;
}
/**
 */
public void setSize( Point dimSize ) 
{
	_dimSize = dimSize;
}
/**
 */
public void setSpan( Point dimSpan ) 
{
	_dimSpan = dimSpan;
}
/**
 */
public void setUILinkManager( XGRUILinkManager uiLinkManager )
{
	_uiLinkManager = uiLinkManager;
}
/**
 */
public void setUISourceName( String strLinkSourceName ) 
{
	_strLinkSourceName = strLinkSourceName;
}
/**
 */
public void setVisible(boolean bVisible)
{

}
/**
 */
public void uiChanged( IXGRDataNode[] dataNodes )
{
//	Trace.functionEntry( this, "uiChanged" );

	if( _list != null )
	{	
		// Clear the list
		//---------------
		_list.removeAll();

		_vectorDataNodes.removeAllElements();

		IXGRDataNode dataNode = null;
		
		// Remove data element listeners
		//------------------------------
		for( int i=0; i<_vectorDataNodes.size(); ++i )
		{
			dataNode = (IXGRDataNode)_vectorDataNodes.elementAt(i);

			if( dataNode instanceof IXGRDataChangedNotifier )
			{
				((IXGRDataChangedNotifier)dataNode).unregisterDataChangedListener( this );
			}
		}
		
		// Get a new list of subnodes with my name
		//----------------------------------------
		if( _dataModel != null && dataNodes != null )
		{
			IXGRDataNode[] dataNodeChildren = null;
			
			for( int i=0; i<dataNodes.length; ++i )
			{
				dataNodeChildren = _dataModel.getDataNodes( _strDataSourceName, dataNodes[i] );
				setData( dataNodeChildren );
			}
		}
	}
	
//	Trace.functionExit( this, "uiChanged" );
}
/**
 * Sent when default selection occurs in the control.
 * <p>
 * For example, on some platforms default selection occurs
 * in a List when the user double-clicks an item.
 * </p>
 */
public void widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent e) {}
/**
 */
public void widgetSelected( SelectionEvent sEvent )
{
	// Run scripts
	//------------
	if( _scriptManager != null )
	{
//		Trace.string( sEvent.toString() );
		
		_scriptManager.evaluateScript( _scriptDescriptorOnChange, _list );
	}

	fireUIChanged();
}
}
