package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import org.eclipse.scripting.internal.uirenderer.*;
import org.eclipse.swt.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.jface.preference.*;
import org.eclipse.scripting.internal.*;
import org.eclipse.scripting.*;
import java.util.*;

public class XGRElementUISwtPreferenceDialog extends PreferenceDialog implements IXGRElement,
							                      IXGRElementCaption,
							                      IXGRElementContainer,
							                      IXGRElementNamed,
							                      IXGRElementEventGeneratorOnLoad,
							                      IXGRElementEventGeneratorOnUnload,
												  IXGRElementUI,
												  ICancel
{

	private   ScriptManager    _scriptManager            = null;
	private   String     	   _strCaption	             = null;
	private   String     	   _strName	                 = null;
	private   ScriptDescriptor _scriptDescriptorOnLoad   = null;
	private   ScriptDescriptor _scriptDescriptorOnUnload = null;
	protected Vector     	   _vectorChildren 	         = new Vector();
	protected boolean          _bCancelPressed           = false;

/**
 */
public XGRElementUISwtPreferenceDialog()
{
	// Specify the name of our translatable resources
	//-----------------------------------------------
//	super( null, "XGRPreferenceDialog" );
	super( null, new PreferenceManager() );

//	Trace.functionEntry( this, "XGRElementUISwtPreferenceDialog" );		
//	Trace.functionExit( this, "XGRElementUISwtPreferenceDialog" );
	
}
/**
 */
public void addChildElement( IXGRElement elementChild ) 
{
//	Trace.functionEntry( this, "addChildElement" );
	
	_vectorChildren.addElement( elementChild );

//	Trace.functionExit( this, "addChildElement" );
}
/**
 */
public void cancelPressed() 
{
	_bCancelPressed = true;
	
	if( _scriptManager != null )
	{
//		Trace.string( "PreferenceDialog Abort" );

		_scriptManager.evaluateScript( _scriptDescriptorOnUnload, this );
	}

	// Close the dialog
	//-----------------
	super.cancelPressed();

	return;
}
/**
 */
public Object createWidget( Object objParent )
{
//	Trace.functionEntry( this, "createWidget" );

	// Create the component's children
	//--------------------------------
	IXGRElementUI elementChild = null;
	Object       objChild     = null;
	
	for( int i=0; i<_vectorChildren.size(); ++i )
	{
		objChild = _vectorChildren.elementAt( i );

		if( objChild instanceof IXGRElementUI )
		{
			elementChild = (IXGRElementUI)objChild;
			elementChild.createWidget( this );
		}
	}

	// Initialize myself
	//------------------
	initialize();
	
//	Trace.functionExit( this, "createWidget" );

	return this;
}
/**
 */
public Vector getChildElements()
{
	return _vectorChildren;
}
/**
 */
public Control getControl()
{
	return null;
}
/**
 */
public String getName()
{
	return _strName;
}
/**
 */
public PreferenceManager getPrefManager()
{
	return super.getPreferenceManager();
}
/**
 */
public Object getWidget() 
{
	return this;
}
/**
 */
public String getWidgetType()
{
	return "org.eclipse.jface.preferences.PreferenceDialog";
}
/**
 */
public Control getWindow() 
{
	return null;
}
/**
 */
public void highlight(boolean bHighlight, org.eclipse.swt.graphics.Color colorHighlight) {}
/**
 */
public void initialize()
{
//	Trace.functionEntry( this, "initialize" );

	// Initialize my properties
	//-------------------------
	if( _strCaption != null )
	{
		super.getShell().setText( _strCaption );
	}

	// Register the widget with the script manager
		//--------------------------------------------
	if( _scriptManager != null && _strName != null && _strName.length() > 0 )
	{
		_scriptManager.declareElement( _strName, this );
	}
	
//	Trace.functionExit ( this, "initialize" );
}
/**
 */
public boolean isCancel() 
{
	return _bCancelPressed;
}
/**
 */
public void okPressed() 
{
	_bCancelPressed = false;
	
	if( _scriptManager != null )
	{
		_scriptManager.evaluateScript( _scriptDescriptorOnUnload, this );
	}

	// Close the dialog
	//-----------------
	super.okPressed();
	
	return;
}
/**
 */
public void setCaptionText( String strCaption )
{
	_strCaption = strCaption;
}
/**
 */
public void setName( String strName )
{
//	Trace.functionEntry( this, "setName" );

	_strName = strName;
	
//	Trace.functionExit( this, "setName" );
}
/**
 */
public void setOnLoadScriptDescriptor( ScriptDescriptor scriptDescriptor )
{
	_scriptDescriptorOnLoad = scriptDescriptor;
}
/**
 */
public void setOnUnloadScriptDescriptor( ScriptDescriptor scriptDescriptor )
{
	_scriptDescriptorOnUnload = scriptDescriptor;
}
/**
 */
public void setScriptManager( ScriptManager scriptManager )
{
	_scriptManager = scriptManager;
}
/**
 */
public void setVisible( boolean bVisible ) 
{
//	Trace.functionEntry( this, "setVisible" );
	
	if( _scriptManager != null )
	{
//		Trace.string( "PreferenceDialog Load" );

		_scriptManager.evaluateScript( _scriptDescriptorOnLoad, this );
	}
	
	super.open();
	
//	Trace.functionExit( this, "setVisible" );
}
}
