package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 */
import org.eclipse.swt.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.scripting.internal.*;
import java.util.Vector;

public class XGRElementUISwtScriptPreferenceDialog implements IXGRElement, 
														     IXGRElementCaption,
														     IXGRElementContainer,
														     IXGRElementNamed,
														     IXGRElementUI,
														     IXGRParentComposite
{
	private ScriptManager 	         _scriptManager            = null;
	private String                   _strCaption               = null;
	private String                   _strName                  = null;
	private ScriptDescriptor 	     _scriptDescriptorOnLoad   = null;
	private ScriptDescriptor   		 _scriptDescriptorOnUnload = null;
	protected ScriptPreferenceViewer _treeViewer               = null;
	protected GridLayout    		 _gridLayout        = null;
	protected Vector     	   		 _vectorChildren 	      = new Vector();

/**
 */
public XGRElementUISwtScriptPreferenceDialog()
{
}
/**
 */
public void addChildElement( IXGRElement elementChild ) 
{
//	Trace.functionEntry( this, "addChildElement" );

	if(elementChild instanceof XGRElementUISwtScriptPreferencePage)
		_vectorChildren.addElement( elementChild );

//	Trace.functionExit( this, "addChildElement" );
}
/**
 */
public Object createWidget( Object objParent )
{
//	Trace.functionEntry( this, "createWidget" );

	// Create the subcomponents
	//-------------------------
	if(_treeViewer == null)
	   _treeViewer = new ScriptPreferenceViewer((Composite)objParent, SWT.NULL, new ScriptPreferenceManager());	

	// Create the component's children
	// In this case, smart guide pages
	//--------------------------------
	IXGRElementUI elementChild = null;
	Object       objChild     = null;

	for( int i=0; i<_vectorChildren.size(); ++i )
	{
		objChild = _vectorChildren.elementAt( i );

		if( objChild instanceof IXGRElementUI )
		{
			elementChild = (IXGRElementUI)objChild;
			elementChild.createWidget( _treeViewer );
		}
	}

	_treeViewer.createContents( (Composite)objParent );
	
	// Initialize myself
	//------------------
	initialize();

//	Trace.functionExit( this, "createWidget" );

	return this;	
}
/**
 */
public Vector getChildElements()
{
	return _vectorChildren;
}
/**
 */
public String getName()
{
	return _strName;
}
/**
 */
public Object getWidget() 
{
	return _treeViewer;
}
/**
 */
public String getWidgetType()
{
	return "org.eclipse.swt.widgets.Composite";
}
/**
 */
public Control getWindow() 
{
	return _treeViewer;
}
/**
 */
public void highlight(boolean bHighlight, org.eclipse.swt.graphics.Color colorHighlight) {}
/**
 */
public void initialize()
{
//	Trace.functionEntry( this, "initialize" );

	
	// Initialize my properties
	//-------------------------
	if(_treeViewer != null){
		GridData gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace   = true;
		
		_treeViewer.setLayoutData( gridData );
	}
		
	// Register the widget with the script manager
	//--------------------------------------------
	if( _scriptManager != null && _strName != null && _strName.length() > 0 )
	{
		_scriptManager.declareElement( _strName, this );
	}
	
//	Trace.functionExit ( this, "initialize" );
}
/**
 */
public void setCaptionText( String strCaption )
{
	_strCaption = strCaption;
}
/**
 */
public void setName( String strName )
{
//	Trace.functionEntry( this, "setName" );

	_strName = strName;
	
//	Trace.functionExit( this, "setName" );
}
/**
 */
public void setParentComposite(Composite compositeParent)
{
	// Create the smart guide
	// The parent composite should have been previously set
	// by the setParentComposite method
	//-----------------------------------------------------
	if( compositeParent instanceof Composite )
	{
		_treeViewer = new ScriptPreferenceViewer( compositeParent, SWT.NULL, new ScriptPreferenceManager() );
	}
}
/**
 */
public void setScriptManager( ScriptManager scriptManager )
{
	_scriptManager = scriptManager;
}
/**
 */
public void setVisible( boolean bVisible ) 
{
}
}
