package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import org.eclipse.swt.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.jface.preference.*;
import org.eclipse.scripting.internal.*;
import java.util.*;

public class XGRElementUISwtScriptPreferencePage extends ScriptPreferencePage implements IXGRElement, 
																		     IXGRElementCaption,
																		     IXGRElementContainer,
																		     IXGRElementEventGeneratorOnReset,
							                						         IXGRElementEventGeneratorOnSubmit,
																		     IXGRElementNamed,
																		     IXGRElementUI,
																		     IXGRElementUILayoutElement 
{
	private Color                    _colorBackground          = null;
	private ScriptManager 	         _scriptManager            = null;
	private String                   _strCaption               = null;
	private String                   _strName                  = null;
	private ScriptDescriptor   	   	 _scriptDescriptorOnReset  = null;
	private ScriptDescriptor   		 _scriptDescriptorOnSubmit = null;
	private Vector                   _vectorSubpages           = null;
	private XGRElementUISwtComposite _xgrComposite             = null;
	protected ScriptPreferenceManager _pm = null;
	protected String nodePath="";

/**
 */
public XGRElementUISwtScriptPreferencePage()
{	
	// Specify the name of our translatable resources
	//-----------------------------------------------
	super( "" );
		
//	Trace.functionEntry( this, "XGRElementUISwtPreferencePage" );

	// Create my composite for my children
	//------------------------------------
	_xgrComposite = new XGRElementUISwtComposite();
	
//	Trace.functionExit( this, "XGRElementUISwtPreferencePage" );
}
/**
 */
public void addChildElement( IXGRElement elementChild )
{
//	Trace.functionEntry( this, "addChildElement" );

	// If my child is a subpage, do not add it to my
	// composite.  Remember it so that I can create
	// it when createWidget() is called.
	//----------------------------------------------
	if( elementChild instanceof XGRElementUISwtScriptPreferencePage )
	{
		if( _vectorSubpages == null )
		{
			_vectorSubpages = new Vector();
		}
		
		_vectorSubpages.addElement( elementChild );
	}
	
	// Otherwise, add it to my composite
	//----------------------------------
	else
	{
		_xgrComposite.addChildElement( elementChild );
	}

//	Trace.functionExit( this, "addChildElement" );
}
/**
 */
public Control createContents( Composite compositeParent ) 
{
	// Create the composite
	//---------------------
	_xgrComposite.createWidget( compositeParent );

	// Initialize myself
	//------------------
	initialize();

	return _xgrComposite.getWindow();
}
/**
 */
public Object createWidget( Object objParent )
{
//	Trace.functionEntry( this, "createWidget" );

	// Add this page to a parent, if a parent exists
	//----------------------------------------------
	if( objParent != null)
	{		
		// If the parent is a script preference dialog, add this page to it as a root page
		// Use the hashcode as my identifier
		//-------------------------------------------------------------------------
		if( objParent instanceof ScriptPreferenceViewer )
		{
			setNodePath(String.valueOf( hashCode() ));
				
			_pm = ((ScriptPreferenceViewer)objParent).getPreferenceManager();				
			_pm.addToRoot( getNodePath(), this );
		}

		// If the parent is another page, add myself to it
		// Use the hashcode as identifiers
		//------------------------------------------------
		else if( objParent instanceof XGRElementUISwtScriptPreferencePage )
		{
			String parentPath = ((XGRElementUISwtScriptPreferencePage)objParent).getNodePath();
		    String path = String.valueOf( hashCode() );
			
		    _pm = ((XGRElementUISwtScriptPreferencePage)objParent).getPreferenceManager();			
			_pm.addTo( parentPath, path, this );

			setNodePath(parentPath + "." + path);
		}
	}

	// Initialize myself
	//------------------
	initialize();

	// Create and initialize my subpages
	//----------------------------------
	if( _vectorSubpages != null )
	{
		IXGRElementUI elementChild = null;
		
		for( int i=0; i<_vectorSubpages.size(); ++i )
		{
			elementChild = (IXGRElementUI)_vectorSubpages.elementAt( i );
			elementChild.createWidget( this );
		}
	}
	
//	Trace.functionExit( this, "createWidget" );

	return this;
}
/**
 */
public void doCancel() 
{
	if( _scriptManager != null )
	{
//		Trace.string( "PreferencePage Abort" );

		_scriptManager.evaluateScript( _scriptDescriptorOnSubmit, this );
	}

	return;
}
/**
 */
public boolean doOk() 
{
	// Note: This doesn't seem to get called
	//--------------------------------------
	
	if( _scriptManager != null )
	{
//		Trace.string( "PreferencePage Submit" );

		_scriptManager.evaluateScript( _scriptDescriptorOnSubmit, this );
	}

	return true;
}
/**
 */
public void enterPage()
{
	if( _scriptManager != null )
	{
//		Trace.string( "PreferencePage Reset" );

		_scriptManager.evaluateScript( _scriptDescriptorOnReset, this );
	}
}
/**
 */
public Vector getChildElements()
{
	return _xgrComposite.getChildElements();
}
/**
 */
public String getName()
{
	return _strName;
}
/**
 */
public String getNodePath() {
	return nodePath;
}
/**
 */
public int getNumberOfColumns() 
{
	return _xgrComposite.getNumberOfColumns();
}
	protected ScriptPreferenceManager getPreferenceManager()
	{
		return _pm;
	}
/**
 */
public Object getWidget() 
{
	return this;
}
/**
 */
public String getWidgetType()
{
	return "org.eclipse.jface.preferences.PreferencePage";
}
/**
 */
public Control getWindow() 
{
	return null;
}
/**
 */
public void highlight(boolean bHighlight, org.eclipse.swt.graphics.Color colorHighlight) {}
/**
 */
public void initialize()
{
//	Trace.functionEntry( this, "initialize" );

	// Initialize my properties
	//-------------------------
	if( _strCaption != null )
	{
		super.setTitle( _strCaption );
	}

	// Register the widget with the script manager
	//--------------------------------------------
	if( _scriptManager != null && _strName != null && _strName.length() > 0 )
	{
		_scriptManager.declareElement( _strName, this );
	}

//	Trace.functionExit( this, "initialize" );
}
/**
 */
public boolean leavePage()
{
	if( _scriptManager != null )
	{
//		Trace.string( "PreferencePage Unload" );

		_scriptManager.evaluateScript( _scriptDescriptorOnSubmit, this );
	}
	
	return true;
}
/**
 */
public void setCaptionText( String strCaption )
{
	_strCaption = strCaption;
}
/**
 */
public void setName( String strName )
{
	_strName = strName;
}
/**
 */
public void setNodePath(String path)
{
	nodePath = path;
}
/**
 */
public void setNumberOfColumns(int iNumberOfColumns)
{
	_xgrComposite.setNumberOfColumns( iNumberOfColumns );
}
/**
 */
public void setOnResetScriptDescriptor( ScriptDescriptor scriptDescriptor )
{
	_scriptDescriptorOnReset = scriptDescriptor;
}
/**
 */
public void setOnSubmitScriptDescriptor( ScriptDescriptor scriptDescriptor )
{
	_scriptDescriptorOnSubmit = scriptDescriptor;
}
/**
 */
public void setScriptManager( ScriptManager scriptManager )
{
	_scriptManager = scriptManager;
}
}
