package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import org.eclipse.scripting.internal.uirenderer.*;
import org.eclipse.scripting.internal.*;
import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import java.util.Vector;

public class XGRElementUISwtTable implements IXGRElement,
 										     IXGRElementContainer,
 										     IXGRElementUIListener,
 										     IXGRElementEventGeneratorOnChange, 
											 IXGRElementNamed,
											 IXGRElementUI,
											 IXGRElementUIDataPresentationElement,
											 IXGRElementUIGridElement,
											 IXGRElementUITableElement,
											 IXGRElementUITextPresentationElement,
											 SelectionListener,
											 IXGRDataChangedListener											 
{
	private Point                      _dimSize                 = null;
	private Point                      _dimSpan                 = null;
	private ScriptDescriptor           _scriptDescriptorOnChange = null;
	private String                     _strDataSourceName       = null;
	private String                     _strLinkSourceName       = null;
	private String                     _strName                 = null;
	private String[]                   _straColumns             = null;
	private String[]                   _straHeadings            = null;
	private Table                      _table                   = null;
	private Vector					   _vectorChildren          = new Vector();
	private Vector                     _vectorUIListeners       = null;
	private IXGRDataModel               _dataModel              = null;
	private XGRUILinkManager _dataPresentationManager = null;
	private XGRFontManager             _fontManager             = null;
	private ScriptManager           _scriptManager           = null;

	private IXGRDataChangedNotifier[][] _dataChangedNotifiers = null;	
/**
 */
public XGRElementUISwtTable()
{
}
/**
 */
public void addChildElement( IXGRElement elementChild )
{
	_vectorChildren.addElement( elementChild );
}
/**
 */
public void addUIListener( IXGRElementUIListener dataListener) 
{
	if( _vectorUIListeners == null )
	{
		_vectorUIListeners = new Vector();
	}

	_vectorUIListeners.addElement( dataListener );

	// Initialize the listener with data
	//----------------------------------
	fireUIChanged();
}
/**
 */
public Object createWidget( Object objParent )
{
	if( objParent instanceof Composite )
	{
		_table = new Table( (Composite)objParent, SWT.BORDER | SWT.MULTI );

		initialize();
	}

	return _table;
}
/**
 */
	public void dataChanged(IXGRDataNode dataElement)
	{
		if(_dataChangedNotifiers != null && _straColumns != null)
		{
			for(int i=0; i<_dataChangedNotifiers.length; i++)
			{
				for(int j=0; j<_straColumns.length; j++)
				{
					IXGRDataChangedNotifier notifier = _dataChangedNotifiers[i][j];
					if(notifier != null && notifier == dataElement)
					{
						TableItem item = _table.getItem(i);
						if(item != null && !item.getText(j).equals(dataElement.getValue()))
						{
							item.setText(j, dataElement.getValue());
						}
					}
				}
			}
		}
/*		
		TableItem[] items = _table.getItems();

		for(int idx=0; idx<items.length; idx++)
		{
			TableItem item = items[idx];
			if(item.getData() == dataElement && !item.getText().equals(dataElement.getValue()))
			  item.setText(dataElement.getValue());
		}
*/		
	}
/**
 */
public void fireUIChanged()
{
//	Trace.functionEntry( this, "fireDataChanged" );
	
	// Tell others that my selection has changed
	//------------------------------------------
	if( _vectorUIListeners != null && _table != null )
	{
		int[] iaSelected = _table.getSelectionIndices();

		IXGRDataNode[] dataNodes = new IXGRDataNode[ iaSelected.length ];
		TableItem item = null;
			
		for( int i=0; i<iaSelected.length; ++i )
		{
			item = _table.getItem( iaSelected[i] );
			
			dataNodes[i] = (IXGRDataNode)item.getData();
		}

		// Notify listeners
		//-----------------
		for( int i=0; i<_vectorUIListeners.size(); ++i )
		{
			((IXGRElementUIListener)(_vectorUIListeners.elementAt(i))).uiChanged( dataNodes );
		}
	}
	
//	Trace.functionExit( this, "fireDataChanged" );
}
/**
 */
public java.util.Vector getChildElements()
{
	return _vectorChildren;
}
/**
 */
public java.lang.String getDataSourceName()
{
	return _strDataSourceName;
}
/**
 */
public String getName()
{
	return _strName;
}
/**
 */
public java.lang.String getUISourceName()
{
	return _strLinkSourceName;
}
/**
 */
public Object getWidget() 
{
	return _table;
}
/**
 */
public String getWidgetType()
{
	return "org.eclipse.swt.widgets.Table";
}
/**
 */
public Control getWindow()
{
	return _table;
}
/**
 */
public void highlight(boolean bHighlight, org.eclipse.swt.graphics.Color colorHighlight) {}
/**
 */
public void initialize()
{
//	Trace.functionEntry( this, "initialize" );
	
	if( _table != null )
	{
		// Font
		//-----
//		_table.setFont( _fontManager.getDefaultFont( _table ) );
		
		GridData gridData = new GridData();

		gridData.horizontalAlignment = GridData.FILL;
		gridData.widthHint           = 300;
		gridData.heightHint          = 100;
		
		// Size
		//-----
		if( _dimSize != null )
		{
			// Width
			//------
			if( _dimSize.x > 0 )
			{
				gridData.widthHint = _dimSize.x;
			}
			
			// Height
			//-------
			if( _dimSize.y > 0 )
			{
				gridData.heightHint = _dimSize.y;
			}
		}

		// Span
		//-----
		if( _dimSpan != null )
		{
			gridData.horizontalSpan      = _dimSpan.x;
			gridData.verticalSpan        = _dimSpan.y;
		}
		
		_table.setLayoutData( gridData );

		
		// Set the heading text, and make the heading visible
		//---------------------------------------------------
		if( _straHeadings != null )
		{
			// Deprecated
//			_table.setText( _straHeadings );
		}
			
		_table.setHeaderVisible(true);


		// Set the column widths in pixels
		// This needs to be done before setting data
		// Otherwise, only the first column data is visible
		//-------------------------------------------------
		int[] iaWidths = new int[ _straColumns.length ];
		
		for( int i=0; i<_straColumns.length; ++i )
		{
			iaWidths[i] = 150;
		}
		
		// Deprecated	
//		_table.setWidths( iaWidths );
		

		// Add data from the data model if not linked to a source
		//-------------------------------------------------------
		if( _dataModel != null )
		{
			if( _strLinkSourceName == null )
			{
				linkToData();
			}

			else if( _dataPresentationManager != null )
			{
				_dataPresentationManager.addUIListenerToSource( this, _strLinkSourceName );
			}
		}

		// Add listeners
		//--------------
		_table.addSelectionListener( this );

		// Register the widget with the script manager
		//--------------------------------------------
		if( _scriptManager != null && _strName != null && _strName.length() > 0 )
		{
			_scriptManager.declareElement( _strName, _table );
		}
	}
	
//	Trace.functionExit( this, "initialize" );
}
/**
 */
public void linkToData()
{
	if( _strDataSourceName != null && _dataModel != null )
	{
		IXGRDataNode[] dataNodes = _dataModel.getDataNodes( _strDataSourceName );
		setData( dataNodes );
	}
}
/**
 */
public void linkToUI() 
{
	if( _dataPresentationManager != null )
	{
		_dataPresentationManager.addUIListenerToSource( this, _strLinkSourceName );
	}
}
/**
 * Set the data for the table.  
 * The input nodes represent table records such as "company.employee".
 * For each record, a column (field) node is searched for.  The name
 * of the column is used ("company.employee.lastname") in the search.
 * The search begins with the record node.  Only the first node returned is used
 * since there should only be one field with the specific column name.
 */
public void setData( IXGRDataNode[] dataNodes )
{
//	Trace.functionEntry( this, "setData" );
	
	// Add the items to the list
	// The input nodes represent table records
	//----------------------------------------
	if( dataNodes != null && _straColumns != null && _table != null )
	{
		// Find column nodes for each record
		//----------------------------------
		if( dataNodes.length > 0 && _straColumns.length > 0 )
		{
			int                  k               = 0;
			String               strEmpty        = new String();
			String[]             straRecord      = new String[ _straColumns.length ];
			TableItem            item            = null;
			IXGRDataNode[] dataNodeaColumn = null;
			boolean              bRecordFound    = true;

			_dataChangedNotifiers = new IXGRDataChangedNotifier[dataNodes.length][_straColumns.length];
			
			for( int i=0; i<dataNodes.length; ++i )
			{
				// For each column, find a data node
				//----------------------------------
				for( int j=0; j<_straColumns.length; j++ ) 
				{
					dataNodeaColumn = _dataModel.getDataNodes( _straColumns[j], dataNodes[i] );

					if( dataNodeaColumn != null )
					{
						// Obtain the column's text string for this record
						// Only use the first entry
						//------------------------------------------------
						straRecord[j] = dataNodeaColumn[0].getValue();
						
						// Add data element listener
						//--------------------------
						if( dataNodeaColumn[0] instanceof IXGRDataChangedNotifier )
						{
							((IXGRDataChangedNotifier)dataNodeaColumn[0]).registerDataChangedListener( this );
							_dataChangedNotifiers[i][j] = (IXGRDataChangedNotifier)dataNodeaColumn[0];
						}
					}
					else
					{
						straRecord[j] = null;
					}

					// If the column node was not found,
					// or there was no text value, set an empty string
					//------------------------------------------------
					if( straRecord[j] == null )
					{
						straRecord[j] = strEmpty;
					}

					// Debug only
					//-----------
					// Trace.string( Integer.toString(i) + " " + Integer.toString(j) + " " + straRecord[j] );
				}

				// Create a new table item for the record
				//---------------------------------------
				item = new TableItem( _table, SWT.NULL, k++ );

				// Remember its data node
				//-----------------------								
				item.setData( dataNodes[i] );

				// Set the item's record text
				//---------------------------
				item.setText( straRecord );
			
//				item.setImage((Image)images.elementAt(logo));
			}
			
			// Select the first item
			//----------------------
			_table.setSelection( 0 );
		}
	}

	fireUIChanged();

//	Trace.functionExit( this, "setData" );
}
/**
 */
public void setDataModel( IXGRDataModel dataModel ) 
{
	_dataModel = dataModel;
}
/**
 */
public void setDataSourceName( String strDataSourceName )
{
	_strDataSourceName = strDataSourceName;
}
/**
 */
public void setFontManager( XGRFontManager fontManager) 
{
	_fontManager = fontManager;
}
/**
 */
public void setName( String strName )
{
	_strName = strName;
}
/**
 */
public void setOnChangeScriptDescriptor( ScriptDescriptor scriptDescriptor )
{
	_scriptDescriptorOnChange = scriptDescriptor;
}
/**
 */
public void setScriptManager( ScriptManager scriptManager )
{
	_scriptManager = scriptManager;
}
/**
 */
public void setSize( Point dimSize ) 
{
	_dimSize = dimSize;
}
/**
 */
public void setSpan( Point dimSpan ) 
{
	_dimSpan = dimSpan;
}
/**
 */
public void setTableColumnNames( String[] straColumns) 
{
//	Trace.functionEntry( this, "setTableColumnNames" );
	
	_straColumns = straColumns;
	
//	Trace.functionExit( this, "setTableColumnNames" );
}
/**
 */
public void setTableHeadings( String[] straHeadings )
{
	_straHeadings = straHeadings;
}
/**
 */
public void setUILinkManager( XGRUILinkManager dataPresentationManager )
{
	_dataPresentationManager = dataPresentationManager;
}
/**
 */
public void setUISourceName( String strLinkSourceName ) 
{
	_strLinkSourceName = strLinkSourceName;
}
/**
 */
public void setVisible(boolean bVisible)
{

}
/**
 */
public void uiChanged( IXGRDataNode[] dataNodes )
{
//	Trace.functionEntry( this, "uiChanged" );

	if( _table != null )
	{		
		// Remove data element listeners
		//------------------------------
		if( _dataChangedNotifiers != null && _straColumns != null )
		{
			for( int i=0; i<_dataChangedNotifiers.length; ++i )
			{
				for( int j=0; j<_straColumns.length; ++j )
				{
					if( _dataChangedNotifiers[i][j] != null )
					{
						_dataChangedNotifiers[i][j].unregisterDataChangedListener( this );
					}
				}
			}
		}
		
		// Reset notifiers list
		//---------------------
		_dataChangedNotifiers = null;
		
		// Clear the list
		//---------------
		_table.removeAll();
		
		// Get a new list of subnodes with my name
		//----------------------------------------
		if( _dataModel != null && dataNodes != null )
		{
			IXGRDataNode[] dataNodeChildren = null;
			
			for( int i=0; i<dataNodes.length; ++i )
			{
				dataNodeChildren = _dataModel.getDataNodes( _strDataSourceName, dataNodes[i] );				
				setData( dataNodeChildren );
			}
		}
	}
	
//	Trace.functionExit( this, "uiChanged" );
}
/**
 * Sent when default selection occurs in the control.
 * <p>
 * For example, on some platforms default selection occurs
 * in a List when the user double-clicks an item.
 * </p>
 */
public void widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent e) {}
/**
 */
public void widgetSelected( SelectionEvent sEvent )
{
	if( _scriptManager != null )
	{
//		Trace.string( sEvent.toString() );

		_scriptManager.evaluateScript( _scriptDescriptorOnChange, _table );
	}

	fireUIChanged();
}
}
