package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import java.util.Vector;

import org.eclipse.scripting.internal.IXGRDataChangedListener;
import org.eclipse.scripting.internal.IXGRDataChangedNotifier;
import org.eclipse.scripting.internal.IXGRDataModel;
import org.eclipse.scripting.internal.IXGRDataNode;
import org.eclipse.scripting.internal.IXGRElement;
import org.eclipse.scripting.internal.IXGRElementCaption;
import org.eclipse.scripting.internal.IXGRElementEventGeneratorOnBlur;
import org.eclipse.scripting.internal.IXGRElementEventGeneratorOnChange;
import org.eclipse.scripting.internal.IXGRElementEventGeneratorOnFocus;
import org.eclipse.scripting.internal.IXGRElementEventGeneratorOnKeyPress;
import org.eclipse.scripting.internal.IXGRElementNamed;
import org.eclipse.scripting.internal.IXGRElementUI;
import org.eclipse.scripting.internal.IXGRElementUIDataPresentationElement;
import org.eclipse.scripting.internal.IXGRElementUIGridElement;
import org.eclipse.scripting.internal.IXGRElementUIListener;
import org.eclipse.scripting.internal.IXGRElementUIReadOnly;
import org.eclipse.scripting.internal.IXGRElementUISizable;
import org.eclipse.scripting.internal.IXGRElementUITextPresentationElement;
import org.eclipse.scripting.internal.ScriptDescriptor;
import org.eclipse.scripting.internal.ScriptManager;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Text;

public abstract class XGRElementUISwtText implements FocusListener,
												     IXGRElement, 
													 IXGRElementCaption,
													 IXGRElementUIListener,
													 IXGRElementEventGeneratorOnBlur,
													 IXGRElementEventGeneratorOnChange,
													 IXGRElementEventGeneratorOnFocus,
													 IXGRElementEventGeneratorOnKeyPress,
													 IXGRElementNamed,
													 IXGRElementUI,
													 IXGRElementUIDataPresentationElement,
													 IXGRElementUIGridElement,
													 IXGRElementUIReadOnly,
													 IXGRElementUISizable,
													 IXGRElementUITextPresentationElement,
													 KeyListener,
													 ModifyListener,
													 IXGRDataChangedListener													 
{
	protected boolean                    _bReadOnly               = false;
	private   Color                      _colorBackground         = null;
	protected Point                      _dimSpan                 = null;
	protected Point                      _dimSize                 = null;
	protected Rectangle                  _rectGrid                = null;
	protected Text                       _text                    = null;
	private   String                     _strCaption              = null;
	private   String                     _strDataSourceName       = null;
	private   String                     _strLinkSourceName       = null;
	private   String                     _strName                 = null;
	private   ScriptDescriptor           _scriptDescriptorOnBlur   = null;
	private   ScriptDescriptor           _scriptDescriptorOnChange = null;
	private   ScriptDescriptor           _scriptDescriptorOnFocus  = null;
	private   ScriptDescriptor           _scriptDescriptorOnKeyPress   = null;
	private   ScriptManager              _scriptManager           = null;
	private   IXGRDataModel              _dataModel               = null;
	private   XGRUILinkManager           _uiLinkManager           = null;
	private   IXGRDataNode[]             _dataNodes               = null;
	private   XGRFontManager             _fontManager             = null;
	private   Vector                     _vectorUIListeners       = null;

/**
 */
public XGRElementUISwtText()
{
}
/**
 */
public void addUIListener( IXGRElementUIListener uiListener )
{
	if( _vectorUIListeners == null )
	{
		_vectorUIListeners = new Vector();
	}

	_vectorUIListeners.addElement( uiListener );

	fireUIChanged();
}
	/**
	*/
	public void dataChanged(IXGRDataNode dataElement)
	{
		if(_dataNodes != null && _dataNodes.length > 0
			&& _dataNodes[0] == dataElement)
		{//synchronize the UI with the dataElement new value
			
			String newValue = dataElement.getValue();
			if(_text.getText() != null && !_text.getText().equals(newValue))
			   _text.setText(newValue);
		}		
	}
/**
 */
public void fireUIChanged()
{
	// Tell others that my selection has changed
	//------------------------------------------
	if( _dataNodes != null && _vectorUIListeners != null && _text != null )
	{
		IXGRDataNode[] dataNodes = new IXGRDataNode[1];

		dataNodes[0] = _dataNodes[0];
		
		// Notify listeners
		//-----------------
		for( int i=0; i<_vectorUIListeners.size(); ++i )
		{
			((IXGRElementUIListener)(_vectorUIListeners.elementAt(i))).uiChanged( dataNodes );
		}
	}
}
/**
 */
public void focusGained( FocusEvent focusEvent ) 
{
//	Trace.functionEntry( this, "focusGained" );
	
	if( _scriptManager != null )
	{
//		Trace.string( focusEvent.toString() );

		_scriptManager.evaluateScript( _scriptDescriptorOnFocus, _text );
	}

//	Trace.functionExit( this, "focusGained" );
}
/**
 */
public void focusLost( FocusEvent focusEvent ) 
{
//	Trace.functionEntry( this, "focusLost" );

	//text value was changed, propagate the change to the element data model
	if(_dataNodes != null && _dataNodes.length > 0
	    && _text != null)
	{
	    IXGRDataNode dataNode = (IXGRDataNode) _dataNodes[0];
	    dataNode.setValue( _text.getText() );
	}
	
	if( _scriptManager != null )
	{
//		Trace.string( focusEvent.toString() );

		_scriptManager.evaluateScript( _scriptDescriptorOnBlur, _text );
	}
	
//	Trace.functionExit( this, "focusLost" );
}
/**
 */
public Composite getComposite()
{
	return null;
}
/**
 * getDataSourceName method comment.
 */
public java.lang.String getDataSourceName()
{
	return _strDataSourceName;
}
/**
 * getGridBounds method comment.
 */
public Rectangle getGridBounds() 
{
	return _rectGrid;
}
/**
 */
public String getName()
{
	return _strName;
}
/**
 */
public java.lang.String getUISourceName()
{
	return _strLinkSourceName;
}
/**
 */
public Object getWidget()
{
	return _text;
}
/**
 */
public String getWidgetType()
{
	return "org.eclipse.swt.widgets.Text";
}
/**
 * getWidget method comment.
 */
public Control getWindow()
{
	return _text;
}
/**
 */
public void highlight(boolean bHighlight, Color colorHighlight )
{
	if( _text != null )
	{
		if( bHighlight == true )
		{
			_colorBackground = _text.getBackground();
			_text.setBackground( colorHighlight );
		}
		else if( _colorBackground != null )
		{
			_text.setBackground( _colorBackground );
		}
	}
}
/**
 */
public void initialize()
{
	if( _text != null )
	{
		// Font
		//-----
//		_text.setFont( _fontManager.getDefaultFont( _text ) );

		// Caption
		//--------
		if( _strCaption != null )
		{
			_text.setText( _strCaption );
		}

		// Add data from the data model if not linked to a source
		//-------------------------------------------------------
		if( _dataModel != null )
		{
			if( _strLinkSourceName == null )
			{
				linkToData();
			}

			else if( _uiLinkManager != null )
			{
				_uiLinkManager.addUIListenerToSource( this, _strLinkSourceName );
			}
		}
		
		// Add focus listener for onfocus and onblur
		// Add key listener for onkeypress
		//------------------------------------------
		if( _scriptManager != null )
		{
			_text.addFocusListener ( this );
			_text.addKeyListener   ( this );
		}

		// Always add the modify listener for ui linking
		//----------------------------------------------
		_text.addModifyListener( this );

		// Register the widget with the script manager
		//--------------------------------------------
		if( _scriptManager != null && _strName != null && _strName.length() > 0 )
		{
			_scriptManager.declareElement( _strName, _text );
		}
	}
}
/**
 */
public void keyPressed( KeyEvent kEvent )
{
}
/**
 * A key has been pressed and released.  Generate onkeypress
 * event.  Onkeypress means a key was pressed and released.
 */
public void keyReleased( KeyEvent e )
{
//	Trace.functionEntry( this, "keyReleased" );
	
	if( _scriptManager != null )
	{
//		Trace.string( kEvent.toString() );

		_scriptManager.evaluateScript( _scriptDescriptorOnKeyPress, _text );
	}
	
//	Trace.functionExit ( this, "keyReleased" );
}
/**
 */
public void linkToData()
{
//	Trace.functionEntry( this, "linkToData" );
//	Trace.object( "_dataModel", _dataModel );
//	Trace.string( _strDataSourceName );
	
	if( _strDataSourceName != null && _dataModel != null )
	{
		_dataNodes = _dataModel.getDataNodes( _strDataSourceName );
		setData( _dataNodes );
	}

//	Trace.functionExit( this, "linkToData" );
}
/**
 */
public void linkToUI() 
{
	if( _uiLinkManager != null )
	{
		_uiLinkManager.addUIListenerToSource( this, _strLinkSourceName );
	}
}
/**
 */
public void modifyText( ModifyEvent mEvent )
{
	if( _scriptManager != null )
	{
//		Trace.string( mEvent.toString() );

		_scriptManager.evaluateScript( _scriptDescriptorOnChange, _text );
	}

	fireUIChanged();
}
/**
 */
public void setCaptionText( String strCaption )
{
	_strCaption = strCaption;
}
/**
 */
public void setData( IXGRDataNode[] dataNodes )
{
//	Trace.functionEntry( this, "setData" );
	
	// Replace the text
	//-----------------
	if( dataNodes != null && _text != null )
	{
		if( dataNodes.length > 0 )
		{
			if(dataNodes[0] instanceof IXGRDataChangedNotifier)
			{
				((IXGRDataChangedNotifier)dataNodes[0]).registerDataChangedListener( this );
			}
				
			_text.setText( dataNodes[0].getValue() );
		}
	}
	
//	Trace.functionExit( this, "setData" );
}
/**
 */
public void setDataModel( IXGRDataModel dataModel ) 
{
//	Trace.functionEntry( this, "setDataModel" );

	_dataModel = dataModel;

//	Trace.object( "_dataModel", _dataModel );
//	Trace.functionExit( this, "setDataModel" );
}
/**
 */
public void setDataSourceName( String strDataSourceName )
{
//	Trace.functionEntry( this, "setDataSourceName" );
//	Trace.string( strDataSourceName );
		
	_strDataSourceName = strDataSourceName;

//	Trace.functionExit( this, "setDataSourceName" );
}
/**
 */
public void setFontManager( XGRFontManager fontManager) 
{
	_fontManager = fontManager;
}
/**
 */
public void setGridBounds( Rectangle rectGrid )
{
	_rectGrid = rectGrid;
}
/**
 */
public void setName( String strName )
{
	_strName = strName;
}
/**
 */
public void setOnBlurScriptDescriptor( ScriptDescriptor scriptDescriptor )
{
//	Trace.functionEntry( this, "setOnBlurScriptDescriptor" );

	_scriptDescriptorOnBlur = scriptDescriptor;
	
//	Trace.functionExit( this, "setOnBlurScriptDescriptor" );
}
/**
 * setOnChangeScriptDescriptor method comment.
 */
public void setOnChangeScriptDescriptor( ScriptDescriptor scriptDescriptor )
{
	_scriptDescriptorOnChange = scriptDescriptor;
}
/**
 */
public void setOnFocusScriptDescriptor( ScriptDescriptor scriptDescriptor )
{
	_scriptDescriptorOnFocus = scriptDescriptor;
}
/**
 */
public void setOnKeyPressScriptDescriptor( ScriptDescriptor scriptDescriptor )
{
	_scriptDescriptorOnKeyPress = scriptDescriptor;
}
/**
 */
public void setReadOnly( boolean bReadOnly )
{
	_bReadOnly = bReadOnly;
}
/**
 */
public void setScriptManager( ScriptManager scriptManager )
{
	_scriptManager = scriptManager;
}
/**
 */
public void setSize( Point dimSize ) 
{
	_dimSize = dimSize;
}
/**
 */
public void setSpan( Point dimSpan ) 
{
	_dimSpan = dimSpan;
}
/**
 */
public void setUILinkManager( XGRUILinkManager uiLinkManager )
{
	_uiLinkManager = uiLinkManager;
}
/**
 */
public void setUISourceName( String strLinkSourceName ) 
{
	_strLinkSourceName = strLinkSourceName;
}
/**
 */
public void setVisible(boolean bVisible)
{
	if( _text != null )
	{
		_text.setVisible( bVisible );
	}
}
/**
 */
public void uiChanged( IXGRDataNode[] dataNodes )
{
//	Trace.functionEntry( this, "uiChanged" );

	if( _text != null )
	{
		// Clear the list
		//---------------
		_text.setText( "" );
		
		// Remove data element listeners
		//------------------------------
		if( _dataNodes != null )
		{
			for( int i=0; i<_dataNodes.length; ++i )
			{
				if( _dataNodes[i] instanceof IXGRDataChangedNotifier )
				{
					((IXGRDataChangedNotifier)_dataNodes[i]).unregisterDataChangedListener( this );
				}
			}
		}

		// Get a new list of subnodes with my name
		//----------------------------------------
		if( _dataModel != null && dataNodes != null )
		{
			if( dataNodes.length > 0 )
			{
				_dataNodes = _dataModel.getDataNodes( _strDataSourceName, dataNodes[0] );

				setData( _dataNodes );
			}
		}
	}
	
//	Trace.functionExit( this, "uiChanged" );
}
}
