package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import java.util.Vector;

import org.eclipse.scripting.internal.IXGRDataChangedListener;
import org.eclipse.scripting.internal.IXGRDataChangedNotifier;
import org.eclipse.scripting.internal.IXGRDataModel;
import org.eclipse.scripting.internal.IXGRDataNode;
import org.eclipse.scripting.internal.IXGRElement;
import org.eclipse.scripting.internal.IXGRElementNamed;
import org.eclipse.scripting.internal.IXGRElementUI;
import org.eclipse.scripting.internal.IXGRElementUIDataPresentationElement;
import org.eclipse.scripting.internal.IXGRElementUIGridElement;
import org.eclipse.scripting.internal.IXGRElementUIListener;
import org.eclipse.scripting.internal.IXGRElementUISizable;
import org.eclipse.scripting.internal.IXGRElementUITextPresentationElement;
import org.eclipse.scripting.internal.ScriptManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

public class XGRElementUISwtTree  implements IXGRElement,
 										     IXGRElementUIListener,
											 IXGRElementNamed,
											 IXGRElementUI,
											 IXGRElementUISizable,
											 IXGRElementUIDataPresentationElement,
											 IXGRElementUIGridElement,
											 IXGRElementUITextPresentationElement,
											 Listener,
											 IXGRDataChangedListener
{
	private Point                      _dimSize                 = null;
	private Point                      _dimSpan                 = null;
	private Tree                       _tree                    = null;
	private Vector                     _vectorChildren          = new Vector();
	private String                     _strDataSourceName       = null;
	private String                     _strLinkSourceName       = null;
	private String                     _strName                 = null;
	private IXGRDataModel              _dataModel               = null;
	private XGRUILinkManager           _uiLinkManager           = null;
	private XGRFontManager             _fontManager             = null;
	private ScriptManager              _scriptManager           = null;
	private IXGRDataNode[]             _dataNodes               = null;
	private Vector                     _vectorUIListeners       = null;

/**
 */
public XGRElementUISwtTree()
{
}
/**
 */
public void addUIListener( IXGRElementUIListener uiListener) 
{
//	Trace.functionEntry( this, "addUIListener" );
	
	if( _vectorUIListeners == null )
	{
		_vectorUIListeners = new Vector();
	}

	_vectorUIListeners.addElement( uiListener );

	// Initialize the listener with data
	//----------------------------------
	fireUIChanged();

//	Trace.functionExit( this, "addUIListener" );
}
/**
 */
public Object createWidget( Object objParent )
{
	if( objParent instanceof Composite )
	{
		_tree = new Tree( (Composite)objParent, SWT.BORDER );

		initialize();
	}

	return _tree;
}
	/**
	*/
	public void dataChanged(IXGRDataNode dataElement)
	{
		TreeItem[] items = getAllItems();

		for(int idx=0; idx<items.length; idx++)
		{
			TreeItem item = items[idx];
			if(item.getData() == dataElement && !item.getText().equals(dataElement.getValue()))
			  item.setText(dataElement.getValue());
		}
	}
protected void expandTree() 
{
	if( _tree != null )
	{
		// Remove any existing tree nodes
		//-------------------------------
		TreeItem[] items = _tree.getItems();
	
		for( int i=0; i<items.length; i++ )
		{
			if(items[i].getData() != null
				&& (items[i].getData() instanceof IXGRDataChangedNotifier))
			{//remove data element listeners for this item
				((IXGRDataChangedNotifier)items[i].getData()).unregisterDataChangedListener(this);
			}		
			
			items[i].dispose();
		}

		if( _dataNodes != null )
		{
			// Add the roots
			//--------------
			String   strText = null;
			TreeItem item    = null;
			
			for( int i=0; i<_dataNodes.length; i++ ) 
			{
				item = new TreeItem( _tree, 0 );
				item.setData( _dataNodes[i] );

				strText = _dataNodes[i].getValue();
				if( strText == null )
				{
					strText = _dataNodes[i].getName();
				}
				
				item.setText( strText );

				// Add a new tree item to indicate children
				//-----------------------------------------
				if( _dataNodes[i].getChildNodes() != null )
				{
					new TreeItem( item, 0 );
				}
			}
		}
	}
}
/**
 */
protected void expandTree( TreeItem item )
{
//	Trace.functionEntry( this, "expandTree" );
//	Trace.string( item.toString() );
	
	// Remove dummy item
	//------------------
	TreeItem[] items = item.getItems();
	
	for( int i=0; i<items.length; i++ )
	{
		if(items[i].getData() != null
			&& (items[i].getData() instanceof IXGRDataChangedNotifier))
		{//remove data element listeners for this item
			((IXGRDataChangedNotifier)items[i].getData()).unregisterDataChangedListener(this);
		}		
		
		items[i].dispose();
	}
	
	// Add real children
	//------------------
	IXGRDataNode dataNode = (IXGRDataNode)item.getData();

	Vector vectorChildNodes = dataNode.getChildNodes();
	
	if( vectorChildNodes != null )
	{
		String              strText       = null;
		TreeItem            itemChild     = null;
		IXGRDataNode dataNodeChild = null;
		
		for( int i=0; i<vectorChildNodes.size(); ++i )
		{
			dataNodeChild = (IXGRDataNode)vectorChildNodes.elementAt( i );

			if( dataNodeChild != null )
			{
				itemChild = new TreeItem( item, 0 );
				itemChild.setData( dataNodeChild );
				((IXGRDataChangedNotifier)dataNodeChild).registerDataChangedListener(this);
				
				strText = dataNodeChild.getValue();
					
				if( strText == null )
				{
					strText = dataNodeChild.getName();
				}
				
				itemChild.setText( strText );

				// Add a new tree item to indicate children
				//-----------------------------------------
				if( dataNodeChild.getChildNodes() != null )
				{
					TreeItem dummy = new TreeItem( itemChild, 0 );
				}
			}
		}
	}

//	Trace.functionExit( this, "expandTree" );
}
/**
 */
public void fireUIChanged()
{
//	Trace.functionEntry( this, "fireUIChanged" );
	
	// Tell others that my selection has changed
	//------------------------------------------
	if( _vectorUIListeners != null && _tree != null )
	{
		TreeItem[] tiaSelected = _tree.getSelection();

		IXGRDataNode[] nodeaSelected = new IXGRDataNode[ tiaSelected.length ];

		if( tiaSelected.length > 0 )
		{
			for( int i=0; i<tiaSelected.length; ++i )
			{
				nodeaSelected[i] = (IXGRDataNode)tiaSelected[i].getData();
			}

			// Notify listeners
			//-----------------
			for( int i=0; i<_vectorUIListeners.size(); ++i )
			{
				((IXGRElementUIListener)(_vectorUIListeners.elementAt(i))).uiChanged( nodeaSelected );
			}
		}
	}
	
//	Trace.functionExit( this, "fireUIChanged" );
}
/**
 */
public TreeItem[] getAllItems()
{
	if( _tree != null )
	{
		TreeItem[] items = _tree.getItems();
		Vector temp = new Vector();
		for(int idx=0; idx<items.length; idx++)
		{
			temp.addElement(items[idx]);
			getAllItems(items[idx], temp);
		}

		
		TreeItem[] result = new TreeItem[temp.size()];
		temp.copyInto(result);
		
		return result;
	}
	
	return null;
}
/**
 */
protected void getAllItems(TreeItem item, Vector items)
{
	TreeItem[] cItems = item.getItems();
	for(int idx=0; idx<cItems.length; idx++)
	{
		items.addElement(cItems[idx]);
		getAllItems(cItems[idx], items);
	}	
}
/**
 */
public java.lang.String getDataSourceName()
{
	return _strDataSourceName;
}
/**
 */
public String getName()
{
	return _strName;
}
/**
 */
public java.lang.String getUISourceName()
{
	return _strLinkSourceName;
}
/**
 */
public Object getWidget()
{
	return _tree;
}
/**
 */
public String getWidgetType()
{
	return "org.eclipse.swt.widgets.Tree";
}
/**
 */
public Control getWindow()
{
	return _tree;
}
/**
 */
public void handleEvent( Event event )
{
//	Shell shell = _tree.getShell ();
//	shell.setCursor (waitCursor);

	switch( event.type )
	{
		case SWT.Expand:    expandTree( (TreeItem)event.item ); break;
		case SWT.Selection: fireUIChanged();                  break;
	}
	
//	shell.setCursor( null );
}
/**
 */
public void highlight(boolean bHighlight, org.eclipse.swt.graphics.Color colorHighlight) {}
/**
 */
public void initialize()
{
//	Trace.functionEntry( this, "initialize" );
	
	if( _tree != null )
	{
		// Font
		//-----
//		_tree.setFont( _fontManager.getDefaultFont( _tree ) );
		
		// Layout
		//-------
		GridData gridData = new GridData();

		gridData.horizontalAlignment = GridData.FILL;
		gridData.widthHint           = 175;
		gridData.heightHint          = 200;

		// Size
		//-----
		if( _dimSize != null )
		{
			// Width
			//------
			if( _dimSize.x > 0 )
			{
				gridData.widthHint = _dimSize.x;
			}
			
			// Height
			//-------
			if( _dimSize.y > 0 )
			{
				gridData.heightHint = _dimSize.y;
			}
		}

		// Span
		//-----
		if( _dimSpan != null )
		{
			gridData.horizontalSpan      = _dimSpan.x;
			gridData.verticalSpan        = _dimSpan.y;
		}
		
		_tree.setLayoutData( gridData );
		

		// Add data from the data model if not linked to a source
		//-------------------------------------------------------
		if( _dataModel != null )
		{
			if( _strLinkSourceName == null )
			{
				linkToData();
			}

			else if( _uiLinkManager != null )
			{
 				_uiLinkManager.addUIListenerToSource( this, _strLinkSourceName );
			}
		}

		// Add listeners
		//--------------
		_tree.addListener( SWT.Expand,    this );
		_tree.addListener( SWT.Selection, this );

		// Register the widget with the script manager
		//--------------------------------------------
		if( _scriptManager != null && _strName != null && _strName.length() > 0 )
		{
			_scriptManager.declareElement( _strName, _tree );
		}
	}
	
//	Trace.functionExit( this, "initialize" );
}
/**
 */
public void linkToData()
{
	if( _strDataSourceName != null && _dataModel != null )
	{
		//remove data element listeners
		if(_dataNodes != null)
		{
			for(int idx=0; idx<_dataNodes.length; idx++)
			   ((IXGRDataChangedNotifier)_dataNodes[idx]).unregisterDataChangedListener(this);
		}
		
		_dataNodes = _dataModel.getDataNodes( _strDataSourceName );

		//add data element listeners		
		if(_dataNodes != null)
		{		
			for(int idx=0; idx<_dataNodes.length; idx++)
			   ((IXGRDataChangedNotifier)_dataNodes[idx]).registerDataChangedListener(this);
		}
			
		setData( _dataNodes );
	}
}
/**
 */
public void linkToUI() 
{
	if( _uiLinkManager != null )
	{
		_uiLinkManager.addUIListenerToSource( this, _strLinkSourceName );
	}
}
/**
 * Set the data for the table.  
 * The input nodes represent table records such as "company.employee".
 * For each record, a column (field) node is searched for.  The name
 * of the column is used ("company.employee.lastname") in the search.
 * The search begins with the record node.  Only the first node returned is used
 * since there should only be one field with the specific column name.
 */
public void setData( IXGRDataNode[] dataNodes )
{
//	Trace.functionEntry( this, "setData" );
	
	// Add the items to the list
	// The input nodes represent table records
	//----------------------------------------
	if( dataNodes != null && _tree != null )
	{
		// Find column nodes for each record
		//----------------------------------
		if( dataNodes.length > 0  )
		{
			expandTree();
			
			// Select the first item
			//----------------------
//			_tree.setSelection( 0 );
		}
	}

	fireUIChanged();

//	Trace.functionExit( this, "setData" );
}
/**
 */
public void setDataModel( IXGRDataModel dataModel ) 
{
	_dataModel = dataModel;
}
/**
 */
public void setDataSourceName( String strDataSourceName )
{
	_strDataSourceName = strDataSourceName;
}
/**
 */
public void setFontManager( XGRFontManager fontManager) 
{
	_fontManager = fontManager;
}
/**
 */
public void setName( String strName )
{
	_strName = strName;
}
/**
 */
public void setScriptManager( ScriptManager scriptManager ) 
{
	_scriptManager = scriptManager;
}
/**
 */
public void setSize( Point dimSize ) 
{
	_dimSize = dimSize;
}
/**
 */
public void setSpan( Point dimSpan ) 
{
	_dimSpan = dimSpan;
}
/**
 */
public void setUILinkManager( XGRUILinkManager uiLinkManager )
{
	_uiLinkManager = uiLinkManager;
}
/**
 */
public void setUISourceName( String strLinkSourceName ) 
{
	_strLinkSourceName = strLinkSourceName;
}
/**
 */
public void setVisible(boolean bVisible)
{

}
/**
 */
public void uiChanged( IXGRDataNode[] dataNodes )
{
//	Trace.functionEntry( this, "uiChanged" );

	if( _tree != null )
	{
		// Clear the list
		//---------------
		_tree.removeAll();

		//remove data element listeners		
		if(_dataNodes != null)
		{
			for(int idx=0; idx<_dataNodes.length; idx++)
			   ((IXGRDataChangedNotifier)_dataNodes[idx]).unregisterDataChangedListener(this);
		}
		
		// Get a new list of subnodes with my name
		//----------------------------------------
		if( _dataModel != null )
		{
			_dataNodes = _dataModel.getDataNodes( _strDataSourceName, dataNodes[0] );
			
			//add data element listeners
			if(_dataNodes != null)
			{
				for(int idx=0; idx<_dataNodes.length; idx++)
				   ((IXGRDataChangedNotifier)_dataNodes[idx]).registerDataChangedListener(this);
			}

			setData( _dataNodes );
		}
	}
	
//	Trace.functionExit( this, "uiChanged" );
}
}
