package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import java.util.Vector;

import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.scripting.ICancel;
import org.eclipse.scripting.internal.IXGRElement;
import org.eclipse.scripting.internal.IXGRElementCaption;
import org.eclipse.scripting.internal.IXGRElementContainer;
import org.eclipse.scripting.internal.IXGRElementEventGeneratorOnLoad;
import org.eclipse.scripting.internal.IXGRElementEventGeneratorOnUnload;
import org.eclipse.scripting.internal.IXGRElementNamed;
import org.eclipse.scripting.internal.IXGRElementUI;
import org.eclipse.scripting.internal.ScriptDescriptor;
import org.eclipse.scripting.internal.ScriptManager;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

public class XGRElementUISwtWizard extends Wizard implements IXGRElement,
							                      IXGRElementCaption,
							                      IXGRElementContainer,
							                      IXGRElementNamed,
							                      IXGRElementEventGeneratorOnLoad,
							                      IXGRElementEventGeneratorOnUnload,
												  IXGRElementUI,
												  ICancel
								
												
{
	protected Vector     	   _vectorChildren 	         = new Vector();
	private   String     	   _strCaption	             = null;
	private   String     	   _strName	                 = null;
	private   ScriptDescriptor _scriptDescriptorOnLoad   = null;
	private   ScriptDescriptor _scriptDescriptorOnUnload = null;
	private   ScriptManager    _scriptManager            = null;
	public    WizardDialog     _wizardDialog             = null;
	protected boolean          _bCancelPressed           = false;
/**
 * 
 */
public XGRElementUISwtWizard() {
	super();

//	ImageDescriptor imageDescriptor = ScriptPlugin.getImageDescriptor("icons/basic/wizban/script_wiz.gif");
//	super.setDefaultPageImageDescriptor(imageDescriptor);
}
/**
 * 
 */
public XGRElementUISwtWizard(boolean bCreateDialog) {
	super();

	if (bCreateDialog == true) {
		_wizardDialog = new WizardDialog(new Shell(), this);
	}

//	ImageDescriptor imageDescriptor = ScriptPlugin.getImageDescriptor("icons/basic/wizban/script_wiz.gif");
//	super.setDefaultPageImageDescriptor(imageDescriptor);
}
/**
 */
public void addChildElement( IXGRElement elementChild ) 
{
//	Trace.functionEntry( this, "addChildElement" );
	
	_vectorChildren.addElement( elementChild );

//	Trace.functionExit( this, "addChildElement" );
}
/**
 * This functions fires an onload event.  Note that due to the Wizard's
 * lazy page creation, the page widgets may not exist yet.
 */
public void addPages()
{
//	Trace.functionEntry( this, "addPages" );
	
	// Fire onload event
	//------------------
	if( _scriptManager != null )
	{
//		Trace.string( "SmartGuide Load" );

		_scriptManager.evaluateScript( _scriptDescriptorOnLoad, this );
	}
	
//	Trace.functionExit( this, "addPages" );
}
/**
 */
public Object createWidget(Object objParent) {
	// Wizards ignore the parent
	//--------------------------

	// Create the component's children
	//--------------------------------
	IXGRElementUI elementChild = null;
	Object objChild = null;

	for (int i = 0; i < _vectorChildren.size(); ++i) {
		objChild = _vectorChildren.elementAt(i);

		if (objChild instanceof IXGRElementUI) {
			elementChild = (IXGRElementUI) objChild;
			elementChild.createWidget(this);
		}
	}

	// Initialize myself
	//------------------
	initialize();

	return this;
}
/**
 */
public void dispose()
{
//	Trace.functionEntry( this, "aboutToClose" );
//	Trace.functionExit( this, "aboutToClose" );
}
/**
 */
public Vector getChildElements()
{
	return _vectorChildren;
}
/**
 */
public String getName()
{
	return _strName;
}
/**
 */
public Object getWidget() 
{
	return this;
}
/**
 */
public String getWidgetType()
{
	return "org.eclipse.jface.sguides.Wizard";
}
/**
 */
public Control getWindow() 
{
	return null;
}
/**
 */
public void highlight(boolean bHighlight, org.eclipse.swt.graphics.Color colorHighlight) {}
/**
 */
public void initialize()
{
//	Trace.functionEntry( this, "initialize" );

	// Initialize my properties
	//-------------------------
	if( _strCaption != null && _wizardDialog != null )
	{
		_wizardDialog.getShell().setText( _strCaption );
	}

	// Register the widget with the script manager
	//--------------------------------------------
	if( _scriptManager != null && _strName != null && _strName.length() > 0 )
	{
		_scriptManager.declareElement( _strName, this );
	}
	
//	Trace.functionExit ( this, "initialize" );
}
/**
 */
public boolean isCancel() 
{
	return _bCancelPressed;
}
/**
 * 
 */
public boolean performCancel() 
{
	_bCancelPressed = true;
	
	if( _scriptManager != null )
	{
//		Trace.string( "SmartGuide Unload" );

		_scriptManager.evaluateScript( _scriptDescriptorOnUnload, this );
	}

	return true;
}
/**
 *
 */
public boolean performFinish() 
{
	_bCancelPressed = false;
	
	if( _scriptManager != null )
	{
		_scriptManager.evaluateScript( _scriptDescriptorOnUnload, this );
	}

	return true;
}
/**
 */
public void setCaptionText( String strCaption )
{
	_strCaption = strCaption;
}
/**
 */
public void setName( String strName )
{
//	Trace.functionEntry( this, "setName" );

	_strName = strName;
	
//	Trace.functionExit( this, "setName" );
}
/**
 */
public void setOnLoadScriptDescriptor( ScriptDescriptor scriptDescriptor )
{
	_scriptDescriptorOnLoad = scriptDescriptor;
}
/**
 */
public void setOnUnloadScriptDescriptor( ScriptDescriptor scriptDescriptor )
{
	_scriptDescriptorOnUnload = scriptDescriptor;
}
/**
 */
public void setScriptManager( ScriptManager scriptManager )
{
	_scriptManager = scriptManager;
}
/**
 */
public void setVisible(boolean bVisible) {
	if (_wizardDialog != null) {
		_wizardDialog.open();
	}
}
}
