package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import java.util.Vector;

import org.eclipse.scripting.internal.IXGRElement;
import org.eclipse.scripting.internal.IXGRElementCaption;
import org.eclipse.scripting.internal.IXGRElementContainer;
import org.eclipse.scripting.internal.IXGRElementEventGeneratorOnLoad;
import org.eclipse.scripting.internal.IXGRElementEventGeneratorOnUnload;
import org.eclipse.scripting.internal.IXGRElementNamed;
import org.eclipse.scripting.internal.IXGRElementUI;
import org.eclipse.scripting.internal.ScriptDescriptor;
import org.eclipse.scripting.internal.ScriptManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

public class XGRElementUISwtWizardBuildTime implements IXGRElement,
							                      IXGRElementCaption,
							                      IXGRElementContainer,
							                      IXGRElementNamed,
							                      IXGRElementEventGeneratorOnLoad,
							                      IXGRElementEventGeneratorOnUnload,
												  IXGRElementUI
								
												
{
	protected Vector     	   _vectorChildren 	         = new Vector();
	private   String     	   _strCaption	             = null;
	private   String     	   _strName	                 = null;
	private   ScriptDescriptor _scriptDescriptorOnLoad   = null;
	private   ScriptDescriptor _scriptDescriptorOnUnload = null;
	private   ScriptManager    _scriptManager            = null;
	protected ScriptWizard _smartGuide               = null; 
	protected Color            _colorBackground          = null;
/**
 */
public XGRElementUISwtWizardBuildTime()
{
	super();

//	Trace.functionEntry( this, "XGRElementUISwtSmartGuideBuildTime" );
//	Trace.functionExit ( this, "XGRElementUISwtSmartGuideBuildTime" );
}
/**
 */
public void addChildElement( IXGRElement elementChild ) 
{
//	Trace.functionEntry( this, "addChildElement" );
	
	_vectorChildren.addElement( elementChild );

//	Trace.functionExit( this, "addChildElement" );
}
/**
 */
public Object createWidget( Object objParent )
{
//	Trace.functionEntry( this, "createWidget" );

	// Create the subcomponents
	//-------------------------
	_smartGuide.createContents( (Composite)objParent );

	// Create the component's children
	// In this case, smart guide pages
	//--------------------------------
	IXGRElementUI elementChild = null;
	Object       objChild     = null;

	for( int i=0; i<_vectorChildren.size(); ++i )
	{
		objChild = _vectorChildren.elementAt( i );

		if( objChild instanceof IXGRElementUI )
		{
			elementChild = (IXGRElementUI)objChild;
			elementChild.createWidget( _smartGuide );
		}
	}
	
	// Initialize myself
	//------------------
	initialize();

	_smartGuide.turnToFirstPage();
	
//	Trace.functionExit( this, "createWidget" );

	return this;
}
/**
 */
public Vector getChildElements()
{
	return _vectorChildren;
}
/**
 */
public String getName()
{
	return _strName;
}
/**
 */
public Object getWidget() 
{
	return _smartGuide;
}
/**
 */
public String getWidgetType()
{
	return "org.eclipse.jface.sguides.SmartGuide";
}
/**
 */
public Control getWindow() 
{
	return _smartGuide;
}
/**
 */
public void highlight(boolean bHighlight, org.eclipse.swt.graphics.Color colorHighlight)
{
	if( bHighlight == true && _smartGuide != null )
	{
		_colorBackground = _smartGuide.getBackground();
		_smartGuide.setBackground( colorHighlight );
	}
	else if( _colorBackground != null )
	{
		_smartGuide.setBackground( _colorBackground );
	}
}
/**
 */
public void initialize()
{
//	Trace.functionEntry( this, "initialize" );

	
	// Initialize my properties
	//-------------------------
	if( _strCaption != null && _smartGuide != null )
	{
//		_smartGuide.setTitle( _strCaption );
	}

	// Register the widget with the script manager
	//--------------------------------------------
	if( _scriptManager != null && _strName != null && _strName.length() > 0 )
	{
		_scriptManager.declareElement( _strName, this );
	}
	
//	Trace.functionExit ( this, "initialize" );
}
/**
 */
public void setCaptionText( String strCaption )
{
	_strCaption = strCaption;
}
/**
 */
public void setName( String strName )
{
//	Trace.functionEntry( this, "setName" );

	_strName = strName;
	
//	Trace.functionExit( this, "setName" );
}
/**
 */
public void setOnLoadScriptDescriptor( ScriptDescriptor scriptDescriptor )
{
	_scriptDescriptorOnLoad = scriptDescriptor;
}
/**
 */
public void setOnUnloadScriptDescriptor( ScriptDescriptor scriptDescriptor )
{
	_scriptDescriptorOnUnload = scriptDescriptor;
}
/**
 */
public void setParentComposite(Composite compositeParent)
{
	// Create the smart guide
	// The parent composite should have been previously set
	// by the setParentComposite method
	//-----------------------------------------------------
	if( compositeParent instanceof Composite )
	{
		_smartGuide = new ScriptWizard( compositeParent, SWT.NULL );
	}
}
/**
 */
public void setScriptManager( ScriptManager scriptManager )
{
	_scriptManager = scriptManager;
}
/**
 */
public void setVisible( boolean bVisible ) 
{
}
}
