package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import java.util.Vector;

import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.scripting.internal.IXGRElement;
import org.eclipse.scripting.internal.IXGRElementCaption;
import org.eclipse.scripting.internal.IXGRElementContainer;
import org.eclipse.scripting.internal.IXGRElementEventGeneratorOnReset;
import org.eclipse.scripting.internal.IXGRElementEventGeneratorOnSubmit;
import org.eclipse.scripting.internal.IXGRElementNamed;
import org.eclipse.scripting.internal.IXGRElementUI;
import org.eclipse.scripting.internal.IXGRElementUILayoutElement;
import org.eclipse.scripting.internal.ScriptDescriptor;
import org.eclipse.scripting.internal.ScriptManager;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

public class XGRElementUISwtWizardPage extends WizardPage implements IXGRElement, 
																			 IXGRElementCaption,
																			 IXGRElementContainer,
																			 IXGRElementEventGeneratorOnSubmit, 
																			 IXGRElementEventGeneratorOnReset,
																			 IXGRElementNamed,
																			 IXGRElementUI,
																			 IXGRElementUILayoutElement 
{
	private ScriptManager            _scriptManager            = null;
	private String                   _strCaption               = null;
	private String                   _strName                  = null;
	private ScriptDescriptor         _scriptDescriptorOnSubmit = null;
	private ScriptDescriptor         _scriptDescriptorOnReset  = null;
	private XGRElementUISwtComposite _xgrComposite             = null;
/**
 */
public XGRElementUISwtWizardPage()
{
	super( "Some string" );
//  Trace.functionEntry( this, "XGRElementUISwtSmartGuidePage" );

	_xgrComposite = new XGRElementUISwtComposite();
	
//  Trace.functionExit( this, "XGRElementUISwtSmartGuidePage" );
}
/**
 */
public void addChildElement( IXGRElement elementChild )
{
//  Trace.functionEntry( this, "addChildElement" );
	
	_xgrComposite.addChildElement( elementChild );

//  Trace.functionExit( this, "addChildElement" );
}
/**
 */
public void createControl( Composite compositeParent )
{
//  Trace.functionEntry( this, "createControl" );

	// Create all my children
	//-----------------------
	_xgrComposite.createWidget( compositeParent );

	// Initialize myself
	//------------------
	initialize();

	super.setControl( _xgrComposite.getWindow() );
		
//  Trace.functionExit( this, "createControl" );
}
/**
 */
public Object createWidget( Object objParent )
{
//  Trace.functionEntry( this, "createWidget" );
	
	// The parent will be a smartguide
	// Add this page to the smartguide
	//--------------------------------
	if( objParent instanceof Wizard )
	{
//      Trace.string( "Adding page to wizard" );
		((Wizard)objParent).addPage( this );
	}
	
//  Trace.functionExit( this, "createWidget" );

	return this;
}
/**
 */
public Vector getChildElements()
{
	return _xgrComposite.getChildElements();
}
/**
 */
public String getName()
{
	return _strName;
}
/**
 */
public int getNumberOfColumns() 
{
	return _xgrComposite.getNumberOfColumns();
}
/**
 */
public Object getWidget() 
{
	return this;
}
/**
 */
public String getWidgetType()
{
	return "org.eclipse.jface.sguides.WizardPage";
}
/**
 */
public Control getWindow() 
{
	return null;
}
/**
 */
public void highlight(boolean bHighlight, org.eclipse.swt.graphics.Color colorHighlight) {}
/**
 */
public void initialize()
{
//  Trace.functionEntry( this, "initialize" );

	// Initialize my properties
	//-------------------------
	if( _strCaption != null )
	{
		super.setTitle( _strCaption );
	}

	// Register the widget with the script manager
	//--------------------------------------------
	if( _scriptManager != null && _strName != null && _strName.length() > 0 )
	{
		_scriptManager.declareElement( _strName, this );
	}

	// Trigger onreset event
	//----------------------
	if( _scriptManager != null )
	{
//      Trace.string( "WizardPage Load" );

		_scriptManager.evaluateScript( _scriptDescriptorOnReset, this );
	}

//  Trace.functionExit( this, "initialize" );
}
/**
 */
public void setCaptionText( String strCaption )
{
	_strCaption = strCaption;
}
/**
 */
public void setErrorMessage(String strMessage)
{
//  Trace.functionEntry( this, "setErrorMessage" );

	// Error messages appear in red text
	//----------------------------------
	super.setErrorMessage( strMessage );

//  Trace.functionExit( this, "setErrorMessage" );
}
/**
 */
public void setMessage(String strMessage)
{
//  Trace.functionEntry( this, "setMessage" );

	// Clear the error message first
	// Otherwise the message will not appear
	// Messages appear in black text
	//--------------------------------------
	super.setErrorMessage(null);
	super.setMessage( strMessage );

//  Trace.functionExit( this, "setMessage" );
}
/**
 */
public void setName( String strName )
{
	_strName = strName;
}
/**
 */
public void setNumberOfColumns(int iNumberOfColumns)
{
	_xgrComposite.setNumberOfColumns( iNumberOfColumns );
}
/**
 */
public void setOnResetScriptDescriptor( ScriptDescriptor scriptDescriptor )
{
	_scriptDescriptorOnReset = scriptDescriptor;
}
/**
 */
public void setOnSubmitScriptDescriptor( ScriptDescriptor scriptDescriptor )
{
	_scriptDescriptorOnSubmit = scriptDescriptor;
}
/**
 */
public void setScriptManager( ScriptManager scriptManager )
{
	_scriptManager = scriptManager;
}
/**
 */
public void setVisible( boolean bVisible )
{
	super.setVisible( bVisible );
	
	if( bVisible == false )
	{
		if( _scriptManager != null )
		{
			_scriptManager.evaluateScript( _scriptDescriptorOnSubmit, this );
		}
	}
}
}
