package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import java.util.Vector;

import org.eclipse.scripting.internal.IXGRElement;
import org.eclipse.scripting.internal.IXGRElementCaption;
import org.eclipse.scripting.internal.IXGRElementContainer;
import org.eclipse.scripting.internal.IXGRElementNamed;
import org.eclipse.scripting.internal.IXGRElementUI;
import org.eclipse.scripting.internal.IXGRElementUILayoutElement;
import org.eclipse.scripting.internal.ScriptDescriptor;
import org.eclipse.scripting.internal.ScriptManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Control;

public class XGRElementUISwtWizardPageBuildTime implements IXGRElement, 
																		     IXGRElementCaption,
																		     IXGRElementContainer,
																		     IXGRElementNamed,
																		     IXGRElementUI,
																		     IXGRElementUILayoutElement 
{
	private   ScriptManager    _scriptManager            = null;
	private   String           _strCaption               = null;
	private   String           _strName                  = null;
	private   ScriptDescriptor _scriptDescriptorOnLoad   = null;
	private   ScriptDescriptor _scriptDescriptorOnUnload = null;
	private   ScriptWizardPage _page                     = null;
	private   int              _iNumberOfColumns         = 1;
	protected GridLayout       _gridLayout               = null;
	protected Vector           _vectorChildren           = new Vector();
	protected Color            _colorBackground          = null;
/**
 */
public XGRElementUISwtWizardPageBuildTime()
{
//	Trace.functionEntry( this, "XGRElementUISwtSmartGuidePageBuildTime" );
//	Trace.functionExit( this, "XGRElementUISwtSmartGuidePageBuildTime" );
}
/**
 */
public void addChildElement( IXGRElement elementChild )
{
	_vectorChildren.addElement( elementChild );
}
/**
 */
public Object createWidget( Object objParent )
{
//	Trace.functionEntry( this, "createWidget" );
	
	// The parent will be a smartguide
	// Add this page to the smartguide
	//--------------------------------
	if( objParent instanceof ScriptWizard )
	{
		_page = new ScriptWizardPage( ((ScriptWizard)objParent).getPageComposite(), SWT.NULL );
		((ScriptWizard)objParent).addPage( _page );
	}

	// Initialize myself, and create my children
	//------------------------------------------
	if( _page != null )
	{
		// Initialize myself
		//------------------
		initialize();
		
		// Create and initialize the component's children
		//-----------------------------------------------
		IXGRElementUI elementChild = null;

		for( int i=0; i<_vectorChildren.size(); ++i )
		{
			elementChild = (IXGRElementUI)_vectorChildren.elementAt( i );
			elementChild.createWidget( _page );
		}
	}
	
//	Trace.functionExit( this, "createWidget" );

	return this;
}
/**
 */
public Vector getChildElements()
{
	return _vectorChildren;
}
/**
 */
public String getName()
{
	return _strName;
}
/**
 */
public int getNumberOfColumns() 
{
	return 1;
}
/**
 */
public Object getWidget() 
{
	return _page;
}
/**
 */
public String getWidgetType()
{
	return "org.eclipse.jface.sguides.SmartGuidePage";
}
/**
 */
public Control getWindow() 
{
	return _page;
}
/**
 */
public void highlight( boolean bHighlight, Color colorHighlight )
{
	if( _page != null )
	{
		if( bHighlight == true )
		{
			_colorBackground = _page.getBackground();
			_page.setBackground( colorHighlight );
		}
		else if( _colorBackground != null )
		{
			_page.setBackground( _colorBackground );
		}
	}
}
/**
 */
public void initialize()
{
//	Trace.functionEntry( this, "initialize" );

	if( _page != null )
	{	
		// Create a grid layout
		// Reset its margin width (default is 5)
		//--------------------------------------
		_gridLayout = new GridLayout();
		_gridLayout.marginWidth = 0;
		_gridLayout.numColumns  = _iNumberOfColumns;
		
		_page.setLayout( _gridLayout );
	}
		
	// Initialize my properties
	//-------------------------
	if( _strCaption != null )
	{
		_page.setTitle( _strCaption );
	}

	// Register the widget with the script manager
	//--------------------------------------------
	if( _scriptManager != null && _strName != null && _strName.length() > 0 )
	{
		_scriptManager.declareElement( _strName, this );
	}

//	Trace.functionExit( this, "initialize" );
}
/**
 */
public void setCaptionText( String strCaption )
{
	_strCaption = strCaption;
}
/**
 */
public void setName( String strName )
{
	_strName = strName;
}
/**
 */
public void setNumberOfColumns(int iNumberOfColumns)
{
	_iNumberOfColumns = iNumberOfColumns;
	return;
}
/**
 */
public void setOnLoadScriptDescriptor( ScriptDescriptor scriptDescriptor )
{
	_scriptDescriptorOnLoad = scriptDescriptor;
}
/**
 */
public void setOnUnloadScriptDescriptor( ScriptDescriptor scriptDescriptor ) 
{
	_scriptDescriptorOnUnload = scriptDescriptor;
}
/**
 */
public void setScriptManager( ScriptManager scriptManager )
{
	_scriptManager = scriptManager;
}
/**
 */
public void setVisible( boolean bVisible )
{
	// No-op
	//------
}
}
