package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import java.util.Vector;

import org.eclipse.scripting.internal.IXGRDataModel;
import org.eclipse.scripting.internal.IXGRElement;
import org.eclipse.scripting.internal.IXGRElementCaption;
import org.eclipse.scripting.internal.IXGRElementContainer;
import org.eclipse.scripting.internal.IXGRElementEventGenerator;
import org.eclipse.scripting.internal.IXGRElementEventGeneratorOnBlur;
import org.eclipse.scripting.internal.IXGRElementEventGeneratorOnChange;
import org.eclipse.scripting.internal.IXGRElementEventGeneratorOnClick;
import org.eclipse.scripting.internal.IXGRElementEventGeneratorOnFocus;
import org.eclipse.scripting.internal.IXGRElementEventGeneratorOnKeyPress;
import org.eclipse.scripting.internal.IXGRElementEventGeneratorOnLoad;
import org.eclipse.scripting.internal.IXGRElementEventGeneratorOnReset;
import org.eclipse.scripting.internal.IXGRElementEventGeneratorOnSubmit;
import org.eclipse.scripting.internal.IXGRElementEventGeneratorOnUnload;
import org.eclipse.scripting.internal.IXGRElementNamed;
import org.eclipse.scripting.internal.IXGRElementScriptElement;
import org.eclipse.scripting.internal.IXGRElementUI;
import org.eclipse.scripting.internal.IXGRElementUIChecked;
import org.eclipse.scripting.internal.IXGRElementUIDataPresentationElement;
import org.eclipse.scripting.internal.IXGRElementUIGridElement;
import org.eclipse.scripting.internal.IXGRElementUIImageElement;
import org.eclipse.scripting.internal.IXGRElementUILayoutElement;
import org.eclipse.scripting.internal.IXGRElementUILayoutTableElement;
import org.eclipse.scripting.internal.IXGRElementUIReadOnly;
import org.eclipse.scripting.internal.IXGRElementUIRunnable;
import org.eclipse.scripting.internal.IXGRElementUISizable;
import org.eclipse.scripting.internal.IXGRElementUITableDataElement;
import org.eclipse.scripting.internal.IXGRElementUITableElement;
import org.eclipse.scripting.internal.IXGRElementUITableHeadingElement;
import org.eclipse.scripting.internal.IXGRElementUITextPresentationElement;
import org.eclipse.scripting.internal.ScriptDescriptor;
import org.eclipse.scripting.internal.ScriptManager;
import org.eclipse.scripting.internal.ScriptNLTranslationManager;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;

public class XGRNodePresentation extends XGRNode
{
	protected int           _iElementType = XGR.UNKNOWN;
	protected IXGRElement   _xgrElement     = null;

	protected static char TRANSLATIONKEY = '%';
/**
 * XGRNodePML constructor comment.
 */
public XGRNodePresentation( XGRPresentationModelElement node, XGRNode xgrNodeParent )
{
	super( node, xgrNodeParent );
}
/**
 */
public IXGRElement createElements( IXGRElement xgrElementParent )
{
//	Trace.functionEntry( this, "createElements" );
	
	// If I am a UI node, create my UI element
	//----------------------------------------
	IXGRElement xgrElementMe = null;

	if( _iElementType != XGR.UNKNOWN )
	{
		// Create a new instance if not precreated
		//----------------------------------------
		if( _xgrElement == null )
		{
			switch( _iElementType )
			{
				case XGR.SCRIPT:                  xgrElementMe = new XGRElementScript();                      break;
				case XGR.BUTTON:                  xgrElementMe = new XGRElementUISwtButton();                 break;        
				case XGR.BREAK:                   xgrElementMe = new XGRElementUISwtBreak();                  break;        
				case XGR.CHECK:                   xgrElementMe = new XGRElementUISwtCheckBox();               break;        
				case XGR.COMBO:                   xgrElementMe = new XGRElementUISwtComboBox();               break;        
				case XGR.COMPOSITE:               xgrElementMe = new XGRElementUISwtComposite();              break;        
				case XGR.COMPOSITEPANEL:          xgrElementMe = new XGRElementUISwtCompositePanel();         break;        
				case XGR.FIELDSET:                xgrElementMe = new XGRElementUISwtFieldSet();               break;        
				case XGR.GROUPBOX:                xgrElementMe = new XGRElementUISwtGroupBox();               break;        
				case XGR.HORZRULE:                xgrElementMe = new XGRElementUISwtHorizontalRule();         break;        
				case XGR.IMG:                     xgrElementMe = new XGRElementUISwtImage();                  break;        
				case XGR.LABEL:                   xgrElementMe = new XGRElementUISwtLabel();                  break;        
				case XGR.TABLELAYOUT:             xgrElementMe = new XGRElementUISwtLayoutTable();            break;        
				case XGR.LIST:                    xgrElementMe = new XGRElementUISwtList();                   break;        
				case XGR.PREFERENCEDIALOG:        xgrElementMe = new XGRElementUISwtPreferenceDialog();       break;        
				case XGR.PREFERENCEPAGE:          xgrElementMe = new XGRElementUISwtPreferencePage();         break;
				case XGR.PROPERTYSHEET:           xgrElementMe = new XGRElementUISwtPropertySheet();          break;        
				case XGR.PROPERTYSHEETCOMPOSITE:  xgrElementMe = new XGRElementUISwtComposite();              break;
				case XGR.SCRIPTPREFERENCEDIALOG:  xgrElementMe = new XGRElementUISwtScriptPreferenceDialog(); break;
				case XGR.SCRIPTPREFERENCEPAGE:    xgrElementMe = new XGRElementUISwtScriptPreferencePage();   break;  
				case XGR.RADIO:                   xgrElementMe = new XGRElementUISwtRadioButton();            break;        
//				case XGR.SHELL:                   xgrElementMe = new XGRElementUISwtShell();                  break;        
				case XGR.WIZARD:                  xgrElementMe = new XGRElementUISwtWizard( true );           break;        
				case XGR.WIZARDBUILDTIME:         xgrElementMe = new XGRElementUISwtWizardBuildTime();        break;     
				case XGR.WIZARDPAGE:              xgrElementMe = new XGRElementUISwtWizardPage();             break;        
	           	case XGR.WIZARDPAGEBUILDTIME:     xgrElementMe = new XGRElementUISwtWizardPageBuildTime();    break;     
	           	case XGR.TABLE:                   xgrElementMe = new XGRElementUISwtTable();                  break;        
	           	case XGR.TABLEDATA:               xgrElementMe = new XGRElementUISwtTableData();              break;        
	           	case XGR.TABLEHEADING:            xgrElementMe = new XGRElementUISwtTableHeading();           break;        
	           	case XGR.TABLEROW:                xgrElementMe = new XGRElementUISwtTableRow();               break;        
	           	case XGR.TEXTAREA:                xgrElementMe = new XGRElementUISwtTextArea();               break;        
	           	case XGR.TEXTFIELD:               xgrElementMe = new XGRElementUISwtTextField();              break;        
				case XGR.TEXTFIELDHIDDEN:          xgrElementMe = new XGRElementUISwtTextFieldHidden();        break;        
//				case XGR.TREE:                    xgrElementMe = new XGRElementUISwtTree();                   break;        
			}
		}

		// Use any preset widget instance
		//-------------------------------
		else
		{
			// Trace.string( "Element already pre-created: "  + _strElementType );
			xgrElementMe = _xgrElement;
		}

		// Add my element to a parent element
		//-----------------------------------
		if( xgrElementMe != null )
		{
			_xgrElement = xgrElementMe;

			// Register with the presentation model element
			//---------------------------------------------
			if( xgrElementMe instanceof IXGRElementUI )
			{
				_node._elementUI = (IXGRElementUI)xgrElementMe;
			}
		
			// Add myself to the parent
			//-------------------------
			if( xgrElementParent != null && xgrElementParent instanceof IXGRElementContainer )
			{
				// Trace.string( "Adding " + xgrElementMe.getClass().getName() + " to " + xgrElementParent.getClass().getName() );

				IXGRElementContainer xgrElementContainer = (IXGRElementContainer)xgrElementParent;
				xgrElementContainer.addChildElement( xgrElementMe );
			}
		}
	
		// Otherwise, I have no UI, send my parent on
		//-------------------------------------------
		else
		{
			xgrElementMe = xgrElementParent;
		}
		
		// Get my children to create their elements
		//-----------------------------------------
		if( _vnodeChildren != null )
		{
			XGRNodePresentation xgrNodeChild = null;
		
			for( int i=0; i< _vnodeChildren.size(); ++i )
			{
				xgrNodeChild = (XGRNodePresentation)_vnodeChildren.elementAt( i );
				xgrNodeChild.createElements( xgrElementMe );
			}
		}
	}
	
//	Trace.functionExit( this, "createElements" );
	
	return xgrElementMe;	
}
/**
 */
public void createWidgets( Object objParent )
{
//	Trace.functionEntry( this, "createWidgets" );
	
	// If I have a UI element, have it create its widget
	// and its child widgets unless, it creates them
	// automatically later during a callback
	//--------------------------------------------------
	Object objMe = null;
	
	if( _xgrElement != null && _xgrElement instanceof IXGRElementUI )
	{
		// Trace.string( _xgrElement.getClass().getName() );
		
		Object objWidget = ((IXGRElementUI)_xgrElement).createWidget( objParent );
	}

	// If I have no UI, try my children
	//---------------------------------
	else
	{
		if( _vnodeChildren != null )
		{
			XGRNodePresentation xgrNodeChild = null;
		
			for( int i=0; i< _vnodeChildren.size(); ++i )
			{
				xgrNodeChild = (XGRNodePresentation)_vnodeChildren.elementAt( i );
				xgrNodeChild.createWidgets( objParent );
			}
		}
	}
	
//	Trace.functionExit( this, "createWidgets" );
}
/**
 */
public void deduceElementTypes( int iParentElementType, boolean bBuildTime, int iUIType )
{
//	Trace.functionEntry( this, "deduceElementTypes" );

	// Deduce my own UI
	//-----------------
	_iElementType = _node._iType;

	// Body
	//-----
	switch( _iElementType )
	{
		case XGR.BODY:
		{
			switch( iUIType )
			{
				case XGR.PANEL:          _iElementType = bBuildTime == false ? XGR.COMPOSITEPANEL : XGR.COMPOSITE;                break;
				case XGR.PREFERENCES:    _iElementType = bBuildTime == false ? XGR.PREFERENCEDIALOG : XGR.SCRIPTPREFERENCEDIALOG; break;
				case XGR.PREFERENCEPAGE: _iElementType = XGR.PREFERENCEDIALOGSTANDIN;                                             break;
				case XGR.PROPERTYSHEET:  _iElementType = XGR.PROPERTYSHEETCOMPOSITE;                                              break;
				case XGR.WIZARD:         _iElementType = bBuildTime == false ? XGR.WIZARD : XGR.WIZARDBUILDTIME;                  break;
			}
			break;
		}

		case XGR.FIELDSET:
		{
			// A fieldset with a title is a group box
			//---------------------------------------
			_iElementType = XGR.FIELDSET;
			
			String strValue = getNodeAttributeValue( _node, "title" );
			
			if( strValue != null )
			{
				_iElementType = strValue.length() > 0 ? XGR.GROUPBOX : XGR.FIELDSET;
			}
			break;
		}

		case XGR.FORM:
		{
			switch( iParentElementType )
			{
				case XGR.PREFERENCEDIALOG:
				case XGR.PREFERENCEDIALOGSTANDIN:
				case XGR.PREFERENCEPAGE:
					_iElementType = XGR.PREFERENCEPAGE;
					break;

				case XGR.PROPERTYSHEETCOMPOSITE:
				    _iElementType = XGR.PROPERTYSHEET;
				    break;
				    
				case XGR.WIZARDPAGEBUILDTIME:
					_iElementType = XGR.SCRIPTPREFERENCEDIALOG;
					break;

				case XGR.SCRIPTPREFERENCEDIALOG:
				case XGR.SCRIPTPREFERENCEPAGE:
					_iElementType = XGR.SCRIPTPREFERENCEPAGE;
					break;
					
				case XGR.WIZARD:
					_iElementType = XGR.WIZARDPAGE;
					break;

				case XGR.WIZARDBUILDTIME:
					_iElementType = XGR.WIZARDPAGEBUILDTIME;
					break;

				case XGR.COMPOSITE:
				case XGR.COMPOSITEPANEL:
					_iElementType = XGR.COMPOSITE;
					break;
			}
			break;
		}

		case XGR.TABLE:
			_iElementType = XGR.TABLELAYOUT;
			break;
	}

//	Trace.string( "Element: " + strNodeName + "  UIType: " + _iElementType );
	
	// Get my children to deduce their UI
	//-----------------------------------
	if( _vnodeChildren != null )
	{
		XGRNodePresentation xgrNodeChild = null;
		
		for( int i=0; i< _vnodeChildren.size(); ++i )
		{
			xgrNodeChild = (XGRNodePresentation)_vnodeChildren.elementAt( i );
			xgrNodeChild.deduceElementTypes( _iElementType, bBuildTime, iUIType );
		}
	}

//	Trace.functionExit( this, "deduceElementTypes" );
	
	return;
}
/**
 */
public void dumpDebugInformation( int iLevel )
{
//	Trace.functionEntry( this, "dumpDebugInformation" );

	// Indentation
	//------------
	StringBuffer strbuf = new StringBuffer();

	for( int i=0; i<iLevel; ++i )
	{
		strbuf.append( "    " );
	}

	// Element type number
	//--------------------
	strbuf.append( Integer.toString( _iElementType ) );

	// Class name
	//-----------
	if( _xgrElement != null )
	{
		strbuf.append( " " );
		strbuf.append( _xgrElement.getClass().getName() );
	}

	System.out.println( strbuf.toString() );

	// Recursively do my children
	//---------------------------
	if( _vnodeChildren != null )
	{
		XGRNodePresentation xgrNodeChild = null;
		
		for( int i=0; i< _vnodeChildren.size(); ++i )
		{
			xgrNodeChild = (XGRNodePresentation)_vnodeChildren.elementAt( i );
			xgrNodeChild.dumpDebugInformation( iLevel + 1 );
		}
	}
	
//	Trace.functionExit( this, "dumpDebugInformation" );
}
/**
 */
public IXGRElementUIDataPresentationElement getDataPresentationElement( String strElementName )
{
//	Trace.functionEntry( this, "getPresentationElement" );

	IXGRElementUIDataPresentationElement elementReturn = null;

	// Determine if this is me
	//------------------------
	if( _xgrElement != null                     && 
		_xgrElement instanceof IXGRElementNamed &&
		_xgrElement instanceof IXGRElementUIDataPresentationElement    ) 
	{
		if( strElementName.equals( ((IXGRElementNamed)_xgrElement).getName() ) == true )
		{
			elementReturn = (IXGRElementUIDataPresentationElement)_xgrElement;
		}
	}

	// Check my children
	//------------------
	if( elementReturn == null && _vnodeChildren != null )
	{
		XGRNode             nodeChild        = null;
		XGRNodePresentation nodePresentation = null;
		
		for( int i=0; i<_vnodeChildren.size(); ++i )
		{
			nodeChild = (XGRNode)_vnodeChildren.elementAt(i);
			
			if( nodeChild instanceof XGRNodePresentation )
			{
				nodePresentation = (XGRNodePresentation)_vnodeChildren.elementAt(i);
				
				elementReturn = nodePresentation.getDataPresentationElement( strElementName );
				
				if( elementReturn != null )
				{
					break;
				}
			}
		}
	}

//	Trace.functionExit( this, "getPresentationElement" );
	
	return elementReturn;
}
/**
 */
public Object getWidget() 
{
	if( _xgrElement != null && _xgrElement instanceof IXGRElementUI )
	{
		return ((IXGRElementUI)_xgrElement).getWidget();
	}
	 
	return null;
}
/**
 */
public void initializeElementProperties( XGRResources resources )
{
//	Trace.functionEntry( this, "initializeElementProperties" );

	// Get my children to initialize first
	//------------------------------------
	if( _vnodeChildren != null )
	{
		XGRNodePresentation xgrNodeChild = null;
		
		for( int i=0; i< _vnodeChildren.size(); ++i )
		{
			xgrNodeChild = (XGRNodePresentation)_vnodeChildren.elementAt( i );
			xgrNodeChild.initializeElementProperties( resources );
		}
	}

	// Initialize my element
	//----------------------
	if( _xgrElement != null )
	{
		// Caption
		//--------
		if( _xgrElement instanceof IXGRElementCaption )
		{
			// Obtain initial caption from the DDML
			//-------------------------------------
			setInitialCaption( (IXGRElementCaption)_xgrElement, resources.getNLTranslationManager() );
		}
		
		// Event generators
		//-----------------
		if( resources.getScriptManager() != null && _xgrElement instanceof IXGRElementEventGenerator )
		{
			((IXGRElementEventGenerator)_xgrElement).setScriptManager( resources.getScriptManager() );
			
			if( _xgrElement instanceof IXGRElementEventGeneratorOnBlur )
			{
				setInitialOnBlurScript( (IXGRElementEventGeneratorOnBlur)_xgrElement );
			}
			
			if( _xgrElement instanceof IXGRElementEventGeneratorOnChange )
			{
				setInitialOnChangeScript( (IXGRElementEventGeneratorOnChange)_xgrElement );
			}

			if( _xgrElement instanceof IXGRElementEventGeneratorOnClick )
			{
				setInitialOnClickScript( (IXGRElementEventGeneratorOnClick)_xgrElement );
			}
			
			if( _xgrElement instanceof IXGRElementEventGeneratorOnFocus )
			{
				setInitialOnFocusScript( (IXGRElementEventGeneratorOnFocus)_xgrElement );
			}

			if( _xgrElement instanceof IXGRElementEventGeneratorOnKeyPress )
			{
				setInitialOnKeyPressScript( (IXGRElementEventGeneratorOnKeyPress)_xgrElement );
			}

			if( _xgrElement instanceof IXGRElementEventGeneratorOnLoad )
			{
				setInitialOnLoadScript( (IXGRElementEventGeneratorOnLoad)_xgrElement );
			}
			
			if( _xgrElement instanceof IXGRElementEventGeneratorOnReset )
			{
				setInitialOnResetScript( (IXGRElementEventGeneratorOnReset)_xgrElement );
			}
			
			if( _xgrElement instanceof IXGRElementEventGeneratorOnSubmit )
			{
				setInitialOnSubmitScript( (IXGRElementEventGeneratorOnSubmit)_xgrElement );
			}

			if( _xgrElement instanceof IXGRElementEventGeneratorOnUnload )
			{
				setInitialOnUnloadScript( (IXGRElementEventGeneratorOnUnload)_xgrElement );
			}
		}
		
		// Name
		//-----
		if( _xgrElement instanceof IXGRElementNamed )
		{
			// Obtain the name, and register the element
			// with the script manager
			//------------------------------------------
			setInitialName( (IXGRElementNamed)_xgrElement, resources.getScriptManager(), resources.getWidgetRegistry() );
		}

		// Checked
		//--------
		if( _xgrElement instanceof IXGRElementUIChecked )
		{
			setInitialChecked( (IXGRElementUIChecked)_xgrElement );
		}
		
		// Data presentation element
		//--------------------------
		if( _xgrElement instanceof IXGRElementUIDataPresentationElement )
		{
			setInitialDataModel( (IXGRElementUIDataPresentationElement)_xgrElement, resources.getDataModel(), resources.getUILinkManager() );
		}
		
		// Image source name
		//------------------
		if( _xgrElement instanceof IXGRElementUIImageElement )
		{
			setInitialImageName( (IXGRElementUIImageElement)_xgrElement );
		}
		
		// Layout
		//-------
		if( _xgrElement instanceof IXGRElementUILayoutElement )
		{
			setInitialLayout( (IXGRElementUILayoutElement)_xgrElement );
		}
		
		// Layout table
		//-------------
		if( _xgrElement instanceof IXGRElementUILayoutTableElement )
		{
			setInitialLayoutTable( (IXGRElementUILayoutTableElement)_xgrElement );
		}

		// Read Only
		//----------
		if( _xgrElement instanceof IXGRElementUIReadOnly )
		{
			setInitialReadOnly( (IXGRElementUIReadOnly)_xgrElement );
		}
		
		// Script
		//-------
		if( resources.getScriptManager() != null && _xgrElement instanceof IXGRElementScriptElement )
		{
			setInitialScript( (IXGRElementScriptElement)_xgrElement, resources.getScriptManager() );
		}

		// Sizable - Uses "width" and "height" not in XHTML
		//-------------------------------------------------
		//		if( _xgrElement instanceof IXGRElementUISizable )
		//		{
		//			setInitialSize( (IXGRElementUISizable )_xgrElement );
		//		}

		// Table
		//------
		if( _xgrElement instanceof IXGRElementUITableElement )
		{
			setInitialTable( (IXGRElementUITableElement)_xgrElement );
		}
		
		// Table data
		//-----------
		else if( _xgrElement instanceof IXGRElementUITableDataElement )
		{
			setInitialTableData( (IXGRElementUITableDataElement)_xgrElement );
		}

		// Table heading
		//--------------
		else if( _xgrElement instanceof IXGRElementUITableHeadingElement )
		{
			setInitialTableHeading( (IXGRElementUITableHeadingElement)_xgrElement );
		}

		// Text presentation element
		//--------------------------
		if( _xgrElement instanceof IXGRElementUITextPresentationElement )
		{
		   ((IXGRElementUITextPresentationElement)_xgrElement).setFontManager( resources.getFontManager() );
		}
	}

//	Trace.functionExit( this, "initializeElementProperties" );
	return;
}
/**
 */
public boolean presetElementInstance( int iType, IXGRElement element)
{
//	Trace.functionEntry( this, "presetElementInstance" );

//	Trace.string( "strType: " + strType + " This type: " + _strElementType );
	
	// If the type is correct, save the instance
	//------------------------------------------
	boolean bFound = false;
	
	if( _iElementType == iType )
	{
//		Trace.string( "Found element instance to preset" );
			
		_xgrElement = element;
		bFound = true;
	}
	
	// Try my children
	//----------------
	if( bFound == false && _vnodeChildren != null )
	{
		XGRNodePresentation xgrNodeChild = null;
		
		for( int i=0; i< _vnodeChildren.size(); ++i )
		{
			xgrNodeChild = (XGRNodePresentation)_vnodeChildren.elementAt( i );
			
			bFound = xgrNodeChild.presetElementInstance( iType, element );
			if( bFound == true )
			{
				break;
			}
		}
	}

//	Trace.functionExit( this, "presetElementInstance" );
	return bFound;
}
/**
 */
public void run() 
{
	boolean bRunning = false;
	
	// If I am a UI component, try to run
	//-----------------------------------
	if( _xgrElement != null )
	{
		if( _xgrElement instanceof IXGRElementUIRunnable )
		{
		  IXGRElementUIRunnable elementUI = (IXGRElementUIRunnable)_xgrElement;
		  elementUI.run();
		  bRunning = true;
		}
	}

	// Otherwise, try all my children
	//-------------------------------
	if( bRunning == false && _vnodeChildren != null )
	{
		XGRNodePresentation xgrNodeChild = null;
		
		for( int i=0; i<_vnodeChildren.size(); ++i )
		{
			xgrNodeChild = (XGRNodePresentation)_vnodeChildren.elementAt( i );
			xgrNodeChild.run();
		}
	}

	return;
}
/**
 */
public void setDataModel( IXGRDataModel dataModel ) 
{
	// Have my children update first
	//------------------------------
	if( _vnodeChildren != null )
	{
		XGRNodePresentation xgrNodeChild = null;
		
		for( int i=0; i< _vnodeChildren.size(); ++i )
		{
			xgrNodeChild = (XGRNodePresentation)_vnodeChildren.elementAt( i );
			xgrNodeChild.setDataModel( dataModel );
		}
	}
	
	if( _xgrElement != null && _xgrElement instanceof IXGRElementUIDataPresentationElement )
	{
		// Give the new data model to the UI element
		//------------------------------------------
		((IXGRElementUIDataPresentationElement)_xgrElement).setDataModel( dataModel );

		// Have the UI element update itself with data
		//--------------------------------------------
		((IXGRElementUIDataPresentationElement)_xgrElement).linkToData();
	}
}
/**
 */
protected void setInitialCaption( IXGRElementCaption element )
{
//	Trace.functionEntry( this, "setInitialCaption" );
	
	String strCaption  = null;

	switch( _node._iType )
	{
		case XGR.BUTTON:
		case XGR.CHECK:
		case XGR.COMBO:
		case XGR.LIST:
		case XGR.RADIO:
		case XGR.TEXTAREA:
		case XGR.TEXTFIELD:
			strCaption = getNodeAttributeValue( _node, "value" );
			break;

		case XGR.BODY:
		case XGR.FIELDSET:
		case XGR.FORM:
			strCaption = getNodeAttributeValue( _node, "title" );
			break;

		case XGR.LABEL:
			strCaption = getNodeTextAttributeString();
			break;
	}

	// Set the caption text
	//---------------------	
	if( strCaption != null )
	{
		element.setCaptionText( strCaption.trim() );
	}

//	Trace.functionExit( this, "setInitialCaption" );
}
/**
 */
protected void setInitialCaption( IXGRElementCaption element, ScriptNLTranslationManager translationManager )
{
//	Trace.functionEntry( this, "setInitialCaption" );
	
	String strCaption  = null;

	switch( _node._iType )
	{
		case XGR.BUTTON:
		case XGR.CHECK:
		case XGR.COMBO:
		case XGR.LIST:
		case XGR.RADIO:
		case XGR.TEXTAREA:
		case XGR.TEXTFIELD:
			strCaption = getNodeAttributeValue( _node, "value" );
			break;

		case XGR.BODY:
		case XGR.FIELDSET:
		case XGR.FORM:
			strCaption = getNodeAttributeValue( _node, "title" );
			break;

		case XGR.LABEL:
			strCaption = getNodeTextAttributeString();
			break;
	}

	// Set the caption text
	//---------------------	
	if( strCaption != null )
	{
		// Translate any strings that begin with a key to
		// National Language equivalent
		//-----------------------------------------------
		if( strCaption.length() > 1 && strCaption.charAt( 0 ) == TRANSLATIONKEY )
		{
			strCaption = translationManager.getTranslatedString( strCaption.substring( 1 ) );
		}
		
		element.setCaptionText( strCaption.trim() );
	}

//	Trace.functionExit( this, "setInitialCaption" );
}
/**
 */
public void setInitialChecked( IXGRElementUIChecked element )
{
	String strChecked = getNodeAttributeValue( _node, "checked" );
	
	if( strChecked != null )
	{
		if( strChecked.equals( "checked" ) == true )
		{
			element.setChecked( true );
		}
	}
}
/**
 */
public void setInitialChecked( IXGRElementUIReadOnly element )
{
	String strReadOnly = getNodeAttributeValue( _node, "readonly" );
	
	if( strReadOnly != null )
	{
		if( strReadOnly.equals( "true" ) == true )
		{
			element.setReadOnly( true );
		}
	}
}
/**
 */
public void setInitialDataModel( IXGRElementUIDataPresentationElement element, IXGRDataModel dataModel, XGRUILinkManager dataPresentationManager ) 
{
//	Trace.functionEntry( this, "setInitialDataModel" );
	
	element.setDataModel( dataModel );
	element.setUILinkManager( dataPresentationManager );

	// Data source name and Link source name are not in XHTML
	//-------------------------------------------------------
	
	// Data source name
	//-----------------
	//	String strSourceName = getNodeAttributeValue( _node, "datasource" );

	//	if( strSourceName != null ) 
	//	{
	//		element.setDataSourceName( strSourceName );
	//	}

	// Link source name
	//-----------------
	//	strSourceName = getNodeAttributeValue( _node, "linksource" );

	//	if( strSourceName != null ) 
	//	{
	//		element.setUISourceName( strSourceName );
	//	}

//	Trace.functionExit( this, "setInitialDataModel" );
}
/**
 */
protected void setInitialImageName( IXGRElementUIImageElement element )
{
	String strImageName = getNodeAttributeValue( _node, "src" );

	if( strImageName != null ) 
	{
		element.setImageName( strImageName );
	}
}
/**
 * This function does a grid layout based on break elements <br>.
 * The number of columns is determined, then dummy label elements
 * are added to each row if the number of elements per row is less
 * than the number of columns in the layout.
 */
protected void setInitialLayout( IXGRElementUILayoutElement element )
{
//	Trace.functionEntry( this, "setInitialLayout" );

	// Obtain a list of child elements
	//--------------------------------
	Vector vectorChildren = element.getChildElements();

	if( vectorChildren != null )
	{
		int iNumberOfChildren = vectorChildren.size();

		if( iNumberOfChildren > 0 )
		{
			// Determine the maximum number of columns
			//----------------------------------------
			IXGRElementUI elementChild = null;

			int iNumberOfRows           = 0;
			int iMaximumNumberOfColumns = 1;
			int iNumberOfColumnsInRow   = 0;
		
			for( int i=0; i<iNumberOfChildren; ++i )
			{
				if( vectorChildren.elementAt(i) instanceof IXGRElementUI )
				{
					elementChild = (IXGRElementUI)vectorChildren.elementAt(i);
	
					if( elementChild instanceof XGRElementUISwtBreak )
					{
						iMaximumNumberOfColumns = Math.max( iMaximumNumberOfColumns,
							 								iNumberOfColumnsInRow    );
						iNumberOfColumnsInRow = 0;
						
						++iNumberOfRows;
					}
					else
					{
						++iNumberOfColumnsInRow;
					}
				}
			}


			// There may not be a break at the end
			//------------------------------------
			if( iNumberOfColumnsInRow > 0 )
			{
				iMaximumNumberOfColumns = Math.max( iMaximumNumberOfColumns,
					 								iNumberOfColumnsInRow    );

				++iNumberOfRows;
			}

			// Initialize the number of columns
			//---------------------------------
			element.setNumberOfColumns( iMaximumNumberOfColumns );

			// Ensure that each row has the same number of elements
			// Insert dummy label elements as required
			//-----------------------------------------------------
			iNumberOfColumnsInRow = 0;
			
			IXGRElementUI elementChildLast = null;
			
			for( int i=0; i<iNumberOfChildren; ++i )
			{
				if( vectorChildren.elementAt(i) instanceof IXGRElementUI )
				{
	
					elementChild = (IXGRElementUI)vectorChildren.elementAt(i);
	
					if( elementChild instanceof XGRElementUISwtBreak )
					{
						// If possible, set the span of the last child in the row
						// to fill the end of the row
						//-------------------------------------------------------
						if( elementChildLast instanceof IXGRElementUIGridElement )
						{
							((IXGRElementUIGridElement)elementChildLast).setSpan( new Point( iMaximumNumberOfColumns - iNumberOfColumnsInRow + 1, 1 ) );
						}

						// Otherwise, add dummy labels to the end of the row
						//--------------------------------------------------
						else
						{
							while( iNumberOfColumnsInRow < iMaximumNumberOfColumns )
							{				
								vectorChildren.insertElementAt( new XGRElementUISwtLabel(), i++ );
						
								++iNumberOfChildren;
								++iNumberOfColumnsInRow;
							}
						}
					
						iNumberOfColumnsInRow = 0;
					}
				
					else
					{
						++iNumberOfColumnsInRow;
					}
				
					elementChildLast = elementChild;
				}
			}
			
			// There may not be a break at the end
			//------------------------------------
			if( iNumberOfColumnsInRow > 0 )
			{
				// If possible, set the span of the last child in the row
				// to fill the end of the row
				//-------------------------------------------------------
				if( elementChildLast instanceof IXGRElementUIGridElement )
				{
					((IXGRElementUIGridElement)elementChildLast).setSpan( new Point( iMaximumNumberOfColumns - iNumberOfColumnsInRow + 1, 1 ) );
				}

				// Otherwise, add dummy labels to the end of the row
				//--------------------------------------------------
				else
				{
					while( iNumberOfColumnsInRow < iMaximumNumberOfColumns )
					{
						vectorChildren.addElement( new XGRElementUISwtLabel() );
						
						++iNumberOfColumnsInRow;
					}
				}
			}
		}
	}
	
//	Trace.functionExit( this, "setInitialLayout" );
	return;
}
/**
 * This lays out elements in a grid layout based on table elements
 * <tr>, and <td>
 * The number of columns is determined, then dummy label elements
 * are added to each row if the number of elements per row is less
 * than the number of columns in the layout.
 */
public void setInitialLayoutTable( IXGRElementUILayoutTableElement element )
{
//	Trace.functionEntry( this, "setInitialLayoutTable" );
	
	// Obtain a list of child elements
	//--------------------------------
	Vector vectorChildren = element.getChildElements();

	if( vectorChildren != null )
	{
		int iNumberOfChildren = vectorChildren.size();

		if( iNumberOfChildren > 0 )
		{
			int iNumberOfRows = 0;
			
			IXGRElementUI           		 elementRow  = null;
			IXGRElementUI         	     elementData = null;
			IXGRElementUITableDataElement elementGrid = null;
			
			// Determine the number of table rows
			//-----------------------------------
			for( int i=0; i<iNumberOfChildren; ++i )
			{
				elementRow = (IXGRElementUI)vectorChildren.elementAt(i);

				if( elementRow instanceof XGRElementUISwtTableRow )
				{
					++iNumberOfRows;
				}
			}

			// Determine the number of columns in each row
			//--------------------------------------------
			int iNumberOfColumnsInRow   = 0;
			int iNumberOfRowChildren    = 0;
			int iRowNumber              = 0;

			int[] iaNumberOfColumns = new int[ iNumberOfRows ];
			
			// Look for table rows
			//--------------------	
			for( int i=0; i<iNumberOfChildren; ++i )
			{
				elementRow = (IXGRElementUI)vectorChildren.elementAt(i);

				if( elementRow instanceof XGRElementUISwtTableRow )
				{
					// Obtain a list of child elements
					//--------------------------------
					Vector vectorRowChildren = ((XGRElementUISwtTableRow)elementRow).getChildElements();

					// Look for table data
					//--------------------
					iNumberOfRowChildren = vectorRowChildren.size();
						
					for( int j=0; j<iNumberOfRowChildren; ++j )
					{
						elementData = (IXGRElementUI)vectorRowChildren.elementAt(j);

						if( elementData instanceof IXGRElementUITableDataElement )
						{
							elementGrid = (IXGRElementUITableDataElement)elementData;
							
							// Obtain column and row span
							//---------------------------
							int iColspan = elementGrid.getColumnSpan();
							int iRowspan = elementGrid.getRowSpan();

							int iLastRowNumber = Math.min( iNumberOfRows - 1,
														   iRowNumber + iRowspan - 1 );

							for( int k=iRowNumber; k<=iLastRowNumber; ++k )
							{
								iaNumberOfColumns[ k ] += iColspan;
							}
						}
					}
					
					++iRowNumber;
				}
			}

			// Debug only
			//-----------
//			for( int i=0; i<iNumberOfRows; ++i )
//			{
//				Trace.string( "Column " + Integer.toString(i) + ": " + Integer.toString( iaNumberOfColumns[i] ) );
//			}
			
			// Determine the maximum number of columns
			//----------------------------------------
			int iMaximumNumberOfColumns = 1;
			
			for( int i=0; i<iNumberOfRows; ++i )
			{
				iMaximumNumberOfColumns = Math.max( iMaximumNumberOfColumns,
						 							iaNumberOfColumns[i]     );
			}
			
//			Trace.value( "NumberOfCols", iMaximumNumberOfColumns );
//			Trace.value( "NumberOfRows", iNumberOfRows );

			// Initialize the number of columns
			//---------------------------------
			element.setNumberOfColumns( iMaximumNumberOfColumns );
		}

		// Continue by setting up the tables children
		//-------------------------------------------
		setInitialLayoutTableElements( element );
	}

//	Trace.functionExit( this, "setInitialLayoutTable" );
	return;
}
/**
 */
public void setInitialLayoutTableElements( IXGRElementUILayoutTableElement element )
{
//	Trace.functionEntry( this, "setInitialLayoutTableElements" );
	
	// Obtain a list of child elements
	//--------------------------------
	Vector vectorChildren = element.getChildElements();
	
	// Create a new vector to hold the children
	//-----------------------------------------
	Vector vectorWidgetElements = new Vector();
		
	int iNumberOfChildren = vectorChildren.size();

	if( iNumberOfChildren > 0 )
	{
		// Determine the maximum number of rows in the layout
		// This is affected by the <td> "rowspan" attribute
		//---------------------------------------------------
		IXGRElement            elementRow    = null;
		IXGRElement            elementData   = null;
		IXGRElement            elementWidget = null;
		IXGRElementUITableDataElement elementTableData   = null;

		int iNumberOfRows 		  = 0;
		int iNumberOfRowChildren  = 0;
		int iNumberOfDataChildren = 0;
		int iColNumber            = 0;
		int iRowNumber            = 0;
		int iTotalNumberOfRows    = 0;
		
		int iNumberOfColumns = element.getNumberOfColumns();

		// Look for table rows and rowspan attributes
		//-------------------------------------------
		for( int i=0; i<iNumberOfChildren; ++i )
		{
			elementRow = (IXGRElement)vectorChildren.elementAt(i);

			if( elementRow instanceof XGRElementUISwtTableRow )
			{
				// Obtain a list of child elements
				//--------------------------------
				Vector vectorRowChildren = ((XGRElementUISwtTableRow)elementRow).getChildElements();

				// Look for table data
				//--------------------
				iNumberOfRowChildren = vectorRowChildren.size();
						
				for( int j=0; j<iNumberOfRowChildren; ++j )
				{
					elementData = (IXGRElement)vectorRowChildren.elementAt(j);
					
					if( elementData instanceof IXGRElementUITableDataElement )
					{
						elementTableData = (IXGRElementUITableDataElement)elementData;
							
						// Obtain row span
						//----------------
						int iRowspan = elementTableData.getRowSpan();

						// Calculate the total number of rows
						//-----------------------------------
						iTotalNumberOfRows = Math.max( iTotalNumberOfRows,
													   iRowNumber + iRowspan );                          
					}
				}
			}
			
			++iRowNumber;
		}

		// Create a 2-Dimensional array to hold widget elements
		//-----------------------------------------------------
//		Trace.value( "iTotalNumberOfRows", iTotalNumberOfRows );
//		Trace.value( "iNumberOfColumns", iNumberOfColumns );
		
		Object[][] oaaMatrix = new Object[ iTotalNumberOfRows ][ iNumberOfColumns ];

		// Place the elements in the matrix
		// Cells that are filled have either the widget element,
		// or a String( "Filled" ).  Empty cells are left null.
		//------------------------------------------------------
		String strFilled = "Filled";

		iRowNumber = 0;
		iColNumber = 0;
			
		// Look for table rows
		//--------------------	
		for( int i=0; i<iNumberOfChildren; ++i )
		{
			elementRow = (IXGRElement)vectorChildren.elementAt(i);

			if( elementRow instanceof XGRElementUISwtTableRow )
			{
				// Obtain a list of child elements
				//--------------------------------
				Vector vectorRowChildren = ((XGRElementUISwtTableRow)elementRow).getChildElements();

				// Look for table data
				//--------------------
				iNumberOfRowChildren = vectorRowChildren.size();
						
				for( int j=0; j<iNumberOfRowChildren; ++j )
				{
					elementData = (IXGRElement)vectorRowChildren.elementAt(j);
					
					if( elementData instanceof IXGRElementUITableDataElement )
					{
						elementTableData = (IXGRElementUITableDataElement)elementData;
							
						// Obtain column and row span
						//---------------------------
						int iColSpan = elementTableData.getColumnSpan();
						int iRowSpan = elementTableData.getRowSpan();

						// Obtain a list of table data child widget elements
						//--------------------------------------------------
						Vector vectorDataChildren = ((IXGRElementContainer)elementTableData).getChildElements();

						// If there is a child widget, use it
						//-----------------------------------
						iNumberOfDataChildren = vectorDataChildren.size();

						if( iNumberOfDataChildren > 0 )
						{
							elementWidget = (IXGRElement)vectorDataChildren.elementAt( 0 );
						}

						// Otherwise, create a label widget
						//---------------------------------
						else
						{
							elementWidget = new XGRElementUISwtLabel();
						}

						// Give the column/row span to the widget element
						// The widget element will set the widget's grid
						// data during widget initialization
						//-----------------------------------------------
						if( elementWidget instanceof IXGRElementUIGridElement )
						{
							((IXGRElementUIGridElement)elementWidget).setSpan( new Point( iColSpan, iRowSpan ) );						
						}
						
						// Find a cell that is not filled in this row
						//-------------------------------------------	
						while( true )
						{
							if( oaaMatrix[ iRowNumber ][ iColNumber ] == null )
							{
								break;
							}
						
							++iColNumber;
						}
						
						// Fill all cells covered with the "Filled" string object
						//-------------------------------------------------------
						for( int k=iRowNumber; k<iRowNumber+iRowSpan; ++k )
						{
							for( int m=iColNumber; m<iColNumber+iColSpan; ++m )
							{
								oaaMatrix[k][m] = strFilled;
							}
						}

						// Fill the current cell with the widget
						//--------------------------------------
						oaaMatrix[ iRowNumber ][ iColNumber ] = elementWidget;
					}
				}
			}
			
			++iRowNumber;
			iColNumber = 0;
		}
		
		// Fill all empty cells with a label
		//----------------------------------
		for( int k=0; k<iTotalNumberOfRows; ++k )
		{
			for( int m=0; m<iNumberOfColumns; ++m )
			{
				if( oaaMatrix[k][m] == null )
				{
					oaaMatrix[k][m] = new XGRElementUISwtLabel();
				}
			}
		}

		// Debug only
		//-----------
		//for( int k=0; k<iTotalNumberOfRows; ++k )
		//{
		//	for( int m=0; m<iNumberOfColumns; ++m )
		//	{
		//		if( oaaMatrix[k][m] == null )
		//		{
		//			Trace.string( "null" );
		//		}
		//		
		//		else
		//		{
		//			Trace.string( Integer.toString(k) + " " +
		//				          Integer.toString(m) + " " + 
		//				          oaaMatrix[k][m].toString() );
		//		}
		//	}
		//}

		// Fill the element vector with all the elements
		//----------------------------------------------
		for( int k=0; k<iTotalNumberOfRows; ++k )
		{
			for( int m=0; m<iNumberOfColumns; ++m )
			{
				if( oaaMatrix[k][m] != strFilled )
				{
					vectorWidgetElements.addElement( oaaMatrix[k][m] );
				}
			}
		}
	}
	
	// Set the elements as children to the grid layout
	//------------------------------------------------
	element.setChildElements( vectorWidgetElements );

//	Trace.functionExit( this, "setInitialLayoutTableElements" );
	return;
}
/**
 * Gives the name and script manager to the element.
 * When the element's widget is created, the widget registers
 * itself with the script manager in its initialize function.
 */
protected void setInitialName( IXGRElementNamed element, ScriptManager scriptManager, XGRWidgetRegistry widgetRegistry )
{
	// This was originally "name", but has been changed
	// to "id" since elements like body, and label do not
	// have a "name" attribute.
	//---------------------------------------------------
	String strName = getNodeAttributeValue( _node, "id" );

	if( strName != null )
	{
		// Give the name and script manager to the element
		//------------------------------------------------
		element.setName( strName );
		element.setScriptManager( scriptManager );

		// Store the element in the widget registry
		//-----------------------------------------
		widgetRegistry.addElement( strName, element );
	}
}
/**
 */
protected void setInitialOnBlurScript( IXGRElementEventGeneratorOnBlur element )
{
//	Trace.functionEntry( this, "setInitialOnBlurScript" );

	XGRPresentationModelAttribute attribute = _node.getAttribute( "onblur" );	

	if( attribute != null && attribute._strValue != null && attribute._strValue.length() > 0 ) 
	{
		element.setOnBlurScriptDescriptor( new ScriptDescriptor( attribute._strValue,
			                                               null,
			                                               attribute._strFilename,
			                                               attribute._iLineNumber,
			                                               attribute._iColumnNumber ) );
	}

//	Trace.functionExit( this, "setInitialOnBlurScript" );
}
/**
 */
protected void setInitialOnChangeScript( IXGRElementEventGeneratorOnChange element )
{
	XGRPresentationModelAttribute attribute = _node.getAttribute( "onchange" );	

	if( attribute != null && attribute._strValue != null && attribute._strValue.length() > 0 ) 
	{
		element.setOnChangeScriptDescriptor( new ScriptDescriptor( attribute._strValue,
			                                               null,
			                                               attribute._strFilename,
			                                               attribute._iLineNumber,
			                                               attribute._iColumnNumber ) );
	}
}
/**
 */
protected void setInitialOnClickScript( IXGRElementEventGeneratorOnClick element )
{
	XGRPresentationModelAttribute attribute = _node.getAttribute( "onclick" );	

	if( attribute != null && attribute._strValue != null && attribute._strValue.length() > 0 ) 
	{
		element.setOnClickScriptDescriptor( new ScriptDescriptor( attribute._strValue,
			                                               null,
			                                               attribute._strFilename,
			                                               attribute._iLineNumber,
			                                               attribute._iColumnNumber ) );
	}
}
/**
 */
protected void setInitialOnFocusScript( IXGRElementEventGeneratorOnFocus element )
{
	XGRPresentationModelAttribute attribute = _node.getAttribute( "onfocus" );	

	if( attribute != null && attribute._strValue != null && attribute._strValue.length() > 0 ) 
	{
		element.setOnFocusScriptDescriptor( new ScriptDescriptor( attribute._strValue,
			                                               null,
			                                               attribute._strFilename,
			                                               attribute._iLineNumber,
			                                               attribute._iColumnNumber ) );
	}
}
/**
 */
protected void setInitialOnKeyPressScript( IXGRElementEventGeneratorOnKeyPress element )
{
	XGRPresentationModelAttribute attribute = _node.getAttribute( "onkeypress" );	

	if( attribute != null && attribute._strValue != null && attribute._strValue.length() > 0 ) 
	{
		element.setOnKeyPressScriptDescriptor( new ScriptDescriptor( attribute._strValue,
			                                               null,
			                                               attribute._strFilename,
			                                               attribute._iLineNumber,
			                                               attribute._iColumnNumber ) );
	}
}
/**
 */
protected void setInitialOnLoadScript( IXGRElementEventGeneratorOnLoad element )
{
	XGRPresentationModelAttribute attribute = _node.getAttribute( "onload" );	

	if( attribute != null && attribute._strValue != null && attribute._strValue.length() > 0 ) 
	{
		element.setOnLoadScriptDescriptor( new ScriptDescriptor( attribute._strValue,
			                                               null,
			                                               attribute._strFilename,
			                                               attribute._iLineNumber,
			                                               attribute._iColumnNumber ) );
	}
}
/**
 */
protected void setInitialOnResetScript( IXGRElementEventGeneratorOnReset element )
{
	XGRPresentationModelAttribute attribute = _node.getAttribute( "onreset" );	

	if( attribute != null && attribute._strValue != null && attribute._strValue.length() > 0 ) 
	{
		element.setOnResetScriptDescriptor( new ScriptDescriptor( attribute._strValue,
			                                               null,
			                                               attribute._strFilename,
			                                               attribute._iLineNumber,
			                                               attribute._iColumnNumber ) );
	}
}
/**
 */
protected void setInitialOnSubmitScript( IXGRElementEventGeneratorOnSubmit element )
{
	XGRPresentationModelAttribute attribute = _node.getAttribute( "onsubmit" );	

	if( attribute != null && attribute._strValue != null && attribute._strValue.length() > 0 ) 
	{
		element.setOnSubmitScriptDescriptor( new ScriptDescriptor( attribute._strValue,
			                                               null,
			                                               attribute._strFilename,
			                                               attribute._iLineNumber,
			                                               attribute._iColumnNumber ) );
	}
}
/**
 */
protected void setInitialOnUnloadScript( IXGRElementEventGeneratorOnUnload element )
{
	XGRPresentationModelAttribute attribute = _node.getAttribute( "onunload" );	

	if( attribute != null && attribute._strValue != null && attribute._strValue.length() > 0 ) 
	{
		element.setOnUnloadScriptDescriptor( new ScriptDescriptor( attribute._strValue,
			                                               null,
			                                               attribute._strFilename,
			                                               attribute._iLineNumber,
			                                               attribute._iColumnNumber ) );
	}
}
/**
 */
public void setInitialReadOnly( IXGRElementUIReadOnly element )
{
	String strReadOnly = getNodeAttributeValue( _node, "readonly" );
	
	if( strReadOnly != null )
	{
		if( strReadOnly.equals( "true" ) == true )
		{
			element.setReadOnly( true );
		}
	}
}
/**
 */
public void setInitialScript( IXGRElementScriptElement element, ScriptManager scriptManager ) 
{
//	Trace.functionEntry( this, "setInitialScript" );

	// The parsers turn #comment child elements into text attributes
	// when creating the presentation model
	//--------------------------------------------------------------
	XGRPresentationModelAttribute attribute = _node.getAttribute( "text" );	

	if( attribute != null && attribute._strValue != null && attribute._strValue.length() > 0 ) 
	{
		element.setScriptManager( scriptManager );
			
		element.setScriptDescriptor( new ScriptDescriptor( attribute._strValue,
														   null,
														   attribute._strFilename,
														   attribute._iLineNumber,
														   attribute._iColumnNumber ) );
	}
	
//	Trace.functionExit( this, "setInitialScript" );
}
/**
 */
public void setInitialSize(IXGRElementUISizable element) 
{
	int iWidth  = 0;
	int iHeight = 0;

	// Width
	//------	
	String strLength = getNodeAttributeValue( _node, "width" );

	if( strLength != null ) 
	{
		try
		{
			iWidth = new Integer( strLength ).intValue();
		}
		catch( Exception ex )
		{
			iWidth = 0;
		}
	}

	// Height
	//-------
	strLength = getNodeAttributeValue( _node, "height" );

	if( strLength != null ) 
	{
		try
		{
			iHeight = new Integer( strLength ).intValue();
		}
		catch( Exception ex )
		{
			iHeight = 0;
		}
	}

	// Set the size if either is non-zero
	//-----------------------------------
	if( iWidth > 0 || iHeight > 0 )
	{
		element.setSize( new Point( iWidth, iHeight ) );
	}
	
}
/**
 */
public void setInitialTable(IXGRElementUITableElement element)
{
	// Look for table headers
	//-----------------------

	// Look for table data
	//--------------------
	// Obtain a list of child elements
	//--------------------------------
	Vector vectorChildren = element.getChildElements();

	if( vectorChildren != null )
	{
		int iNumberOfChildren = vectorChildren.size();

		if( iNumberOfChildren > 0 )
		{
			int iNumberOfRows = 0;
			
			IXGRElementUI           		 elementRow  = null;
			IXGRElementUI         	     elementData = null;
			
			// Determine the number of table rows
			//-----------------------------------
			for( int i=0; i<iNumberOfChildren; ++i )
			{
				elementRow = (IXGRElementUI)vectorChildren.elementAt(i);

				if( elementRow instanceof XGRElementUISwtTableRow )
				{
					++iNumberOfRows;
				}
			}

			// Determine the number of columns in each row
			//--------------------------------------------
			int iNumberOfColumns        = 0;
			int iNumberOfHeaders        = 0;
			int iNumberOfRowChildren    = 0;
			int iRowNumber              = 0;

			int[] iaNumberOfColumns = new int[ iNumberOfRows ];
			
			// Look for table rows and count table headers / table data
			//---------------------------------------------------------
			for( int i=0; i<iNumberOfChildren; ++i )
			{
				elementRow = (IXGRElementUI)vectorChildren.elementAt(i);

				if( elementRow instanceof XGRElementUISwtTableRow )
				{
					// Obtain a list of child elements
					//--------------------------------
					Vector vectorRowChildren = ((XGRElementUISwtTableRow)elementRow).getChildElements();

					// Look for table headers and table data
					//--------------------------------------
					iNumberOfRowChildren = vectorRowChildren.size();
						
					for( int j=0; j<iNumberOfRowChildren; ++j )
					{
						elementData = (IXGRElementUI)vectorRowChildren.elementAt(j);

						if( elementData instanceof IXGRElementUITableHeadingElement )
						{
							++iNumberOfHeaders;
						}
						
						else if( elementData instanceof IXGRElementUITableDataElement )
						{
							++iNumberOfColumns;
						}
					}
					
					++iRowNumber;
				}
			}

			// Create arrays to hold values
			//-----------------------------
			String[] straHeaders = new String[ iNumberOfHeaders ];
			String[] straColumns = new String[ iNumberOfColumns ];

			int iHeader = 0;
			int iColumn = 0;
			
			// Look for table rows and obtain table headers / table data
			//----------------------------------------------------------
			for( int i=0; i<iNumberOfChildren; ++i )
			{
				elementRow = (IXGRElementUI)vectorChildren.elementAt(i);

				if( elementRow instanceof XGRElementUISwtTableRow )
				{
					// Obtain a list of child elements
					//--------------------------------
					Vector vectorRowChildren = ((XGRElementUISwtTableRow)elementRow).getChildElements();

					// Look for table headers and table data
					//--------------------------------------
					iNumberOfRowChildren = vectorRowChildren.size();
						
					for( int j=0; j<iNumberOfRowChildren; ++j )
					{
						elementData = (IXGRElementUI)vectorRowChildren.elementAt(j);

						if( elementData instanceof IXGRElementUITableHeadingElement )
						{
							straHeaders[ iHeader++ ] = ((IXGRElementUITableHeadingElement)elementData).getValue();
						}
						
						else if( elementData instanceof IXGRElementUITableDataElement )
						{
							straColumns[ iColumn++ ] = ((IXGRElementUITableDataElement)elementData).getValue();
						}
					}
				}
			}

			// Initialize the table with headers / column names
			//-------------------------------------------------	
			element.setTableHeadings   ( straHeaders );
			element.setTableColumnNames( straColumns );
		}
	}
}
/**
 */
protected void setInitialTableData( IXGRElementUITableDataElement element )
{
	if( element != null )
	{
		String strValue = null;
		int    iValue   = 0;

		// Column span
		//------------
		strValue = getNodeAttributeValue( _node, "colspan" );
		if( strValue != null )
		{
			try
			{
				iValue = new Integer( strValue ).intValue();
			}
			catch( Exception ex ){}

			if( iValue < 1 ) iValue = 1;

			element.setColumnSpan( iValue );
		}

		// Row span
		//---------
		strValue = getNodeAttributeValue( _node, "rowspan" );
		if( strValue != null )
		{
			try
			{
				iValue = new Integer( strValue ).intValue();
			}
			catch( Exception ex ){}

			if( iValue < 1 ) iValue = 1;

			element.setRowSpan( iValue );
		}

		// Text value
		//-----------
		strValue = getNodeTextAttributeString();
		if( strValue != null )
		{
			element.setValue( strValue );
		}
	}
	
	return;
}
/**
 */
protected void setInitialTableHeading( IXGRElementUITableHeadingElement element )
{
	// Text value
	//-----------
	String strValue = getNodeTextAttributeString();
	
	if( strValue != null )
	{
		element.setValue( strValue );
	}
}
/**
 */
public void setParentElement(Composite parent)
{
	// Desktop New Smartguide Build Time
	//----------------------------------
	if( _iElementType == XGR.WIZARDBUILDTIME )
	{
	   XGRElementUISwtWizardBuildTime element = new XGRElementUISwtWizardBuildTime();
	   element.setParentComposite( parent );
			
		presetElementInstance( XGR.WIZARDBUILDTIME, element );
	}
			

	// Preference dialog Build Time
	//----------------------------------
	else if( _iElementType == XGR.SCRIPTPREFERENCEDIALOG )
	{
	   XGRElementUISwtScriptPreferenceDialog element = new XGRElementUISwtScriptPreferenceDialog();
	   element.setParentComposite( parent );
			
		presetElementInstance( XGR.SCRIPTPREFERENCEDIALOG, element );
	}

	// Panel Build Time
	//-----------------
	else if( _iElementType == XGR.COMPOSITE )
	{
	   XGRElementUISwtComposite element = new XGRElementUISwtComposite();
//	   element.setParentComposite( parent );
			
		presetElementInstance( XGR.COMPOSITE, element );
	}		
}
/**
 */
public void setVisible( boolean bVisible ) 
{
	// If I am a UI component, set my visibility
	//------------------------------------------
	if( _xgrElement != null )
	{
		if( _xgrElement instanceof IXGRElementUI )
		{
		  IXGRElementUI elementUI = (IXGRElementUI)_xgrElement;
		  elementUI.setVisible( bVisible );
		  return;
		}
	}

	// Otherwise, try all my children
	//-------------------------------
	if( _vnodeChildren != null )
	{
		XGRNodePresentation xgrNodeChild = null;
		
		for( int i=0; i<_vnodeChildren.size(); ++i )
		{
			xgrNodeChild = (XGRNodePresentation)_vnodeChildren.elementAt( i );
			xgrNodeChild.setVisible( bVisible );
		}
	}

	return;
}
}
