package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * This parser takes an xml tree and creates a presentation element tree.
 */

public class XGRPresentationModel
{
	protected XGRPresentationModelElement _elementCurrent = null;
	protected XGRPresentationModelElement _elementRoot    = null;
	protected int                          _iLine                = -1; // 0 based
	protected int                          _iColumn              = -1; // 0 based
/**
 */
public XGRPresentationModel() 
{
}
/**
 */
private void addAttributeToElement( XGRPresentationModelElement  element,
	                               String                       strElementAttributeName,
	                               XGRXmlElement                elementXml,
	                               String                       strXmlAttributeName,
	                               String                       strDefaultValue           )
{
	XGRXmlAttribute attributeXml = elementXml.getAttribute( strXmlAttributeName );

	XGRPresentationModelAttribute attribute = null;
		
	if( attributeXml != null )
	{
		attribute = new XGRPresentationModelAttribute( strElementAttributeName,
													   attributeXml._strValue != null ? attributeXml._strValue : strDefaultValue,
													   attributeXml._strFilename,
													   attributeXml._iLineNumber,
													   attributeXml._iColumnNumber );
	}
	
	else
	{
		attribute = new XGRPresentationModelAttribute( strElementAttributeName,
													   strDefaultValue );
	}
	
	element.addAttribute( attribute );
	
}
/**
 */
public XGRPresentationModelElement getRootElement()
{
	return _elementRoot;
}
/**
 */
public void parse( XGRXmlElement elementXmlRoot ) throws Exception
{
//	Trace.functionEntry( this, "parse" );

	if( elementXmlRoot._strName.equals( "html" ) == true )
	{
		_elementRoot = new XGRPresentationModelElement( null, "html", XGR.HTML );

		int iNumberOfChildElements = elementXmlRoot.getNumberOfChildElements();

		for( int i=0; i<iNumberOfChildElements; ++i )
		{
			parseRecursive( _elementRoot, elementXmlRoot.getChildElementAt( i ) );
		}
	}
	
//	Trace.functionExit( this, "parse" );
}
/**
 * This method recursively creates presentation elements from xml elements.
 * If a presentation element is created, all of its child attributes are created as well.
 * Script text stores line/column number information in the presentation attribute.
 */
private void parseRecursive( XGRPresentationModelElement elementParent, XGRXmlElement elementXml )
{
//	Trace.functionEntry( this, "parseRecursive" );
//	Trace.string( elementXml._strName );
	
	XGRPresentationModelElement   element   = null;
	XGRPresentationModelAttribute attribute = null;
	
	if( elementXml._strName.equals( "#comment" ) == true )
	{
		if( elementParent._iType == XGR.SCRIPT )
		{
			attribute = elementParent.getAttribute( "text" );
			if( attribute != null )
			{
				XGRXmlAttribute attributeXml = elementXml.getAttribute( "text" );

				if( attributeXml != null )
				{
					attribute._strValue      = attributeXml._strValue;
					attribute._strFilename   = attributeXml._strFilename;
					attribute._iLineNumber   = attributeXml._iLineNumber;
					attribute._iColumnNumber = attributeXml._iColumnNumber;
				}
			}
		}
	}
	
	else if( elementXml._strName.equals( "#text" ) == true )
	{
		if( elementParent._iType == XGR.LABEL  ||
			elementParent._iType == XGR.SCRIPT    )
		{
			attribute = elementParent.getAttribute( "text" );

			if( attribute != null )
			{
				XGRXmlAttribute attributeXml = elementXml.getAttribute( "text" );

				if( attributeXml != null )
				{
					attribute._strValue      = attributeXml._strValue;
					attribute._strFilename   = attributeXml._strFilename;
					attribute._iLineNumber   = attributeXml._iLineNumber;
					attribute._iColumnNumber = attributeXml._iColumnNumber;
				}
			}
		}

		else
		{
			element = new XGRPresentationModelElement( elementParent, "label", XGR.LABEL );
			addAttributeToElement( element, "id", elementXml, "id", "" );
			addAttributeToElement( element, "text", elementXml, "text", "" );
		}
	}
		
	else if( elementXml._strName.equals( "body" ) == true )
	{
		element = new XGRPresentationModelElement( elementParent, elementXml._strName, XGR.BODY );
		addAttributeToElement( element, "id", elementXml, "id", "" );
		addAttributeToElement( element, "onload"    , elementXml, "onload"    , "" );
		addAttributeToElement( element, "onunload"  , elementXml, "onunload"  , "" );
	}
	
	else if( elementXml._strName.equals( "br" ) == true )
	{
		element = new XGRPresentationModelElement( elementParent, elementXml._strName, XGR.BREAK );
	}
	
	else if( elementXml._strName.equals( "button" ) == true )
	{
		element = new XGRPresentationModelElement( elementParent, elementXml._strName, XGR.BUTTON );
		addAttributeToElement( element, "id", elementXml, "id", "" );
		addAttributeToElement( element, "onclick", elementXml, "onclick", "" );
		addAttributeToElement( element, "value", elementXml, "value", "" );
	}

	else if( elementXml._strName.equals( "fieldset" ) == true )
	{
		element = new XGRPresentationModelElement( elementParent, elementXml._strName, XGR.FIELDSET );
		addAttributeToElement( element, "id", elementXml, "id", "" );
		addAttributeToElement( element, "title", elementXml, "title", "" );
	}
	
	else if( elementXml._strName.equals( "form" ) == true )
	{
		element = new XGRPresentationModelElement( elementParent, elementXml._strName, XGR.FORM );
		addAttributeToElement( element, "id", elementXml, "id", "" );
		addAttributeToElement( element, "title", elementXml, "title", "" );
		addAttributeToElement( element, "onreset"  , elementXml, "onreset" , "" );
		addAttributeToElement( element, "onsubmit" , elementXml, "onsubmit", "" );
	}
	
	else if( elementXml._strName.equals( "head" ) == true )
	{
		element = new XGRPresentationModelElement( elementParent, elementXml._strName, XGR.HEAD );
	}
	
	else if( elementXml._strName.equals( "hr" ) == true )
	{
		element = new XGRPresentationModelElement( elementParent, elementXml._strName, XGR.HORZRULE );
	}
	
	else if( elementXml._strName.equals( "img" ) == true )
	{
		element = new XGRPresentationModelElement( elementParent, elementXml._strName, XGR.IMG );
		addAttributeToElement( element, "id", elementXml, "id", "" );
		addAttributeToElement( element, "src", elementXml, "src", "" );
	}
	
	else if( elementXml._strName.equals( "input" ) == true )
	{
		// Determine the type of input
		//----------------------------
		String strInputType = elementXml.getAttributeValue( "type" );

		if( strInputType == null )
		{
			element = new XGRPresentationModelElement( elementParent, "text field", XGR.TEXTFIELD );
			addAttributeToElement( element, "id", elementXml, "id", "" );
			addAttributeToElement( element, "onblur"    , elementXml, "onblur"    , "" );
			addAttributeToElement( element, "onchange"  , elementXml, "onchange"  , "" );
			addAttributeToElement( element, "onfocus"   , elementXml, "onfocus"   , "" );
			addAttributeToElement( element, "onkeypress", elementXml, "onkeypress", "" );
			addAttributeToElement( element, "value", elementXml, "value", "" );
		}
		
		// Button
		//-------
		else if( strInputType.equals( "button" ) == true )
		{
			element = new XGRPresentationModelElement( elementParent, "button", XGR.BUTTON );
			addAttributeToElement( element, "id", elementXml, "id", "" );
			addAttributeToElement( element, "onclick", elementXml, "onclick", "" );
			addAttributeToElement( element, "value", elementXml, "value", "" );
		}

		// CheckBox
		//---------
		else if( strInputType.equals( "checkbox" ) == true )
		{
			element = new XGRPresentationModelElement( elementParent, "check", XGR.CHECK );
			addAttributeToElement( element, "checked", elementXml, "checked", "" );
			addAttributeToElement( element, "id", elementXml, "id", "" );
			addAttributeToElement( element, "onclick", elementXml, "onclick", "" );
			addAttributeToElement( element, "value", elementXml, "value", "" );
		}
		
		// Text hidden
		//------------
		else if( strInputType.equals( "hidden" ) == true )
		{
			element = new XGRPresentationModelElement( elementParent, "hidden", XGR.TEXTFIELDHIDDEN );
			addAttributeToElement( element, "id", elementXml, "id", "" );
			addAttributeToElement( element, "value", elementXml, "value", "" );
		}
		
		// RadioButton
		//------------
		else if( strInputType.equals( "radio" ) == true )
		{
			element = new XGRPresentationModelElement( elementParent, "radio", XGR.RADIO );
			addAttributeToElement( element, "checked", elementXml, "checked", "" );
			addAttributeToElement( element, "id", elementXml, "id", "" );
			addAttributeToElement( element, "onclick", elementXml, "onclick", "" );
			addAttributeToElement( element, "value", elementXml, "value", "" );
		}
			
		// Text
		//-----
		else if( strInputType.equals( "text" ) == true )
		{
			element = new XGRPresentationModelElement( elementParent, "text field", XGR.TEXTFIELD );
			addAttributeToElement( element, "id", elementXml, "id", "" );
			addAttributeToElement( element, "onblur"    , elementXml, "onblur"    , "" );
			addAttributeToElement( element, "onchange"  , elementXml, "onchange"  , "" );
			addAttributeToElement( element, "onfocus"   , elementXml, "onfocus"   , "" );
			addAttributeToElement( element, "onkeypress", elementXml, "onkeypress", "" );
			addAttributeToElement( element, "value", elementXml, "value", "" );
		}
	}

	else if( elementXml._strName.equals( "label" ) == true )
	{
		element = new XGRPresentationModelElement( elementParent, elementXml._strName, XGR.LABEL );
		addAttributeToElement( element, "id", elementXml, "id", "" );
		addAttributeToElement( element, "text", elementXml, "text", "" );
	}

	else if( elementXml._strName.equals( "meta" ) == true )
	{
		element = new XGRPresentationModelElement( elementParent, elementXml._strName, XGR.META );
		addAttributeToElement( element, "http-equiv", elementXml, "http-equiv", "Content-Script-Type" );
		addAttributeToElement( element, "content"   , elementXml, "content"   , "text/javascript"     );
	}
			
	else if( elementXml._strName.equals( "select" ) == true )
	{
		String strValue = elementXml.getAttributeValue( "size" );
		try
		{
			int iSize = new Integer( strValue ).intValue();
			if( iSize > 1 )
			{
				element = new XGRPresentationModelElement( elementParent, elementXml._strName, XGR.LIST );
			}
			
			else
			{
				element = new XGRPresentationModelElement( elementParent, elementXml._strName, XGR.COMBO );
			}
		}
		
		catch( Exception ex )
		{
			element = new XGRPresentationModelElement( elementParent, elementXml._strName, XGR.COMBO );			
		}

		addAttributeToElement( element, "id", elementXml, "id", "" );
		addAttributeToElement( element, "onchange", elementXml, "onchange", "" );
	}
	
	else if( elementXml._strName.equals( "script" ) == true )
	{
		element = new XGRPresentationModelElement( elementParent, elementXml._strName, XGR.SCRIPT );
		addAttributeToElement( element, "text", elementXml, "text", "" );
		addAttributeToElement( element, "type", elementXml, "type", "" );
	}
	
	else if( elementXml._strName.equals( "table" ) == true )
	{
		element = new XGRPresentationModelElement( elementParent, elementXml._strName, XGR.TABLELAYOUT );
		addAttributeToElement( element, "id", elementXml, "id", "" );
	}

	else if( elementXml._strName.equals( "td" ) == true )
	{
		element = new XGRPresentationModelElement( elementParent, elementXml._strName, XGR.TABLEDATA );
		addAttributeToElement( element, "colspan", elementXml, "colspan", "1" );
		addAttributeToElement( element, "rowspan", elementXml, "rowspan", "1" );
	}

	else if( elementXml._strName.equals( "th" ) == true )
	{
		element = new XGRPresentationModelElement( elementParent, elementXml._strName, XGR.TABLEHEADING );
		elementParent._iType = XGR.TABLE;
	}
	
	else if( elementXml._strName.equals( "textarea" ) == true )
	{
		element = new XGRPresentationModelElement( elementParent, elementXml._strName, XGR.TEXTAREA );
		addAttributeToElement( element, "id", elementXml, "id", "" );
		addAttributeToElement( element, "onblur"    , elementXml, "onblur"    , "" );
		addAttributeToElement( element, "onchange"  , elementXml, "onchange"  , "" );
		addAttributeToElement( element, "onfocus"   , elementXml, "onfocus"   , "" );
		addAttributeToElement( element, "onkeypress", elementXml, "onkeypress", "" );
	}
	
	else if( elementXml._strName.equals( "tr" ) == true )
	{
		element = new XGRPresentationModelElement( elementParent, elementXml._strName, XGR.TABLEROW );
	}

	// Add the new element to its parent.
	// Recursively create child elements
	//----------------------------------
	if( element != null )
	{
		// Add to parent element
		//----------------------
		if( elementParent != null )
		{
			elementParent.addChildElement( element );
		}
		
		// Recurse
		//--------
		int iNumberOfChildElements = elementXml.getNumberOfChildElements();

		for( int i=0; i<iNumberOfChildElements; ++i )
		{
			parseRecursive( element, elementXml.getChildElementAt( i ) );
		}	
	}

//	Trace.functionExit( this, "parseRecursive" );
}
}
