package org.eclipse.scripting.internal.uirenderer;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import java.util.Vector;

import org.eclipse.scripting.internal.IXGRElementUI;

public class XGRPresentationModelElement 
{
	public String                       _strName             = null;
	public Vector                       _vectorChildElements = null;
	public Vector                       _vectorAttributes    = null;
	public XGRPresentationModelElement  _elementParent       = null;
	public IXGRElementUI                _elementUI           = null;
	public int                          _iType               = 0;

/**
 */
public XGRPresentationModelElement( XGRPresentationModelElement elementParent, String strName, int iType )
{
	_elementParent = elementParent;
	_strName       = strName;
	_iType         = iType;
	
}
/**
 */
public void addAttribute( XGRPresentationModelAttribute parserAttribute ) 
{
	if( _vectorAttributes == null )
	{
		_vectorAttributes = new Vector();
	}

	_vectorAttributes.addElement( parserAttribute );
}
/**
 */
public void addChildElement( XGRPresentationModelElement parserElement ) 
{
	if( _vectorChildElements == null )
	{
		_vectorChildElements = new Vector();
	}

	_vectorChildElements.addElement( parserElement );
}
/**
 */
public XGRPresentationModelAttribute getAttribute( String strAttributeName ) 
{
	if( _vectorAttributes != null )
	{
		XGRPresentationModelAttribute attribute = null;
		
		for( int i=0; i<_vectorAttributes.size(); ++i )
		{
			attribute = (XGRPresentationModelAttribute)_vectorAttributes.elementAt( i );
			
			if(attribute._strName != null && attribute._strName.equals( strAttributeName ) == true )
			{
				return attribute;
			}
		}
	}

	return null;
}
/**
 */
public String getAttributeValue( String strAttributeName )
{
	XGRPresentationModelAttribute attribute = getAttribute( strAttributeName );

	if( attribute != null )
	{
		return attribute._strValue;
	}
	
	return null;
}
/**
 */
public XGRPresentationModelElement getChildElement( String strElementName )
{
	if( _vectorChildElements != null )
	{
		XGRPresentationModelElement elementChild = null;
		
		for( int i=0; i<_vectorChildElements.size(); ++i )
		{
			elementChild = (XGRPresentationModelElement)_vectorChildElements.elementAt( i );

			if( elementChild._strName != null && strElementName.equals( elementChild._strName ) == true )
			{
				return elementChild;
			}
		}
	}

	return null;
}
/**
 */
public XGRPresentationModelElement getChildElementAt( int iIndex )
{
	return _vectorChildElements != null ? (XGRPresentationModelElement)_vectorChildElements.elementAt( iIndex ) : null;
}
/**
 */
public int getNumberOfAttributes() 
{
	return _vectorAttributes != null ? _vectorAttributes.size() : 0;
}
/**
 */
public int getNumberOfChildElements() 
{
	return _vectorChildElements != null ? _vectorChildElements.size() : 0;
}
/**
 */
public XGRPresentationModelElement getParentElement()
{
	return _elementParent;
}
/**
 */
public void insertChildElementAt( XGRPresentationModelElement parserElement, int iIndex ) 
{
	if( _vectorChildElements == null )
	{
		_vectorChildElements = new Vector();
	}

	_vectorChildElements.insertElementAt( parserElement, iIndex );
}
/**
 * 
 */
public void printElementString() {}
/**
 * 
 */
public void printElementString( StringBuffer strb ) {

	printElementString(strb, 0);
}
/**
 * Prints the a single line of xml representing the element
 * and its attributes starting at this element.
 */
public void printElementString( StringBuffer strb, int iIndentation )
{
//	Trace.functionEntry( this, "printElementTree" );
	
	// Indentation
	//------------
	for( int i=0; i<iIndentation; ++i )
	{
		strb.append( ' ' );
	}

	// Begin element
	//--------------
	strb.append( '<' );
		
	if( _strName != null )
	{
		printElementStringName( strb, true );
	}
	else
	{
		strb.append( "undefined" );
	}

	// Print out my attributes and their values
	//-----------------------------------------
	if( _vectorAttributes != null && _vectorAttributes.size() > 0 )
	{
		for( int i=0; i<_vectorAttributes.size(); ++i )
		{
			strb.append( ' ' );
			((XGRPresentationModelAttribute)_vectorAttributes.elementAt( i )).printAttributeString( strb );
		}
	}
	
	// Get my children to print themselves
	//------------------------------------
	if( _vectorChildElements != null && _vectorChildElements.size() > 0 )
	{
		strb.append( ">\n\r" );
		
		for( int i=0; i<_vectorChildElements.size(); ++i )
		{
			((XGRPresentationModelElement)_vectorChildElements.elementAt( i )).printElementString( strb, iIndentation + 4 );
		}
		
//		strb.append( "\n\r" );

		// End element
		//------------
		for( int i=0; i<iIndentation; ++i )
		{
			strb.append( ' ' );
		}			

		strb.append( "</" );
		
		if( _strName != null )
		{
			printElementStringName( strb, false );
		}
		else
		{
			strb.append( "undefined" );
		}

		strb.append( ">\n\r" );
	}

	// End element without children
	//-----------------------------
	else
	{
		strb.append( " />\n\r" );
	}

//	Trace.functionExit( this, "printElementTree" );
}
/**
 */
public void printElementStringName(StringBuffer strb, boolean bBeginElement ) {

	switch( _iType )
	{
		case XGR.CHECK:
			strb.append( "input" );
			if( bBeginElement == true ) strb.append( " type=\"check\"" );
			break;

		case XGR.BUTTON:
			strb.append( "input" );
			if( bBeginElement == true ) strb.append( " type=\"button\"" );
			break;

		case XGR.RADIO:
			strb.append( "input" );
			if( bBeginElement == true ) strb.append( " type=\"radio\"" );
			break;
			
		case XGR.TEXTFIELD:
			strb.append( "input" );
			if( bBeginElement == true ) strb.append( " type=\"text\"" );
			break;

		case XGR.LIST:
			strb.append( "select" );
			if( bBeginElement == true ) strb.append( " size=\"3\"" );
			break;
			
		case XGR.COMBO:
			strb.append( "select" );
			if( bBeginElement == true ) strb.append( " size=\"1\"" );
			break;

		default:
			strb.append( _strName );
			break;
	}
}
/**
 * Debug function that prints the entire element tree, and each element's
 * attributes starting at this element.
 */
public void printElementTree()
{
//	Trace.functionEntry( this, "printElementTree" );
	
	printElementTree( 0 );

//	Trace.functionExit( this, "printElementTree" );
}
/**
 * Debug function that prints the entire element tree, and each element's
 * attributes starting at this element.
 */
public void printElementTree( int iIndentation )
{
//	Trace.functionEntry( this, "printElementTree" );
	
	// Print the name of this element
	//-------------------------------
	StringBuffer strb = new StringBuffer();

	for( int i=0; i<iIndentation; ++i )
	{
		strb.append( ' ' );
	}
	
	if( _strName != null )
	{
		strb.append( "<" + _strName + ">" );
	}
	else
	{
		strb.append( "<>" );
	}

	System.out.println( strb.toString() );

	// Print out my attributes
	//------------------------
	if( _vectorAttributes != null )
	{		
		for( int i=0; i<_vectorAttributes.size(); ++i )
		{
			((XGRPresentationModelAttribute)_vectorAttributes.elementAt( i )).printAttribute( iIndentation + 2 );
		}
	}

	// Get my children to print themselves
	//------------------------------------
	if( _vectorChildElements != null )
	{
		for( int i=0; i<_vectorChildElements.size(); ++i )
		{
			((XGRPresentationModelElement)_vectorChildElements.elementAt( i )).printElementTree( iIndentation + 4 );
		}
	}

//	Trace.functionExit( this, "printElementTree" );
}
/**
 */
public void removeChildElement( XGRPresentationModelElement elementChild )
{
	_vectorChildElements.removeElement( elementChild );
}
/**
 */
public String toString()
{
	return _strName != null ? _strName : new String();
}
}
