package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExecutableExtension;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.scripting.internal.ScriptPlugin;
import org.eclipse.scripting.internal.ScriptStrings;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.part.ViewPart;

public class ScriptConsole extends ViewPart implements IExecutableExtension
{
	protected ScriptConsoleViewer      _viewer      = null;
	protected ScriptConsoleClearAction _actionClear = null;
/**
 */
public ScriptConsole()
{
	super();
	setTitle( ScriptStrings.getString( "S_Script_Console" ) );
}
/**
 * Adds the clear button to the toolbar.
 */
private void addContributions() {
	
	// Create the toolbar clear button
	//--------------------------------
	_actionClear = new ScriptConsoleClearAction( _viewer, "clear" );
	_actionClear.setText( ScriptStrings.getString( "S_Clear" ) );
	_actionClear.setToolTipText( ScriptStrings.getString( "S_Clear" ) );
	_actionClear.setEnabled( true );
	
	ImageDescriptor imageDescriptor = ScriptPlugin.getImageDescriptor( "icons/full/clcl16/scriptcons_clear_edit.gif" );
	_actionClear.setImageDescriptor( imageDescriptor );
	
	// Add actions to the local tool bar
	//----------------------------------
	IToolBarManager tbm = getViewSite().getActionBars().getToolBarManager();    
	
	tbm.add(_actionClear);

	// Register with action service
	//-----------------------------
//	IActionService actionService = getSite().getActionBars().getActionService();
//	actionService.setGlobalActionHandler("clear", _actionClear);	
}
/**
 * Creates the SWT controls for a part.
 */
public void createPartControl(Composite compositeParent) {

	_viewer = new ScriptConsoleViewer( compositeParent );
	
	addContributions();
}
/**
 * Asks the part to take focus within the desktop.
 */
public void setFocus() {}
/**
 * This method just calls its super class' function.
 */
public void setInitializationData( IConfigurationElement configElement,
								   String                strName,
								   Object                objData )
{
//	Trace.functionEntry( this, "setInitializationData" );

	super.setInitializationData( configElement, strName, objData );
	
//	Trace.functionExit( this, "setInitializationData" );
}
}
