package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import org.eclipse.jface.viewers.ContentViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Text;

public class ScriptConsoleViewer extends ContentViewer
{
	protected static ScriptConsoleViewer _consoleInstance = null;

	protected Button       _buttonClear     = null;
	protected Composite    _compositeClient = null;
	protected Rectangle    _rectBounds      = null;
	protected StringBuffer _stringBuffer    = new StringBuffer();	
	protected Text         _textArea        = null;
/**
 * There is only one instance of the script console that can be obtained
 * by calling the static getInstance member function.
 */
public ScriptConsoleViewer( Composite compositeParent ) 
{
//	Trace.functionEntry( this, "ScriptConsole" );

	_consoleInstance = this;
	
	createControl( compositeParent );

//	Trace.functionExit( this, "ScriptConsole" );
}
/**
 */
public void clear() 
{
	// Clear button pressed
	//---------------------
	if( _stringBuffer != null )
	{
		_stringBuffer.setLength( 0 );
	}

	else if( _textArea != null )
	{
		_textArea.setText("");
	}
}
/**
 */
private Control createControl( Composite compositeParent )
{
//	Trace.functionEntry( this, "createWindow" );

	// Create a composite for my children
	//-----------------------------------
	_compositeClient = new Composite( compositeParent, SWT.NULL );
	
	// Create a grid layout
	// Reset its margin width (default is 5)
	//--------------------------------------
	GridLayout gridLayout = new GridLayout();
	gridLayout.marginWidth = 0;
	gridLayout.numColumns = 1;
	_compositeClient.setLayout( gridLayout );

//	Font fontFixed = new Font( compositeParent.getDisplay(), "courier", 8, SWT.NORMAL );

	// Text Area
	//----------
	_textArea = new Text( _compositeClient, SWT.MULTI | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL  | SWT.READ_ONLY );

//	_textArea.setFont( fontFixed );
	
	GridData gridData = new GridData();
		
	gridData.horizontalAlignment = GridData.FILL;
	gridData.verticalAlignment   = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.grabExcessVerticalSpace   = true;
		
	_textArea.setLayoutData( gridData );

	// Set the text from the string buffer
	// Remove the string buffer
	//------------------------------------
	if( _consoleInstance != null && _consoleInstance._stringBuffer != null )
	{
		_textArea.setText( _consoleInstance._stringBuffer.toString() );
	}

	// Set the instance to this instance
	//----------------------------------
	_stringBuffer = null;
	_consoleInstance = this;

//	Trace.functionExit( this, "createWindow" );	
	return _compositeClient;
}
/**
 * Returns the primary control associated with this viewer.
 */
public Control getControl() {
	return _compositeClient;
}
/**
 */
public static ScriptConsoleViewer getInstance() 
{
	// Return the one and only instance
	//---------------------------------
	return _consoleInstance;
}
public ISelection getSelection() {
	return StructuredSelection.EMPTY;
}
/**
 * Outputs the string to the scrip console text area
 */
public void println( String strText ) 
{
//	Trace.functionEntry( this, "println" );
	
	if( strText != null )
	{
		if(  _stringBuffer != null )
		{
			_stringBuffer.append( strText );
			_stringBuffer.append( "\n" );
		}

		else if( _textArea != null && _textArea.isDisposed() == false )
		{
			_textArea.append( strText );
			_textArea.append( "\n" );			
		}
	}
	
//	Trace.functionExit( this, "println" );
}
/**
 * Refreshes this viewer completely, getting new contents
 * for the input from the content provider.
 */
public void refresh() {}
/**
 */
public void setSelection(ISelection selection, boolean reveal) {}
}
