package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import org.eclipse.core.runtime.model.*;
import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.scripting.*;
import org.eclipse.scripting.internal.*;
import org.eclipse.jface.wizard.*;
import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import java.util.*;
import java.net.*;

public abstract class ScriptDevelopmentPreferencePageWorkbenchAbstractWizard extends ScriptAbstractPreferencePage {
	
	private ScriptDevelopmentWizardPageConfirmation   _wizardConfirmation   = null;
	private ScriptDevelopmentWizardPageFileName       _wizardFileName       = null;
	private ScriptDevelopmentWizardPageIconName       _wizardIconName       = null;
//	private ScriptDevelopmentWizardPageScriptLanguage _wizardLanguage       = null;
	
	private java.io.File                     _containerScript      = null;

	private static String _strExtensionPointID = "org.eclipse.ui.newWizards";
/**
 */
public void addScriptWizardPages(Wizard wizard, boolean newScript) {
	// Create the wizard pages
	//------------------------
	_wizardFileName = new ScriptDevelopmentWizardPageFileName(this, wizard, "File Name");
	_wizardIconName = new ScriptDevelopmentWizardPageIconName(this, wizard, "Icon Name");
	//	_wizardLanguage       = new ScriptDevelopmentWizardPageScriptLanguage( this, wizard, "Script Language" );

	// Add the wizard pages to the wizard
	//-----------------------------------
	wizard.addPage(_wizardIconName);
	//	wizard.addPage( _wizardLanguage       );
	wizard.addPage(_wizardFileName);

	// Initialize the pages
	//---------------------
	if (newScript == false) {
		ExtensionModel extensionModel = getExtensionModel();
		ConfigurationElementModel element = getConfigurationSubElement(extensionModel, "wizard");

		if (element != null) {

			_wizardIconName.setIconName(getPropertyValue(element, "icon"));

			String strProperty = getPropertyValue(element, "class");
			if (strProperty != null) {
				String strFile = _wizardFileName.getFileNameFromClassString(strProperty);
				_wizardFileName.setDefaultFileName(strFile);
			}

			strProperty = getPropertyValue(element, "icon");
			if (strProperty != null) {
				String strIcon = _wizardIconName.getIconNameFromIconString(strProperty);
				_wizardIconName.setIconName(strIcon);
			}
		}
	}
}
/**
 */
public abstract String getScriptAdapterName();
/**
 * Allows the preference page to modify a new or existing plugin extension.
 * This method is called when the add/modify wizard is completing.
 */
public void modifyPluginExtension(ExtensionModel extensionEntry, boolean newScript) {
	
	URL url = null;
	
	// Compute the icon file name
	//---------------------------
	String strIconFilename = null;

	if (_wizardIconName.isDefaultIcon() == true) {
		strIconFilename = _wizardIconName.getDefaultIconName();
	}
	else {
		try{
			url = new URL( "file", null, -1, _wizardIconName.getIconName());
			strIconFilename = url.toString();
		}
		catch( MalformedURLException ex ){
			strIconFilename = _wizardIconName.getDefaultIconName();
		}
	}

	// Compute script file name
	//-------------------------
	String strScriptFilename = null;

	try{
		url = new URL( "file", null, -1, _wizardFileName.getFileName());
		strScriptFilename = url.toString();
	}
	catch( MalformedURLException ex ){
		strScriptFilename = _wizardFileName.getFileName();
	}

		
	// Add action element and its attributes
	//--------------------------------------
	ConfigurationElementModel[] elementsExtension = new ConfigurationElementModel[1];
	
	elementsExtension[0] = createConfigurationElement( extensionEntry, "wizard" );
	
	ConfigurationPropertyModel[] properties = new ConfigurationPropertyModel[5];
	
	properties[0] = createConfigurationProperty("id", getContiguousName());
	properties[1] = createConfigurationProperty("name", getScriptName());
	properties[2] = createConfigurationProperty("category", ScriptStrings.getString("S_Scripts") );
	properties[3] = createConfigurationProperty("icon", strIconFilename);
	
	StringBuffer strbClass = new StringBuffer( getScriptAdapterName() );
	strbClass.append( ":" );
	strbClass.append( "ui=" );
	strbClass.append( strScriptFilename );
	strbClass.append( " uitype=wizard" );
	strbClass.append( " language=" );
//	strbClass.append( _wizardLanguage.getScriptLanguage() );
	strbClass.append( "javascript" );

	properties[4] = createConfigurationProperty("class", strbClass.toString() );
	
	elementsExtension[0].setProperties( properties );

	// Hook it all together
	//---------------------
	extensionEntry.setSubElements(elementsExtension);
}
/**
 */
public void syncScriptWizardPages()
{
	// Set the file extension in the script file name
	// Set the default icon name
	//-----------------------------------------------
	_wizardFileName.setScriptFileExtension( "xhtml" );
	
	// Update confirmation page
	//-------------------------
//	_wizardConfirmation.removeAllFiles();
	
//	_wizardConfirmation.addFile( _wizardFileName.getFileName() );
	
//	if (_wizardIconName.isDefaultIcon() == false) {
//		_wizardConfirmation.addFile(_wizardIconName.getIconName());
//	}
}
}
