package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import org.eclipse.core.runtime.model.*;
import org.eclipse.core.runtime.*;
import org.eclipse.core.resources.*;
import org.eclipse.scripting.*;
import org.eclipse.scripting.internal.ScriptConstants;
import org.eclipse.jface.*;
import org.eclipse.jface.wizard.*;
import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import java.net.*;
import java.util.*;

public class ScriptDevelopmentPreferencePageWorkbenchEditor extends ScriptAbstractPreferencePage {
	
	private Button  _buttonResourceTypeAdd    = null;
	private Button  _buttonResourceTypeRemove = null;
	private Table   _tableResourceType        = null;

	private ScriptDevelopmentWizardPageFileName       _wizardFileName       = null;
	private ScriptDevelopmentWizardPageIconName       _wizardIconName       = null;
	private ScriptDevelopmentWizardPageResourceType   _wizardResourceType   = null;
//	private ScriptDevelopmentWizardPageScriptLanguage _wizardLanguage       = null;

	private java.io.File                     _containerScript      = null;

	private static String _strAdapterID        = "org.eclipse.scripting.adapter.workbenchEditor";
	private static String _strExtensionPointID = "org.eclipse.ui.editors";
/**
 */
public void addScriptWizardPages(Wizard wizard, boolean newScript) {
	// Create the wizard pages
	//------------------------
	_wizardFileName = new ScriptDevelopmentWizardPageFileName(this, wizard, "File Name");
	_wizardIconName = new ScriptDevelopmentWizardPageIconName(this, wizard, "Icon Name");
	//	_wizardLanguage       = new ScriptDevelopmentWizardPageScriptLanguage( this, wizard, "Script Language" );
	_wizardResourceType = new ScriptDevelopmentWizardPageResourceType(this, wizard, "Resource Type");

	// Add the wizard pages to the wizard
	//-----------------------------------
	wizard.addPage(_wizardResourceType);
	//	wizard.addPage( _wizardLanguage     );
	wizard.addPage(_wizardIconName);
	wizard.addPage(_wizardFileName);

	// Initialize the pages
	//---------------------
	if (newScript == false) {
		ExtensionModel extensionModel = getExtensionModel();
		ConfigurationElementModel element = getConfigurationSubElement(extensionModel, "editor");

		if (element != null) {
			String strProperty = getPropertyValue(element, "class");
			if (strProperty != null) {
				String strFile = _wizardFileName.getFileNameFromClassString(strProperty);
				_wizardFileName.setDefaultFileName(strFile);
			}

			strProperty = getPropertyValue(element, "icon");
			if (strProperty != null) {
				String strIcon = _wizardIconName.getIconNameFromIconString(strProperty);
				_wizardIconName.setIconName(strIcon);
			}

			strProperty = getPropertyValue(element, "extensions");
			if (strProperty != null) {
				_wizardResourceType.setFileExtension(strProperty);
			}
		}
	}
}
/**
 * Returns the identification string of the plugin extension adapter.
 */
public String getAdapterID() {
	return _strAdapterID;
}
/**
 * Returns the extension point identifier for this script type.
 */
public String getExtensionPointID() {
	
	return _strExtensionPointID;
}
/**
 * Allows the preference page to modify a new or existing plugin extension.
 * This method is called when the add/modify wizard is completing.
 */
public void modifyPluginExtension(ExtensionModel extensionEntry, boolean newScript) {

	URL url = null;

	// Compute the icon file name
	//---------------------------
	String strIconFilename = null;

	if (_wizardIconName.isDefaultIcon() == true) {
		strIconFilename = _wizardIconName.getDefaultIconName();
	}
	else {
		try{
			url = new URL( "file", null, -1, _wizardIconName.getIconName());
			strIconFilename = url.toString();
		}
		catch( MalformedURLException ex ){
			strIconFilename = _wizardIconName.getDefaultIconName();
		}
	}

	// Compute script file name
	//-------------------------
	String strScriptFilename = null;

	try{
		url = new URL( "file", null, -1, _wizardFileName.getFileName());
		strScriptFilename = url.toString();
	}
	catch( MalformedURLException ex ){
		strScriptFilename = _wizardFileName.getFileName();
	}

	// Obtain the file extension string
	//---------------------------------
	String strFileExtension = _wizardResourceType.getSelectedResourceType();

	// editor
	//-------
	ConfigurationElementModel[] elementsExtension = new ConfigurationElementModel[1];

	elementsExtension[0] = createConfigurationElement(extensionEntry, "editor");

	ConfigurationPropertyModel[] properties = null;

	if (strFileExtension != null && strFileExtension.length() > 0)
		properties = new ConfigurationPropertyModel[5];
	else
		properties = new ConfigurationPropertyModel[4];

	properties[0] = createConfigurationProperty("id", getContiguousName());
	properties[1] = createConfigurationProperty("name", getScriptName());
	properties[2] = createConfigurationProperty("icon", strIconFilename);

	if (strFileExtension != null && strFileExtension.length() > 0)
		properties[3] = createConfigurationProperty("extensions", strFileExtension);

	// Class attribute
	//----------------
	StringBuffer strbClass = new StringBuffer();
	strbClass.append(ScriptConstants.getPluginID());
	strbClass.append("/");
	strbClass.append(ScriptAdapterWorkbenchResourceEditor.class.getName());
	strbClass.append(":");

	strbClass.append("ui=");
	strbClass.append(strScriptFilename);

	//	if (_wizardLanguage.getScriptLanguage() != null) {
	//		strbClass.append(" language=");
	//		strbClass.append(_wizardLanguage.getScriptLanguage());
	//	}

	strbClass.append(" language=javascript");
	
	if (strFileExtension != null && strFileExtension.length() > 0)
		properties[4] = createConfigurationProperty("class", strbClass.toString());
	else
		properties[3] = createConfigurationProperty("class", strbClass.toString());

	elementsExtension[0].setProperties(properties);

	// Hook it all together
	//---------------------
	extensionEntry.setSubElements(elementsExtension);
}
/**
 * Synchronizes the file name page with the action type page and the
 * script language page.  Sets the file extension on the file name page.
 */
public void syncScriptWizardPages() {
	
	// Set the file extension in the script file name
	// Set the default icon name
	//-----------------------------------------------
	_wizardFileName.setScriptFileExtension("xhtml");
}
}
