package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.scripting.ScriptAbstractPreferencePage;
import org.eclipse.scripting.internal.ScriptPlugin;
import org.eclipse.scripting.internal.ScriptStrings;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.dialogs.ContainerSelectionDialog;
import org.eclipse.ui.dialogs.ResourceSelectionDialog;


public class ScriptDevelopmentWizard extends Wizard
{
	protected boolean                 _bNewScript         = true;
	protected IConfigurationElement   _configuration      = null;
	protected IWorkbench              _workbench          = null;
	protected IStructuredSelection    _selection          = null;

	protected ScriptAbstractPreferencePage _preferencePage = null;

//	protected ScriptDevelopmentWizardPageConfirmation  _pageConfirmation   = null;
//	protected ScriptDevelopmentWizardPageContainerName _pageContainerName  = null;
	protected ScriptDevelopmentWizardPageScriptName    _pageScriptName     = null;
//	protected ScriptDevelopmentWizardPageWarning       _pageWarning        = null;
/**
 */
public ScriptDevelopmentWizard(ScriptAbstractPreferencePage preferencePage, boolean bNewScript)
{
	super();

	ImageDescriptor imageDescriptor = ScriptPlugin.getImageDescriptor( "icons/full/wizban/script_wiz.gif" );
	setDefaultPageImageDescriptor( imageDescriptor );

	_bNewScript = bNewScript;
	_preferencePage = preferencePage;

	if (bNewScript == true)
	{
		setWindowTitle( ScriptStrings.getString( "S_New_Script") );
	}

	else
	{
		setWindowTitle( ScriptStrings.getString( "S_Modify_Script" ) );
	}
}
/**
 * 
 */
public void addPages() {

	// Create the wizard pages
	//------------------------
	_pageScriptName = new ScriptDevelopmentWizardPageScriptName(_preferencePage, this, "Script Name", _bNewScript );
	
	// Script Name page
	//-----------------
	addPage(_pageScriptName);

	// Script type specific wizard pages
	//----------------------------------
	_preferencePage.addScriptWizardPages(this, _bNewScript);

	// Initialize pages
	//-----------------
	if (_bNewScript == false) {
		_pageScriptName.setDefaultScriptName(_preferencePage.getScriptName());
	}

	_pageScriptName.setScriptNames( _preferencePage.getScriptNames());
}
/**
 * Gives an opportunity to the preference page to synchronize its wizard pages
 */
public IWizardPage getNextPage(WizardPage page) {

	_preferencePage.syncScriptWizardPages();

	return super.getNextPage(page);
}
/**
 * init method comment.
 */
public Wizard init( IWorkbench workbench, IStructuredSelection selection )
{
	_workbench = workbench;
	_selection = selection;
	
	return this;
}
/**
 * Lets the preference page know that the wizard has completed.
 * @return boolean
 */
public boolean performFinish()
{
	// Tell the preference page that we're finished
	//---------------------------------------------
	_preferencePage.wizardComplete( _bNewScript );
	
	// Let the wizard know that the dialog can be closed
	//--------------------------------------------------
	return true;
}
/**
 * Displays the container selection dialog.
 * The dialog will be displayed with the given IFolder as the root.
 * A single
 * IFolder is returned.  If the folderRoot parameter is null, the
 * open solution is used as the root folder.
 */
public static IContainer promptForWorkspaceContainer(Shell shell, IContainer containerRoot, String strMessage) throws Exception {
	IContainer containerSelected = null;

	try {
		if (containerRoot == null) {
			IWorkspace workspace = ResourcesPlugin.getWorkspace();
			containerRoot = workspace.getRoot();
		}

		// Create the container selection dialog and wait for a result
		// If container is null then the dialog will show all projects
		//------------------------------------------------------------
		ContainerSelectionDialog dialog = new ContainerSelectionDialog(shell, containerRoot, false, strMessage);

		// Create a new shell, otherwise
		// the controls will not be active
		//--------------------------------
		dialog.setBlockOnOpen(true);
		dialog.open();

		// Obtain the list of selected containers
		//---------------------------------------
		Object[] objSelected = dialog.getResult();

		// Select the first container
		//---------------------------
		if (objSelected.length > 0) {
			if (objSelected[0] instanceof IPath) {
				IWorkspace workspace = ResourcesPlugin.getWorkspace();
				containerSelected = (IContainer) workspace.getRoot().findMember(((IPath) objSelected[0]).toString());
			}
		}
	}

	// Catch and rethrow all exceptions
	//---------------------------------
	catch (Exception ex) {
		containerSelected = null;
		throw ex;
	}

	return containerSelected;
}
/**
 * Displays the resource selection dialog.  An array
 * of IResources is returned.
 */
public static IFile[] promptForWorkspaceFiles(Shell shell, IContainer containerRoot, String strMessage) throws Exception {

	IFile[] fileArray = null;

	try {
		if (containerRoot == null) {
			IWorkspace workspace = ResourcesPlugin.getWorkspace();
			containerRoot = workspace.getRoot();
		}
		
		// Show the resource selection dialog with folderRoot as the root
		//---------------------------------------------------------------
		ResourceSelectionDialog dialog = new ResourceSelectionDialog(shell, containerRoot, strMessage);

		// Create a new shell, otherwise
		// the controls will not be active
		//--------------------------------
		dialog.open();

		// Store the selected resource vector elements into an array
		//----------------------------------------------------------
		Object[] objResources = dialog.getResult();

		// Count the number of files
		// (They currently are all files,
		//  but it's a weird design returning a vector of IResource)
		//----------------------------------------------------------
		if (objResources != null) {
			int iNumberOfFiles = 0;

			for (int i = 0; i < objResources.length; ++i) {
				if (objResources[i] instanceof IFile) {
					++iNumberOfFiles;
				}
			}

			fileArray = new IFile[iNumberOfFiles];

			for (int i = 0; i < objResources.length; ++i) {
				if (objResources[i] instanceof IFile) {
					fileArray[i] = (IFile) objResources[i];
				}
			}
		}
	}

	// Catch and rethrow all exceptions
	//---------------------------------
	catch (Exception ex) {
		throw ex;
	}

	finally {
		if (fileArray == null) {
			fileArray = new IFile[0];
		}
	}

	return fileArray;
}
}
