package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * A preference page that asks for an action type of
 * <ul>
 * <li>no user interface, just a script file</li>
 * <li>wizard</li>
 * <li>preferences</li>
 * <li>panel</li>
 * </ul>
 */
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.scripting.ScriptAbstractPreferencePage;
import org.eclipse.scripting.internal.ScriptStrings;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Listener;

public class ScriptDevelopmentWizardPageActionType extends WizardPage implements Listener
{
	// The items in the list must be in the same order
	// as this enumerated type.  Undefined indicates
	// no user interface, only a script file.
	//------------------------------------------------
	public static final int undefined   = -1;
	public static final int wizard      =  0;
	public static final int preferences =  1;

	protected int    _iDefaultUserInterfaceType = undefined;
	protected Button _radioExecuteScript        = null;
	protected Button _radioPresentUserInterface = null;
	protected List   _listUserInterfaces        = null;
	protected Wizard _wizard                    = null;



/**
 */
public ScriptDevelopmentWizardPageActionType( ScriptAbstractPreferencePage preferencePage, Wizard wizard, String strName )
{
	super( strName );
	_wizard = wizard;
	setTitle( strName );
	setDescription( ScriptStrings.getString( "S_Select_the_type_of_action" ) );
}
/**
 */
public void createControl( Composite compositeParent )
{
	// Content
	//--------
	Composite compositeContent = new Composite( compositeParent, SWT.NULL );

	GridLayout layout= new GridLayout();
	compositeContent.setLayout( layout );
	
	GridData gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.verticalAlignment   = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.grabExcessVerticalSpace   = true;
	compositeContent.setLayoutData( gridData );
	
	// Radio Button: Execute script from a file
	//-----------------------------------------
	_radioExecuteScript = new Button( compositeContent, SWT.RADIO );
	_radioExecuteScript.setText( ScriptStrings.getString( "S_Execute_a_script_from_a_file" ) );
	_radioExecuteScript.setSelection( true );
	
	gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	_radioExecuteScript.setLayoutData( gridData );


	// Radio Button: Present User Interface
	//-------------------------------------	
	_radioPresentUserInterface = new Button( compositeContent, SWT.RADIO );
	_radioPresentUserInterface.setText( ScriptStrings.getString( "S_Present_a_user_interface" ) );
	_radioPresentUserInterface.setSelection( false );

	gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	_radioPresentUserInterface.setLayoutData( gridData );

		
	// List: User Interfaces
	//----------------------
	_listUserInterfaces = new List( compositeContent, SWT.BORDER );
	
	gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.verticalAlignment   = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.grabExcessVerticalSpace   = true;
	gridData.horizontalIndent = 16;
	_listUserInterfaces.setLayoutData( gridData );
	
	// Initialize the list
	// These must be added in the same order as the enumeration
	//---------------------------------------------------------
	_listUserInterfaces.add( ScriptStrings.getString( "S_Wizard_dialog" ) );
	_listUserInterfaces.add( ScriptStrings.getString( "S_Preference_dialog" ) );

	_listUserInterfaces.select( 0 );
	_listUserInterfaces.setEnabled( false );
	
	// Initialize
	//-----------
	if( _iDefaultUserInterfaceType == undefined ) {
		_radioExecuteScript.setSelection( true );
	}
	
	else {
		_radioPresentUserInterface.setSelection( true );
		_listUserInterfaces.setSelection( _iDefaultUserInterfaceType );
	}
	
	// Add listeners
	//--------------
	_radioExecuteScript       .addListener( SWT.Selection, this );
	_radioPresentUserInterface.addListener( SWT.Selection, this );

	setControl( compositeContent );
}
/**
 * Returns the type of user interface as defined by the enumerated type.
 * A value of undefined indicates no user interface, script file only.
 */
public int getUserInterfaceType() {

	// Determine user interface type
	//------------------------------
	if (_radioPresentUserInterface != null && _radioPresentUserInterface.getSelection() == true) {
		return _listUserInterfaces.getSelectionIndex();
	}

	return undefined;
}
/**
 */
public void handleEvent( Event event ) {

	// Radio button: Script file only radio button
	//--------------------------------------------
	if (event.widget == _radioExecuteScript) {
		
		// Enable / disable list box
		//--------------------------
		_listUserInterfaces.setEnabled(_radioPresentUserInterface.getSelection());
	} 

	// Radio button: User interface file with scripts in it
	//-----------------------------------------------------
	else if (event.widget == _radioPresentUserInterface) {
		
		// Enable / disable list box
		//--------------------------
		_listUserInterfaces.setEnabled( _radioPresentUserInterface.getSelection() );
	}
}
/**
 */
public void setDefaultUserInterfaceType(int iType) {
}
}
