package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.scripting.ScriptAbstractPreferencePage;
import org.eclipse.scripting.internal.ScriptStrings;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;

public class ScriptDevelopmentWizardPageFileName extends WizardPage implements Listener
{
	protected Button _buttonBrowseContainer       = null;
	protected Button _buttonBrowseFile            = null;	
//	protected Label  _labelFileExtension = null;
	protected Text   _textContainer      = null;
	protected Text   _textFile           = null;
	protected String _strFileName        = null;
	protected String _strDefaultFilename = null;
	protected IContainer _container = null;
/**
 */
public ScriptDevelopmentWizardPageFileName( ScriptAbstractPreferencePage preferencePage, Wizard wizard, String strName )
{
	super( strName );
	setTitle( strName );
	setDescription( ScriptStrings.getString("S_Select_a_file_for_this_script") );
}
/**
 */
public void createControl(Composite compositeParent) {
	// Content
	//--------
	Composite compositeContent = new Composite(compositeParent, SWT.NULL);

	GridLayout layout = new GridLayout();
	layout.numColumns = 3;
	compositeContent.setLayout(layout);

	GridData gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.verticalAlignment = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.grabExcessVerticalSpace = true;
	compositeContent.setLayoutData(gridData);

	// Label: Container
	//-----------------
	Label labelContainer = new Label(compositeContent, SWT.NULL);
	labelContainer.setText(ScriptStrings.getString("S_Container") + ":");

	// Text: Container
	//----------------
	_textContainer = new Text(compositeContent, SWT.BORDER);

	gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.widthHint = 200;
	_textContainer.setLayoutData(gridData);

	// Button: Browse
	//---------------
	_buttonBrowseContainer = new Button(compositeContent, SWT.PUSH);
	_buttonBrowseContainer.setText(ScriptStrings.getString("S_Browse"));

	// Label: File name
	//-----------------
	Label labelFile = new Label(compositeContent, SWT.NULL);
	labelFile.setText(ScriptStrings.getString("S_File") + ":");

	// Text: File name
	//----------------
	_textFile = new Text(compositeContent, SWT.BORDER);

	gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.widthHint = 200;
	_textFile.setLayoutData(gridData);

	// Label: File extension
	//----------------------
	//	_labelFileExtension = new Label( compositeContent, SWT.NULL );
	//	_labelFileExtension.setText( ".html" );

	// Button: Browse
	//---------------
	_buttonBrowseFile = new Button(compositeContent, SWT.PUSH);
	_buttonBrowseFile.setText(ScriptStrings.getString("S_Browse"));

	// Initialize
	//-----------
	if (_strDefaultFilename != null) {
		_textFile.setText(_strDefaultFilename);
	}

	setPageComplete(false);

	// Add listeners
	//--------------
	_textFile.addListener(SWT.Modify, this);
	_textContainer.addListener(SWT.Modify, this);
	_buttonBrowseFile.addListener(SWT.Selection, this);
	_buttonBrowseContainer.addListener(SWT.Selection, this);

	setControl(compositeContent);
}
/**
 */
public String getFileName() {
	
	StringBuffer strbufFilename = new StringBuffer();

	// Append the base file name
	//--------------------------
	if (_textFile != null ) {

		strbufFilename.append(_textFile.isDisposed() ? _strFileName : _textFile.getText());
	}

	return strbufFilename.toString();
}
/**
 */
public String getFileNameFromClassString(String strClass) {

	String strFile = null;
	
	if (strClass != null) {
		
		// Look for ui
		//------------
		int iIndex = strClass.indexOf("ui=");
		if (iIndex >= 0)
			strFile = strClass.substring(iIndex + 3);

		// Look for file
		//--------------
		else{
			iIndex = strClass.indexOf("file=");
			if (iIndex >= 0)
				strFile = strClass.substring(iIndex + 5);
		}

		// Find the end
		//-------------
		if (strFile != null) {
			// Look for next parameter's = sign
			//---------------------------------
			iIndex = strFile.indexOf("=");

			if (iIndex > 0) {

				// Look for previous space
				//------------------------
				int iIndexSpace = strFile.lastIndexOf(' ', iIndex);

				if (iIndexSpace > 0) {
					strFile = strFile.substring(0, iIndexSpace);
				}
			}
			
			URL url = null;
			try{
				url = new URL( strFile );
				strFile = url.getFile();
			}
			catch( MalformedURLException ex ){
				strFile = null;
			}
		}
	}

	return strFile;
}
/**
 */
public void handleEvent(Event event) {

	// Modify event
	//-------------
	if (event.widget == _textFile || event.widget == _textContainer) {
		updatePageComplete();
	}

	// Selection event
	//----------------

	else if (event.widget == _buttonBrowseContainer) {

		try {
			_container = ScriptDevelopmentWizard.promptForWorkspaceContainer(getShell(), null, "Select container");

			if (_container != null) {
				// Place the string in the entry field
				//------------------------------------
				_textContainer.setText(_container.getFullPath().toString());
			}

			updatePageComplete();
		}

		// Catch all exceptions
		//---------------------
		catch (Exception ex) {
		}
	}

	else if (event.widget == _buttonBrowseFile) {

		try {
			IFile[] files = ScriptDevelopmentWizard.promptForWorkspaceFiles(getShell(), _container, "Select file");

			if (files != null && files.length > 0) {
				// Place the string in the entry field
				//------------------------------------
				_textFile.setText(files[0].getLocation().toString());
			}
		}

		// Catch all exceptions
		//---------------------
		catch (Exception ex) {
		}
	}
}
/**
 */
public void setDefaultFileName(String strName) {
	
	_strDefaultFilename = strName;
}
/**
 * Sets the text to be displayed as the default file extension.
 */
public void setScriptFileExtension(String strFileExtension) {

//	if( _labelFileExtension != null ){
//		_labelFileExtension.setText( "." + strFileExtension );
//	}
}
/**
 */
public void setVisible(boolean bVisible) {

	// When first in this page, or coming back to this page
	// see if the file name is ok
	//-----------------------------------------------------
	if (bVisible == true) {
		updatePageComplete();
	}

	// Call the super class, otherwise, nothing happens
	//-------------------------------------------------
	super.setVisible( bVisible );
}
/**
 */
protected void updatePageComplete() {

	// Browse button
	//--------------
	_buttonBrowseFile.setEnabled(false);

	IWorkspace workspace = ResourcesPlugin.getWorkspace();
	String pathString = _textContainer.getText();

	IStatus result = workspace.validatePath(pathString, IResource.PROJECT | IResource.FOLDER | IResource.FILE);

	if (result.isOK() && workspace.getRoot().exists(new Path(pathString))) {
		IResource resource = workspace.getRoot().findMember(new Path(pathString));
		if (resource instanceof IFile) {
			resource = resource.getParent();
		}
		if (resource instanceof IContainer) {
			_container = (IContainer) resource;
			_buttonBrowseFile.setEnabled(true);
		}
	}

	// Page complete
	//--------------
	if (_textFile.getText().length() > 0) {

		try {
			URL url = new URL("file:/" + _textFile.getText());
			setPageComplete(true);
		}
		catch (java.net.MalformedURLException ex) {
			setPageComplete(false);
		}

	}
	else {
		setPageComplete(false);
	}
}
}
