package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.net.URL;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IPluginDescriptor;
import org.eclipse.core.runtime.IPluginRegistry;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.scripting.ScriptAbstractPreferencePage;
import org.eclipse.scripting.internal.ScriptConstants;
import org.eclipse.scripting.internal.ScriptStrings;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;

public class ScriptDevelopmentWizardPageIconName extends WizardPage implements Listener
{
	protected Button _buttonBrowseContainer       = null;
	protected Button _buttonBrowseFile            = null;	
	protected Text   _textContainer      = null;
	protected Text   _textFile           = null;
	protected Button _checkboxDefaultIcon = null;
	protected String _strIconName         = new String();
	protected String _strDefaultIconName  = null;
	protected IContainer _container = null;

	protected static final String STRING_DEFAULT_ICON_FILENAME = "icons/full/etool16/script_scp.gif";
/**
 */
public ScriptDevelopmentWizardPageIconName( ScriptAbstractPreferencePage preferencePage, Wizard wizard, String strName )
{
	super( strName );
	setTitle( strName );
	setDescription(ScriptStrings.getString("S_Select_an_icon_for_this_script"));
}
/**
 * Takes a filename consisting of /project/filename and converts it
 * to a path relative to where the file is actually stored in the
 * workspace.  This is used to specify an icon that resides in the
 * workspace in a plugin file.
 */
public String convertFilenameToWorkspaceRelativePath(String strFilename) {

	// This will be replaced by URL code
	//----------------------------------
	
	// Obtain the full absolute path of the resource
	//----------------------------------------------
	IPath pathResource = new Path( strFilename );

	IWorkspace workspace = ResourcesPlugin.getWorkspace();

	IFile fileResource = workspace.getRoot().getFile( pathResource );

	pathResource = fileResource.getLocation();

	// Obtain the full absolute path of the user scripts plugin
	//---------------------------------------------------------
	IPluginRegistry registry = org.eclipse.core.runtime.Platform.getPluginRegistry();

	IPluginDescriptor descriptorUserScripts = registry.getPluginDescriptor( "com.ibm.eclipse.internal.core.scripting.userscripts");

	URL urlUserScriptsPluginPath = descriptorUserScripts.getInstallURL();

	String strUserScriptsPluginPath = urlUserScriptsPluginPath.getFile();
	
	IPath pathUserScriptsPlugin = new Path( strUserScriptsPluginPath );

	// Determine where the paths match
	//--------------------------------
	int iNumberOfSegments = pathUserScriptsPlugin.segmentCount();

	StringBuffer strbPath = new StringBuffer();
	
	for( int i=0; i<iNumberOfSegments; ++i )
	{
		strbPath.append( "../" );
	}

	String strSegmentOne = pathResource.segment(0);

	String strResource = pathResource.toString();
		
	int iIndexSegmentOne = strResource.indexOf( strSegmentOne );

	strbPath.append( strResource.substring( iIndexSegmentOne ) );
	
	return strResource.substring( iIndexSegmentOne );
}
/**
 */
public void createControl(Composite compositeParent) {

	// Content
	//--------
	Composite compositeContent = new Composite(compositeParent, SWT.NULL);

	GridLayout layout = new GridLayout();
	layout.numColumns = 3;
	compositeContent.setLayout(layout);

	GridData gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.verticalAlignment = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.grabExcessVerticalSpace = true;
	compositeContent.setLayoutData(gridData);

	// Checkbox: Default icon
	//-----------------------
	_checkboxDefaultIcon = new Button(compositeContent, SWT.CHECK);
	_checkboxDefaultIcon.setText(ScriptStrings.getString("S_Use_default_icon"));

	gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.horizontalSpan = 3;
	_checkboxDefaultIcon.setLayoutData(gridData);

	// Label: Container
	//-----------------
	Label labelContainer = new Label(compositeContent, SWT.NULL);
	labelContainer.setText(ScriptStrings.getString("S_Container") + ":");

	// Text: Container
	//----------------
	_textContainer = new Text(compositeContent, SWT.BORDER);

	gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.widthHint = 200;
	_textContainer.setLayoutData(gridData);

	// Button: Browse
	//---------------
	_buttonBrowseContainer = new Button(compositeContent, SWT.PUSH);
	_buttonBrowseContainer.setText(ScriptStrings.getString("S_Browse"));

	// Label: File name
	//-----------------
	Label labelFile = new Label(compositeContent, SWT.NULL);
	labelFile.setText(ScriptStrings.getString("S_File") + ":");

	// Text: File name
	//----------------
	_textFile = new Text(compositeContent, SWT.BORDER);

	gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.widthHint = 200;
	_textFile.setLayoutData(gridData);

	// Button: Browse
	//---------------
	_buttonBrowseFile = new Button(compositeContent, SWT.PUSH);
	_buttonBrowseFile.setText(ScriptStrings.getString("S_Browse"));

	// Initialize
	//-----------
	String strIconName = getIconNameFromIconString(_strIconName);

	if (strIconName != null && strIconName.length() > 0) {
		_textFile.setText(strIconName);
		_checkboxDefaultIcon.setSelection(false);
		_textFile.setEnabled(true);
		_buttonBrowseFile.setEnabled(true);
		_textContainer.setEnabled(true);
		_buttonBrowseContainer.setEnabled(true);
	}
	else {
		_checkboxDefaultIcon.setSelection(true);
		_textFile.setEnabled(false);
		_buttonBrowseFile.setEnabled(false);
		_textContainer.setEnabled(false);
		_buttonBrowseContainer.setEnabled(false);
	}

	updatePageComplete();

	// Add listeners
	//--------------
	_buttonBrowseFile.addListener(SWT.Selection, this);
	_buttonBrowseContainer.addListener(SWT.Selection, this);
	_checkboxDefaultIcon.addListener(SWT.Selection, this);
	_textFile.addListener(SWT.Modify, this);
	_textContainer.addListener(SWT.Modify, this);

	setControl(compositeContent);
}
/**
 * Returns the default icon filename.  This file is installed in the same
 * folder as the plugin definition file.
 */
public String getDefaultIconName() {

	IPluginRegistry registry = org.eclipse.core.runtime.Platform.getPluginRegistry();

	IPluginDescriptor descriptorUserScripts = registry.getPluginDescriptor( ScriptConstants.getPluginID() );

	URL urlScriptsPluginPath = descriptorUserScripts.getInstallURL();

	String strFile = urlScriptsPluginPath.toExternalForm() + STRING_DEFAULT_ICON_FILENAME;
	
	return strFile;
}
/**
 * Returns the current icon name.
 */
public String getIconName() {
	
	return _textFile.isDisposed() ? _strIconName : _textFile.getText();
}
/**
 */
public String getIconNameFromIconString(String strIcon) {

	if (strIcon != null) {
	    
		if (strIcon.indexOf("file:") == 0) {
	        
			// Eliminate the "file:/" url at the beginning
			//--------------------------------------------
			if (strIcon.length() > 6) {

				strIcon = strIcon.substring(6);
			}
		}
		else
			strIcon = new String();
	}
	return strIcon;
}
/**
 */
public void handleEvent(Event event) {

	// Modify event
	//-------------
	if (event.widget == _textFile || event.widget == _textContainer) {
		updatePageComplete();
	}

	// Selection event
	//----------------

	else if (event.widget == _buttonBrowseContainer) {

		try {
			_container = ScriptDevelopmentWizard.promptForWorkspaceContainer(getShell(), null, "Select container");

			if (_container != null) {
				// Place the string in the entry field
				//------------------------------------
				_textContainer.setText(_container.getName());
			}

			updatePageComplete();
		}

		// Catch all exceptions
		//---------------------
		catch (Exception ex) {
		}
	}

	else if (event.widget == _buttonBrowseFile) {

		try {
			IFile[] files = ScriptDevelopmentWizard.promptForWorkspaceFiles(getShell(), _container, "Select file");

			if (files != null && files.length > 0) {
				// Place the string in the entry field
				//------------------------------------
				_textFile.setText(files[0].getLocation().toString());
			}
		}

		// Catch all exceptions
		//---------------------
		catch (Exception ex) {
		}
	}

	else if (event.widget == _checkboxDefaultIcon) {

		if (_checkboxDefaultIcon.getSelection() == true) {
			_textFile.setEnabled(false);
			_buttonBrowseFile.setEnabled(false);
			_textContainer.setEnabled(false);
			_buttonBrowseContainer.setEnabled(false);
		}
		else {
			_textFile.setEnabled(true);
			_buttonBrowseFile.setEnabled(true);
			_textContainer.setEnabled(true);
			_buttonBrowseContainer.setEnabled(true);

		}

		updatePageComplete();
	}

	return;
}
/**
 */
public boolean isDefaultIcon() {

	if (_checkboxDefaultIcon != null) {
		return _checkboxDefaultIcon.getSelection() == true;
	}

	return true;
}
/**
 * Sets the default icon name
 */
public void setDefaultIconName( String strIconName ) {
	_strDefaultIconName = strIconName;

	return;
}
/**
 */
public void setIconName(String strIconName) {
	_strIconName = strIconName;
	return;
}
/**
 */
public void setVisible(boolean bVisible) {

	if (bVisible == true) {

		// When first in this page, or coming back to this page
		// see if the file name is ok
		//-----------------------------------------------------
		updatePageComplete();
	}

	// Call the super class, otherwise, nothing happens
	//-------------------------------------------------
	super.setVisible( bVisible );

	return;
}
/**
 */
protected void updatePageComplete() {

	// Check for default icon
	//-----------------------
	if (_checkboxDefaultIcon.getSelection() == true) {
		_textFile.setEnabled(false);
		_buttonBrowseFile.setEnabled(false);
		_textContainer.setEnabled(false);
		_buttonBrowseContainer.setEnabled(false);

		setPageComplete(true);
		return;
	}

	// Not default icon
	//-----------------
	else {
		_textContainer.setEnabled(true);
		_buttonBrowseContainer.setEnabled(true);
		_textFile.setEnabled(true);
		_buttonBrowseFile.setEnabled(_textContainer.getText().length() != 0);

		if (_textFile.getText().length() == 0) {
			setPageComplete(false);
		}
		else {
			setPageComplete(true);
		}
	}

	return;
}
}
