package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * This class is not currently used.
 * A preference page that lists the currently available script languages
 * and asks the user to select one.
 */
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.scripting.ScriptAbstractPreferencePage;
import org.eclipse.scripting.internal.ScriptLanguageDescriptor;
import org.eclipse.scripting.internal.ScriptPlugin;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Listener;

public class ScriptDevelopmentWizardPageScriptLanguage extends WizardPage implements Listener
{
	protected Wizard _wizard                         = null;
	protected List   _listLanguages                  = null;
	protected String _strDefaultScriptLanguage       = null;
	protected String _strScriptLanguage              = "javascript";
	protected String _strScriptLanguageFileExtension = "js";
	protected ScriptAbstractPreferencePage _preferencePage = null;
/**
 */
public ScriptDevelopmentWizardPageScriptLanguage( ScriptAbstractPreferencePage preferencePage, Wizard wizard, String strName )
{
	super( strName );
	_wizard = wizard;
	_preferencePage = preferencePage;
	setTitle( strName );
	setDescription("S_Select_a_script_language_to_use");
}
/**
 * Presents a list of plugged in script languages.  These are
 * extensions of the script language extension point.
 */
public void createControl( Composite compositeParent )
{
	// Content
	//--------
	Composite compositeContent = new Composite( compositeParent, SWT.NULL );

	GridLayout layout= new GridLayout();
	compositeContent.setLayout( layout );
	
	GridData gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.verticalAlignment   = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.grabExcessVerticalSpace   = true;
	compositeContent.setLayoutData( gridData );
	
	// List
	//-----
	_listLanguages = new List( compositeContent, SWT.BORDER );
	
	gridData = new GridData();
	gridData.horizontalAlignment       = GridData.FILL;
	gridData.verticalAlignment         = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.grabExcessVerticalSpace   = true;
	_listLanguages.setLayoutData( gridData );

	// Initialize the list with script engine extensions
	//--------------------------------------------------
	ScriptPlugin scriptPlugin = ScriptPlugin.getPluginInstance();

	// Activate the plugin if not activated
	//-------------------------------------
	if( scriptPlugin == null ) {
		scriptPlugin = ScriptPlugin.bootstrap();
	}

	if( scriptPlugin != null )	{
		
		String[] straScriptLanguages = scriptPlugin.getScriptLanguageNames();

		for( int i=0; i<straScriptLanguages.length; ++i )
		{
			_listLanguages.add( straScriptLanguages[i] );
		}

		if( _listLanguages.getItemCount() > 0 )
		{
			// Select the first language
			//--------------------------
			_listLanguages.select( 0 );

			// Initialize the selected script language file extension
			//-------------------------------------------------------
			determineScriptLanguageFileExtension();
		}
	}

	// Add listeners
	//--------------
	_listLanguages.addListener( SWT.Selection, this );
	
	setControl( compositeContent );
}
/**
 * The list contains a list of script language
 * names.  A set of script descriptors is obtained in order to find the file
 * extension of the script type.
 */
public void determineScriptLanguageFileExtension() {
	
	// Item of list of languages has been selected
	// Store the selected script language string
	// Store the selected script language file extension
	//--------------------------------------------------
	String[] straSelected = _listLanguages.getSelection();
	
	if( straSelected != null && straSelected.length > 0 ) {

		// Only one can be selected at a time
		//-----------------------------------
		_strScriptLanguage = straSelected[0];

		// Obtain the single script plugin instance
		//-----------------------------------------
		ScriptPlugin scriptPlugin = ScriptPlugin.getPluginInstance();

		// Activate the plugin if not activated
		//-------------------------------------
		if( scriptPlugin == null ) {
			scriptPlugin = ScriptPlugin.bootstrap();
		}
		
		if( scriptPlugin != null ) {
			
			// Obtain the list of script descriptors
			//--------------------------------------
			ScriptLanguageDescriptor[] descriptors = scriptPlugin.getScriptLanguageDescriptors();

			// Find the script descriptor with the selected name
			//--------------------------------------------------
			for( int i=0; i<descriptors.length; ++i ) {
				
				if( descriptors[i].getScriptLanguage().equals( _strScriptLanguage ) == true ) {

					// Store the file extension
					//-------------------------
					_strScriptLanguageFileExtension = descriptors[i].getFileExtension();
					break;
				}
			}
		}
	}
}
/**
 * Returns the selected script language.
 */
public String getScriptLanguage() {
	
	return _strScriptLanguage;
}
/**
 * Returns the selected script language.
 */
public String getScriptLanguageFileExtension() {
	
	return _strScriptLanguageFileExtension;
}
/**
 * Handles the list selection event.  The list contains a list of script language
 * names.  A set of script descriptors is obtained in order to find the file
 * extension of the script type.
 */
public void handleEvent(Event event) {

	// List selection event
	//---------------------
	determineScriptLanguageFileExtension();
}
/**
 * Sets the default script language to be selected at initialization.
 */
public void setDefaultScriptLanguage(String strScriptLanguage) {
	_strDefaultScriptLanguage = strScriptLanguage;
}
}
