package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * A preference page that asks for a descriptive name for a script.
 */
import java.util.List;

import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.scripting.ScriptAbstractPreferencePage;
import org.eclipse.scripting.internal.ScriptStrings;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;

public class ScriptDevelopmentWizardPageScriptName extends WizardPage implements Listener
{
	protected ScriptAbstractPreferencePage _preferencePage = null;
	protected Text   _textScriptName  = null;
	protected String _strDefaultScriptName = null;
	protected List    _listScriptNames = null;
	protected boolean _bNewScript = false;
/**
 */
public ScriptDevelopmentWizardPageScriptName( ScriptAbstractPreferencePage preferencePage, Wizard wizard, String strName, boolean bNewScript )
{
	super( strName );

	_preferencePage = preferencePage;
	_bNewScript     = bNewScript;
	
	setTitle( strName );
	setDescription(ScriptStrings.getString("S_Specify_a_descriptive_name_for_this_script"));
}
/**
 */
public void createControl( Composite compositeParent )
{
	// Content
	//--------
	Composite compositeContent = new Composite( compositeParent, SWT.NULL );

	GridLayout layout = new GridLayout();
	layout.numColumns = 2;
	compositeContent.setLayout( layout );
	
	GridData gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.verticalAlignment   = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.grabExcessVerticalSpace   = true;
	compositeContent.setLayoutData( gridData );
	
	// Label: Script name
	//-----------------
	Label labelScriptName = new Label( compositeContent, SWT.NULL );
	labelScriptName.setText( ScriptStrings.getString("S_Script_name") + ":" );

	// Text: Script name
	//----------------
	_textScriptName = new Text( compositeContent, SWT.BORDER );

	gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.widthHint = 200;
	_textScriptName.setLayoutData( gridData );

	// Initialize
	//-----------
	if( _strDefaultScriptName != null )
	{
		_textScriptName.setText( _strDefaultScriptName );
	}

	// Add listeners
	//--------------
	_textScriptName.addListener( SWT.Modify, this );
	
	setControl( compositeContent );
}
/**
 * Returns the name currently in the script name text field.
 */
public String getScriptName() {
	return _textScriptName.getText();
}
/**
 */
public void handleEvent(Event event)
{
	// Modify event
	//-------------
	if( event.widget == _textScriptName )
	{
		updatePageComplete();
	}
}
/**
 * Sets the name into the script name text field.
 */
public void setDefaultScriptName(String name) {
	_strDefaultScriptName = name;
}
/**
 */
public void setScriptNames(List listScriptNames) {
	_listScriptNames = listScriptNames;
}
/**
 * Lets the preference page know what the current script name is.
 */
public void setVisible(boolean bVisible) {

	if (bVisible == true) {
		updatePageComplete();
	}

	else {
		// Let the preference page know the script name
		//---------------------------------------------
		_preferencePage.setScriptName(_textScriptName.getText());
	}

	// Call the super class, otherwise, nothing happens
	//-------------------------------------------------
	super.setVisible( bVisible );
	
	return;
}
/**
 */
protected void updatePageComplete() {

	// Check for zero length
	//----------------------
	if (_textScriptName.getText().length() == 0) {
		setPageComplete(false);
		return;
	}

	// Check for invalid filename characters
	//--------------------------------------
	if (_bNewScript == true && _listScriptNames != null) {

		String strText = _textScriptName.getText();

		for (int i = 0; i < _listScriptNames.size(); ++i) {
	        
			if (strText.equals((String) _listScriptNames.get(i)) == true) {
	            
				setPageComplete(false);
				return;
			}
		}
	}

	setPageComplete(true);
}
}
