package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.core.runtime.IExecutableExtension;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.DocumentEvent;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.scripting.internal.ScriptPlugin;
import org.eclipse.scripting.internal.ScriptStrings;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.BusyIndicator;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.custom.TextChangedEvent;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.TypedEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.editors.text.TextEditor;
import org.eclipse.ui.texteditor.DefaultRangeIndicator;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
import org.eclipse.ui.views.properties.IPropertySheetPage;

public class ScriptUIEditor extends TextEditor implements IExecutableExtension, KeyListener, MouseListener, SelectionListener {
	protected Composite                         _compositeClient = null;
	protected Composite                         _compositeTextEditor = null;
	protected TabFolder                         _tabFolder = null;
	protected ScriptUIPreviewEditorPreviewPanel _preview = null;
	protected ScriptUIEditorColorProvider        fColorProvider = null;
	protected ScriptUIEditorDesigner            _designer = null;
	protected ScriptUIEditorContentOutlinePage  _outlinePage = null;
	protected ScriptUIEditorPropertySheetPage   _propertySheet = null;
//	protected ScriptUIEditorActionManager       _actionManager = null;
	protected ScriptUIEditorButtonPalette       _palette = null;
	protected boolean                           _bApplyNeeded = false;
	protected Button                            _buttonApply = null;
	protected ScriptUIEditorModelElement        _elementSelected = null;

	protected static ScriptUIEditorDocumentProvider    _documentProvider = new ScriptUIEditorDocumentProvider();

	protected static final String STRING_EMPTY               = "";
	protected static final String STRING_POUNDCOMMENT        = "#comment";
	protected static final String STRING_POUNDTEXT           = "#text";
	protected static final String STRING_BODY                = "body";
	protected static final String STRING_BR                  = "br";
	protected static final String STRING_BUTTON              = "button";
	protected static final String STRING_CHECKBOX            = "checkbox";
	protected static final String STRING_CHECKED             = "checked";
	protected static final String STRING_COLSPAN             = "colspan";
	protected static final String STRING_COMMENT             = "comment";
	protected static final String STRING_CONTENT             = "content";
	protected static final String STRING_CONTENT_SCRIPT_TYPE = "Content-Script-Type";
	protected static final String STRING_FIELDSET            = "fieldset";
	protected static final String STRING_FORM                = "form";
	protected static final String STRING_HEAD                = "head";
	protected static final String STRING_HIDDEN              = "hidden";
	protected static final String STRING_HR                  = "hr";
	protected static final String STRING_HTML                = "html";
	protected static final String STRING_HTTP_EQUIV          = "http-equiv";
	protected static final String STRING_ID                  = "id";
	protected static final String STRING_IMG                 = "img";
	protected static final String STRING_INPUT               = "input";
	protected static final String STRING_LABEL               = "label";
	protected static final String STRING_META                = "meta";
	protected static final String STRING_ONBLUR              = "onblur";
	protected static final String STRING_ONCHANGE            = "onchange";
	protected static final String STRING_ONCLICK             = "onclick";
	protected static final String STRING_ONFOCUS             = "onfocus";
	protected static final String STRING_ONKEYPRESS          = "onkeypress";
	protected static final String STRING_ONLOAD              = "onload";
	protected static final String STRING_ONRESET             = "onreset";
	protected static final String STRING_ONSUBMIT            = "onsubmit";
	protected static final String STRING_ONUNLOAD            = "onunload";
	protected static final String STRING_RADIO               = "radio";
	protected static final String STRING_ROWSPAN             = "rowspan";
	protected static final String STRING_SCRIPT              = "script";
	protected static final String STRING_SRC                 = "src";
	protected static final String STRING_SELECT              = "select";
	protected static final String STRING_SIZE                = "size";
	protected static final String STRING_TABLE               = "table";
	protected static final String STRING_TD                  = "td";
	protected static final String STRING_TEXT                = "text";
	protected static final String STRING_TEXTAREA            = "textarea";
	protected static final String STRING_TEXT_JAVASCRIPT     = "text/javascript";
	protected static final String STRING_TEXT_FIELD          = "text field";
	protected static final String STRING_TH                  = "th";
	protected static final String STRING_TITLE               = "title";
	protected static final String STRING_TR                  = "tr";
	protected static final String STRING_TYPE                = "type";
	protected static final String STRING_VALUE               = "value";
	
	public static final int UNKNOWN                 =  0;
	public static final int BODY                    =  1;
	public static final int BREAK                   =  2;
	public static final int BUTTON                  =  3;
	public static final int CHECK                   =  4;
	public static final int COMBO                   =  5;
	public static final int COMMENT                 =  6;
	public static final int COMPOSITE               =  7;
	public static final int COMPOSITEPANEL          =  8;
	public static final int FIELDSET                =  9;
	public static final int FORM                    = 10;
	public static final int GROUPBOX                = 11;
	public static final int HEAD                    = 12;
	public static final int HTML                    = 13;
	public static final int HORZRULE                = 14;
	public static final int IMG                     = 15;
	public static final int LABEL                   = 16;
	public static final int META                    = 17;
	public static final int LIST                    = 18;
	public static final int PANEL                   = 19;
	public static final int PREFERENCES             = 20;
	public static final int PREFERENCEDIALOG        = 21;
	public static final int PREFERENCEDIALOGSTANDIN = 22;
	public static final int PREFERENCEPAGE          = 23;
	public static final int PROPERTYSHEET           = 24;
	public static final int PROPERTYSHEETCOMPOSITE  = 25;
	public static final int RADIO                   = 26;
	public static final int SCRIPT                  = 27;
	public static final int SCRIPTPREFERENCEDIALOG  = 28;
	public static final int SCRIPTPREFERENCEPAGE    = 29;
	public static final int TABLE                   = 30;
	public static final int TABLEDATA               = 31;
	public static final int TABLEHEADING            = 32;
	public static final int TABLELAYOUT             = 33;
	public static final int TABLEROW                = 34;
	public static final int TEXTAREA                = 35;
	public static final int TEXTFIELD               = 36;
	public static final int TEXTFIELDHIDDEN         = 37;
	public static final int WIZARD                  = 38;
	public static final int WIZARDBUILDTIME         = 39;
	public static final int WIZARDPAGEBUILDTIME     = 40;
	public static final int WIZARDPAGE              = 41;

	public static final int ADDASCHILD              = 50;
	public static final int ADDASSIBLING            = 51;
	public static final int DELETE                  = 52;
	public static final int MOVEHIGHER              = 53;
	public static final int MOVELOWER               = 54;
/**
 */
public ScriptUIEditor() {
	super();
	_documentProvider.setEditor(this);
}
/**
 */
public void createPartControl(Composite compositeParent)
{
	// Client composite
	// This has to be here.  Otherwise, the title above the tabs
	// is not drawn correctly
	//----------------------------------------------------------
	_compositeClient = new Composite( compositeParent, SWT.NULL );
	
	GridLayout layout= new GridLayout();
	layout.numColumns = 1;
	_compositeClient.setLayout( layout );

	GridData gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.verticalAlignment   = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.grabExcessVerticalSpace   = true;
	_compositeClient.setLayoutData( gridData );

	// Composite for palette / apply button
	//-------------------------------------
	Composite compositeFooter = new Composite( _compositeClient, SWT.NULL );
	layout= new GridLayout();
	layout.verticalSpacing = 0;
	layout.marginHeight = 2;
	layout.numColumns = 2;
	compositeFooter.setLayout( layout );

	gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	compositeFooter.setLayoutData( gridData );

	// Palette
	//--------
	_palette = new ScriptUIEditorButtonPalette( compositeFooter, this );
	gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	_palette.setLayoutData( gridData );

	// Apply button
	//-------------
	_buttonApply = new Button( compositeFooter, SWT.NULL );
	_buttonApply.setText( ScriptStrings.getString("S_Apply") );
	_buttonApply.setToolTipText( ScriptStrings.getString("S_Apply_text_changes" ));
	_buttonApply.setImage(ScriptPlugin.getImageDescriptor("icons/full/clcl16/apply_co.gif").createImage());

	gridData = new GridData();
	gridData.horizontalAlignment = GridData.HORIZONTAL_ALIGN_END;
	gridData.verticalAlignment = GridData.VERTICAL_ALIGN_CENTER;
	_buttonApply.setLayoutData( gridData );
	_buttonApply.setVisible( false );
	
	// Tab Folder
	//-----------
	_tabFolder = new TabFolder( _compositeClient, SWT.NULL );
	gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.verticalAlignment   = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.grabExcessVerticalSpace   = true;
	_tabFolder.setLayoutData( gridData );
	
	// Source view
	//------------
	_compositeTextEditor = new Composite( _tabFolder, SWT.BORDER );
	gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.verticalAlignment   = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.grabExcessVerticalSpace   = true;
	_compositeTextEditor.setLayoutData( gridData );
	FillLayout fillLayout = new FillLayout();
	_compositeTextEditor.setLayout( fillLayout );

	TabItem tabItem = new TabItem( _tabFolder, SWT.NULL );
	tabItem.setText( ScriptStrings.getString("S_Source" ));
	tabItem.setControl( _compositeTextEditor );
	tabItem.setData( _compositeTextEditor );

	// Design view
	//------------
/*	
	_designer = new ScriptUIEditorDesigner( _tabFolder, this );
	tabItem = new TabItem( _tabFolder, SWT.NULL );
	tabItem.setText( ScriptStrings.getString("Design") );
	tabItem.setControl( _designer );
	tabItem.setData( _designer );
*/	
	// Preview view
	//-------------
	tabItem = new TabItem( _tabFolder, SWT.NULL );
	tabItem.setText( ScriptStrings.getString("S_Preview") );

	_preview   = new ScriptUIPreviewEditorPreviewPanel( _tabFolder, true );
	tabItem.setControl( _preview );
	tabItem.setData( _preview );

	// Render the user interface
	// The document provider has the document now
	// Performance upgrade:
	// Set its visibility first, otherwise the preview
	// page thinks it's visible even though it isn't,
	// and performs a render when not required.
	//------------------------------------------------
	_preview.setVisible( false );
	updatePreview();
	
	// Call super class
	//-----------------
	super.createPartControl( _compositeTextEditor );

	// Font
	//-----
//	Font font = new Font( getSourceViewer().getTextWidget().getDisplay(), "courier", 10, SWT.NULL );
//	getSourceViewer().getTextWidget().setFont( font );

	// Range indicator
	//----------------
	getSourceViewer().setRangeIndicator( new DefaultRangeIndicator() );
	// Add listeners to the text editor
	//---------------------------------
	getSourceViewer().getTextWidget().addKeyListener( this );
	getSourceViewer().getTextWidget().addMouseListener( this );
	_tabFolder.addSelectionListener( this );
	_buttonApply.addSelectionListener( this );
}
/**
 * Determines which element was selected, and notifies the outline page
 * and property sheet page.
 */
protected void cursorMoved(TypedEvent event) {

	//	System.out.println("cursorMoved");

	// Obtain the caret position in the document
	//------------------------------------------
	if (event.widget instanceof StyledText) {
		StyledText styledText = (StyledText) event.widget;

		// Zero based
		//-----------
		int iLineNumber = styledText.getLineAtOffset(styledText.getCaretOffset());

		// Determine which element is on the line
		//---------------------------------------
		ScriptUIEditorModelElement modelElementRoot = _documentProvider.getElementRoot();

		ScriptUIEditorModelElement element = modelElementRoot.getElementFromLineNumber(iLineNumber);

		selectionChanged( element, false );		
	}
}
	/**
	 */
	public void dispose() {
		if (fColorProvider != null) {
			fColorProvider.dispose();
			fColorProvider= null;
		}
		super.dispose();
	}
/**
 * Obtains the text editor's text, does an XML parse, then a model parse, then updates
 * the outline page and preview page.  Returns an error message if an error occurs.
 */
public String doApplyTextChanges() {

	String strErrorMessage = null;

	if (_bApplyNeeded == true) {
		
		// Determine if this is in script element
		//---------------------------------------
		if (_documentProvider.getDocument(getEditorInput()) != null) {

			// Obtain the user modified text
			//------------------------------
			String strDocument = _documentProvider.getDocument(getEditorInput()).get();
			
			// Generate XML elements and attributes
			//-------------------------------------
			ScriptUIEditorXmlParser parser = new ScriptUIEditorXmlParser();
			parser.setString(strDocument);

			ScriptUIEditorXmlElement elementXmlRoot = null;
			try {
				parser.parse();
				elementXmlRoot = parser.getRootElement();
				setErrorMessage(null);
			}
			catch (Exception ex) {
				elementXmlRoot = null;
				strErrorMessage = ex.getMessage();
			}

			// Generate xhtml elements and attributes
			//---------------------------------------
			ScriptUIEditorModelElement elementRoot = null;

			if (elementXmlRoot != null) {

				ScriptUIEditorModel model = _documentProvider._elementRoot.getModel();

				try {
					model.parse(elementXmlRoot);
					elementRoot = model.getRootElement();

					// Reset the error message if no exception occurs
					//-----------------------------------------------
					setErrorMessage(null);
				}
				catch (Exception ex) {
					strErrorMessage = ex.getMessage();
					elementRoot = null;
				}

				// Replace the old model with the new
				//-----------------------------------
				if (elementRoot != null) {

					// Add mandatory elements
					//-----------------------
					model.makeValid();

					// Save user location information
					//-------------------------------
					int iTopIndex = getSourceViewer().getTextWidget().getTopIndex();

					int iLine = getSourceViewer().getTextWidget().getLineAtOffset(getSourceViewer().getTextWidget().getCaretOffset());
					ScriptUIEditorModelElement element = elementRoot.getElementFromLineNumber(iLine);

					// Generate new text
					//------------------
					StringBuffer strb = new StringBuffer(10000);
					model.printElementString(strb);

					getSourceViewer().getTextWidget().setVisible(false);
					
					// Replace old text with new text
					// Performance: Document.set()
					//   1.8 seconds for small document
					//   2.4 seconds for a larger one
					//---------------------------------
					_documentProvider.getDocument(getEditorInput()).set(strb.toString());

					// Update outline, and preview
					//----------------------------
					_palette.setEnabled( true );

					_outlinePage.setEnabled( true );

					_outlinePage.setInput(elementRoot);

					updatePreview( strb.toString() );

					// Select the element
					//-------------------
					_bApplyNeeded = false;
					
					// Go to where the user was in the editor
					//---------------------------------------
					goToElement(element);
					getSourceViewer().setTopIndex(iTopIndex);
					getSourceViewer().getTextWidget().setVisible(true);
					
					// Reset text changed mode
					//------------------------
					_buttonApply.setVisible(false);
				}
			}
		}
	}
	
	return strErrorMessage;
}
/**
 * Called after a value displayed by the property sheet is edited.
 */
public void doElementAttributeChange(ScriptUIEditorModelElement element) {

	_documentProvider.elementChanged( getEditorInput(), element );
	updatePreview();
	goToElement( element );
}
/**
 */
public void doElementCreate(ScriptUIEditorModelElement elementParent, int iType, String strName, int iLocation) {

	ScriptUIEditorModelElement elementNew = ScriptUIEditorModel.createChildElement(iType, strName, elementParent, iLocation);
	getSourceViewer().getTextWidget().setVisible(false);
	_documentProvider.elementChanged( getEditorInput(), elementParent);

	_outlinePage.setInput(_documentProvider._elementRoot );
	goToElement(elementNew);	
	updatePreview();

	selectionChanged( elementNew, false );
	getSourceViewer().getTextWidget().setVisible(true);
	return;
}
/**
 */
public void doElementDelete(ScriptUIEditorModelElement element) {

	if (element != null) {
	    
		ScriptUIEditorModelElement elementParent = element.getParent();
		
		if( elementParent == null ){
			return;
		}
		
		int iIndex = elementParent.getChildElementIndex(element);
		
		if( iIndex < 0 ){
			return;
		}

		getSourceViewer().getTextWidget().setVisible(false);
		
		ScriptUIEditorModel.deleteElement(element);

		_documentProvider.elementChanged(getEditorInput(), elementParent);
		_outlinePage.doElementDelete(element);

		updatePreview();

		// Go to the new element at the same location
		//-------------------------------------------
		if (iIndex < elementParent.getNumberOfChildElements()) {
			goToElement(elementParent.getChildElementAt(iIndex));
		}

		// Go to the previous element
		//---------------------------
		else if (iIndex > 0) {
			goToElement(elementParent.getChildElementAt(iIndex - 1));
		}

		// No more child elements
		//-----------------------
		else {
			goToElement(elementParent);
		}

		getSourceViewer().getTextWidget().setVisible(true);
	}
}
/**
 */
public void doElementMove(ScriptUIEditorModelElement elementSource, ScriptUIEditorModelElement elementTarget, boolean bAddAsChild, boolean bAfterTarget) {

	if (elementSource != null && elementTarget != null) {

		if (ScriptUIEditorModel.isMoveValid(elementSource, elementTarget, bAddAsChild, bAfterTarget) == true) {

			boolean bMoved = ScriptUIEditorModel.moveElement(elementSource, elementTarget, bAddAsChild, bAfterTarget);

			if (bMoved == true) {
				_documentProvider.elementChanged(getEditorInput(), _documentProvider._elementRoot);
				_outlinePage.setInput(_documentProvider._elementRoot);
				updatePreview();
				goToElement(elementSource);
			}
		}
	}
}
/**
 */
public void doElementMoveDown(ScriptUIEditorModelElement element) {

	if (element != null) {

		ScriptUIEditorModelElement elementParent = element.getParent();

		boolean bMoved = ScriptUIEditorModel.moveElementLower(element);

		if (bMoved == true) {
			_documentProvider.elementChanged(getEditorInput(), elementParent);
			_outlinePage.setInput(_documentProvider._elementRoot);
			updatePreview();
			goToElement(element);
		}
	}
}
/**
 */
public void doElementMoveUp(ScriptUIEditorModelElement element) {

	if (element != null) {

		ScriptUIEditorModelElement elementParent = element.getParent();

		boolean bMoved = ScriptUIEditorModel.moveElementHigher(element);

		if (bMoved == true) {
			_documentProvider.elementChanged(getEditorInput(), elementParent);
			_outlinePage.setInput(_documentProvider._elementRoot);
			updatePreview();
			goToElement(element);
		}
	}
}
/**
 */
public void doRevertToSaved() {
	
	super.doRevertToSaved();
	
	BusyIndicator.showWhile( getDisplay(), new Runnable() {
		public void run() {
			
			doApplyTextChanges();

		}
	});

}
/**
 */
public void editorContextMenuAboutToShow(MenuManager menu) {
	super.editorContextMenuAboutToShow(menu);
	//   addAction(menu, "ContentAssistProposal");
	//   addAction(menu, "ContentAssistTip");
}
/**
 */
public Object getAdapter(Class classRequired) {

	// Outline page
	//-------------
	if (classRequired.equals(IContentOutlinePage.class) == true) {

		if (_outlinePage == null) {
			ScriptUIEditorModelElement elementRoot = _documentProvider.getElementRoot();
			_outlinePage = new ScriptUIEditorContentOutlinePage(elementRoot, _documentProvider, this);
		}

		return _outlinePage;
	}

	// Property sheet page
	//--------------------
	else if (classRequired.equals(IPropertySheetPage.class) == true) {

		if (_propertySheet == null) {
			_propertySheet = new ScriptUIEditorPropertySheetPage(this);
		}

		return _propertySheet;
	}

	// Other
	//------
	return super.getAdapter(classRequired);
}
/**
 */
public Display getDisplay() {
	return null;
}
/**
 */
public Shell getShell() {
	return super.getSite().getShell();
}
/**
 * Moves the cursor, highlight range, outline view to the element.
 */
public void goToElement(ScriptUIEditorModelElement element) {

	if (element != null) {

		// Find the element that has this line
		// The model may have changed and the element specified
		// may no longer exist in the model
		//-----------------------------------------------------
		element = _documentProvider._elementRoot.getElementFromLineNumber(element._iLineNumberFirst);

		// Notify the outline page, and property page
		//-------------------------------------------
		if (element != null) {
			StructuredSelection selection = new StructuredSelection(element);

			if (_outlinePage != null) {
				_outlinePage.setSelection(selection);
			}

			if (_propertySheet != null) {
				_propertySheet.selectionChanged(this, selection);
			}
		}
	}
}
/**
 * Initializes the editor when created from scratch.
 * 
 * This method is called soon after part construction and marks 
 * the start of the extension lifecycle.  At the end of the
 * extension lifecycle <code>dispose</code> will be invoked
 * to terminate the lifecycle.
 */
public void init(IEditorSite site, IEditorInput input) throws PartInitException {

	// Check input
	//------------	
	if (!(input instanceof IFileEditorInput))
		throw new PartInitException("Invalid Input: " + input + ".  Input must implement IFileEditorInput");

	// Save input
	//-----------
	setSite(site);
	setInput(input);

	/*
		// Update titles.
		setTitle(input.getName());
		setTitleToolTip(input.getToolTipText());
		ImageDescriptor desc = input.getImageDescriptor();
		if (desc != null) {
			oleTitleImage = desc.createImage();
			setTitleImage(oleTitleImage);
		}
	
		// Listen for part activation.
		site.getPage().addPartListener(partListener);
	*/
}
	/**
	 * This is called during construction of TextEditor
	 */
	protected void initializeEditor() {

		super.initializeEditor();

		// The document provider must be a static
		//---------------------------------------
		setDocumentProvider( _documentProvider );
		fColorProvider= new ScriptUIEditorColorProvider();

		ScriptUIEditorEnvironment.connect(this);
		ScriptUIEditorSourceViewerConfiguration config = new ScriptUIEditorSourceViewerConfiguration();

		// This is being changed from public to protected so is ok.
		//---------------------------------------------------------
		setSourceViewerConfiguration(config);
	}
/**
 */
public void keyPressed(org.eclipse.swt.events.KeyEvent e) {}
/**
 */
public void keyReleased(KeyEvent kEvent) {

	if (_bApplyNeeded == false) {
		
		switch (kEvent.keyCode) {
			case SWT.ARROW_DOWN :
			case SWT.ARROW_LEFT :
			case SWT.ARROW_RIGHT :
			case SWT.ARROW_UP :
			case SWT.END :
			case SWT.HOME :
			case SWT.PAGE_DOWN :
			case SWT.PAGE_UP :
				cursorMoved(kEvent);
				break;

			default :
				break;
		}
	}
}
/**
 */
public void mouseDoubleClick(MouseEvent e) {
}
/**
 */
public void mouseDown(MouseEvent mEvent) {

}
/**
 */
public void mouseUp(MouseEvent mEvent) {

	if (_bApplyNeeded == false) {
		cursorMoved(mEvent);
	}
}
/**
 * Called by the content outline page with the element that has been selected.
 */
public void selectionChanged(ScriptUIEditorModelElement element, boolean bFromOutlinePage) {

	if (element == null) {
		return;
	}

	if (_elementSelected != element) {

		_elementSelected = element;

		IDocument document = _documentProvider.getDocument(getEditorInput());

		if (document != null) {
			try {
				int iOffsetFirst = document.getLineOffset(element._iLineNumberFirst);
				int iOffsetLast = document.getLineOffset(element._iLineNumberLast) + document.getLineLength(element._iLineNumberLast);
				setHighlightRange(iOffsetFirst, iOffsetLast - iOffsetFirst, bFromOutlinePage);
			}
			catch (BadLocationException ex) {
			}
		}

		// Notify the outline page, and property page
		//-------------------------------------------
		if (_bApplyNeeded == false) {
			ISelection selection = new StructuredSelection(element);

			if (bFromOutlinePage == false && _outlinePage != null) {
				_outlinePage.setSelection(selection);
			}

			if (_propertySheet != null) {
				_propertySheet.selectionChanged(this, selection);
			}
		}

		_palette.selectionChanged(element);
	}
}
/**
 * Sets the error message line with the text string.
 */
public void setErrorMessage(String strMessage) {

	ScriptUIEditorContextContributor c = ScriptUIEditorContextContributor.getInstance();

	if (c != null) {

		if (strMessage == null) {
			c.setErrorMessage("");
		}
		else{
			c.setErrorMessage(strMessage);
		}
	}
}
/**
 * This overrides the super class in order to override the move cursor
 * parameter.  The instance variable bCursorMoved tells us that this
 * has been called as a result of the user moving the cursor with the
 * mouse or keyboard, and therefore, the editor should not scroll the text.
 * Otherwise, the highlight range has been requested as a result of a selection
 * on the ouline page which should scroll the text. 
 */
public void setHighlightRange(int iOrigin, int iExtent, boolean bMoveCursor) {

	// Disable if selection changed in outline page if apply needed
	//-------------------------------------------------------------
	if (_bApplyNeeded == false) {
		try {
			super.setHighlightRange(iOrigin, iExtent, bMoveCursor);
		}
		catch (Exception ex) {
			
		}
	}
}
/**
 * Called from document provider textReplaced and textSet
 */
public void textChanged(DocumentEvent event) {

	if (_bApplyNeeded == false) {
	    
		_bApplyNeeded = true;
		_buttonApply.setVisible(true);
		_palette.setEnabled(false);
		resetHighlightRange();

		if (_outlinePage != null) {
			_outlinePage.setEnabled(false);
		}

		if (_propertySheet != null) {
			ISelection selection = new StructuredSelection(_documentProvider.getElementRoot());
			_propertySheet.selectionChanged(this, selection);
		}
	}
}
/**
 * Called from document provider textReplaced and textSet
 */
public void textChanged(TextChangedEvent event) {

	if (_bApplyNeeded == false) {
	    
		_bApplyNeeded = true;
		_buttonApply.setVisible(true);
		_palette.setEnabled(false);
		resetHighlightRange();

		if (_outlinePage != null) {
			_outlinePage.setEnabled(false);
		}

		if (_propertySheet != null) {
			ISelection selection = new StructuredSelection(_documentProvider.getElementRoot());
			_propertySheet.selectionChanged(this, selection);
		}
	}
}
/**
 */
protected void updatePreview() {
	if (_documentProvider._elementRoot != null) {
		IDocument document = _documentProvider.getDocument(getEditorInput());
		if (document != null) {
			String strDocument = document.get();
			_preview.setPresentationString(strDocument);
			_preview.renderWhenVisible();
		}
	}

}
/**
 */
protected void updatePreview(String strDocument) {
	_preview.setPresentationString(strDocument);
	_preview.renderWhenVisible();

}
/**
 */
public void widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent e) {}
/**
 */
public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {

	// Apply button
	//-------------
	if (e.widget == _buttonApply) {
		
		BusyIndicator.showWhile( getDisplay(), new Runnable() {
			public void run() {
				
				boolean bSuccessful = true;
				String strErrorMessage = doApplyTextChanges();
				if( strErrorMessage != null )
				{
					MessageDialog.openError( getShell(), ScriptStrings.getString("S_An_error_was_detected"), strErrorMessage );
					setErrorMessage( strErrorMessage );
				}
			}
		});

		getSourceViewer().getTextWidget().setFocus();
	}

	return;
}
}
