package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.util.Enumeration;
import java.util.Hashtable;

import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.scripting.internal.ScriptPlugin;
import org.eclipse.swt.custom.BusyIndicator;

public class ScriptUIEditorActionManager {
	
	protected ScriptUIEditorAction       _actionChildOrSibling = null;
	protected ScriptUIEditorAction       _actionDelete         = null;
	protected Hashtable                  _hashIDToAction       = new Hashtable();
	protected boolean                    _bAddChildNotSibling  = true;
	protected ScriptUIEditor             _editor               = null;
	protected ScriptUIEditorModelElement _elementSelected      = null;
	protected String                     _strIconPath          = "icons/full/ctool16/";
/**
 */
public ScriptUIEditorActionManager( ScriptUIEditor editor) {
	super();
	_editor = editor;
}
/**
 */
protected ScriptUIEditorAction createAction( IToolBarManager tbm, int iId, String strLabel, String strName, String strImage ) {
	
	ScriptUIEditorAction action = new ScriptUIEditorAction( this, strLabel, strName, iId );
	action.setToolTipText( strLabel );

	ImageDescriptor imageDescriptor = ScriptPlugin.getImageDescriptor( _strIconPath + strImage );
	
	action.setImageDescriptor( imageDescriptor );
	action.setEnabled( false );
	_hashIDToAction.put( new Integer( iId ), action );

	tbm.add( action );

	return action;
}
/**
 */
protected void createActions( IToolBarManager tbm ) {
/*
	// Separator
	//----------
	tbm.add(new Separator());
	
	// Create non-toggle buttons
	//--------------------------	
	createAction( tbm, ScriptUIEditor.BREAK    , "Break"          , "br",        "break.gif" );
	createAction( tbm, ScriptUIEditor.BUTTON   , "Button"         , "button",    "button.gif" );
	createAction( tbm, ScriptUIEditor.CHECK    , "Check"          , "check",     "checkbox.gif" );
	createAction( tbm, ScriptUIEditor.COMBO    , "Combo"          , "combo",     "combobox.gif" );
	createAction( tbm, ScriptUIEditor.FIELDSET , "Fieldset"       , "fieldset",  "fieldset.gif" );
	createAction( tbm, ScriptUIEditor.FORM     , "Form"           , "form",      "form.gif" );
	createAction( tbm, ScriptUIEditor.HORZRULE , "Horizontal Rule", "hr",        "horzrule.gif" );
	createAction( tbm, ScriptUIEditor.IMG      , "Image"          , "img",       "image.gif" );
	createAction( tbm, ScriptUIEditor.LABEL    , "Label"          , "label",     "label.gif" );
	createAction( tbm, ScriptUIEditor.LIST     , "List"           , "list",      "listbox.gif" );
	createAction( tbm, ScriptUIEditor.META     , "Meta"           , "meta",      "meta.gif" );					
	createAction( tbm, ScriptUIEditor.RADIO    , "Radio"          , "radio",     "radiobutton.gif" );
	createAction( tbm, ScriptUIEditor.SCRIPT   , "Script"         , "script",    "script.gif" );
	createAction( tbm, ScriptUIEditor.TABLE    , "Table"          , "table",     "table.gif" );
	createAction( tbm, ScriptUIEditor.TABLEDATA, "TableData"      , "td",        "tabledata.gif" );
	createAction( tbm, ScriptUIEditor.TABLEROW , "TableRow"       , "tr",        "tablerow.gif" );
	createAction( tbm, ScriptUIEditor.TEXTAREA , "TextArea"       , "textarea",  "textarea.gif" );
	createAction( tbm, ScriptUIEditor.TEXTFIELD, "TextField"      , "textfield", "textfield.gif" );
	
	// Separator
	//----------
	tbm.add(new Separator());
	
	// Child or sibling toggle button
	// By setting a boolean value, this value will be
	// automatically toggled when pressed.  A call
	// to getValue returns the current state.
	//-----------------------------------------------
	_actionChildOrSibling = new ScriptUIEditorAction( this, "Add as Child or Sibling", null, ScriptUIEditor.CHILDORSIBLING );
	_actionChildOrSibling.setToolTipText( "Add as Child or Sibling" );
	ImageDescriptor imageDescriptor = ScriptPlugin.getImageDescriptor( "icons/full/ctool16/addaschild.gif" );
	_actionChildOrSibling.setImageDescriptor( imageDescriptor );
	_actionChildOrSibling.setChecked( true );
	tbm.add( _actionChildOrSibling );
	
	// Separator
	//----------
	tbm.add(new Separator());
	
	// Delete
	//-------
	createAction( tbm, ScriptUIEditor.DELETE, "Delete element", "deleteelement", "delete.gif" );
	
	// Separator
	//----------
	tbm.add(new Separator());

	// Move higher / lower within parent
	//----------------------------------
	createAction( tbm, ScriptUIEditor.MOVEHIGHER, "Move higher", "movehigher", "movehigher.gif" );
	createAction( tbm, ScriptUIEditor.MOVELOWER, "Move lower", "movelower", "movelower.gif" );
*/
}
/**
 */
protected void enableAction(int iId, boolean bEnable) {
	
	Integer intId = new Integer( iId );

	ScriptUIEditorAction action = (ScriptUIEditorAction)_hashIDToAction.get( intId );

	if( action.isEnabled() != bEnable )
	{
		action.setEnabled( bEnable );
	}
}
/**
 */
protected void enableAllActions(boolean bEnable) {
	
	ScriptUIEditorAction action = null;
	Enumeration  enum   = _hashIDToAction.elements();
	
	while( enum.hasMoreElements() == true )
	{
		action = (ScriptUIEditorAction)enum.nextElement();

		if( action.isEnabled() != bEnable )
		{
			action.setEnabled( bEnable );
		}
	}
}
/**
 * Responds to menu items / toolbar buttons.
 */
public void run(final ScriptUIEditorAction action) {
	BusyIndicator.showWhile( _editor.getDisplay(), new Runnable() {
		public void run() {
			// Child / sibling
			//----------------
			if (action._iId == ScriptUIEditor.ADDASCHILD) {
				_bAddChildNotSibling = action.isChecked();
				updateActions();
			}

			// Delete
			//-------
			else if (action._iId == ScriptUIEditor.DELETE) {
				ScriptUIEditorModelElement elementParent = _elementSelected.getParent();

				ScriptUIEditorModel.deleteElement(_elementSelected);
				_editor._documentProvider.changed(elementParent);
			}

			// Move higher
			//------------
			else if (action._iId == ScriptUIEditor.MOVEHIGHER) {
				ScriptUIEditorModelElement elementParent = _elementSelected.getParent();

				boolean bMoved = ScriptUIEditorModel.moveElementHigher(_elementSelected);

				if (bMoved == true) {
					_editor._documentProvider.changed(elementParent);
				}
			}

			// Move lower
			//-----------
			else if (action._iId == ScriptUIEditor.MOVELOWER) {
				ScriptUIEditorModelElement elementParent = _elementSelected.getParent();

				boolean bMoved = ScriptUIEditorModel.moveElementLower(_elementSelected);

				if (bMoved == true) {
					_editor._documentProvider.changed(elementParent);
				}
			}

			// Creation of new element
			//------------------------
			else {
				ScriptUIEditorModelElement elementParent = _bAddChildNotSibling ? _elementSelected : _elementSelected._elementParent;
				ScriptUIEditorModelElement elementNew = ScriptUIEditorModel.createChildElement(action._iId, action._strName, elementParent, 0);
				_editor._documentProvider.changed(elementParent);
				ISelection selection = new StructuredSelection(elementNew);
				_editor._outlinePage.setSelection(selection);
			}
		}
	});
}
/**
 */
public void selectionChanged(ScriptUIEditorModelElement element ) {
	_elementSelected = element;
	updateActions();
}
/**
 */
public void updateActions() {
/*
	if (_elementSelected != null) {

		ScriptUIEditorModelElement elementParent = _elementSelected.getParent();

		if (elementParent != null) {

			int iType = _bAddChildNotSibling == true ? _elementSelected._iType : elementParent._iType;

			int iNumberOfChildren = _bAddChildNotSibling == true ? _elementSelected.getNumberOfChildElements() : elementParent.getNumberOfChildElements();

			switch (iType) {
				case ScriptUIEditor.BODY :
					enableAllActions(false);
					enableAction(ScriptUIEditor.FORM, true);
					enableAction(ScriptUIEditor.SCRIPT, true);
					break;

				case ScriptUIEditor.FORM :
				case ScriptUIEditor.FIELDSET :
					enableAllActions(true);
					enableAction(ScriptUIEditor.META, false);
					enableAction(ScriptUIEditor.FORM, false);
					enableAction(ScriptUIEditor.SCRIPT, false);
					enableAction(ScriptUIEditor.TABLEDATA, false);
					enableAction(ScriptUIEditor.TABLEROW, false);
					break;

				case ScriptUIEditor.HEAD :
					enableAllActions(false);
					enableAction(ScriptUIEditor.META, true);
					enableAction(ScriptUIEditor.SCRIPT, true);
					break;
				case ScriptUIEditor.HTML :
					enableAllActions(false);
					break;

				case ScriptUIEditor.TABLE :
					enableAllActions(false);
					enableAction(ScriptUIEditor.TABLEROW, true);
					enableAction(ScriptUIEditor.DELETE, true);
					break;

				case ScriptUIEditor.TABLEROW :
					enableAllActions(false);
					enableAction(ScriptUIEditor.TABLEDATA, true);
					enableAction(ScriptUIEditor.DELETE, true);
					break;

				case ScriptUIEditor.TABLEDATA :
					if (iNumberOfChildren == 0) {
						enableAllActions(true);
						enableAction(ScriptUIEditor.BREAK, false );
						enableAction(ScriptUIEditor.FORM, false);
						enableAction(ScriptUIEditor.META, false);
						enableAction(ScriptUIEditor.SCRIPT, false);
						enableAction(ScriptUIEditor.TABLE, false);
						enableAction(ScriptUIEditor.TABLEDATA, false);
						enableAction(ScriptUIEditor.TABLEROW, false);
					}
					else {
						enableAllActions(false);
						enableAction(ScriptUIEditor.DELETE, true);
					}
					break;

				default :
					enableAllActions(false);
					enableAction(ScriptUIEditor.DELETE, true);
					break;
			}

			// Move higher/lower
			//------------------
			if (_elementSelected._iType != ScriptUIEditor.HTML && _elementSelected._iType != ScriptUIEditor.HEAD && _elementSelected._iType != ScriptUIEditor.BODY) {
	            
				int iIndex = elementParent._vectorChildElements.indexOf(_elementSelected);

				// Higher towards zero
				//--------------------
				int iTotal = elementParent.getNumberOfChildElements();

				if (iTotal > 1 && iIndex > 0) {
					enableAction(ScriptUIEditor.MOVEHIGHER, true);
				}
				else {
					enableAction(ScriptUIEditor.MOVEHIGHER, false);
				}

				// Lower away from zero
				//---------------------
				if (iTotal > 1 && iIndex < iTotal - 1) {
					enableAction(ScriptUIEditor.MOVELOWER, true);
				}
				else {
					enableAction(ScriptUIEditor.MOVELOWER, false);
				}
			}
			else
			{
			   enableAction(ScriptUIEditor.MOVEHIGHER, false);
			   enableAction(ScriptUIEditor.MOVELOWER, false);
			}
		}

		// No parent element
		//------------------
		else {
			enableAllActions(false);
		}
	}
*/
}
}
