package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.util.Enumeration;
import java.util.Hashtable;

import org.eclipse.scripting.internal.ScriptPlugin;
import org.eclipse.scripting.internal.ScriptStrings;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.BusyIndicator;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;

public class ScriptUIEditorButtonPalette implements SelectionListener
{
	protected final static String _strHotIconPath = "icons/full/clcl16/";
	protected final static String _strDisabledIconPath = "icons/full/dlcl16/";
	protected final static String _strEnabledIconPath = "icons/full/elcl16/";
	
	protected ScriptUIEditor          _editor = null;
	protected ScriptUIEditorModelElement _elementSelected = null;
	protected Hashtable               _hashIDToButton = new Hashtable();
	protected ToolItem                  _radioAddAsChild   = null;
	protected ToolItem                  _radioAddAsSibling = null;
	protected ToolItem                  _buttonDelete = null;
	protected ToolBar                 _toolBar = null;
	protected boolean                   _bEnabled = true;

	// Button data associated with push buttons
	//-----------------------------------------
	protected class ButtonData {
		protected int _iType      = 0;
		protected String _strName = null;
	}
/**
 */
public ScriptUIEditorButtonPalette(Composite compositeParent, ScriptUIEditor editor) {

	_editor = editor;

	_toolBar = new ToolBar( compositeParent, SWT.FLAT );

	_radioAddAsChild = createPaletteButton(SWT.RADIO, "S_Add_as_child", "Add as child", ScriptUIEditor.ADDASCHILD, "addaschild_co.gif", false);

	_radioAddAsSibling = createPaletteButton(SWT.RADIO, "S_Add_as_sibling", "Add as sibling", ScriptUIEditor.ADDASSIBLING, "addassibling_co.gif", false);

	
	createPaletteButton( SWT.SEPARATOR, null, null, 0, null, false );
	
	createPaletteButton(SWT.PUSH, "S_Form", ScriptUIEditor.STRING_FORM, ScriptUIEditor.FORM, "form_edit.gif", true);	
	createPaletteButton(SWT.PUSH, "S_Fieldset", ScriptUIEditor.STRING_FIELDSET, ScriptUIEditor.FIELDSET, "fieldset_edit.gif", true);
	
	createPaletteButton( SWT.SEPARATOR, null, null, 0, null, false );

	createPaletteButton(SWT.PUSH, "S_Label", ScriptUIEditor.STRING_LABEL, ScriptUIEditor.LABEL, "label_edit.gif", true);
	createPaletteButton(SWT.PUSH, "S_Text_field", ScriptUIEditor.STRING_INPUT, ScriptUIEditor.TEXTFIELD, "textfield_edit.gif", true);
	createPaletteButton(SWT.PUSH, "S_Text_area", ScriptUIEditor.STRING_TEXTAREA, ScriptUIEditor.TEXTAREA, "textarea_edit.gif", true);
	createPaletteButton(SWT.PUSH, "S_Button", ScriptUIEditor.STRING_INPUT, ScriptUIEditor.BUTTON, "button_edit.gif", true);
	createPaletteButton(SWT.PUSH, "S_Radio_button", ScriptUIEditor.STRING_INPUT, ScriptUIEditor.RADIO, "radiobutton_edit.gif", true);
	createPaletteButton(SWT.PUSH, "S_Check_box", ScriptUIEditor.STRING_INPUT, ScriptUIEditor.CHECK, "checkbox_edit.gif", true);
	createPaletteButton(SWT.PUSH, "S_Combo_box", ScriptUIEditor.STRING_SELECT, ScriptUIEditor.COMBO, "combobox_edit.gif", true);
	createPaletteButton(SWT.PUSH, "S_List_box", ScriptUIEditor.STRING_SELECT, ScriptUIEditor.LIST, "listbox_edit.gif", true);
	createPaletteButton(SWT.PUSH, "S_Horizontal_rule", ScriptUIEditor.STRING_HR, ScriptUIEditor.HORZRULE, "horzrule_edit.gif", true);
	createPaletteButton(SWT.PUSH, "S_Image", ScriptUIEditor.STRING_IMG, ScriptUIEditor.IMG, "image_edit.gif", true);
	
	createPaletteButton( SWT.SEPARATOR, null, null, 0, null, false );
	
	createPaletteButton(SWT.PUSH, "S_Table", ScriptUIEditor.STRING_TABLE, ScriptUIEditor.TABLE, "table_edit.gif", true);
	createPaletteButton(SWT.PUSH, "S_Table_row", ScriptUIEditor.STRING_TR, ScriptUIEditor.TABLEROW, "tablerow_edit.gif", true);
	createPaletteButton(SWT.PUSH, "S_Table_data", ScriptUIEditor.STRING_TD, ScriptUIEditor.TABLEDATA, "tabledata_edit.gif", true);
	
	createPaletteButton( SWT.SEPARATOR, null, null, 0, null, false );
	
	createPaletteButton(SWT.PUSH, "S_Meta", ScriptUIEditor.STRING_META, ScriptUIEditor.META, "meta_edit.gif" , true);
	createPaletteButton(SWT.PUSH, "S_Script", ScriptUIEditor.STRING_SCRIPT, ScriptUIEditor.SCRIPT, "script_edit.gif", true);
	
	createPaletteButton( SWT.SEPARATOR, null, null, 0, null, false );

	createPaletteButton(SWT.PUSH, "S_Comment", ScriptStrings.getString("S_comment"), ScriptUIEditor.COMMENT, "comment_edit.gif", true);
	createPaletteButton(SWT.PUSH, "S_Break", ScriptUIEditor.STRING_BR, ScriptUIEditor.BREAK, "break_edit.gif", true);

	createPaletteButton( SWT.SEPARATOR, null, null, 0, null, false );

//	_buttonMoveUp = createPaletteButton(SWT.PUSH, "S_Move_up", "Move up", ScriptUIEditor.MOVEHIGHER, "moveup.gif", false);
//	_buttonMoveDown = createPaletteButton(SWT.PUSH, "S_Move_down", "Move down", ScriptUIEditor.MOVELOWER, "movedown.gif", false);

//	createPaletteButton( SWT.SEPARATOR, null, null, 0, null, false );

	_buttonDelete = createPaletteButton(SWT.PUSH, "S_Delete", "Delete", ScriptUIEditor.DELETE, "delete_edit.gif", false);
	
	enableAllButtons(false);
	_radioAddAsChild.setEnabled(true);
	_radioAddAsSibling.setEnabled(true);
	_radioAddAsSibling.setSelection(true);
}
/**
 * Creates a palette button.
 */
private ToolItem createPaletteButton(int iStyle, String strText, String strName, int iType, String strImage, boolean bAddToList) {

	ToolItem toolItem = null;

	if (iStyle != SWT.SEPARATOR) {
		toolItem = new ToolItem(_toolBar, iStyle);
		toolItem.addSelectionListener(this);
		toolItem.setImage(ScriptPlugin.getImageDescriptor(_strEnabledIconPath + strImage).createImage());
		toolItem.setHotImage(ScriptPlugin.getImageDescriptor(_strHotIconPath + strImage).createImage());
		toolItem.setDisabledImage(ScriptPlugin.getImageDescriptor(_strDisabledIconPath + strImage).createImage());
		toolItem.setToolTipText( ScriptStrings.getString( strText ));
		ButtonData data = new ButtonData();
		data._iType = iType;
		data._strName = strName;
		toolItem.setData(data);

		if( bAddToList == true )
		{
			_hashIDToButton.put(new Integer(iType), toolItem);
		}
	}
	else {
		toolItem = new ToolItem(_toolBar, SWT.SEPARATOR);
	}

	return toolItem;
}
/**
 */
private void enableAllButtons( boolean bEnable )
{
	Enumeration enum   = _hashIDToButton.elements();
	
	ToolItem item = null;
	
	while( enum.hasMoreElements() == true )
	{
		item = (ToolItem)enum.nextElement();
		item.setEnabled( bEnable );
	}
}
/**
 */
private void enableButton( int iType, boolean bEnable )
{
	Integer intType = new Integer( iType );

	ToolItem item = (ToolItem)_hashIDToButton.get( intType );

	if( item != null && item.getEnabled() != bEnable )
	{
		item.setEnabled( bEnable );
	}
}
/**
 */
public void selectionChanged(ScriptUIEditorModelElement element ) {
	_elementSelected = element;
	updateButtons();
}
/**
 */
public void setEnabled(boolean bEnabled) {
	_bEnabled = bEnabled;
	updateButtons();
}
/**
 */
public void setLayoutData(GridData gridData) {
	if (_toolBar != null) {
		_toolBar.setLayoutData(gridData);
	}
}
/**
 * Updates all toolbar buttons.
 */
private void updateButtons() {

	_buttonDelete.setEnabled( _bEnabled );
//	_buttonMoveUp.setEnabled( _bEnabled );
//	_buttonMoveDown.setEnabled( _bEnabled );

	if( _bEnabled == false ) {
		enableAllButtons(false);
		_radioAddAsChild.setEnabled( false );
		_radioAddAsSibling.setEnabled( false );
		return;
	}	
		
	if (_elementSelected != null) {

		ScriptUIEditorModelElement elementParent = _elementSelected.getParent();

		// Handle html element
		//--------------------
		if (elementParent == null) {
			elementParent = _elementSelected;
		}
		
		int iType = _elementSelected._iType;

		// Set add as children, add as sibling buttons
		//--------------------------------------------
		_radioAddAsChild.setEnabled(true);
		_radioAddAsSibling.setEnabled(true);

		switch (iType) {
			case ScriptUIEditor.HTML :
				_radioAddAsChild.removeSelectionListener(this);
				_radioAddAsSibling.removeSelectionListener(this);
				_radioAddAsSibling.setSelection(false);
				_radioAddAsSibling.setEnabled(false);
				_radioAddAsChild.setEnabled(true);
				_radioAddAsChild.setSelection(true);
				_radioAddAsChild.addSelectionListener(this);
				_radioAddAsSibling.addSelectionListener(this);
				break;

			case ScriptUIEditor.HEAD :
			case ScriptUIEditor.BODY :
				_radioAddAsChild.removeSelectionListener(this);
				_radioAddAsSibling.removeSelectionListener(this);
				_radioAddAsSibling.setSelection(false);
				_radioAddAsSibling.setEnabled(false);
				_radioAddAsChild.setEnabled(true);
				_radioAddAsChild.setSelection(true);
				_radioAddAsChild.addSelectionListener(this);
				_radioAddAsSibling.addSelectionListener(this);
				break;

			case ScriptUIEditor.FORM :
			case ScriptUIEditor.FIELDSET :
			case ScriptUIEditor.TABLE :
			case ScriptUIEditor.TABLEROW :
				_radioAddAsChild.setEnabled(true);
				break;

			case ScriptUIEditor.TABLEDATA :
				if (_elementSelected.getNumberOfChildElements() == 0) {
					_radioAddAsChild.setEnabled(true);
				}
				else {
					_radioAddAsChild.setEnabled(false);
				}
				break;


			default :
				_radioAddAsChild.removeSelectionListener(this);
				_radioAddAsSibling.removeSelectionListener(this);
				_radioAddAsChild.setSelection(false);
				_radioAddAsChild.setEnabled(false);
				_radioAddAsSibling.setSelection(true);
				_radioAddAsChild.addSelectionListener(this);
				_radioAddAsSibling.addSelectionListener(this);
				break;
		}

		// Enable / disable add buttons
		//-----------------------------
		iType = _radioAddAsChild.getSelection() == true ? _elementSelected._iType : elementParent._iType;
		int iNumberOfChildren = _radioAddAsChild.getSelection() == true ? _elementSelected.getNumberOfChildElements() : elementParent.getNumberOfChildElements();

		switch (iType) {
			case ScriptUIEditor.HTML :
				enableAllButtons(false);
				enableButton(ScriptUIEditor.COMMENT, true);
				break;

			case ScriptUIEditor.BODY :
				enableAllButtons(false);
				enableButton(ScriptUIEditor.COMMENT, true);
				enableButton(ScriptUIEditor.FORM, true);
				enableButton(ScriptUIEditor.SCRIPT, true);
				break;

			case ScriptUIEditor.FORM :
			case ScriptUIEditor.FIELDSET :
				enableAllButtons(true);
				enableButton(ScriptUIEditor.META, false);
				enableButton(ScriptUIEditor.FORM, false);
				enableButton(ScriptUIEditor.SCRIPT, false);
				enableButton(ScriptUIEditor.TABLEDATA, false);
				enableButton(ScriptUIEditor.TABLEROW, false);
				break;

			case ScriptUIEditor.HEAD :
				enableAllButtons(false);
				enableButton(ScriptUIEditor.COMMENT, true);
				enableButton(ScriptUIEditor.META, true);
				enableButton(ScriptUIEditor.SCRIPT, true);
				break;

			case ScriptUIEditor.TABLE :
				enableAllButtons(false);
				enableButton(ScriptUIEditor.COMMENT, true);
				enableButton(ScriptUIEditor.TABLEROW, true);
				break;

			case ScriptUIEditor.TABLEROW :
				enableAllButtons(false);
				enableButton(ScriptUIEditor.COMMENT, true);
				enableButton(ScriptUIEditor.TABLEDATA, true);
				break;

			case ScriptUIEditor.TABLEDATA :
				if (iNumberOfChildren == 0) {
					enableAllButtons(true);
					enableButton(ScriptUIEditor.FORM, false);
					enableButton(ScriptUIEditor.META, false);
					enableButton(ScriptUIEditor.SCRIPT, false);
					enableButton(ScriptUIEditor.TABLE, false);
					enableButton(ScriptUIEditor.TABLEDATA, false);
					enableButton(ScriptUIEditor.TABLEROW, false);
				}
				else {
					//						_radioAddAsChild.setEnabled( false );
					enableAllButtons(false);
				}
				break;

			default :
				enableAllButtons(false);
				break;
		}

		// Move higher/lower
		//------------------
		if (_elementSelected._iType != ScriptUIEditor.HTML && _elementSelected._iType != ScriptUIEditor.HEAD && _elementSelected._iType != ScriptUIEditor.BODY) {

			int iIndex = elementParent._vectorChildElements.indexOf(_elementSelected);

			// Higher towards zero
			//--------------------
			int iTotal = elementParent.getNumberOfChildElements();

			if (iTotal > 1 && iIndex > 0) {
//				_buttonMoveUp.setEnabled(true);
			}
			else {
//				_buttonMoveUp.setEnabled(false);
			}

			// Lower away from zero
			//---------------------
			if (iTotal > 1 && iIndex < iTotal - 1) {
//				_buttonMoveDown.setEnabled(true);
			}
			else {
//				_buttonMoveDown.setEnabled(false);
			}

			// Delete
			//-------
			_buttonDelete.setEnabled(true);
		}

		else {
//			_buttonMoveUp.setEnabled(false);
//			_buttonMoveDown.setEnabled(false);
			_buttonDelete.setEnabled(false);
		}
	}

	// No parent element
	//------------------
	else {
		enableAllButtons(false);
	}
}
/**
 */
public void widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent e) {}
/**
 */
public void widgetSelected(final SelectionEvent sEvent) {

	BusyIndicator.showWhile( _editor.getDisplay(), new Runnable() {
		public void run() {

			// Add as child/sibling
			//---------------------
			if (sEvent.widget == _radioAddAsChild || sEvent.widget == _radioAddAsSibling) {
				updateButtons();
			}

			// Delete
			//-------
			else if (sEvent.widget == _buttonDelete) {
				_editor.doElementDelete(_elementSelected);
			}

			// Move higher
			//------------
//			else if (sEvent.widget == _buttonMoveUp) {
//				_editor.doElementMoveUp(_elementSelected);
//			}

			// Move lower
			//-----------
//			else if (sEvent.widget == _buttonMoveDown) {
//				_editor.doElementMoveDown(_elementSelected);
//			}

			// Creation of new element
			//------------------------
			else if (sEvent.widget instanceof ToolItem) {
				ToolItem toolItem = (ToolItem) sEvent.widget;

				ButtonData data = (ButtonData) toolItem.getData();

				if (data != null) {
					ScriptUIEditorModelElement elementParent = _radioAddAsChild.getSelection() ? _elementSelected : _elementSelected._elementParent;
					int iIndex = _radioAddAsChild.getSelection() ? 0 : elementParent.getChildElementIndex( _elementSelected ) + 1;
					_editor.doElementCreate(elementParent, data._iType, data._strName, iIndex );
				}
			}

		}
	});
}
}
