package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.util.Iterator;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.views.contentoutline.ContentOutlinePage;

public class ScriptUIEditorContentOutlinePage extends ContentOutlinePage {

	protected ScriptUIEditorModelElement     _elementRoot = null;
	protected ScriptUIEditor                 _textEditor  = null;
	protected ScriptUIEditorDocumentProvider _documentProvider = null;

/**
 */
public ScriptUIEditorContentOutlinePage(Object input, ScriptUIEditorDocumentProvider documentProvider, ScriptUIEditor textEditor) {
	super();

	_textEditor = textEditor;
	_documentProvider = documentProvider;
	
	if( input instanceof ScriptUIEditorModelElement )
	{
		_elementRoot = (ScriptUIEditorModelElement)input;
	}
}
/**
 */
public void createControl(Composite parent) {
	super.createControl(parent);

	// Create a tree viewer for drag and drop
	//---------------------------------------
	//	treeViewer = new ScriptUIEditorTreeViewer(new Tree(parent, SWT.BORDER + SWT.MULTI), this);

	TreeViewer treeViewer = getTreeViewer();

	// Add drag/drop support for the tree viewer
	//------------------------------------------
	int ops = DND.DROP_MOVE;

	Transfer[] transfers = new Transfer[] { ScriptUIEditorIntegerTransfer.getInstance()};

	treeViewer.addDragSupport(ops, transfers, new ScriptUIEditorTreeViewerDragAdapter(treeViewer));
	treeViewer.addDropSupport(ops, transfers, new ScriptUIEditorTreeViewerDropAdapter(treeViewer, this));

	// Add tree viewer listeners
	//--------------------------
	treeViewer.addSelectionChangedListener(this);

	// Create content providers for the tree viewer
	//---------------------------------------------
	treeViewer.setContentProvider(new ScriptUIEditorTreeContentProvider());
	treeViewer.setLabelProvider(new ScriptUIEditorLabelProvider());

	// Set initial expansion level
	//----------------------------
	treeViewer.setAutoExpandLevel(4);

	// Give the root element to the tree viewer
	//-----------------------------------------
	treeViewer.setInput(_elementRoot);

}
/**
 * Removes the element from the tree
 */
public void doElementDelete(ScriptUIEditorModelElement element) {

	if (getTreeViewer().getControl() != null && getTreeViewer().getControl().isDisposed() == false) {
		getTreeViewer().remove(element);
	}
}
/**
 * Overrides the super class because it doesn't check for widgetDisposed.
 */
public ISelection getSelection() {
	if (getTreeViewer().getTree() != null && getTreeViewer().getTree().isDisposed() == false) {
		return getTreeViewer().getSelection();
	}
	
	return null;
}
/**
 */
public void performDrop(int iHashSource, ScriptUIEditorModelElement elementTarget, boolean bAddAsChild, boolean bDropAfter )
{
	ScriptUIEditorModelElement elementSource = _elementRoot.findElementFromHashValue( iHashSource );

	if( elementSource != null )
	{
		_textEditor.doElementMove( elementSource, elementTarget, bAddAsChild, bDropAfter );
	}
}
/**
 */
public void selectionChanged(SelectionChangedEvent event) {
	super.selectionChanged(event);

	ISelection selection = event.getSelection();

	// Update text editor
	//-------------------
	if (_textEditor != null) {
		if (selection.isEmpty() == false) {
			Iterator iter = ((IStructuredSelection) selection).iterator();

			Object objIter = iter.next();

			if (objIter instanceof ScriptUIEditorModelElement) {
				ScriptUIEditorModelElement element = (ScriptUIEditorModelElement) objIter;
				_textEditor.selectionChanged(element, true);
			}
		}

		// Nothing selected
		//-----------------
		else {
			_textEditor.selectionChanged(null, true);
		}
	}
}
/**
 */
public void setEnabled(boolean bEnable) {
	
	if (getTreeViewer() != null) {
		Control control = getTreeViewer().getControl();

		if (control.isDisposed() == false) {
			control.setEnabled(bEnable);
		}
	}
}
/**
 * Sets the input of the outline page
 */
public void setInput(Object input) {
	if (input instanceof ScriptUIEditorModelElement) {
		_elementRoot = (ScriptUIEditorModelElement) input;
	}

	update();
}
/**
 * Overrides the super class because it doesn't check for widgetDisposed.
 */
public void setSelection(ISelection selection) {

	if (getTreeViewer().getTree() != null && getTreeViewer().getTree().isDisposed() == false) {
		getTreeViewer().setSelection(selection);
	}
}
/**
 * Updates the outline page.
 */
public void update() {
	if (getTreeViewer() != null) {
		Control control = getTreeViewer().getControl();

		if (control.isDisposed() == false) {
			control.setRedraw(false);
			getTreeViewer().setInput(_elementRoot);
			control.setRedraw(true);
		}
	}
}
}
