package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.util.Enumeration;
import java.util.Hashtable;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.scripting.internal.ScriptPlugin;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.texteditor.BasicTextEditorActionContributor;

/**
 */
public class ScriptUIEditorContextContributor extends BasicTextEditorActionContributor 
{
//	private ScriptUIEditorContent _content             = null;
	private EditorAction          _actionChildOrSibling = null;
	private Hashtable             _hashIDToAction      = new Hashtable();
	private boolean               _bAddChildNotSibling = true;
	protected int                     _iSelectedPresentationType = ScriptUIEditor.UNKNOWN;
	protected int                     _iSelectedParentPresentationType = ScriptUIEditor.UNKNOWN;
	protected int                     _iSelectedChildren = 0;
	protected int                     _iSelectedParentChildren = 0;
	protected IStatusLineManager    _statusLineManager = null;
	
	private static ScriptUIEditorContextContributor _instance = null;
	
	class EditorAction extends Action
	{
		private int _iXgrAction;
		
		public EditorAction( String strLabel, int iXgrAction )
		{
			super( strLabel );
			_iXgrAction = iXgrAction;
		}
		
		public void run()
		{
//			Trace.object( "content", _content );
			
			if( _iXgrAction == ScriptUIEditor.ADDASCHILD )
			{
				_bAddChildNotSibling = isChecked();
				updateActions();
			}
			
			else
			{
//				_content.itemAdd( _iXgrAction, _bAddChildNotSibling );
			}
		}
	}
/**
 */
public ScriptUIEditorContextContributor()
{
	if( _instance == null )
	{
		_instance = this;
	}
	
	// Create "Child or sibling" toggle button
	// By setting a boolean value, this value will be
	// automatically toggled when pressed.  A call
	// to getValue returns the current state.
	//-----------------------------------------------
	_actionChildOrSibling = new EditorAction( "Add as Child or Sibling", ScriptUIEditor.ADDASCHILD );
	_actionChildOrSibling.setToolTipText( "Add as Child or Sibling" );
	ImageDescriptor imageDescriptor = ScriptPlugin.getImageDescriptor( "icons/full/ctool16/addaschild.gif" );
	_actionChildOrSibling.setImageDescriptor( imageDescriptor );
	_actionChildOrSibling.setChecked( true );

	// Create non-toggle buttons
	//--------------------------	
	createAction( ScriptUIEditor.BREAK    , "Break"          , "break.gif" );
	createAction( ScriptUIEditor.BUTTON   , "Button"         , "button.gif" );
	createAction( ScriptUIEditor.CHECK    , "Check"          , "checkbox.gif" );
	createAction( ScriptUIEditor.COMBO    , "Combo"          , "combobox.gif" );
	createAction( ScriptUIEditor.COMMENT  , "Comment"        , "comment.gif" );
	createAction( ScriptUIEditor.FIELDSET , "Fieldset"       , "groupbox.gif" );
	createAction( ScriptUIEditor.FORM     , "Form"           , "form.gif" );
	createAction( ScriptUIEditor.HORZRULE , "Horizontal Rule", "horzrule.gif" );
	createAction( ScriptUIEditor.IMG      , "Image"          , "image.gif" );
	createAction( ScriptUIEditor.LABEL    , "Label"          , "label.gif" );
	createAction( ScriptUIEditor.LIST     , "List"           , "listbox.gif" );
	createAction( ScriptUIEditor.META     , "Meta"           , "meta.gif" );					
	createAction( ScriptUIEditor.RADIO    , "Radio"          , "radiobutton.gif" );
	createAction( ScriptUIEditor.SCRIPT   , "Script"         , "scriptcatagory.gif" );
	createAction( ScriptUIEditor.TABLE    , "Table"          , "table.gif" );
	createAction( ScriptUIEditor.TABLEDATA, "TableData"      , "tabledata.gif" );
	createAction( ScriptUIEditor.TABLEROW , "TableRow"       , "tablerow.gif" );
	createAction( ScriptUIEditor.TEXTAREA , "TextArea"       , "textarea.gif" );
	createAction( ScriptUIEditor.TEXTFIELD, "TextField"      , "textfield.gif" );
}
/**
 * Contributes menus and actions into the provided menu manager.
 */
public void contributeToMenu(IMenuManager menuManager) {
	
	// Editor-specific menu
	//---------------------
//	MenuManager mmScriptEditor = new MenuManager("&Script UI Editor");
	
//	mmScriptEditor.appendToGroup("group1", "action1", action1);
//	mmScriptEditor.appendToGroup("group1", "action2", action2);
//	mmScriptEditor.appendToGroup("group1", "action3", action3);

	
	// It is important to append the menu to the
	// group "additions". This group is created
	// between "Project" and "Tools" menus
	// for this purpose.
	//------------------------------------------
//	menuManager.insertAfter("additions", mmScriptEditor);

	
/*	
	// Editor-specitic menu
	MenuManager readmeMenu = new MenuManager("&Readme");
	// It is important to append the menu to the
	// group "additions". This group is created
	// between "Project" and "Tools" menus
	// for this purpose.
	mbm.insertAfter("additions", readmeMenu);
	readmeMenu.add(action1);
	readmeMenu.add(action2);
	readmeMenu.add(action3);
*/
}
/**
 * Adds sample actions desktop status line.
 */
public void contributeToStatusLine(IStatusLineManager slm) {
	_statusLineManager = slm;
}
/**
 * Adds the sample actions to the tool bar.
 */
public void contributeToToolBar( IToolBarManager tbm )
{
//	tbm.addSeparator( "ScriptUIEditor" );
/*
	// Toggle button
	//--------------
	tbm.add(new Separator("ScriptUIEditor"));
	tbm.add( _actionChildOrSibling );
	
	// Non-toggle buttons
	//-------------------
	EditorAction action = null;
	
	Enumeration enumActions = _hashIDToAction.elements();

	while( enumActions.hasMoreElements() == true )
	{
		action = (EditorAction)enumActions.nextElement();
		tbm.add( action );
	}
*/
/*
	tbm.add(new Separator("ReadmeEditor"));
	tbm.add(action1);
	tbm.add(action2);
	tbm.add(action3);
*/
}
/**
 */
private EditorAction createAction( int iId, String strLabel, String strImage )
{
	EditorAction action = new EditorAction( strLabel, iId );
	action.setToolTipText( strLabel );

	ImageDescriptor imageDescriptor = ScriptPlugin.getImageDescriptor( "icons/full/ctool16/" + strImage );
	
	action.setImageDescriptor( imageDescriptor );

	action.setEnabled( false );
	_hashIDToAction.put( new Integer( iId ), action );

	return action;
}
/**
 * Does anything necessary as a result of the editor changing
 */
public void editorChanged(IEditorPart editor)
{
//	Trace.functionEntry( this, "editorChanged" );
//	Trace.functionExit ( this, "editorChanged" );
}
/**
 */
private void enableAction( int iType, boolean bEnable )
{
	Integer intType = new Integer( iType );

	EditorAction action = (EditorAction)_hashIDToAction.get( intType );

	if( action.isEnabled() != bEnable )
	{
		action.setEnabled( bEnable );
	}
}
/**
 */
private void enableAllActions( boolean bEnable )
{
	EditorAction action = null;
	Enumeration  enum   = _hashIDToAction.elements();
	
	while( enum.hasMoreElements() == true )
	{
		action = (EditorAction)enum.nextElement();

		if( action.isEnabled() != bEnable )
		{
			action.setEnabled( bEnable );
		}
	}
}
/**
 */
public static ScriptUIEditorContextContributor getInstance()
{
	return _instance;
}
/**
 */
public boolean isAddAsSibling()
{
	return false;
}
/**
 */
public void setErrorMessage(String strMessage) {
	
	// Set the message text
	//---------------------
	if (_statusLineManager != null) {
		_statusLineManager.setErrorMessage(strMessage);
	}
}
/**
 */
private void updateActions()
{
	int iType = _bAddChildNotSibling == true ? 
				_iSelectedPresentationType : _iSelectedParentPresentationType;

	int iNumberOfChildren = _bAddChildNotSibling == true ? 
				_iSelectedChildren : _iSelectedParentChildren;
	
	switch( iType )
	{
		case ScriptUIEditor.BODY:
			enableAllActions( false );
			enableAction( ScriptUIEditor.FORM,   true );
			enableAction( ScriptUIEditor.SCRIPT, true );
			break;

		case ScriptUIEditor.FORM:
		case ScriptUIEditor.FIELDSET:
			enableAllActions( true );
			enableAction( ScriptUIEditor.META,      false );
			enableAction( ScriptUIEditor.FORM,      false );
			enableAction( ScriptUIEditor.SCRIPT,    false );
			enableAction( ScriptUIEditor.TABLEDATA, false );
			enableAction( ScriptUIEditor.TABLEROW,  false );
			break;
			
		case ScriptUIEditor.HEAD:
			enableAllActions( false );
			enableAction( ScriptUIEditor.META,   true );
			enableAction( ScriptUIEditor.SCRIPT, true );
			break;
			
		case ScriptUIEditor.TABLE:
			enableAllActions( false );
			enableAction( ScriptUIEditor.TABLEROW, true );
			break;
			
		case ScriptUIEditor.TABLEROW:
			enableAllActions( false );
			enableAction( ScriptUIEditor.TABLEDATA, true );
			break;
			
		case ScriptUIEditor.TABLEDATA:
			if( iNumberOfChildren == 0 )
			{
				enableAllActions( true );
				enableAction( ScriptUIEditor.FORM,      false );
				enableAction( ScriptUIEditor.META,      false );
				enableAction( ScriptUIEditor.SCRIPT,    false );
				enableAction( ScriptUIEditor.TABLE,     false );
				enableAction( ScriptUIEditor.TABLEDATA, false );
				enableAction( ScriptUIEditor.TABLEROW,  false );
			}
			else
			{
				enableAllActions( false );
			}
			break;		
			
		default:
			enableAllActions( false );
			break;
	}
}
/**
 */
public void updateActions( int iType, int iNumberOfChildren, int iParentType, int iNumberOfParentChildren )
{
	_iSelectedPresentationType       = iType;
	_iSelectedParentPresentationType = iParentType;
	_iSelectedChildren               = iNumberOfChildren;
	_iSelectedParentChildren         = iNumberOfParentChildren;
	
	updateActions();
}
}
