package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;

public class ScriptUIEditorDesignerCanvas extends Canvas implements PaintListener {
	protected ScriptUIEditorDesigner  _designer = null;
/**
 */
public ScriptUIEditorDesignerCanvas(Composite compositeParent, ScriptUIEditorDesigner designer) {
	super(compositeParent, SWT.NULL );

	// Layout
	//-------
	GridLayout layout = new GridLayout();
	layout.numColumns   = 1;
	layout.verticalSpacing = 0;
	layout.marginHeight = 0; 
	layout.marginWidth  = 0;
	setLayout( layout );
	
	_designer = designer;

	GridData gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.verticalAlignment   = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.grabExcessVerticalSpace   = true;
	setLayoutData( gridData );
	setLayoutData( gridData );

	// Add listeners
	//--------------
	addPaintListener( this );

	if( designer._editor._documentProvider != null && designer._editor._documentProvider.getElementRoot() != null )
	{
		updateLayout();
	}
}
/**
 * Gets the widget bounds.
 */
public Rectangle getBounds () {

	if( _designer._editor._documentProvider != null )
	{
		ScriptUIEditorModelElement elementRoot = _designer._editor._documentProvider.getElementRoot();
		Point ptSize = elementRoot.calculateBounds( new Point( 0, 0 ) );
		return new Rectangle (0, 0, ptSize.x, ptSize.y);
	}

	return new Rectangle(0, 0, 1, 1);

}
/**
 */
public void paintControl(PaintEvent event) 
{
	if( _designer._editor._documentProvider != null )
	{
		event.gc.setForeground( getDisplay().getSystemColor( SWT.COLOR_BLACK ) );


		ScriptUIEditorModelElement elementRoot = _designer._editor._documentProvider.getElementRoot();
		Point ptSize = elementRoot.calculateBounds( new Point( 0, 0 ) );
	
		event.gc.drawRectangle( 0, 0, ptSize.x - 1, ptSize.y - 1 );

		elementRoot.paintControl( event.gc );
	}
}
/**
 */
public void updateLayout() {

	if( _designer._editor._documentProvider != null )
	{
		ScriptUIEditorModelElement elementRoot = _designer._editor._documentProvider.getElementRoot();
	
		Point ptSize = elementRoot.calculateBounds( new Point( 0, 0 ) );
	
		setBounds( 0, 0, ptSize.x, ptSize.y );

		redraw();
	}
}
}
