package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */



import java.util.ArrayList;
import java.util.List;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.TextPresentation;
import org.eclipse.jface.text.TypedRegion;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.IWordDetector;
import org.eclipse.jface.text.rules.RuleBasedDamagerRepairer;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.text.rules.SingleLineRule;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.rules.WhitespaceRule;
import org.eclipse.jface.text.rules.WordRule;

/**
 * A rule based JavaDoc scanner.
 */
public class ScriptUIEditorDocScanner extends RuleBasedScanner {
		
		
	/**
	 * A key word detector.
	 */
	static class ScriptUIEditorDocWordDetector implements IWordDetector {

		/**
		 * @see IWordDetector#isWordStart
		 */
		public boolean isWordStart(char c) {
			return (c == '@');
		}

		/**
		 * @see IWordDetector#isWordPart
		 */
		public boolean isWordPart(char c) {
			return Character.isLetter(c);
		}
	};
	
	
	private static String[] fgKeywords= {"@author", "@deprecated", "@exception", "@param", "@return", "@see", "@serial", "@serialData", "@serialField", "@since", "@throws", "@version"};
	
	
	public ScriptUIEditorDocScanner(ScriptUIEditorColorProvider provider) {
		super();
		
		IToken keyword= new Token(new TextAttribute(provider.getColor(ScriptUIEditorColorProvider.JAVADOC_KEYWORD)));
		IToken tag= new Token(new TextAttribute(provider.getColor(ScriptUIEditorColorProvider.JAVADOC_TAG)));
		IToken link= new Token(new TextAttribute(provider.getColor(ScriptUIEditorColorProvider.JAVADOC_LINK)));
		
		
		List list= new ArrayList();
		
		// Add rule for tags.
		list.add(new SingleLineRule("<", ">", tag));
		
		// Add rule for links.
		list.add(new SingleLineRule("{", "}", link));
		
		// Add generic whitespace rule.
		list.add(new WhitespaceRule(new ScriptUIEditorWhitespaceDetector()));
		
		// Add word rule for keywords.
		WordRule wordRule= new WordRule(new ScriptUIEditorDocWordDetector());
		for (int i= 0; i < fgKeywords.length; i++)
			wordRule.addWord(fgKeywords[i], keyword);
		list.add(wordRule);
		
		IRule[] result= new IRule[list.size()];
		list.toArray(result);
		setRules(result);
	}
}
