package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import org.eclipse.swt.dnd.ByteArrayTransfer;
import org.eclipse.swt.dnd.TransferData;

/**
 * This class transfers integer-specified data (an instance
 * of ScriptUIEditorIntegerTransferData) within a tree viewer.
 * The transfer data contains the hash value of the source 
 * ScriptUIEditorModelElement that is to be moved to another part of
 * the tree viewer.
 *
 */
public class ScriptUIEditorIntegerTransfer extends ByteArrayTransfer {
	public static final String TYPE_NAME = "integer-transfer-format";
	protected static final int TYPEID = registerType(TYPE_NAME);
	private static ScriptUIEditorIntegerTransfer _instance = new ScriptUIEditorIntegerTransfer();
/**
 */
public ScriptUIEditorIntegerTransfer() {
	super();
}
/**
 * Returns the singleton instance of the TextTransfer class.
 */
public static ScriptUIEditorIntegerTransfer getInstance () {
	return _instance;
}
/**
 */
protected int[] getTypeIds() {
	return new int[] {TYPEID};
}
public String[] getTypeNames() {
	return new String[] {TYPE_NAME};
}
/**
 * Converts the given string and byte array to a native
 * representation.
 */
protected void javaToNative (Object data, TransferData transferData){
	Integer intData = (Integer)data;
	if (data == null) return;
	try {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		DataOutputStream dataOut = new DataOutputStream(out);
		dataOut.writeInt(intData.intValue());
		dataOut.close();
		super.javaToNative(out.toByteArray(), transferData);
	} catch (IOException e) {
		e.printStackTrace();
	}	
}
/**
 *
 */
protected Object nativeToJava(TransferData transferData) {
	try {
		byte[] bytes = (byte[]) super.nativeToJava(transferData);
		ByteArrayInputStream in = new ByteArrayInputStream(bytes);
		DataInputStream dataIn = new DataInputStream(in);
		int iValue = dataIn.readInt();
		return new Integer(iValue);
	} catch (IOException e) {
		e.printStackTrace();
	}
	//can't get here
	return null;
}
}
