package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.scripting.internal.ScriptStrings;
import org.eclipse.scripting.internal.views.ScriptUIEditorModelElement.IntBuffer;

public class ScriptUIEditorModel extends ScriptUIEditorModelElement
{
	protected ScriptUIEditorModelElement     _elementCurrent = null;
	protected int                            _iLine                = -1; // 0 based
	protected int                            _iColumn              = -1; // 0 based
	protected ScriptUIEditorDocumentProvider _documentProvider     = null;
/**
 */
public ScriptUIEditorModel( ScriptUIEditorDocumentProvider documentProvider ) 
{
	super( null, null, "root", -1, -1, -1 );
	_model = this;
	_documentProvider = documentProvider;
}
/**
 */
private static void addAttributeToElement( ScriptUIEditorModelElement  element,
	                               String                       strElementAttributeName,
	                               ScriptUIEditorXmlElement     elementXml,
	                               String                       strXmlAttributeName,
	                               String                       strDefaultValue           )
{
	ScriptUIEditorModelAttribute attribute = null;
	ScriptUIEditorXmlAttribute attributeXml = null;
	
	if( elementXml != null )
	{
		attributeXml = elementXml.getAttribute( strXmlAttributeName );
	}
		
	if( attributeXml != null )
	{
		attribute = new ScriptUIEditorModelAttribute( strElementAttributeName,
													   attributeXml._strValue != null ? attributeXml._strValue.trim() : strDefaultValue,
													   attributeXml._strFilename,
													   attributeXml._iLineNumber,
													   attributeXml._iColumnNumber );
	}
	
	else
	{
		attribute = new ScriptUIEditorModelAttribute( strElementAttributeName,
													   strDefaultValue );
	}
	
	element.addAttribute( attribute );
	
	return;	
}
/**
 */
public static ScriptUIEditorModelElement createChildElement(int iType, String strName, ScriptUIEditorModelElement elementParent, int iLocation) {
	
	ScriptUIEditorModelElement element = null;
	
	if( elementParent != null )
	{
		element = new ScriptUIEditorModelElement( elementParent._model, elementParent, strName, iType, -1, -1 );

		elementParent.addChildElement( element, iLocation );
		
		switch( iType )
		{
			case ScriptUIEditor.BREAK:			
				break;

			case ScriptUIEditor.BUTTON:
				addAttributeToElement( element, "id", null, "id", "" );
				addAttributeToElement( element, "onclick", null, "onclick", "" );
				addAttributeToElement( element, "value", null, "value", "" );
				break;
		
			case ScriptUIEditor.CHECK:
				addAttributeToElement( element, "checked", null, "checked", "" );
				addAttributeToElement( element, "id", null, "id", "" );
				addAttributeToElement( element, "onclick", null, "onclick", "" );
				addAttributeToElement( element, "value", null, "value", "" );
				break;
			
			case ScriptUIEditor.COMBO:
				addAttributeToElement( element, "id", null, "id", "" );
				addAttributeToElement( element, "onchange", null, "onchange", "" );		break;

			case ScriptUIEditor.COMMENT:
				addAttributeToElement( element, "#text", null, "", "" );
			case ScriptUIEditor.FIELDSET:
				addAttributeToElement( element, "id", null, "id", "" );
				addAttributeToElement( element, "title", null, "title", "" );
				break;

			case ScriptUIEditor.FORM:
				addAttributeToElement( element, "id", null, "id", "" );
				addAttributeToElement( element, "title", null, "title", "" );
				addAttributeToElement( element, "onreset"  , null, "onreset" , "" );
				addAttributeToElement( element, "onsubmit" , null, "onsubmit", "" );
				break;

			case ScriptUIEditor.HORZRULE:
				break;

			case ScriptUIEditor.IMG:
				addAttributeToElement( element, "id", null, "id", "" );
				addAttributeToElement( element, "src", null, "src", "" );
				break;

			case ScriptUIEditor.LABEL:
				addAttributeToElement( element, "id", null, "id", "" );
				addAttributeToElement( element, "#text", null, "text", "" );
				break;
			
			case ScriptUIEditor.LIST:
				addAttributeToElement( element, "id", null, "id", "" );
				addAttributeToElement( element, "onchange", null, "onchange", "" );
				break;

			case ScriptUIEditor.RADIO:
				addAttributeToElement( element, "checked", null, "checked", "" );
				addAttributeToElement( element, "id", null, "id", "" );
				addAttributeToElement( element, "onclick", null, "onclick", "" );
				addAttributeToElement( element, "value", null, "value", "" );
				break;
				
			case ScriptUIEditor.META:
				addAttributeToElement( element, "http-equiv", null, "http-equiv", "Content-Script-Type" );
				addAttributeToElement( element, "content"   , null, "content"   , "text/javascript"     );
				break;

			case ScriptUIEditor.SCRIPT:
				addAttributeToElement( element, "#text", null, "text", "" );
				addAttributeToElement( element, "type", null, "type", "" );
				break;

			case ScriptUIEditor.TABLE:
				addAttributeToElement( element, "id", null, "id", "" );
				break;
			
			case ScriptUIEditor.TABLEDATA:
				addAttributeToElement( element, "colspan", null, "colspan", "1" );
				addAttributeToElement( element, "rowspan", null, "rowspan", "1" );
				break;
			
			case ScriptUIEditor.TABLEROW:
				break;

			case ScriptUIEditor.TEXTAREA:
				addAttributeToElement( element, "id", null, "id", "" );
				addAttributeToElement( element, "onblur"    , null, "onblur"    , "" );
				addAttributeToElement( element, "onchange"  , null, "onchange"  , "" );
				addAttributeToElement( element, "onfocus"   , null, "onfocus"   , "" );
				addAttributeToElement( element, "onkeypress", null, "onkeypress", "" );
				break;

			case ScriptUIEditor.TEXTFIELD:
				addAttributeToElement( element, "id", null, "id", "" );
				addAttributeToElement( element, "onblur"    , null, "onblur"    , "" );
				addAttributeToElement( element, "onchange"  , null, "onchange"  , "" );
				addAttributeToElement( element, "onfocus"   , null, "onfocus"   , "" );
				addAttributeToElement( element, "onkeypress", null, "onkeypress", "" );
				addAttributeToElement( element, "value", null, "value", "" );
				break;
		}
	}

	return element;
}
/**
 */
public static void deleteElement( ScriptUIEditorModelElement element ) {

	if (element != null) {
		ScriptUIEditorModelElement elementParent = element.getParent();

		if (elementParent != null) {
			elementParent.removeChildElement( element );
		}
	}
}
/**
 * Called by one of the elements to say that one of its attributes has changed.
 * Notifies the document provider which will in turn notify the editor, which
 * will update the various views.
 */
public void elementAttributeChanged(ScriptUIEditorModelElement element) {

	if( _documentProvider != null )
	{
		_documentProvider.elementAttributeChanged( element );
	}
}
/**
 * This overrides the default behaviour since this is the root element
 * that really is not a part of the element tree.
 */
public void generateLineNumbers() {

	// Since this is the dummy root element, tell
	// my children (only one) to do the work
	//-------------------------------------------
	if (_vectorChildElements != null && _vectorChildElements.size() > 0) {

		for (int i = 0; i < _vectorChildElements.size(); ++i) {
			((ScriptUIEditorModelElement) _vectorChildElements.elementAt(i)).generateLineNumbers(new IntBuffer());
		}

	}

}
/**
 */
public ScriptUIEditorModelElement getRootElement()
{
	return this;
}
/**
 * Determines whether a move of the source element to either before or after the target element
 * is valid.
 */
public static boolean isMoveValid(ScriptUIEditorModelElement elementSource, ScriptUIEditorModelElement elementTarget, boolean bAddAsChild, boolean bMoveAfter) {

	if (elementSource == null || elementTarget == null) {
		return false;
	}

	// Disable moves of html, head, body
	//----------------------------------
	int iSourceType = elementSource._iType;
	
	if (iSourceType == ScriptUIEditor.HTML || iSourceType == ScriptUIEditor.HEAD || iSourceType == ScriptUIEditor.BODY) {
		return false;
	}

	// Ensure target is not one of the source's children
	//--------------------------------------------------
	if (elementSource.hasDescendent(elementTarget) == true) {
		return false;
	}

	// If the parents are the same, then it's ok
	//------------------------------------------
	ScriptUIEditorModelElement elementParentSource = elementSource.getParent();
	ScriptUIEditorModelElement elementParentTarget = bAddAsChild ? elementTarget : elementTarget.getParent();

	if (elementParentSource == elementParentTarget) {
		return true;
	}

	// See if target parent can contain the source type
	//-------------------------------------------------
	switch (elementParentTarget._iType) {
		
		case ScriptUIEditor.HTML :
			return iSourceType == ScriptUIEditor.COMMENT ? true : false;

		case ScriptUIEditor.BODY :
			switch (iSourceType) {
				case ScriptUIEditor.COMMENT :
				case ScriptUIEditor.FORM :
				case ScriptUIEditor.SCRIPT :
					return true;
				default :
					return false;
			}

		case ScriptUIEditor.FORM :
		case ScriptUIEditor.FIELDSET :
			switch (iSourceType) {
				case ScriptUIEditor.META :
				case ScriptUIEditor.FORM :
				case ScriptUIEditor.SCRIPT :
				case ScriptUIEditor.TABLEDATA :
				case ScriptUIEditor.TABLEROW :
					return false;
				default :
					return true;
			}

		case ScriptUIEditor.HEAD :
			switch (iSourceType) {
				case ScriptUIEditor.COMMENT :
				case ScriptUIEditor.META :
				case ScriptUIEditor.SCRIPT :
					return true;
				default :
					return false;
			}

		case ScriptUIEditor.TABLE :
			switch (iSourceType) {
				case ScriptUIEditor.COMMENT :
				case ScriptUIEditor.TABLEROW :
					return true;
				default :
					return false;
			}

		case ScriptUIEditor.TABLEROW :
			switch (iSourceType) {

				case ScriptUIEditor.COMMENT :
				case ScriptUIEditor.TABLEDATA :
					return true;
				default :
					return false;
			}

		case ScriptUIEditor.TABLEDATA :
			if (elementParentTarget.getNumberOfChildElements() == 0) {
				switch (iSourceType) {
					case ScriptUIEditor.FORM :
					case ScriptUIEditor.META :
					case ScriptUIEditor.SCRIPT :
					case ScriptUIEditor.TABLE :
					case ScriptUIEditor.TABLEDATA :
					case ScriptUIEditor.TABLEROW :
						return false;
					default :
						return true;
				}
			}
			else {
				return false;
			}

		default :
			return false;
	}
}
/**
 * Ensures that the base xhtml elements exist by creating missing elements
 */
protected void makeValid() {

	// Html
	//-----
	ScriptUIEditorModelElement elementHtml = this.getChildElement( "html" );
	if( elementHtml == null )
	{
		elementHtml = new ScriptUIEditorModelElement( this, this, "html", ScriptUIEditor.HTML, -1, -1 );
		this.addChildElement( elementHtml );
	}

	// Html/Head
	//----------
	ScriptUIEditorModelElement elementHead = elementHtml.getChildElement( "head" );
	if( elementHead == null )
	{
		elementHead = new ScriptUIEditorModelElement( this, elementHtml, "head", ScriptUIEditor.HEAD, -1, -1 );
		elementHtml.addChildElement( elementHead, 0 );
	}

	// Html/Head/Script
	//-----------------
	ScriptUIEditorModelElement elementScript = elementHead.getChildElement( "script" );
	if( elementScript == null )
	{
		elementScript = new ScriptUIEditorModelElement( this, elementHead, "script", ScriptUIEditor.SCRIPT, -1, -1 );
		elementHead.addChildElement( elementScript, 0 );
		addAttributeToElement( elementScript, "#text", null, "text", "" );
		addAttributeToElement( elementScript, "type", null, "type", "" );
	}
	
	// Html/Body
	//----------
	ScriptUIEditorModelElement elementBody = elementHtml.getChildElement( "body" );
	if( elementBody == null )
	{
		elementBody = new ScriptUIEditorModelElement( this, elementHtml, "body", ScriptUIEditor.BODY, -1, -1 );
		elementHtml.addChildElement( elementBody );
		addAttributeToElement( elementBody, "id", null, "id", "" );
		addAttributeToElement( elementBody, "onload"    , null, "onload"    , "" );
		addAttributeToElement( elementBody, "onunload"  , null, "onunload"  , "" );
	}
	
	// Html/Body/Form
	//---------------
	ScriptUIEditorModelElement elementForm = elementBody.getChildElement( "form" );
	if( elementForm == null )
	{
		elementForm = new ScriptUIEditorModelElement( this, elementBody, "form", ScriptUIEditor.FORM, -1, -1 );
		elementBody.addChildElement( elementForm );
		addAttributeToElement( elementForm, "id", null, "id", "" );
		addAttributeToElement( elementForm, "title", null, "title", "" );
		addAttributeToElement( elementForm, "onreset"  , null, "onreset" , "" );
		addAttributeToElement( elementForm, "onsubmit" , null, "onsubmit", "" );

		// Add a label
		//------------
		ScriptUIEditorModelElement elementLabel = new ScriptUIEditorModelElement( this, elementForm, "label", ScriptUIEditor.LABEL, -1, -1 );
		elementForm.addChildElement( elementLabel );
		addAttributeToElement( elementLabel, "id", null, "id", "" );
		addAttributeToElement( elementLabel, "#text", null, "text", "label" );
	}
}
/**
 */
public static boolean moveElement(ScriptUIEditorModelElement elementSource, ScriptUIEditorModelElement elementTarget, boolean bAddAsChild, boolean bAfterTarget) {

	//	System.out.println("Model:moveElement " + elementSource.toString() + " " + elementTarget.toString() + " " + (bAfterTarget == true ? "after" : "before"));

	// Moving always creates a sibling of the target
	//----------------------------------------------
	if (elementSource != null && elementTarget != null && elementSource != elementTarget) {

		ScriptUIEditorModelElement elementParentSource = elementSource.getParent();
		ScriptUIEditorModelElement elementParentTarget = bAddAsChild ? elementTarget : elementTarget.getParent();

		if (elementParentSource != elementParentTarget) {

			// Add sibling to target parent
			//-----------------------------	
			int iIndex = elementParentTarget.getChildElementIndex(elementTarget);

			// Add as child
			//-------------
			if (bAddAsChild == true) {
				elementParentTarget.addChildElement(elementSource, 0);
			}
			else {
				elementParentTarget.addChildElement(elementSource, bAfterTarget == false ? iIndex : iIndex + 1);
			}

			// Remove from original location
			//------------------------------
			elementParentSource.removeChildElement(elementSource);

			// Reparent the source element
			//----------------------------
			elementSource._elementParent = elementParentTarget;
			return true;
		}

		else {
			int iIndexSource = elementParentSource.getChildElementIndex(elementSource);
			int iIndexTarget = elementParentTarget.getChildElementIndex(elementTarget);

			// Add as child
			//-------------
			if (bAddAsChild == true) {
   	            elementParentSource._vectorChildElements.removeElementAt(iIndexSource);
	            elementParentTarget.addChildElement(elementSource, 0);
			}

			else {
				// Move before
				//------------
				if (bAfterTarget == false) {
					if (iIndexSource < iIndexTarget) {
						elementParentTarget._vectorChildElements.add(iIndexTarget, elementSource);
						elementParentSource._vectorChildElements.removeElementAt(iIndexSource);
					}
					else {
						elementParentSource._vectorChildElements.removeElementAt(iIndexSource);
						elementParentTarget._vectorChildElements.add(iIndexTarget, elementSource);
					}
				}

				// Move after
				//-----------
				else {
					if (iIndexSource < iIndexTarget) {
						elementParentTarget._vectorChildElements.add(iIndexTarget + 1, elementSource);
						elementParentSource._vectorChildElements.removeElementAt(iIndexSource);
					}
					else {
						elementParentSource._vectorChildElements.removeElementAt(iIndexSource);
						elementParentTarget._vectorChildElements.add(iIndexTarget + 1, elementSource);
					}
				}
			}

			return true;
		}
	}
	return false;
}
/**
 */
public static boolean moveElementHigher( ScriptUIEditorModelElement element ) {

	boolean bMoved = false;

	// Moving higher means moving closer to zero
	//------------------------------------------	
	if (element != null) {
		ScriptUIEditorModelElement elementParent = element.getParent();

		if (elementParent != null && elementParent.getNumberOfChildElements() > 1 ) {
			int iIndex = elementParent._vectorChildElements.indexOf( element);
			if( iIndex > 0 )
			{
				elementParent._vectorChildElements.removeElementAt( iIndex );
				elementParent._vectorChildElements.add( iIndex - 1, element );
				bMoved = true;
			}
		}
	}

	return bMoved;
}
/**
 */
public static boolean moveElementLower( ScriptUIEditorModelElement element ) {

	boolean bMoved = false;

	// Moving lower means moving away from zero
	//-----------------------------------------	
	if (element != null) {
		ScriptUIEditorModelElement elementParent = element.getParent();

		if (elementParent != null && elementParent.getNumberOfChildElements() > 1 ) {
			int iIndex = elementParent._vectorChildElements.indexOf( element);
			if( iIndex < elementParent.getNumberOfChildElements() - 1 )
			{
				elementParent._vectorChildElements.removeElementAt( iIndex );
				elementParent._vectorChildElements.add( iIndex + 1, element );
				bMoved = true;
			}
		}
	}

	return bMoved;
}
/**
 */
public void parse( ScriptUIEditorXmlElement elementXmlFirst ) throws Exception
{
//	Trace.functionEntry( this, "parse" );

	removeAllChildElements();

	ScriptUIEditorModelException exception = null;
	
	if( elementXmlFirst._strName.equals( ScriptUIEditor.STRING_HTML ) == true )
	{
		ScriptUIEditorModelElement elementFirst = new ScriptUIEditorModelElement( this, null, "html", ScriptUIEditor.HTML, elementXmlFirst._iLineNumberFirst, elementXmlFirst._iLineNumberLast );

		int iNumberOfChildElements = elementXmlFirst.getNumberOfChildElements();

		for( int i=0; i<iNumberOfChildElements; ++i )
		{
			try
			{
				parseRecursive( this, elementFirst, elementXmlFirst.getChildElementAt( i ) );
			}
			catch( ScriptUIEditorModelException ex )
			{
				exception = ex;
			}
		}

		addChildElement( elementFirst );
		elementFirst._elementParent = this;
	}

	if( exception != null )
	{
		throw( exception );
	}
	
//	Trace.functionExit( this, "parse" );
}
/**
 * This method recursively creates presentation elements from xml elements.
 * If a presentation element is created, all of its child attributes are created as well.
 * Script text stores line/column number information in the presentation attribute.
 */
private void parseRecursive( ScriptUIEditorModel model, ScriptUIEditorModelElement elementParent, ScriptUIEditorXmlElement elementXml ) throws ScriptUIEditorModelException
{
//	Trace.functionEntry( this, "parseRecursive" );
//	Trace.string( elementXml._strName );
	
	ScriptUIEditorModelElement   element   = null;
	ScriptUIEditorModelAttribute attribute = null;
	ScriptUIEditorModelException exception = null;
	
	if( elementXml._strName.equals( ScriptUIEditor.STRING_POUNDCOMMENT ) == true )
	{
		// Script text
		//------------
		if( elementParent._iType == ScriptUIEditor.SCRIPT )
		{
			attribute = elementParent.getAttribute( ScriptUIEditor.STRING_TEXT );
			if( attribute != null )
			{
				ScriptUIEditorXmlAttribute attributeXml = elementXml.getAttribute( ScriptUIEditor.STRING_TEXT );

				if( attributeXml != null )
				{
					attribute._strName          = ScriptUIEditor.STRING_POUNDTEXT;
					attribute._strValue         = attributeXml._strValue;
					attribute._strFilename      = attributeXml._strFilename;
					attribute._iLineNumber      = attributeXml._iLineNumber;
					attribute._iColumnNumber    = attributeXml._iColumnNumber;
				}
			}
		}

		// Comment element
		//----------------
		else
		{
			element = new ScriptUIEditorModelElement( model, elementParent, ScriptStrings.getString("S_comment"), ScriptUIEditor.COMMENT, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
			addAttributeToElement( element, ScriptUIEditor.STRING_POUNDTEXT, elementXml, ScriptUIEditor.STRING_TEXT, ScriptUIEditor.STRING_EMPTY );
		}
	}

	// Script or label text
	//---------------------	
	else if( elementXml._strName.equals( ScriptUIEditor.STRING_POUNDTEXT ) == true )
	{
		if( elementParent._iType == ScriptUIEditor.LABEL  ||
			elementParent._iType == ScriptUIEditor.SCRIPT    )
		{
			attribute = elementParent.getAttribute( ScriptUIEditor.STRING_POUNDTEXT );

			if( attribute != null )
			{
				ScriptUIEditorXmlAttribute attributeXml = elementXml.getAttribute( ScriptUIEditor.STRING_TEXT );

				if( attributeXml != null )
				{
					attribute._strName       = ScriptUIEditor.STRING_POUNDTEXT;
					attribute._strValue      = attributeXml._strValue.trim();
					attribute._strFilename   = attributeXml._strFilename;
					attribute._iLineNumber   = attributeXml._iLineNumber;
					attribute._iColumnNumber = attributeXml._iColumnNumber;
				}
			}
		}

		else
		{
			element = new ScriptUIEditorModelElement( model, elementParent, ScriptUIEditor.STRING_LABEL, ScriptUIEditor.LABEL, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
			addAttributeToElement( element, ScriptUIEditor.STRING_ID,        elementXml, ScriptUIEditor.STRING_ID,   ScriptUIEditor.STRING_EMPTY);
			addAttributeToElement( element, ScriptUIEditor.STRING_POUNDTEXT, elementXml, ScriptUIEditor.STRING_TEXT, ScriptUIEditor.STRING_EMPTY );
		}
	}
		
	else if( elementXml._strName.equals( ScriptUIEditor.STRING_BODY ) == true )
	{
		element = new ScriptUIEditorModelElement( model, elementParent, elementXml._strName, ScriptUIEditor.BODY, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
		addAttributeToElement( element, ScriptUIEditor.STRING_ID,       elementXml, ScriptUIEditor.STRING_ID,       ScriptUIEditor.STRING_EMPTY );
		addAttributeToElement( element, ScriptUIEditor.STRING_ONLOAD,   elementXml, ScriptUIEditor.STRING_ONLOAD,   ScriptUIEditor.STRING_EMPTY );
		addAttributeToElement( element, ScriptUIEditor.STRING_ONUNLOAD, elementXml, ScriptUIEditor.STRING_ONUNLOAD, ScriptUIEditor.STRING_EMPTY );
	}
	
	else if( elementXml._strName.equals( ScriptUIEditor.STRING_BR ) == true )
	{
		element = new ScriptUIEditorModelElement( model, elementParent, elementXml._strName, ScriptUIEditor.BREAK, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
	}
	
	else if( elementXml._strName.equals( ScriptUIEditor.STRING_BUTTON ) == true )
	{
		element = new ScriptUIEditorModelElement( model, elementParent, ScriptUIEditor.STRING_INPUT, ScriptUIEditor.BUTTON, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
		addAttributeToElement( element, ScriptUIEditor.STRING_ID,      elementXml, ScriptUIEditor.STRING_ID,      ScriptUIEditor.STRING_EMPTY );
		addAttributeToElement( element, ScriptUIEditor.STRING_ONCLICK, elementXml, ScriptUIEditor.STRING_ONCLICK, ScriptUIEditor.STRING_EMPTY );
		addAttributeToElement( element, ScriptUIEditor.STRING_VALUE,   elementXml, ScriptUIEditor.STRING_VALUE,   ScriptUIEditor.STRING_EMPTY );
	}

	else if( elementXml._strName.equals( ScriptUIEditor.STRING_FIELDSET ) == true )
	{
		element = new ScriptUIEditorModelElement( model, elementParent, elementXml._strName, ScriptUIEditor.FIELDSET, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
		addAttributeToElement( element, ScriptUIEditor.STRING_ID,    elementXml, ScriptUIEditor.STRING_ID,    ScriptUIEditor.STRING_EMPTY );
		addAttributeToElement( element, ScriptUIEditor.STRING_TITLE, elementXml, ScriptUIEditor.STRING_TITLE, ScriptUIEditor.STRING_EMPTY );
	}
	
	else if( elementXml._strName.equals( ScriptUIEditor.STRING_FORM ) == true )
	{
		element = new ScriptUIEditorModelElement( model, elementParent, elementXml._strName, ScriptUIEditor.FORM, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
		addAttributeToElement( element, ScriptUIEditor.STRING_ID,       elementXml, ScriptUIEditor.STRING_ID,       ScriptUIEditor.STRING_EMPTY );
		addAttributeToElement( element, ScriptUIEditor.STRING_TITLE,    elementXml, ScriptUIEditor.STRING_TITLE,    ScriptUIEditor.STRING_EMPTY );
		addAttributeToElement( element, ScriptUIEditor.STRING_ONRESET , elementXml, ScriptUIEditor.STRING_ONRESET , ScriptUIEditor.STRING_EMPTY );
		addAttributeToElement( element, ScriptUIEditor.STRING_ONSUBMIT, elementXml, ScriptUIEditor.STRING_ONSUBMIT, ScriptUIEditor.STRING_EMPTY );
	}
	
	else if( elementXml._strName.equals( ScriptUIEditor.STRING_HEAD ) == true )
	{
		element = new ScriptUIEditorModelElement( model, elementParent, elementXml._strName, ScriptUIEditor.HEAD, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
	}
	
	else if( elementXml._strName.equals( ScriptUIEditor.STRING_HR ) == true )
	{
		element = new ScriptUIEditorModelElement( model, elementParent, elementXml._strName, ScriptUIEditor.HORZRULE, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
	}
	
	else if( elementXml._strName.equals( ScriptUIEditor.STRING_IMG ) == true )
	{
		element = new ScriptUIEditorModelElement( model, elementParent, elementXml._strName, ScriptUIEditor.IMG, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
		addAttributeToElement( element, ScriptUIEditor.STRING_ID,  elementXml, ScriptUIEditor.STRING_ID,  ScriptUIEditor.STRING_EMPTY );
		addAttributeToElement( element, ScriptUIEditor.STRING_SRC, elementXml, ScriptUIEditor.STRING_SRC, ScriptUIEditor.STRING_EMPTY );
	}
	
	else if( elementXml._strName.equals( ScriptUIEditor.STRING_INPUT ) == true )
	{
		// Determine the type of input
		//----------------------------
		String strInputType = elementXml.getAttributeValue( ScriptUIEditor.STRING_TYPE );

		if( strInputType == null )
		{
			element = new ScriptUIEditorModelElement( model, elementParent, ScriptUIEditor.STRING_INPUT, ScriptUIEditor.TEXTFIELD, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
			addAttributeToElement( element, ScriptUIEditor.STRING_ID,         elementXml, ScriptUIEditor.STRING_ID,         ScriptUIEditor.STRING_EMPTY );
			addAttributeToElement( element, ScriptUIEditor.STRING_ONBLUR    , elementXml, ScriptUIEditor.STRING_ONBLUR    , ScriptUIEditor.STRING_EMPTY );
			addAttributeToElement( element, ScriptUIEditor.STRING_ONCHANGE  , elementXml, ScriptUIEditor.STRING_ONCHANGE  , ScriptUIEditor.STRING_EMPTY );
			addAttributeToElement( element, ScriptUIEditor.STRING_ONFOCUS   , elementXml, ScriptUIEditor.STRING_ONFOCUS   , ScriptUIEditor.STRING_EMPTY );
			addAttributeToElement( element, ScriptUIEditor.STRING_ONKEYPRESS, elementXml, ScriptUIEditor.STRING_ONKEYPRESS, ScriptUIEditor.STRING_EMPTY );
			addAttributeToElement( element, ScriptUIEditor.STRING_VALUE,      elementXml, ScriptUIEditor.STRING_VALUE     , ScriptUIEditor.STRING_EMPTY );
		}
		
		// Button
		//-------
		else if( strInputType.equals( ScriptUIEditor.STRING_BUTTON ) == true )
		{
			element = new ScriptUIEditorModelElement( model, elementParent, ScriptUIEditor.STRING_INPUT, ScriptUIEditor.BUTTON, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
			addAttributeToElement( element, ScriptUIEditor.STRING_ID,      elementXml, ScriptUIEditor.STRING_ID,      ScriptUIEditor.STRING_EMPTY );
			addAttributeToElement( element, ScriptUIEditor.STRING_ONCLICK, elementXml, ScriptUIEditor.STRING_ONCLICK, ScriptUIEditor.STRING_EMPTY );
			addAttributeToElement( element, ScriptUIEditor.STRING_VALUE,   elementXml, ScriptUIEditor.STRING_VALUE,   ScriptUIEditor.STRING_EMPTY );
		}

		// CheckBox
		//---------
		else if( strInputType.equals( ScriptUIEditor.STRING_CHECKBOX ) == true )
		{
			element = new ScriptUIEditorModelElement( model, elementParent, ScriptUIEditor.STRING_INPUT, ScriptUIEditor.CHECK, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
			addAttributeToElement( element, ScriptUIEditor.STRING_CHECKED, elementXml, ScriptUIEditor.STRING_CHECKED, ScriptUIEditor.STRING_EMPTY );
			addAttributeToElement( element, ScriptUIEditor.STRING_ID,      elementXml, ScriptUIEditor.STRING_ID, ScriptUIEditor.STRING_EMPTY );
			addAttributeToElement( element, ScriptUIEditor.STRING_ONCLICK, elementXml, ScriptUIEditor.STRING_ONCLICK, ScriptUIEditor.STRING_EMPTY );
			addAttributeToElement( element, ScriptUIEditor.STRING_VALUE,   elementXml, ScriptUIEditor.STRING_VALUE, ScriptUIEditor.STRING_EMPTY );
		}
		
		// Text hidden
		//------------
		else if( strInputType.equals( ScriptUIEditor.STRING_HIDDEN ) == true )
		{
			element = new ScriptUIEditorModelElement( model, elementParent, ScriptUIEditor.STRING_INPUT, ScriptUIEditor.TEXTFIELDHIDDEN, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
			addAttributeToElement( element, ScriptUIEditor.STRING_ID,    elementXml, ScriptUIEditor.STRING_ID,    ScriptUIEditor.STRING_EMPTY );
			addAttributeToElement( element, ScriptUIEditor.STRING_VALUE, elementXml, ScriptUIEditor.STRING_VALUE, ScriptUIEditor.STRING_EMPTY );
		}
		
		// RadioButton
		//------------
		else if( strInputType.equals( ScriptUIEditor.STRING_RADIO ) == true )
		{
			element = new ScriptUIEditorModelElement( model, elementParent, ScriptUIEditor.STRING_INPUT, ScriptUIEditor.RADIO, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
			addAttributeToElement( element, ScriptUIEditor.STRING_CHECKED, elementXml, ScriptUIEditor.STRING_CHECKED, ScriptUIEditor.STRING_EMPTY );
			addAttributeToElement( element, ScriptUIEditor.STRING_ID,      elementXml, ScriptUIEditor.STRING_ID,      ScriptUIEditor.STRING_EMPTY );
			addAttributeToElement( element, ScriptUIEditor.STRING_ONCLICK, elementXml, ScriptUIEditor.STRING_ONCLICK, ScriptUIEditor.STRING_EMPTY );
			addAttributeToElement( element, ScriptUIEditor.STRING_VALUE,   elementXml, ScriptUIEditor.STRING_VALUE,   ScriptUIEditor.STRING_EMPTY );
		}
			
		// Text
		//-----
		else if( strInputType.equals( ScriptUIEditor.STRING_TEXT ) == true )
		{
			element = new ScriptUIEditorModelElement( model, elementParent, ScriptUIEditor.STRING_INPUT, ScriptUIEditor.TEXTFIELD, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
			addAttributeToElement( element, ScriptUIEditor.STRING_ID,         elementXml, ScriptUIEditor.STRING_ID,         ScriptUIEditor.STRING_EMPTY );
			addAttributeToElement( element, ScriptUIEditor.STRING_ONBLUR,     elementXml, ScriptUIEditor.STRING_ONBLUR,     ScriptUIEditor.STRING_EMPTY );
			addAttributeToElement( element, ScriptUIEditor.STRING_ONCHANGE,   elementXml, ScriptUIEditor.STRING_ONCHANGE,   ScriptUIEditor.STRING_EMPTY );
			addAttributeToElement( element, ScriptUIEditor.STRING_ONFOCUS,    elementXml, ScriptUIEditor.STRING_ONFOCUS,    ScriptUIEditor.STRING_EMPTY );
			addAttributeToElement( element, ScriptUIEditor.STRING_ONKEYPRESS, elementXml, ScriptUIEditor.STRING_ONKEYPRESS, ScriptUIEditor.STRING_EMPTY );
			addAttributeToElement( element, ScriptUIEditor.STRING_VALUE,      elementXml, ScriptUIEditor.STRING_VALUE,      ScriptUIEditor.STRING_EMPTY );
		}
		else
		{
			element = new ScriptUIEditorModelElement( model, elementParent, ScriptUIEditor.STRING_INPUT, ScriptUIEditor.TEXTFIELD, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
			exception = new ScriptUIEditorModelException( ScriptStrings.getString( "S_Unknown_input_type" ) + ": " + strInputType );
		}
	}

	else if( elementXml._strName.equals( ScriptUIEditor.STRING_LABEL ) == true )
	{
		element = new ScriptUIEditorModelElement( model, elementParent, elementXml._strName, ScriptUIEditor.LABEL, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
		addAttributeToElement( element, ScriptUIEditor.STRING_ID,   elementXml, ScriptUIEditor.STRING_ID,   ScriptUIEditor.STRING_EMPTY );
		addAttributeToElement( element, ScriptUIEditor.STRING_POUNDTEXT, elementXml, ScriptUIEditor.STRING_TEXT, ScriptUIEditor.STRING_EMPTY );
	}

	else if( elementXml._strName.equals( ScriptUIEditor.STRING_META ) == true )
	{
		element = new ScriptUIEditorModelElement( model, elementParent, elementXml._strName, ScriptUIEditor.META, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
		addAttributeToElement( element, ScriptUIEditor.STRING_HTTP_EQUIV, elementXml, ScriptUIEditor.STRING_HTTP_EQUIV, ScriptUIEditor.STRING_CONTENT_SCRIPT_TYPE );
		addAttributeToElement( element, ScriptUIEditor.STRING_CONTENT   , elementXml, ScriptUIEditor.STRING_CONTENT   , ScriptUIEditor.STRING_TEXT_JAVASCRIPT     );
	}
			
	else if( elementXml._strName.equals( ScriptUIEditor.STRING_SELECT ) == true )
	{
		String strValue = elementXml.getAttributeValue( ScriptUIEditor.STRING_SIZE );
		try
		{
			int iSize = new Integer( strValue ).intValue();
			if( iSize > 1 )
			{
				element = new ScriptUIEditorModelElement( model, elementParent, elementXml._strName, ScriptUIEditor.LIST, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
			}
			
			else
			{
				element = new ScriptUIEditorModelElement( model, elementParent, elementXml._strName, ScriptUIEditor.COMBO, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
			}
		}
		
		catch( Exception ex )
		{
			element = new ScriptUIEditorModelElement( model, elementParent, elementXml._strName, ScriptUIEditor.COMBO, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );			
		}

		addAttributeToElement( element, ScriptUIEditor.STRING_ID, elementXml, ScriptUIEditor.STRING_ID, ScriptUIEditor.STRING_EMPTY );
		addAttributeToElement( element, ScriptUIEditor.STRING_ONCHANGE, elementXml, ScriptUIEditor.STRING_ONCHANGE, ScriptUIEditor.STRING_EMPTY );
	}
	
	else if( elementXml._strName.equals( ScriptUIEditor.STRING_SCRIPT ) == true )
	{
		element = new ScriptUIEditorModelElement( model, elementParent, elementXml._strName, ScriptUIEditor.SCRIPT, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
		addAttributeToElement( element, ScriptUIEditor.STRING_POUNDTEXT,  elementXml, ScriptUIEditor.STRING_TEXT, ScriptUIEditor.STRING_EMPTY );
		addAttributeToElement( element, ScriptUIEditor.STRING_TYPE,       elementXml, ScriptUIEditor.STRING_TYPE, ScriptUIEditor.STRING_EMPTY );
	}
	
	else if( elementXml._strName.equals( ScriptUIEditor.STRING_TABLE ) == true )
	{
		element = new ScriptUIEditorModelElement( model, elementParent, elementXml._strName, ScriptUIEditor.TABLE, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
		addAttributeToElement( element, ScriptUIEditor.STRING_ID, elementXml, ScriptUIEditor.STRING_ID, ScriptUIEditor.STRING_EMPTY );
	}

	else if( elementXml._strName.equals( ScriptUIEditor.STRING_TD ) == true )
	{
		element = new ScriptUIEditorModelElement( model, elementParent, elementXml._strName, ScriptUIEditor.TABLEDATA, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
		addAttributeToElement( element, ScriptUIEditor.STRING_COLSPAN, elementXml, ScriptUIEditor.STRING_COLSPAN, "1" );
		addAttributeToElement( element, ScriptUIEditor.STRING_ROWSPAN, elementXml, ScriptUIEditor.STRING_ROWSPAN, "1" );
	}

	else if( elementXml._strName.equals( ScriptUIEditor.STRING_TH ) == true )
	{
		element = new ScriptUIEditorModelElement( model, elementParent, elementXml._strName, ScriptUIEditor.TABLEHEADING, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
		elementParent._iType = ScriptUIEditor.TABLE;
	}
	
	else if( elementXml._strName.equals( ScriptUIEditor.STRING_TEXTAREA ) == true )
	{
		element = new ScriptUIEditorModelElement( model, elementParent, elementXml._strName, ScriptUIEditor.TEXTAREA, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
		addAttributeToElement( element, ScriptUIEditor.STRING_ID,         elementXml, ScriptUIEditor.STRING_ID,         ScriptUIEditor.STRING_EMPTY );
		addAttributeToElement( element, ScriptUIEditor.STRING_ONBLUR,     elementXml, ScriptUIEditor.STRING_ONBLUR,     ScriptUIEditor.STRING_EMPTY );
		addAttributeToElement( element, ScriptUIEditor.STRING_ONCHANGE  , elementXml, ScriptUIEditor.STRING_ONCHANGE  , ScriptUIEditor.STRING_EMPTY );
		addAttributeToElement( element, ScriptUIEditor.STRING_ONFOCUS   , elementXml, ScriptUIEditor.STRING_ONFOCUS   , ScriptUIEditor.STRING_EMPTY );
		addAttributeToElement( element, ScriptUIEditor.STRING_ONKEYPRESS, elementXml, ScriptUIEditor.STRING_ONKEYPRESS, ScriptUIEditor.STRING_EMPTY );
	}
	
	else if( elementXml._strName.equals( ScriptUIEditor.STRING_TR ) == true )
	{
		element = new ScriptUIEditorModelElement( model, elementParent, elementXml._strName, ScriptUIEditor.TABLEROW, elementXml._iLineNumberFirst, elementXml._iLineNumberLast );
	}

	else
	{
		exception = new ScriptUIEditorModelException( ScriptStrings.getString( "S_Invalid_element_name" + ": " + elementXml._strName ) );
	}

	// Add the new element to its parent.
	// Recursively create child elements
	//----------------------------------
	if( element != null )
	{
		// Add to parent element
		//----------------------
		if( elementParent != null )
		{
			elementParent.addChildElement( element );
		}
		
		// Recurse
		//--------
		int iNumberOfChildElements = elementXml.getNumberOfChildElements();

		for( int i=0; i<iNumberOfChildElements; ++i )
		{
			try
			{
				parseRecursive( model, element, elementXml.getChildElementAt( i ) );
			}
			catch( ScriptUIEditorModelException ex )
			{
				exception = ex;
			}
		}	
	}

	if( exception != null )
	{
		throw( exception );
	}


//	Trace.functionExit( this, "parseRecursive" );
}
/**
 * This overrides the default behaviour since this is the root element
 * that really is not a part of the element tree.
 */
public void printElementString(StringBuffer strb, IntBuffer intbLineNumber, int iIndentation) {

	// Since this is the dummy root element, tell
	// my children (only one) to do the work
	//-------------------------------------------
	if (_vectorChildElements != null && _vectorChildElements.size() > 0) {

		for (int i = 0; i < _vectorChildElements.size(); ++i) {
			((ScriptUIEditorModelElement) _vectorChildElements.elementAt(i)).printElementString(strb, intbLineNumber, iIndentation);
		}

	}

}
}
