package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
import java.util.Iterator;
import java.util.Vector;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.scripting.internal.IXGRElementUI;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

public class ScriptUIEditorModelElement implements IAdaptable, IPropertySource
{
	public String                       _strName             = null;
	public Vector                       _vectorChildElements = null;
	public Vector                       _vectorAttributes    = null;
	public ScriptUIEditorModelElement   _elementParent       = null;
	public IXGRElementUI                _elementUI           = null;
	public int                          _iType               = 0;
	public int                          _iLineNumberFirst    = 0;
	public int                          _iLineNumberLast     = 0;
	public Rectangle                    _rectBounds          = new Rectangle(0,0,0,0);
	public ScriptUIEditorModel          _model               = null;

	static public class IntBuffer {
		public int iValue = 0;

		public IntBuffer() {}
		
		public IntBuffer(int iInitialValue) {
			this.iValue = iInitialValue;
		}

		public int getIntValue() {
			return this.iValue;
		}

		public void setIntValue(int iNewValue) {
			this.iValue = iNewValue;
		}
	}
/**
 */
public ScriptUIEditorModelElement( ScriptUIEditorModel model, ScriptUIEditorModelElement elementParent, String strName, int iType, int iLineNumberFirst, int iLineNumberLast )
{
	_model         = model;
	_elementParent = elementParent;
	_strName       = strName;
	_iType         = iType;
	_iLineNumberFirst  = iLineNumberFirst;
	_iLineNumberLast   = iLineNumberLast;
}
/**
 */
public void addAttribute( ScriptUIEditorModelAttribute parserAttribute ) 
{
	if( _vectorAttributes == null )
	{
		_vectorAttributes = new Vector();
	}

	_vectorAttributes.addElement( parserAttribute );
}
/**
 */
public void addChildElement( ScriptUIEditorModelElement parserElement ) 
{
	if( _vectorChildElements == null )
	{
		_vectorChildElements = new Vector();
	}

	_vectorChildElements.addElement( parserElement );
}
/**
 */
public void addChildElement( ScriptUIEditorModelElement parserElement, int iIndex ) 
{
	if( _vectorChildElements == null )
	{
		_vectorChildElements = new Vector();
	}

	_vectorChildElements.add( iIndex, parserElement );
}
/**
 * Returns width/height
 */
public Point calculateBounds( Point ptOrigin ) {

	switch( _iType )
	{
		case ScriptUIEditor.FORM:
		case ScriptUIEditor.FIELDSET:
		case ScriptUIEditor.GROUPBOX:
		case ScriptUIEditor.BODY:
		case ScriptUIEditor.HTML:
		    Point ptOriginChildren = new Point( ptOrigin.x, ptOrigin.y );
		    ptOriginChildren.x += 8;
		    ptOriginChildren.y += 8;
		    
			Point ptSize = calculateContainerBounds( ptOriginChildren );
			_rectBounds.x = ptOrigin.x;
			_rectBounds.y = ptOrigin.y;
			_rectBounds.width  = ptSize.x;
			_rectBounds.height = ptSize.y;
			break;

		default:
			_rectBounds.x = ptOrigin.x;
			_rectBounds.y = ptOrigin.y;
			_rectBounds.width  = 100;
			_rectBounds.height =  30;
			break;
	}

	return new Point( _rectBounds.x, _rectBounds.y );
}
/**
 * Returns preferred cell dimensions
 */
protected Point calculateContainerBounds(Point ptOrigin) {
	int iMaxX = 0;
	int iMaxY = 0;

	int iMaxRowX = 0;
	int iMaxRowY = 0;

	ScriptUIEditorModelElement elementChild = null;

	for (int i = 0; i < _vectorChildElements.size(); ++i) {
		
		elementChild = (ScriptUIEditorModelElement) _vectorChildElements.elementAt(i);
		
		switch (elementChild._iType) {
			
			case ScriptUIEditor.BODY :
			case ScriptUIEditor.FORM :
			case ScriptUIEditor.FIELDSET :
			case ScriptUIEditor.GROUPBOX :
			case ScriptUIEditor.HEAD :
			case ScriptUIEditor.TABLE :
			case ScriptUIEditor.TABLEDATA :
			case ScriptUIEditor.TABLEROW :
				Point ptChild = elementChild.calculateContainerBounds(ptOrigin);
				iMaxRowX += ptChild.x;
				iMaxRowY = Math.max(iMaxRowY, ptChild.y);
				break;

			case ScriptUIEditor.BREAK :
				iMaxX = Math.max(iMaxX, iMaxRowX);
				iMaxY += iMaxRowY;
				iMaxRowX = 0;
				iMaxRowY = 0;
				break;

			case ScriptUIEditor.META :
			case ScriptUIEditor.SCRIPT :
				continue;

			default :
				Point ptChild1 = new Point(30, 100);
				iMaxRowX += ptChild1.x
				;
				iMaxRowY = Math.max(iMaxRowY, ptChild1.y);
				break;
		}
	}

	iMaxX = Math.max(iMaxX, iMaxRowX);
	iMaxY += iMaxRowY;

	if (iMaxX == 0 || iMaxY == 0) {
		return new Point(200, 200);
	}
	else {
		return new Point(iMaxX + 8, iMaxY + 8);
	}
}
/**
 * Searches the element tree for an element with the hash value.  This is
 * used during drag/drop.
 */
public ScriptUIEditorModelElement findElementFromHashValue(int iHashValue) {

	if (this.hashCode() == iHashValue) {
		return this;
	}
	
	// Get my children to find the element
	//------------------------------------
	if (_vectorChildElements != null) {
		ScriptUIEditorModelElement elementChild = null;
		ScriptUIEditorModelElement elementFound = null;
		
		for (int i = 0; i < _vectorChildElements.size(); ++i) {
			elementChild = (ScriptUIEditorModelElement)_vectorChildElements.elementAt(i);
			elementFound = elementChild.findElementFromHashValue( iHashValue );
			if( elementFound != null )
			{
				return elementFound;
			}
		}
	}
	
	return null;
}
/**
 * Prints the a single line of xml representing the element
 * and its attributes starting at this element.
 */
public void generateLineNumbers(IntBuffer intbLineNumber) {

	// Comment element
	//----------------
	if (_iType == ScriptUIEditor.COMMENT) {

		generateLineNumbersComment(intbLineNumber);
	}

	// Non comment
	//------------
	else {
		// Indentation
		//------------
//		printElementStringIndentation(strb, iIndentation);

		// Begin element
		//--------------
		_iLineNumberFirst = intbLineNumber.iValue;
//		strb.append('<');
//		printElementStringName(strb, true);

		// Print out my attributes and their values
		//-----------------------------------------
		ScriptUIEditorModelAttribute attribute = null;
		ScriptUIEditorModelAttribute attributeText = null;

		if (_vectorAttributes != null && _vectorAttributes.size() > 0) {

			for (int i = 0; i < _vectorAttributes.size(); ++i) {
				attribute = (ScriptUIEditorModelAttribute) _vectorAttributes.elementAt(i);

				if (attribute._strName.equals("#text")) {
					attributeText = attribute;
				}

				else {
//					strb.append(' ');
//					attribute.printAttributeString(strb);
				}
			}
		}

		// Print out any #text attribute
		//------------------------------
		if (attributeText != null) {

			// Script text lines, end element
			//-------------------------------
			if (_iType == ScriptUIEditor.SCRIPT) {

//				strb.append(">");
//				printElementStringNewLine(strb, intbLineNumber);
				intbLineNumber.iValue++;

				// Print out the text
				//-------------------
//				printElementStringIndentation(strb, iIndentation + 4);
//				strb.append("<![CDATA[");
				if (attributeText._strValue != null && attributeText._strValue.length() > 0) {

					generateLineNumbersScript(intbLineNumber, attributeText._strValue);
				}
				else {
//	                printElementStringNewLine(strb, intbLineNumber);
					intbLineNumber.iValue++;
//	                printElementStringNewLine(strb, intbLineNumber);
					intbLineNumber.iValue++;
				}
//				printElementStringIndentation(strb, iIndentation + 4);
//				strb.append("]]>");
//				printElementStringNewLine(strb, intbLineNumber);
				intbLineNumber.iValue++;

				// End element
				//------------
//				printElementStringIndentation(strb, iIndentation);
//				strb.append("</");
//				printElementStringName(strb, false);
//				strb.append(">");
				_iLineNumberLast = intbLineNumber.iValue;
//				printElementStringNewLine(strb, intbLineNumber);
				intbLineNumber.iValue++;
			}

			// Label text string, end element
			//-------------------------------
			else {
//				strb.append(">");
//				printElementStringLabel(strb, attributeText._strValue);
//				strb.append("</");
//				printElementStringName(strb, false);
//				strb.append(">");
				_iLineNumberLast = intbLineNumber.iValue;
//				printElementStringNewLine(strb, intbLineNumber);
				intbLineNumber.iValue++;
			}
		}

		else {
			// Print the rest of the element string
			// with the end element on a separate line
			//----------------------------------------
			if (_vectorChildElements != null && _vectorChildElements.size() > 0 ||
				_iType == ScriptUIEditor.FIELDSET ||
				_iType == ScriptUIEditor.FORM ||
				_iType == ScriptUIEditor.BODY ||
				_iType == ScriptUIEditor.TABLE ||
				_iType == ScriptUIEditor.TABLEDATA ||
				_iType == ScriptUIEditor.TABLEROW ||
				_iType == ScriptUIEditor.HTML) {

				// Terminate the begin element
				//----------------------------	
//				strb.append(">");
//				printElementStringNewLine(strb, intbLineNumber);
				intbLineNumber.iValue++;

				// Get my children to print themselves
				//------------------------------------
				if (_vectorChildElements != null) {
					for (int i = 0; i < _vectorChildElements.size(); ++i) {

//						((ScriptUIEditorModelElement) _vectorChildElements.elementAt(i)).printElementString(strb, intbLineNumber, iIndentation + 4);
						((ScriptUIEditorModelElement) _vectorChildElements.elementAt(i)).generateLineNumbers(intbLineNumber);
					}
				}

				// End element
				//------------
//				printElementStringIndentation(strb, iIndentation);
//				strb.append("</");
//				printElementStringName(strb, false);
//				strb.append(">");
				_iLineNumberLast = intbLineNumber.iValue;
//				printElementStringNewLine(strb, intbLineNumber);
				intbLineNumber.iValue++;
			}

			// End element on a single line
			//-----------------------------
			else {
//				strb.append(" />");
				_iLineNumberLast = intbLineNumber.iValue;
//				printElementStringNewLine(strb, intbLineNumber);
				intbLineNumber.iValue++;
			}
		}
	}
}
/**
 * Prints the a single line of xml representing the element
 * and its attributes starting at this element.
 */
protected void generateLineNumbersComment(IntBuffer intbLineNumber) {

	// Indentation
	//------------
//	printElementStringIndentation(strb, iIndentation);

	// Begin comment
	//--------------
	_iLineNumberFirst = intbLineNumber.iValue;

	ScriptUIEditorModelAttribute attributeText = getAttribute("#text");
	
	// Format lines of text
	//---------------------
	if (attributeText != null) {
		String strText = attributeText._strValue.toString();

		// Single line comment
		//--------------------
		if (strText.indexOf('\n') == -1) {
//			strb.append("<!-- " );
//			strb.append( strText.trim() );
//			strb.append( " -->" );
			_iLineNumberLast = _iLineNumberFirst;
//			printElementStringNewLine( strb, intbLineNumber );
			intbLineNumber.iValue++;
		}

		// Multiple line comment
		//----------------------
		else {
	        // Begin comment
	        //--------------
//		    strb.append("<!--");
//			printElementStringNewLine( strb, intbLineNumber );
			intbLineNumber.iValue++;
//			printElementStringMultipleLineText( strb, intbLineNumber, iIndentation + 4, strText );
			generateLineNumbersMultipleLineText( intbLineNumber, strText );
			
			// End comment
			//------------
//			printElementStringIndentation(strb, iIndentation);
//			strb.append("-->");
			_iLineNumberLast = intbLineNumber.iValue;
//			printElementStringNewLine( strb, intbLineNumber );
			intbLineNumber.iValue++;
		}
	}

	return;
}
/**
 * Prints the a single line of xml representing the element
 * and its attributes starting at this element.
 */
protected void generateLineNumbersMultipleLineText(IntBuffer intbLineNumber, String strText) {

	// Check for single line
	//----------------------
	if (strText.indexOf('\n') == -1) {
//		printElementStringIndentation(strb, iIndentation);
//		strb.append(strText.trim());
//		printElementStringNewLine(strb, intbLineNumber);
		intbLineNumber.iValue++;
	}

	// Multiple lines
	//---------------
	else {
		int iIndexStart = -1;
		int iIndexEnd = -1;

		do {
			// Not last line
			//--------------
			if ((iIndexEnd = strText.indexOf('\n', iIndexStart + 1)) != -1) {
//				printElementStringIndentation(strb, iIndentation);
//				strb.append(strText.substring(iIndexStart + 1, iIndexEnd).trim());
//				printElementStringNewLine(strb, intbLineNumber);
				intbLineNumber.iValue++;
				iIndexStart = iIndexEnd;
			}

			// Last line
			//----------
			else {
//				printElementStringIndentation(strb, iIndentation);
//				strb.append(strText.substring(iIndexStart + 1).trim());
//				printElementStringNewLine(strb, intbLineNumber);
				intbLineNumber.iValue++;
				break;
			}
		}
		while (iIndexEnd != -1 && iIndexEnd < strText.length() - 1);
	}
	return;
}
/**
 * Prints the a single line of xml representing the element
 * and its attributes starting at this element.
 */
protected void generateLineNumbersScript(IntBuffer intbLineNumber, String strText) {

	if (strText != null && strText.length() > 0) {

		char[] charaText = new char[strText.length()];

		strText.getChars(0, strText.length(), charaText, 0);

//		int iLocalIndentation = iIndentation;

		boolean bForMode = false;
		boolean bNewLineMode = true;
		boolean bStringMode = false;
		boolean bMultiLineCommentMode = false;
		boolean bSingleLineCommentMode = false;

		int iForDepth = 0;

		for (int i = 0; i < charaText.length; ++i) {

			switch (charaText[i]) {

				case '/' :
					if (bNewLineMode == true) {
//						printElementStringNewLine(strb, intbLineNumber);
						intbLineNumber.iValue++;
//						printElementStringIndentation(strb, iLocalIndentation);
						bNewLineMode = false;
					}
//					strb.append('/');
					if (bStringMode == false) {
						if (i < charaText.length - 1) {
							if (charaText[i + 1] == '/') {
								bSingleLineCommentMode = true;
//								strb.append("/");
								i++;
							}
							else if (charaText[i + 1] == '*') {
								bMultiLineCommentMode = true;
//								strb.append("*");
								i++;
							}
						}
					}
					break;
					
				case '*' :
					if (bNewLineMode == true) {
//						printElementStringNewLine(strb, intbLineNumber);
						intbLineNumber.iValue++;
//						printElementStringIndentation(strb, iLocalIndentation);
						bNewLineMode = false;
					}
//					strb.append('*');
					if (bMultiLineCommentMode == true) {
						if (i < charaText.length - 1) {
							if (charaText[i + 1] == '/') {
								bSingleLineCommentMode = true;
//								strb.append("/");
								i++;
								bNewLineMode = true;
							}
						}
					}
					break;

				case '{' :
//					printElementStringNewLine(strb, intbLineNumber);
					intbLineNumber.iValue++;
//					printElementStringIndentation(strb, iLocalIndentation);
//					strb.append('{');
//					iLocalIndentation += 4;
					bNewLineMode = true;
					break;

				case '}' :
//					printElementStringNewLine(strb, intbLineNumber);
					intbLineNumber.iValue++;
//					iLocalIndentation -= 4;
//					printElementStringIndentation(strb, iLocalIndentation);
//					strb.append('}');
					bNewLineMode = true;
					break;

				case 'f' :
					if (bNewLineMode == true) {
//						printElementStringNewLine(strb, intbLineNumber);
						intbLineNumber.iValue++;
//						printElementStringIndentation(strb, iLocalIndentation);
						bNewLineMode = false;
					}
//					strb.append('f');
					if (bStringMode == false) {
						if (i < charaText.length - 2) {
							if (charaText[i + 1] == 'o' && charaText[i + 2] == 'r') {
								bForMode = true;
//								strb.append("or");
								i += 2;
							}
						}
					}
					break;

				case '(' :
//					strb.append('(');
					if (bForMode == true) {
						iForDepth++;
					}
					break;

				case ')' :
//					strb.append(')');
					if (bForMode == true) {
						if (--iForDepth <= 0) {
							iForDepth = 0;
							bForMode = false;
						}
					}
					break;

				case ';' :
//					strb.append(';');
					if (bForMode == false) {
						bNewLineMode = true;
					}
					break;

				case '"' :
					bStringMode = !bStringMode;
//					strb.append('"');
					break;

				case '\n' :
					if (bSingleLineCommentMode == true) {
						bSingleLineCommentMode = false;
						bNewLineMode = true;
					}
					else if (bMultiLineCommentMode == true) {
						bNewLineMode = true;
					}
					break;

				case '\r' :
					break;

				case ' ' :
//					if (bNewLineMode == false) {
//						strb.append(' ');
//					}
					break;
					
				case '\t' :
//					if (bNewLineMode == false) {
//						strb.append("    ");
//					}
					break;

				default :
					if (bNewLineMode == true) {
//						printElementStringNewLine(strb, intbLineNumber);
						intbLineNumber.iValue++;
//						printElementStringIndentation(strb, iLocalIndentation);
						bNewLineMode = false;
					}
//					strb.append(charaText[i]);
					break;
			}
		}
//		printElementStringNewLine(strb, intbLineNumber);
		intbLineNumber.iValue++;
	}
	return;
}
/**
 * Returns an object which is an instance of the given class
 * associated with this object. Returns <code>null</code> if
 * no such object can be found.
 */
public Object getAdapter(Class adapter)
{
	if (adapter == IPropertySource.class)
	{
		return this;
	}
	
	return null;
}
/**
 */
public ScriptUIEditorModelAttribute getAttribute( String strAttributeName ) 
{
	if( _vectorAttributes != null )
	{
		ScriptUIEditorModelAttribute attribute = null;
		
		for( int i=0; i<_vectorAttributes.size(); ++i )
		{
			attribute = (ScriptUIEditorModelAttribute)_vectorAttributes.elementAt( i );
			
			if(attribute._strName != null && attribute._strName.equals( strAttributeName ) == true )
			{
				return attribute;
			}
		}
	}

	return null;
}
/**
 */
public String getAttributeValue( String strAttributeName )
{
	ScriptUIEditorModelAttribute attribute = getAttribute( strAttributeName );

	if( attribute != null )
	{
		return attribute._strValue;
	}
	
	return null;
}
/**
 */
public ScriptUIEditorModelElement getChildElement( String strElementName )
{
	if( _vectorChildElements != null )
	{
		ScriptUIEditorModelElement elementChild = null;
		
		for( int i=0; i<_vectorChildElements.size(); ++i )
		{
			elementChild = (ScriptUIEditorModelElement)_vectorChildElements.elementAt( i );

			if( elementChild._strName != null && strElementName.equals( elementChild._strName ) == true )
			{
				return elementChild;
			}
		}
	}

	return null;
}
/**
 */
public ScriptUIEditorModelElement getChildElementAt( int iIndex )
{
	if( _vectorChildElements == null ){
		return null;
	}
	
	if( iIndex >= 0 && iIndex < _vectorChildElements.size() ){
		return (ScriptUIEditorModelElement)_vectorChildElements.elementAt( iIndex );
	}
	
	return null;
}
/**
 */
public int getChildElementIndex(ScriptUIEditorModelElement elementChild) {

	if( _vectorChildElements != null )
	{
		return _vectorChildElements.indexOf( elementChild );
	}
	
	return -1;
}
/**
 */
public Iterator getChildren() {

	if( _vectorChildElements == null )
	{
		_vectorChildElements = new Vector();
	}
	return _vectorChildElements.iterator();
}
/**
 * Returns a value for this object that can be editted in a property sheet.
 */
public Object getEditableValue()
{
	return this;
}
/**
 */
public ScriptUIEditorModelElement getElementFromLineNumber(int iLineNumber) {

	ScriptUIEditorModelElement element = null;

	// See if I am in range
	//---------------------
	if( (iLineNumber >= _iLineNumberFirst && iLineNumber <= _iLineNumberLast) ||
		(_iLineNumberFirst == -1 && _iLineNumberLast == -1 ) ){

		element = this;
			
		// See if one of my children is in range
		//--------------------------------------
		ScriptUIEditorModelElement elementChild = null;
		
		if (_vectorChildElements != null && _vectorChildElements.size() > 0) {

			for (int i = 0; i < _vectorChildElements.size(); ++i) {
				elementChild = ((ScriptUIEditorModelElement) _vectorChildElements.elementAt(i)).getElementFromLineNumber(iLineNumber);
				if( elementChild != null )
				{
					element = elementChild;
					break;
				}
			}
		}
	}

	return element;
}
/**
 */
public ScriptUIEditorModel getModel() {
	return _model;
}
/**
 */
public int getNumberOfAttributes() 
{
	return _vectorAttributes != null ? _vectorAttributes.size() : 0;
}
/**
 */
public int getNumberOfChildElements() 
{
	return _vectorChildElements != null ? _vectorChildElements.size() : 0;
}
/**
 */
public ScriptUIEditorModelElement getParent()
{
	return _elementParent;
}
/**
 */
public IPropertyDescriptor[] getPropertyDescriptors() {
	
	Vector vectorDescriptors = new Vector();

	if (_vectorAttributes != null) {
		PropertyDescriptor descriptor = null;

		for (int i = 0; i < _vectorAttributes.size(); ++i) {
			ScriptUIEditorModelAttribute attribute = (ScriptUIEditorModelAttribute) _vectorAttributes.elementAt(i);

			// Script text is non-editable in the property table
			//--------------------------------------------------
			if (_iType == ScriptUIEditor.SCRIPT && attribute._strName.equals("#text") == true) {
				descriptor = new PropertyDescriptor(attribute._strName, attribute._strName);
			}

			// All other attributes are editable in the property table
			//--------------------------------------------------------
			else {
				descriptor = new TextPropertyDescriptor(attribute._strName, attribute._strName);
			}

			vectorDescriptors.add(descriptor);
		}
	}

	IPropertyDescriptor[] propertyDescriptors = new IPropertyDescriptor[vectorDescriptors.size()];

	vectorDescriptors.copyInto( propertyDescriptors );

	return propertyDescriptors;
}
/**
 * Returns the current value for the named property.
 */
public Object getPropertyValue(Object objName) {

	for (int i = 0; i < _vectorAttributes.size(); ++i) {
		ScriptUIEditorModelAttribute attribute = (ScriptUIEditorModelAttribute) _vectorAttributes.elementAt(i);

		if (attribute._strName == objName) {
			return attribute._strValue;
		}
	}

	return null;
}
/**
 */
public String getTypeName(int iType) {

	switch( iType )
	{
		case ScriptUIEditor.BODY :      return "body";
		case ScriptUIEditor.BREAK :     return "break";
		case ScriptUIEditor.BUTTON:     return "button";
		case ScriptUIEditor.COMBO:      return "combo";
		case ScriptUIEditor.FORM :      return "form";
		case ScriptUIEditor.FIELDSET :  return "fieldset";
		case ScriptUIEditor.GROUPBOX :  return "groupbox";
		case ScriptUIEditor.HORZRULE :  return "horizontalrule";
		case ScriptUIEditor.HEAD :      return "head";
		case ScriptUIEditor.HTML :      return "html";
		case ScriptUIEditor.IMG:        return "image";
		case ScriptUIEditor.LABEL:      return "label";
		case ScriptUIEditor.LIST :      return "list";
		case ScriptUIEditor.META :      return "meta";
		case ScriptUIEditor.RADIO:      return "radio";
		case ScriptUIEditor.SCRIPT :    return "script";
		case ScriptUIEditor.TABLE :     return "table";
		case ScriptUIEditor.TABLEDATA : return "tabledata";
		case ScriptUIEditor.TABLEROW :  return "tablerow";
		case ScriptUIEditor.TEXTAREA :   return "textarea";
		case ScriptUIEditor.TEXTFIELD :  return "textfield";
		default:						return "unknown";
	}
}
/**
 */
public boolean hasChildren() {

	if( _vectorChildElements != null )
	{
		return _vectorChildElements.size() > 0;
	}
	
	return false;
}
/**
 * Determines whether the element is a descendent of this one.
 */
public boolean hasDescendent(ScriptUIEditorModelElement element) {

	if (_vectorChildElements == null || _vectorChildElements.size() == 0) {
		return false;
	}

	// Get my children to find the element
	//------------------------------------
	ScriptUIEditorModelElement elementChild = null;

	for (int i = 0; i < _vectorChildElements.size(); ++i) {
		
		elementChild = (ScriptUIEditorModelElement) _vectorChildElements.elementAt(i);
		
		if (elementChild == element || elementChild.hasDescendent(element) == true) {
			return true;
		}
	}

	return false;
}
/**
 */
public void insertChildElementAt( ScriptUIEditorModelElement element, int iIndex ) 
{
	if( _vectorChildElements == null )
	{
		_vectorChildElements = new Vector();
	}

	_vectorChildElements.insertElementAt( element, iIndex );
}
/**
 * Returns whether the property value has changed from the default.
 */
public boolean isPropertySet(Object property)
{
	return false;
}
/**
 * Returns whether the value of the property with the given id has changed from
 * its default value. Returns <code>false</code> if the notion of default value
 * is not meaningful for the specified property, or if this source does not have
 * the specified property.
 */
public boolean isPropertySet(java.lang.String id) {
	return false;
}
/**
 */
public void paintControl( GC gc ) 
{
//	Trace.functionEntry( this, "paintControl" );
	
//	gc.setForeground( gc.getDisplay().getSystemColor( SWT.COLOR_BLACK ) );

//	Trace.string( getTypeName( _iType ) + " " + _rectBounds.toString() );
	
	if( _rectBounds.isEmpty() == false )
	{
		gc.drawRectangle( _rectBounds.x, _rectBounds.y, _rectBounds.width, _rectBounds.height );
	}

	Iterator iter = getChildren();
	while( iter.hasNext() )
	{
		((ScriptUIEditorModelElement)iter.next()).paintControl( gc );
	}

//	Trace.functionExit( this, "paintControl" );
}
/**
 */
public void printElementString( StringBuffer strb ) {

	printElementString(strb, new IntBuffer(), 0);
}
/**
 * Prints the a single line of xml representing the element
 * and its attributes starting at this element.
 */
public void printElementString(StringBuffer strb, int iLineNumber, int iIndentation) {

	printElementString( strb, new IntBuffer( iLineNumber ), iIndentation );
	return;
}
/**
 * Prints the a single line of xml representing the element
 * and its attributes starting at this element.
 */
public void printElementString(StringBuffer strb, IntBuffer intbLineNumber, int iIndentation) {

	// Comment element
	//----------------
	if (_iType == ScriptUIEditor.COMMENT) {

		printElementStringComment(strb, intbLineNumber, iIndentation);
	}

	// Non comment
	//------------
	else {
		// Indentation
		//------------
		printElementStringIndentation(strb, iIndentation);

		// Begin element
		//--------------
		_iLineNumberFirst = intbLineNumber.iValue;
		strb.append('<');
		printElementStringName(strb, true);

		// Print out my attributes and their values
		//-----------------------------------------
		ScriptUIEditorModelAttribute attribute = null;
		ScriptUIEditorModelAttribute attributeText = null;

		if (_vectorAttributes != null && _vectorAttributes.size() > 0) {

			for (int i = 0; i < _vectorAttributes.size(); ++i) {
				attribute = (ScriptUIEditorModelAttribute) _vectorAttributes.elementAt(i);

				if (attribute._strName.equals("#text")) {
					attributeText = attribute;
				}

				else {
					strb.append(' ');
					attribute.printAttributeString(strb);
				}
			}
		}

		// Print out any #text attribute
		//------------------------------
		if (attributeText != null) {

			// Script text lines, end element
			//-------------------------------
			if (_iType == ScriptUIEditor.SCRIPT) {

				strb.append(">");
				printElementStringNewLine(strb, intbLineNumber);

				// Print out the text
				//-------------------
				printElementStringIndentation(strb, iIndentation + 4);
				strb.append("<![CDATA[");
				if (attributeText._strValue != null && attributeText._strValue.length() > 0) {

					printElementStringScript(strb, intbLineNumber, iIndentation + 4, attributeText._strValue);
				}
				else {
					printElementStringNewLine(strb, intbLineNumber);
					printElementStringNewLine(strb, intbLineNumber);
				}
				printElementStringIndentation(strb, iIndentation + 4);
				strb.append("]]>");
				printElementStringNewLine(strb, intbLineNumber);

				// End element
				//------------
				printElementStringIndentation(strb, iIndentation);
				strb.append("</");
				printElementStringName(strb, false);
				strb.append(">");
				_iLineNumberLast = intbLineNumber.iValue;
				printElementStringNewLine(strb, intbLineNumber);
			}

			// Label text string, end element
			//-------------------------------
			else {
				strb.append(">");
				printElementStringLabel(strb, attributeText._strValue);
				strb.append("</");
				printElementStringName(strb, false);
				strb.append(">");
				_iLineNumberLast = intbLineNumber.iValue;
				printElementStringNewLine(strb, intbLineNumber);
			}
		}

		else {
			// Print the rest of the element string
			// with the end element on a separate line
			//----------------------------------------
			if (_vectorChildElements != null
				&& _vectorChildElements.size() > 0
				|| _iType == ScriptUIEditor.FIELDSET
				|| _iType == ScriptUIEditor.FORM
				|| _iType == ScriptUIEditor.BODY
				|| _iType == ScriptUIEditor.TABLE
				|| _iType == ScriptUIEditor.TABLEDATA
				|| _iType == ScriptUIEditor.TABLEROW
				|| _iType == ScriptUIEditor.HTML) {

				// Terminate the begin element
				//----------------------------	
				strb.append(">");
				printElementStringNewLine(strb, intbLineNumber);

				// Get my children to print themselves
				//------------------------------------
				if (_vectorChildElements != null) {
					for (int i = 0; i < _vectorChildElements.size(); ++i) {
						((ScriptUIEditorModelElement) _vectorChildElements.elementAt(i)).printElementString(strb, intbLineNumber, iIndentation + 4);
					}
				}

				// End element
				//------------
				printElementStringIndentation(strb, iIndentation);
				strb.append("</");
				printElementStringName(strb, false);
				strb.append(">");
				_iLineNumberLast = intbLineNumber.iValue;
				printElementStringNewLine(strb, intbLineNumber);
			}

			// End element on a single line
			//-----------------------------
			else {
				strb.append(" />");
				_iLineNumberLast = intbLineNumber.iValue;
				printElementStringNewLine(strb, intbLineNumber);
			}
		}
	}
}
/**
 * Prints the a single line of xml representing the element
 * and its attributes starting at this element.
 */
protected void printElementStringComment(StringBuffer strb, IntBuffer intbLineNumber, int iIndentation) {

	// Indentation
	//------------
	printElementStringIndentation(strb, iIndentation);

	// Begin comment
	//--------------
	_iLineNumberFirst = intbLineNumber.iValue;

	ScriptUIEditorModelAttribute attributeText = getAttribute("#text");
	
	// Format lines of text
	//---------------------
	if (attributeText != null) {
		String strText = attributeText._strValue.toString();

		// Single line comment
		//--------------------
		if (strText.indexOf('\n') == -1) {
			strb.append("<!-- " );
			strb.append( strText.trim() );
			strb.append( " -->" );
			_iLineNumberLast = _iLineNumberFirst;
			printElementStringNewLine( strb, intbLineNumber );
		}

		// Multiple line comment
		//----------------------
		else {
	        // Begin comment
	        //--------------
		    strb.append("<!--");
			printElementStringNewLine( strb, intbLineNumber );				
			printElementStringMultipleLineText( strb, intbLineNumber, iIndentation + 4, strText );
			
			// End comment
			//------------
			printElementStringIndentation(strb, iIndentation);
			strb.append("-->");
			_iLineNumberLast = intbLineNumber.iValue;
			printElementStringNewLine( strb, intbLineNumber );
		}
	}

	return;
}
/**
 * Outputs a number of space characters.
 */
protected void printElementStringIndentation(StringBuffer strb, int iIndentation) {
	for (int i = 0; i < iIndentation; ++i) {
		strb.append(' ');
	}
}
/**
 * Replaces carriage returns / line feeds / tabs with spaces
 */
protected void printElementStringLabel(StringBuffer strb, String strText) {

	if (strText != null && strText.length() > 0) {

		char[] charaText = new char[strText.length()];

		strText.getChars(0, strText.length(), charaText, 0);

		boolean bNewLineMode = false;

		for (int i = 0; i < charaText.length; ++i) {

			switch (charaText[i]) {

				case '\n' :
					strb.append(' ' );
					bNewLineMode = true;
					break;
					
				case '\t' :
					if (bNewLineMode == false) {
						strb.append(' ');
					}
					break;

				case ' ' :
					if (bNewLineMode == false) {
						strb.append(' ');
					}
					break;

				case '\r' :
					break;

				default :
					strb.append(charaText[i]);
					bNewLineMode = false;
					break;
			}
		}
	}
	return;
}
/**
 * Prints the a single line of xml representing the element
 * and its attributes starting at this element.
 */
protected void printElementStringMultipleLineText(StringBuffer strb, IntBuffer intbLineNumber, int iIndentation, String strText) {

	// Check for single line
	//----------------------
	if (strText.indexOf('\n') == -1) {
		printElementStringIndentation(strb, iIndentation);
		strb.append(strText.trim());
		printElementStringNewLine(strb, intbLineNumber);
	}

	// Multiple lines
	//---------------
	else {
		int iIndexStart = -1;
		int iIndexEnd = -1;

		do {
			// Not last line
			//--------------
			if ((iIndexEnd = strText.indexOf('\n', iIndexStart + 1)) != -1) {
				printElementStringIndentation(strb, iIndentation);
				strb.append(strText.substring(iIndexStart + 1, iIndexEnd).trim());
				printElementStringNewLine(strb, intbLineNumber);
				iIndexStart = iIndexEnd;
			}

			// Last line
			//----------
			else {
				printElementStringIndentation(strb, iIndentation);
				strb.append(strText.substring(iIndexStart + 1).trim());
				printElementStringNewLine(strb, intbLineNumber);
				break;
			}
		}
		while (iIndexEnd != -1 && iIndexEnd < strText.length() - 1);
	}
	return;
}
/**
 */
public void printElementStringName(StringBuffer strb, boolean bBeginElement ) {

	switch( _iType )
	{
		case ScriptUIEditor.CHECK:
			strb.append( "input" );
			if( bBeginElement == true ) strb.append( " type=\"checkbox\"" );
			break;

		case ScriptUIEditor.BUTTON:
			strb.append( "input" );
			if( bBeginElement == true ) strb.append( " type=\"button\"" );
			break;

		case ScriptUIEditor.RADIO:
			strb.append( "input" );
			if( bBeginElement == true ) strb.append( " type=\"radio\"" );
			break;
			
		case ScriptUIEditor.TEXTFIELD:
			strb.append( "input" );
			if( bBeginElement == true ) strb.append( " type=\"text\"" );
			break;

		case ScriptUIEditor.LIST:
			strb.append( "select" );
			if( bBeginElement == true ) strb.append( " size=\"3\"" );
			break;
			
		case ScriptUIEditor.COMBO:
			strb.append( "select" );
			if( bBeginElement == true ) strb.append( " size=\"1\"" );
			break;

		default:
			strb.append( _strName != null ? _strName : "undefined" );
			break;
	}
}
/**
 * Prints the a single line of xml representing the element
 * and its attributes starting at this element.
 */
protected void printElementStringNewLine(StringBuffer strb, IntBuffer intbLineNumber) {

	intbLineNumber.iValue++;
	strb.append( "\n" );
}
/**
 * Prints the a single line of xml representing the element
 * and its attributes starting at this element.
 */
protected void printElementStringScript(StringBuffer strb, IntBuffer intbLineNumber, int iIndentation, String strText) {

	if (strText != null && strText.length() > 0) {

		char[] charaText = new char[strText.length()];

		strText.getChars(0, strText.length(), charaText, 0);

		int iLocalIndentation = iIndentation;

		boolean bForMode = false;
		boolean bNewLineMode = true;
		boolean bStringMode = false;
		boolean bMultiLineCommentMode = false;
		boolean bSingleLineCommentMode = false;

		int iForDepth = 0;

		for (int i = 0; i < charaText.length; ++i) {

			switch (charaText[i]) {

				case '/' :
					if (bNewLineMode == true) {
						printElementStringNewLine(strb, intbLineNumber);
						printElementStringIndentation(strb, iLocalIndentation);
						bNewLineMode = false;
					}
					strb.append('/');
					if (bStringMode == false) {
						if (i < charaText.length - 1) {
							if (charaText[i + 1] == '/') {
								bSingleLineCommentMode = true;
								strb.append("/");
								i++;
							}
							else if (charaText[i + 1] == '*') {
								bMultiLineCommentMode = true;
								strb.append("*");
								i++;
							}
						}
					}
					break;
					
				case '*' :
					if (bNewLineMode == true) {
						printElementStringNewLine(strb, intbLineNumber);
						printElementStringIndentation(strb, iLocalIndentation);
						bNewLineMode = false;
					}
					strb.append('*');
					if (bMultiLineCommentMode == true) {
						if (i < charaText.length - 1) {
							if (charaText[i + 1] == '/') {
								bSingleLineCommentMode = true;
								strb.append("/");
								i++;
								bNewLineMode = true;
							}
						}
					}
					break;

				case '{' :
					printElementStringNewLine(strb, intbLineNumber);
					printElementStringIndentation(strb, iLocalIndentation);
					strb.append('{');
					iLocalIndentation += 4;
					bNewLineMode = true;
					break;

				case '}' :
					printElementStringNewLine(strb, intbLineNumber);
					iLocalIndentation -= 4;
					printElementStringIndentation(strb, iLocalIndentation);
					strb.append('}');
					bNewLineMode = true;
					break;

				case 'f' :
					if (bNewLineMode == true) {
						printElementStringNewLine(strb, intbLineNumber);
						printElementStringIndentation(strb, iLocalIndentation);
						bNewLineMode = false;
					}
					strb.append('f');
					if (bStringMode == false) {
						if (i < charaText.length - 2) {
							if (charaText[i + 1] == 'o' && charaText[i + 2] == 'r') {
								bForMode = true;
								strb.append("or");
								i += 2;
							}
						}
					}
					break;

				case '(' :
					strb.append('(');
					if (bForMode == true) {
						iForDepth++;
					}
					break;

				case ')' :
					strb.append(')');
					if (bForMode == true) {
						if (--iForDepth <= 0) {
							iForDepth = 0;
							bForMode = false;
						}
					}
					break;

				case ';' :
					strb.append(';');
					if (bForMode == false) {
						bNewLineMode = true;
					}
					break;

				case '"' :
					bStringMode = !bStringMode;
					strb.append('"');
					break;

				case '\n' :
					if (bSingleLineCommentMode == true) {
						bSingleLineCommentMode = false;
						bNewLineMode = true;
					}
					else if (bMultiLineCommentMode == true) {
						bNewLineMode = true;
					}
					break;

				case '\r' :
					break;

				case ' ' :
					if (bNewLineMode == false) {
						strb.append(' ');
					}
					break;
					
				case '\t' :
					if (bNewLineMode == false) {
						strb.append("    ");
					}
					break;

				default :
					if (bNewLineMode == true) {
						printElementStringNewLine(strb, intbLineNumber);
						printElementStringIndentation(strb, iLocalIndentation);
						bNewLineMode = false;
					}
					strb.append(charaText[i]);
					break;
			}
		}
		printElementStringNewLine(strb, intbLineNumber);
	}
	return;
}
/**
 * Debug function that prints the entire element tree, and each element's
 * attributes starting at this element.
 */
public void printElementTree()
{
//	Trace.functionEntry( this, "printElementTree" );
	
	printElementTree( 0 );

//	Trace.functionExit( this, "printElementTree" );
}
/**
 * Debug function that prints the entire element tree, and each element's
 * attributes starting at this element.
 */
public void printElementTree( int iIndentation )
{
//	Trace.functionEntry( this, "printElementTree" );
	
	// Print the name of this element
	//-------------------------------
	StringBuffer strb = new StringBuffer();

	for( int i=0; i<iIndentation; ++i )
	{
		strb.append( ' ' );
	}
	
	if( _strName != null )
	{
		strb.append( "<" + _strName + ">" );
	}
	else
	{
		strb.append( "<>" );
	}

	strb.append( "{" );
	strb.append( Integer.toString(_iLineNumberFirst) );
	strb.append( ", " );
	strb.append( Integer.toString(_iLineNumberLast) );
	strb.append( "}" );

	System.out.println( strb.toString() );

	// Print out my attributes
	//------------------------
	if( _vectorAttributes != null )
	{		
		for( int i=0; i<_vectorAttributes.size(); ++i )
		{
			((ScriptUIEditorModelAttribute)_vectorAttributes.elementAt( i )).printAttribute( iIndentation + 2 );
		}
	}

	// Get my children to print themselves
	//------------------------------------
	if( _vectorChildElements != null )
	{
		for( int i=0; i<_vectorChildElements.size(); ++i )
		{
			((ScriptUIEditorModelElement)_vectorChildElements.elementAt( i )).printElementTree( iIndentation + 4 );
		}
	}

//	Trace.functionExit( this, "printElementTree" );
}
/**
 *
 */
public void removeAllChildElements() {
	if (_vectorChildElements != null) {
		_vectorChildElements.removeAllElements();
	}
}
/**
 */
public void removeChildElement( ScriptUIEditorModelElement elementChild )
{
	_vectorChildElements.removeElement( elementChild );
}
/**
 * Resets the property with the given id to its default value if possible.
 * Does nothing if the notion of default value is not meaningful for 
 * the specified property, or if the property's value cannot be changed,
 * or if this source does not have the specified property.
 */
public void resetPropertyValue(Object id) {}
/**
 * Sets the named property to the given value.
 */
public void setPropertyValue(Object objName, Object objValue) {
	
	String strName  = null;
	String strValue = null;

	if (objName instanceof String && objValue instanceof String) {
		strName  = (String) objName;
		strValue = (String) objValue;
	}

	if (strName != null && strValue != null && _vectorAttributes != null) {
	    
		for (int i = 0; i < _vectorAttributes.size(); ++i) {

			ScriptUIEditorModelAttribute attribute = (ScriptUIEditorModelAttribute) _vectorAttributes.elementAt(i);

			if (attribute._strName.equals(strName) == true) {
				attribute._strValue = strValue;

				// Notify the model that this element has changed
				//-----------------------------------------------
				if (_model != null) {
					_model.elementAttributeChanged(this);
				}
			}
		}
	}
}
/**
 */
public String toString()
{
	return _strName != null ? _strName : new String();
}
}
