package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */


import org.eclipse.swt.graphics.RGB;

import org.eclipse.jface.text.DefaultAutoIndentStrategy;
import org.eclipse.jface.text.IAutoIndentStrategy;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextDoubleClickStrategy;
import org.eclipse.jface.text.ITextHover;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.contentassist.ContentAssistant;
import org.eclipse.jface.text.contentassist.IContentAssistant;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.jface.text.rules.RuleBasedDamagerRepairer;
import org.eclipse.jface.text.source.IAnnotationHover;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewerConfiguration;


/**
 * Example configuration for an <code>SourceViewer</code> which shows Java code.
 */
public class ScriptUIEditorSourceViewerConfiguration extends SourceViewerConfiguration {
	
	/**
	 * Default constructor.
	 */
	public ScriptUIEditorSourceViewerConfiguration() {
	}
	/**
	 * @see SourceViewerConfiguration#getVerticalRulerHover
	 */
	public IAnnotationHover getAnnotationHover(ISourceViewer sourceViewer) {
		return new ScriptUIEditorAnnotationHover();
	}
	/**
	 * @see ISourceViewerConfiguration#getAutoIndentStrategy
	 */
	public IAutoIndentStrategy getAutoIndentStrategy(ISourceViewer sourceViewer, String contentType) {
		return (contentType == null ? new ScriptUIEditorAutoIndentStrategy() : new DefaultAutoIndentStrategy());
	}
	/**
	 * @see ISourceViewerConfiguration#getContentAssistant
	 */
	public IContentAssistant getContentAssistant(ISourceViewer sourceViewer) {

		ContentAssistant assistant= new ContentAssistant();
		assistant.setContentAssistProcessor(new ScriptUIEditorCompletionProcessor(), IDocument.DEFAULT_CONTENT_TYPE);
		assistant.setContentAssistProcessor(new ScriptUIEditorJavaDocCompletionProcessor(), ScriptUIEditorPartitionScanner.JAVA_DOC);

		assistant.enableAutoActivation(true);
		assistant.setAutoActivationDelay(500);
		assistant.setProposalPopupOrientation(assistant.PROPOSAL_OVERLAY);
		assistant.setContextInformationPopupOrientation(assistant.CONTEXT_INFO_ABOVE);
		assistant.setContextInformationPopupBackground(ScriptUIEditorEnvironment.getScriptUIEditorColorProvider().getColor(new RGB(150, 150, 0)));

		return assistant;
	}
	/**
	 * @see ISourceViewerConfiguration#getDefaultPrefix
	 */
	public String getDefaultPrefix(ISourceViewer sourceViewer, String contentType) {
		return (contentType == null ? "//" : null);
	}
	/**
	 * @see ISourceViewerConfiguration#getDoubleClickStrategy
	 */
	public ITextDoubleClickStrategy getDoubleClickStrategy(ISourceViewer sourceViewer, String contentType) {
		return new ScriptUIEditorDoubleClickSelector();
	}
	/**
	 * @see ISourceViewerConfiguration#getIndentPrefixes
	 */
	public String[] getIndentPrefixes(ISourceViewer sourceViewer, String contentType) {
		return new String[] {"\t", "    "};
	}
	/**
	 * @see ISourceViewerConfiguration#getPresentationReconciler
	 */
	public IPresentationReconciler getPresentationReconciler(ISourceViewer sourceViewer) {

		ScriptUIEditorColorProvider provider= ScriptUIEditorEnvironment.getScriptUIEditorColorProvider();
		PresentationReconciler reconciler= new PresentationReconciler();

		RuleBasedDamagerRepairer dr= new RuleBasedDamagerRepairer(
			ScriptUIEditorEnvironment.getScriptUIEditorCodeScanner(),
			new TextAttribute(provider.getColor(provider.DEFAULT))
		);
		reconciler.setDamager(dr, IDocument.DEFAULT_CONTENT_TYPE);
		reconciler.setRepairer(dr, IDocument.DEFAULT_CONTENT_TYPE);

		dr= new RuleBasedDamagerRepairer(
			ScriptUIEditorEnvironment.getScriptUIEditorDocScanner(),
			new TextAttribute(provider.getColor(provider.JAVADOC_DEFAULT))
		);
		reconciler.setDamager(dr, ScriptUIEditorPartitionScanner.JAVA_DOC);
		reconciler.setRepairer(dr, ScriptUIEditorPartitionScanner.JAVA_DOC);

		dr= new RuleBasedDamagerRepairer(
			null,
			new TextAttribute(provider.getColor(provider.MULTI_LINE_COMMENT))
		);		
		reconciler.setDamager(dr, ScriptUIEditorPartitionScanner.JAVA_MULTILINE_COMMENT);
		reconciler.setRepairer(dr, ScriptUIEditorPartitionScanner.JAVA_MULTILINE_COMMENT);

		return reconciler;
	}
	/**
	 * @see ISourceViewerConfiguration#getSupportedContentTypes
	 */
	public String[] getSupportedContentTypes(ISourceViewer sourceViewer) {
		return new String[] { IDocument.DEFAULT_CONTENT_TYPE, ScriptUIEditorPartitionScanner.JAVA_DOC,ScriptUIEditorPartitionScanner.JAVA_MULTILINE_COMMENT };
	}
	/**
	 * @see ISourceViewerConfiguration#getTabWidth
	 */
	public int getTabWidth(ISourceViewer sourceViewer) {
		return 4;
	}
	/**
	 * @see SourceViewerConfiguration#getTextHover(ISourceViewer, String)
	 */
	public ITextHover getTextHover(ISourceViewer sourceViewer, String contentType) {
		return new ScriptUIEditorTextHover();
	}
}
