package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 */
import java.util.Vector;

public class ScriptUIEditorXmlElement 
{
	public String        _strName             = null;
	public Vector        _vectorChildElements = null;
	public Vector        _vectorAttributes    = null;
	public ScriptUIEditorXmlElement _elementParent       = null;
	public int           _iLineNumberFirst = 0;
	public int           _iLineNumberLast  = 0;
/**
 */
public ScriptUIEditorXmlElement( ScriptUIEditorXmlElement elementParent, String strName, int iLineNumberFirst )
{
	_elementParent    = elementParent;
	_strName          = strName;
	_iLineNumberFirst = _iLineNumberLast = iLineNumberFirst;
}
/**
 */
public void addAttribute( ScriptUIEditorXmlAttribute attribute ) 
{
	if( _vectorAttributes == null )
	{
		_vectorAttributes = new Vector();
	}

	_vectorAttributes.addElement( attribute );
}
/**
 */
public void addChildElement( ScriptUIEditorXmlElement element ) 
{
	if( _vectorChildElements == null )
	{
		_vectorChildElements = new Vector();
	}

	_vectorChildElements.addElement( element );
}
/**
 */
public ScriptUIEditorXmlAttribute getAttribute( String strAttributeName ) 
{
	if( _vectorAttributes != null )
	{
		ScriptUIEditorXmlAttribute attribute = null;
		
		for( int i=0; i<_vectorAttributes.size(); ++i )
		{
			attribute = (ScriptUIEditorXmlAttribute)_vectorAttributes.elementAt( i );
			
			if( attribute._strName != null && attribute._strName.equals( strAttributeName ) == true )
			{
				return attribute;
			}
		}
	}

	return null;
}
/**
 */
public String getAttributeValue( String strAttributeName )
{
	ScriptUIEditorXmlAttribute attribute = getAttribute( strAttributeName );

	if( attribute != null )
	{
		return attribute._strValue;
	}
	
	return null;
}
/**
 */
public ScriptUIEditorXmlElement getChildElement( String strElementName )
{
	if( _vectorChildElements != null )
	{
		ScriptUIEditorXmlElement elementChild = null;
		
		for( int i=0; i<_vectorChildElements.size(); ++i )
		{
			elementChild = (ScriptUIEditorXmlElement)_vectorChildElements.elementAt( i );

			if( elementChild._strName != null && strElementName.equals( elementChild._strName ) == true )
			{
				return elementChild;
			}
		}
	}

	return null;
}
/**
 */
public ScriptUIEditorXmlElement getChildElementAt( int iIndex )
{
	return _vectorChildElements != null ? (ScriptUIEditorXmlElement)_vectorChildElements.elementAt( iIndex ) : null;
}
/**
 */
public int getNumberOfAttributes() 
{
	return _vectorAttributes != null ? _vectorAttributes.size() : 0;
}
/**
 */
public int getNumberOfChildElements() 
{
	return _vectorChildElements != null ? _vectorChildElements.size() : 0;
}
/**
 */
public ScriptUIEditorXmlElement getParentElement()
{
	return _elementParent;
}
/**
 * Debug function that prints the entire element tree, and each element's
 */
public void printElementTree()
{
//	Trace.functionEntry( this, "printElementTree" );
	
	printElementTree( 0 );

//	Trace.functionExit( this, "printElementTree" );
}
/**
 * Debug function that prints the entire element tree, and each element's
 */
public void printElementTree( int iIndentation )
{
//	Trace.functionEntry( this, "printElementTree" );
	
	// Print the name of this element
	//-------------------------------
	StringBuffer strb = new StringBuffer();

	for( int i=0; i<iIndentation; ++i )
	{
		strb.append( ' ' );
	}
	
	if( _strName != null )
	{
		strb.append( "<" + _strName + ">" );
	}
	else
	{
		strb.append( "<>" );
	}

	System.out.println( strb.toString() );

	// Print out my attributes
	//------------------------
	if( _vectorAttributes != null )
	{		
		for( int i=0; i<_vectorAttributes.size(); ++i )
		{
			((ScriptUIEditorXmlAttribute)_vectorAttributes.elementAt( i )).printAttribute( iIndentation + 2 );
		}
	}

	// Get my children to print themselves
	//------------------------------------
	if( _vectorChildElements != null )
	{
		for( int i=0; i<_vectorChildElements.size(); ++i )
		{
			((ScriptUIEditorXmlElement)_vectorChildElements.elementAt( i )).printElementTree( iIndentation + 4 );
		}
	}

//	Trace.functionExit( this, "printElementTree" );
}
/**
 */
public void removeChildElement( ScriptUIEditorXmlElement elementChild )
{
	_vectorChildElements.removeElement( elementChild );
}
/**
 */
public String toString()
{
	return _strName != null ? _strName : new String();
}
}
