package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.io.InputStream;

import org.eclipse.core.resources.IFile;
import org.eclipse.scripting.internal.IXGRReader;
import org.eclipse.scripting.internal.ScriptReaderLineInputStream;
import org.eclipse.scripting.internal.ScriptReaderString;
import org.eclipse.scripting.internal.ScriptReaderWorkbenchFile;
import org.eclipse.scripting.internal.ScriptStrings;

public class ScriptUIEditorXmlParser
{
	private final static int MODE_UNKNOWN           =  0;
	private final static int MODE_ATTRIBUTESTART    =  1;
	private final static int MODE_CDATA             =  2;
	private final static int MODE_CDATASTART        =  3;
	private final static int MODE_COMMENT           =  4;
	private final static int MODE_COMMENTSTART      =  5;
	private final static int MODE_DOCTYPESTART      =  6;
	private final static int MODE_ELEMENT           =  7;
	private final static int MODE_ELEMENTSTART      =  8;
	private final static int MODE_ELEMENTENDSTART   =  9;
	private final static int MODE_TEXT              = 10;
	private final static int MODE_TEXTSTART         = 11;
	private final static int MODE_XMLSTART          = 12;
	

	private   int                          _iMode                = MODE_UNKNOWN;
	protected IXGRReader                   _reader               = null;
	protected String                       _strFilename          = null;
	protected String                       _strLine              = null;
	protected ScriptUIEditorXmlAttribute              _parserTextAttributeCurrent = null;
	protected ScriptUIEditorXmlElement                _parserElementCurrent = null;
	protected ScriptUIEditorXmlElement                _parserElementRoot    = null;
	protected int                          _iLine                = -1; // 0 based
	protected int                          _iColumn              = -1; // 0 based


/**
 */
public ScriptUIEditorXmlParser() 
{
//	Trace.functionEntry( this, "XGRPresentationParser" );
//	Trace.functionExit( this, "XGRPresentationParser" );	
}
/**
 */
public ScriptUIEditorXmlElement getRootElement()
{
	return _parserElementRoot;
}
/**
 */
protected void handleAttribute() throws ScriptUIEditorXmlParserException
{
//	Trace.functionEntry( this, "handleAttribute" );

//	Trace.value( "Line number", _iLine );
//	Trace.stringValue( "Line", _strLine.substring( _iColumn ) );
	
	StringBuffer strbName  = new StringBuffer();
	StringBuffer strbValue = new StringBuffer();

	// Need to save the first column of the value
	//-------------------------------------------
	int iValueColumnNumber = -1;

	char character = ' ';

	// Obtain the attribute's name
	//----------------------------
	for( ; _iColumn<_strLine.length(); ++_iColumn )
	{
		character = _strLine.charAt( _iColumn );
		
		if( character == ' ' || character == '\t' || character == '=' )
		{
			break;
		}
		
		else if( character == '\n' || character == '\r' )
		{
			throw new ScriptUIEditorXmlParserException( ScriptStrings.getString("S_Unexpected_end_of_line"), _strFilename, _strLine, _iLine, _iColumn );
		}
		
		else
		{
			strbName.append( character );
		}
	}

	// Look for equal sign
	//--------------------
	for( ; _iColumn<_strLine.length(); ++_iColumn )
	{
		character = _strLine.charAt( _iColumn );
		
		if( character == ' ' || character == '\t' )
		{
			continue;
		}
		
		else if( character == '=' )
		{
			_iColumn++;
			break;
		}
	}

	// Look for double quote
	//----------------------
	for( ; _iColumn<_strLine.length(); ++_iColumn )
	{
		character = _strLine.charAt( _iColumn );
		
		if( character == ' ' || character == '\t' )
		{
			continue;
		}
		
		else if( character == '\"' )
		{
			// Remember where the value of the attribute started
			//--------------------------------------------------
			iValueColumnNumber = ++_iColumn;
			break;
		}
	}

	// Look for trailing double quote
	//-------------------------------
	for( ; _iColumn<_strLine.length(); ++_iColumn )
	{
		character = _strLine.charAt( _iColumn );
		
		if( character == '\"' )
		{
			_iColumn++;
			_iMode = MODE_ELEMENT;
			break;
		}

		// Add the character to the value string
		//--------------------------------------	
		else
		{
			strbValue.append( character );
		}
	}

	// Create an attribute object, and add it to
	// its parent element object
	//------------------------------------------
	if( strbName.length() > 0 )
	{
		ScriptUIEditorXmlAttribute attribute = new ScriptUIEditorXmlAttribute( strbName.toString(), strbValue.toString(), _strFilename, _iLine, iValueColumnNumber );
		if( _parserElementCurrent != null )
		{
			_parserElementCurrent.addAttribute( attribute );
		}
		
//		Trace.stringValue( strbName.toString(), strbValue.toString() );
	}
	
//	Trace.functionExit( this, "handleAttribute" );
}
/**
 */
protected void handleCDATA() 
{
//	Trace.functionEntry( this, "handleCommentBlock" );

	String strText = null;
	
	int iIndex = _strLine.indexOf( "]]>", _iColumn );
	
	if( iIndex > _iColumn )
	{
		strText = _strLine.substring( _iColumn, iIndex );
//		Trace.string( strText );

		// Point to after the -->
		//-----------------------
		_iColumn = iIndex + 3;
		_iMode = MODE_UNKNOWN;
	}
	
	else
	{
		strText = _strLine.substring( _iColumn );
		
//		Trace.string( strText );
		_iColumn = _strLine.length();
		_iMode = MODE_CDATA;
	}

	// Append the text to the current text attribute's text value
	//-----------------------------------------------------------
	if( strText != null && _parserTextAttributeCurrent != null )
	{
		_parserTextAttributeCurrent._strValue = _parserTextAttributeCurrent._strValue + "\n" + strText;
//		Trace.stringValue( "string", _parserTextAttributeCurrent._strValue );
	}
	
//	Trace.functionExit( this, "handleCommentBlock" );
}
/**
 */
protected void handleCDATAStart()
{
//	Trace.functionEntry( this, "handleCommentStart" );

	String strText = null;

	// Remember where the content starts
	//----------------------------------
	int iCDataStart = _iColumn + 9;
	
	int iIndex = _strLine.indexOf( "]]>" );
	
	if( iIndex > _iColumn + 9 )
	{
		strText = _strLine.substring( _iColumn + 9, iIndex );
		_iColumn = iIndex + 3;
		_iMode = MODE_UNKNOWN;
	}
	
	else
	{
		strText = _strLine.substring( _iColumn + 9 );
		_iColumn = _strLine.length();
		_iMode = MODE_CDATA;
	}
	
	if( strText != null )
	{
//		Trace.string( strText );
		
		// Create a #text element
		//-----------------------
		ScriptUIEditorXmlElement parserElement = new ScriptUIEditorXmlElement( _parserElementCurrent, "#text", _iLine  );

		// Add the child element to its parent
		//------------------------------------
		if( _parserElementCurrent != null )
		{
			_parserElementCurrent.addChildElement( parserElement );
		}

		// Create a "text" attribute
		//--------------------------
		ScriptUIEditorXmlAttribute attribute = new ScriptUIEditorXmlAttribute( "text", strText, _strFilename, _iLine, iCDataStart );
		parserElement.addAttribute( attribute );

		// Remember this attribute for the next line
		//------------------------------------------
		if( _iMode == MODE_CDATA )
		{
			_parserTextAttributeCurrent = attribute;
		}
	}
	
//	Trace.functionExit( this, "handleCommentStart" );
}
/**
 */
protected void handleComment() 
{
//	Trace.functionEntry( this, "handleCommentBlock" );

	String strText = null;
	
	int iIndex = _strLine.indexOf( "-->", _iColumn );
	
	if( iIndex > _iColumn )
	{
		strText = _strLine.substring( _iColumn, iIndex );
//		Trace.string( strText );

		// Point to after the -->
		//-----------------------
		_iColumn = iIndex + 3;
		_iMode = MODE_UNKNOWN;
	}
	
	else
	{
		strText = _strLine.substring( _iColumn );
		
//		Trace.string( strText );
		_iColumn = _strLine.length();
		_iMode = MODE_COMMENT;
	}

	// Append the text to the current text attribute's text value
	//-----------------------------------------------------------
	if( strText != null && _parserTextAttributeCurrent != null )
	{
		_parserTextAttributeCurrent._strValue = _parserTextAttributeCurrent._strValue + "\n" + strText;
//		Trace.stringValue( "string", _parserTextAttributeCurrent._strValue );
	}
	
//	Trace.functionExit( this, "handleCommentBlock" );
}
/**
 */
protected void handleCommentStart()
{
//	Trace.functionEntry( this, "handleCommentStart" );

	String strText = null;

	// Remember where the content starts
	//----------------------------------
	int iCommentStart = _iColumn + 4;
	
	int iIndex = _strLine.indexOf( "-->" );
	
	if( iIndex > _iColumn + 4 )
	{
		strText = _strLine.substring( _iColumn + 4, iIndex );
		_iColumn = iIndex + 3;
		_iMode = MODE_UNKNOWN;
	}
	
	else
	{
		strText = _strLine.substring( _iColumn + 4 );
		_iColumn = _strLine.length();
		_iMode = MODE_COMMENT;
	}
	
	if( strText != null )
	{
//		Trace.string( strText );
		
		// Create a COMMENT element
		//-------------------------
		ScriptUIEditorXmlElement parserElement = new ScriptUIEditorXmlElement( _parserElementCurrent, "#comment", _iLine  );

		// Add the child element to its parent
		//------------------------------------
		if( _parserElementCurrent != null )
		{
			_parserElementCurrent.addChildElement( parserElement );
		}

		// Create a "text" attribute
		//--------------------------
		ScriptUIEditorXmlAttribute attribute = new ScriptUIEditorXmlAttribute( "text", strText, _strFilename, _iLine, iCommentStart );
		parserElement.addAttribute( attribute );

		// Remember this attribute for the next line
		//------------------------------------------
		if( _iMode == MODE_COMMENT )
		{
			_parserTextAttributeCurrent = attribute;
		}
	}
	
//	Trace.functionExit( this, "handleCommentStart" );
}
/**
 */
protected void handleDoctype()
{
//	Trace.functionEntry( this, "handleDoctype" );

	int iIndex = _strLine.indexOf( ">" );;
	
	if( iIndex > _iColumn )
	{
		_iColumn = iIndex + 1;
		_iMode = MODE_UNKNOWN;
	}
	else
	{
		_iColumn = _strLine.length();
	}
	
//	Trace.functionExit( this, "handleDoctype" );
}
/**
 */
protected void handleElement()
{
//	Trace.functionEntry( this, "handleElement" );
	
//	Trace.stringValue( "Line", _strLine.substring( _iColumn ) );
	
	for( ; _iColumn<_strLine.length(); ++_iColumn )
	{
		char character = _strLine.charAt( _iColumn );

		// Look for non space
		//-------------------	
		if( character == ' ' || character == '\t' || character == '\r' || character == '\n' )
		{
			continue;
		}

		// End of element start
		//---------------------
		else if( character == '>' )
		{
			_iColumn++;
			_iMode = MODE_UNKNOWN;
			break;
		}

		// End of element
		//---------------
		else if( _strLine.indexOf( "/>", _iColumn ) == _iColumn )
		{
			_iColumn += 2;
			_iMode = MODE_UNKNOWN;

			// Pop the stack
			// Replace the current element with its parent
			//--------------------------------------------
			if( _parserElementCurrent != null )
			{
				_parserElementCurrent._iLineNumberLast = _iLine;
				_parserElementCurrent = _parserElementCurrent._elementParent;
			}
			break;
		}

		// Attribute start
		//----------------
		else
		{
			_iMode = MODE_ATTRIBUTESTART;
			break;
		}
	}

	
//	Trace.functionExit( this, "handleElement" );
}
/**
 */
protected void handleElementEnd() throws ScriptUIEditorXmlParserException
{
//	Trace.functionEntry( this, "handleElementEnd" );

	// </x>
	//-----
	int iIndex = _strLine.indexOf( '>', _iColumn );

	if( iIndex > _iColumn + 2 )
	{
		
		String strElementName = _strLine.substring( _iColumn + 2, iIndex );

//		Trace.stringValue( "Element Name", strElementName );

		if( _parserElementCurrent == null ){
			throw new ScriptUIEditorXmlParserException( ScriptStrings.getString( "S_Unexpected_end_element" ) + ": </" + strElementName + ">", _strFilename, _strLine, _iLine, _iColumn );			
		}

		if( strElementName.equals( _parserElementCurrent._strName ) == false )
		{
			throw new ScriptUIEditorXmlParserException( ScriptStrings.getString( "S_Expecting_end_element" ) + ": </" + _parserElementCurrent._strName + ">", _strFilename, _strLine, _iLine, _iColumn );
		}
			
		_iColumn = iIndex + 1;
		_iMode = MODE_UNKNOWN;
		
		// Pop the stack
		// Replace the current element with its parent
		//--------------------------------------------
		if( _parserElementCurrent != null )
		{
			_parserElementCurrent._iLineNumberLast = _iLine;
			_parserElementCurrent = _parserElementCurrent._elementParent;
		}
	}

	else
	{
		throw new ScriptUIEditorXmlParserException( ScriptStrings.getString("S_Expecting") + ": \">\"", _strFilename, _strLine, _iLine, _iColumn );
	}
	
//	Trace.functionExit( this, "handleElementEnd" );
}
/**
 */
protected void handleElementStart() throws ScriptUIEditorXmlParserException
{
//	Trace.functionEntry( this, "handleElementStart" );
	
	if( _strLine.indexOf( "<!--", _iColumn ) == _iColumn )
	{
		_iMode = MODE_COMMENTSTART;
	}
	
	else if( _strLine.indexOf( "<![CDATA[" ) == _iColumn )
	{
		_iMode = MODE_CDATASTART;
	}
	
	else if( _strLine.indexOf( "<!", _iColumn ) == _iColumn )
	{
		_iMode = MODE_DOCTYPESTART;
	}
		
	else if( _strLine.indexOf( "<?", _iColumn ) == _iColumn )
	{
		_iMode = MODE_XMLSTART;
	}

	else if( _strLine.indexOf( "</", _iColumn ) == _iColumn )
	{
		_iMode = MODE_ELEMENTENDSTART;
	}

	// Real element
	//-------------
	else
	{
		// Look for first non space character
		//-----------------------------------
		++_iColumn;
		
		StringBuffer strbName = new StringBuffer();

		for( ; _iColumn<_strLine.length(); ++_iColumn )
		{
			if( _strLine.charAt( _iColumn ) != ' ' &&
				_strLine.charAt( _iColumn ) != '\t'   )
			{
				break;
			}	
		}

		// Look for name
		//--------------
		char character = ' ';

		for( ; _iColumn<_strLine.length(); ++_iColumn )
		{
			character = _strLine.charAt( _iColumn );

			// End of element name
			//--------------------
			if( character == '>' )
			{
				_iColumn++;
				_iMode = MODE_UNKNOWN;
				break;
			}

			else if( character == '\r' || character == '\n' )
			{
				throw new ScriptUIEditorXmlParserException( ScriptStrings.getString("S_Expecting") + ": '>'", _strFilename, _strLine, _iLine, _iColumn );
			}
			
			else if( _strLine.indexOf( "/>", _iColumn ) == _iColumn )
			{
				_iColumn += 2;
				_iMode = MODE_UNKNOWN;
				break;
			}
			
			// Part of name
			//-------------
			else if( character != ' ' && character != '\t' )
			{
				strbName.append( character );
			}

			// End of name
			//------------
			else
			{
				_iMode = MODE_ELEMENT;
				break;
			}
		}

		// Create a new element node, and link it to its parent
		//-----------------------------------------------------
		if( strbName.length() > 0 )
		{
			ScriptUIEditorXmlElement parserElement = new ScriptUIEditorXmlElement( _parserElementCurrent, strbName.toString(), _iLine );

			// Add the child element to its parent
			//------------------------------------
			if( _parserElementCurrent != null )
			{
				_parserElementCurrent.addChildElement( parserElement );
			}

			// This must be the root element
			// since it doesn't have a parent
			//-------------------------------
			else
			{
				_parserElementRoot = parserElement;
			}

			// Set this element as the current element
			//----------------------------------------
			_parserElementCurrent = parserElement;
		}
		
//		Trace.stringValue( "Element Name", strbName.toString() );
	}

//	Trace.functionExit( this, "handleElementStart" );
}
/**
 */
protected void handleText()
{
//	Trace.functionEntry( this, "handleText" );

	String strText = null;
	
	int iIndex = _strLine.indexOf( "<", _iColumn );
	
	if( iIndex > _iColumn )
	{
		strText = _strLine.substring( _iColumn, iIndex );
//		Trace.string( strText );
		_iColumn = iIndex;
		_iMode = MODE_UNKNOWN;
	}
	
	else
	{
		strText = _strLine.substring( _iColumn );
		
//		Trace.string( strText );
		_iColumn = _strLine.length();
		_iMode = MODE_TEXT;
	}

	// Append the text to the current text attribute's text value
	//-----------------------------------------------------------
	if( strText != null && _parserTextAttributeCurrent != null )
	{
		_parserTextAttributeCurrent._strValue = _parserTextAttributeCurrent._strValue + "\n" + strText;
	}
	
//	Trace.functionExit( this, "handleText" );
}
/**
 */
protected void handleTextStart()
{
//	Trace.functionEntry( this, "handleTextStart" );

	String strText = null;

	// Remember the start of this attribute
	//-------------------------------------
	int iTextStart = _iColumn;
	
	int iIndex = _strLine.indexOf( "<", _iColumn );
	
	if( iIndex > _iColumn )
	{
		strText = _strLine.substring( _iColumn, iIndex );
//		Trace.string( strText );
		_iColumn = iIndex;
		_iMode = MODE_UNKNOWN;
	}
	
	else
	{
		strText = _strLine.substring( _iColumn );
		
//		Trace.string( strText );
		_iColumn = _strLine.length();
		_iMode = MODE_TEXT;
	}

	if( strText != null )
	{
		// Create a TEXT element
		//----------------------
		ScriptUIEditorXmlElement parserElement = new ScriptUIEditorXmlElement( _parserElementCurrent, "#text", _iLine );

		// Add the child element to its parent
		//------------------------------------
		if( _parserElementCurrent != null )
		{
			_parserElementCurrent.addChildElement( parserElement );
		}

		// Create a "text" attribute
		//--------------------------
		ScriptUIEditorXmlAttribute attribute = new ScriptUIEditorXmlAttribute( "text", strText, _strFilename, _iLine, iTextStart );
		parserElement.addAttribute( attribute );

		// Remember this attribute for the next line
		//------------------------------------------
		if( _iMode == MODE_TEXT )
		{
			_parserTextAttributeCurrent = attribute;
		}
	}
	
//	Trace.functionExit( this, "handleTextStart" );
}
/**
 */
protected void handleUnknown()
{
//	Trace.functionEntry( this, "handleUnknown" );
	
	for( ; _iColumn<_strLine.length(); ++_iColumn )
	{
		char character = _strLine.charAt( _iColumn );
		
		if( character == ' ' || character == '\t' || character == '\n' || character == '\r' )
		{
			continue;
		}

		else if( character == '<' )
		{
			_iMode = MODE_ELEMENTSTART;
			break;
		}
		
		else
		{
			_iMode = MODE_TEXTSTART;
			break;
		}
	}

//	Trace.functionExit( this, "handleUnknown" );
}
/**
 */
protected void handleXml()
{
//	Trace.functionEntry( this, "handleXml" );
		
	int iIndex = _strLine.indexOf( "?>" );
	
	if( iIndex > _iColumn )
	{
		_iColumn = iIndex + 2;
		_iMode = MODE_UNKNOWN;
	}
	else
	{
		_iColumn = _strLine.length();
	}
	
//	Trace.functionExit( this, "handleXml" );
}
/**
 */
public void parse() throws Exception
{
//	Trace.functionEntry( this, "parse" );

	_iMode = MODE_UNKNOWN;
	
	do
	{
		try
		{
			_strLine = _reader.readLine();
		}
		catch( Exception ex )
		{
			throw( ex );
		}
		
		if( _strLine != null )
		{
//			Trace.stringValue( "New Line", _strLine );
			processLine();
		}
	} while( _strLine != null );


	if( _parserElementCurrent != null )
	{
		throw new ScriptUIEditorXmlParserException( ScriptStrings.getString( "S_Expecting_end_element" ) + ": </" + _parserElementCurrent._strName + ">", _strFilename, _strLine, _iLine, _iColumn );
	}
	
//	Trace.functionExit( this, "parse" );
}
/**
 */
protected void processLine() throws ScriptUIEditorXmlParserException
{
//	Trace.functionEntry( this, "processLine" );

	_iLine++;
	_iColumn = 0;

	// Continue in the mode that was in effect at the end of the last line
	//--------------------------------------------------------------------
	while( _iColumn < _strLine.length() )
	{
		switch( _iMode )
		{
			case MODE_UNKNOWN:           handleUnknown();      break;
			case MODE_ATTRIBUTESTART:    handleAttribute();    break;
			case MODE_CDATA:             handleCDATA();        break;
			case MODE_CDATASTART:        handleCDATAStart();   break;
			case MODE_COMMENT:           handleComment();      break;
			case MODE_COMMENTSTART:      handleCommentStart(); break;
			case MODE_DOCTYPESTART:      handleDoctype();      break;
			case MODE_ELEMENT:           handleElement();      break;
			case MODE_ELEMENTSTART:      handleElementStart(); break;
			case MODE_ELEMENTENDSTART:   handleElementEnd();   break;
			case MODE_TEXT:              handleText();         break;
			case MODE_TEXTSTART:         handleTextStart();    break;
			case MODE_XMLSTART:          handleXml();          break;
//			default: Trace.string( "Unknown mode: " + Integer.toString( _iMode ) );
		}
	}

//	Trace.functionExit( this, "processLine" );
}
/**
 */
public void setFile( IFile file ) throws Exception
{
	if( file != null )
	{
		_strFilename = file.getFullPath().toString();

		// Open the file
		// This reader always replace tab characters with spaces
		//------------------------------------------------------
		_reader = new ScriptReaderWorkbenchFile( file );
	}
}
/**
 *
 */
public void setInputStream( InputStream inputStream ) throws Exception
{
	if( inputStream != null )
	{
		// Open the file
		// This reader always replace tab characters with spaces
		//------------------------------------------------------
		_reader = new ScriptReaderLineInputStream( inputStream );
	}
}
/**
 * Sets the string buffer containing the xml text.
 */
public void setString( String strContent ) {
	_reader = new ScriptReaderString( strContent );
}
}
