package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.jface.viewers.ContentViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.scripting.internal.ScriptRunnerExtended;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.part.FileEditorInput;

public class ScriptUIPreviewEditorContent extends ContentViewer
															
{
	protected Composite                         _compositeClient         = null;
	protected ScriptRunnerExtended              _scriptRunner            = null;
	protected ScriptUIPreviewEditorPreviewPanel _previewPanel            = null;
	protected String                            _strPresentationFilename = null;
	protected String                            _strPresentationType     = "wizard";

/**
 * Constructor.
 */
public ScriptUIPreviewEditorContent( Composite compositeParent ) 
{
//	super.setHandleInputStrategy( this );

	// Client composite
	// This has to be here.  Otherwise, the title above the tabs
	// is not drawn correctly
	//----------------------------------------------------------
	_compositeClient = new Composite( compositeParent, SWT.NULL );
	
	GridLayout layout= new GridLayout();
	_compositeClient.setLayout( layout );

	GridData gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.verticalAlignment   = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.grabExcessVerticalSpace   = true;
	_compositeClient.setLayoutData( gridData );

	// Create preview panel child
	//---------------------------
	_previewPanel = new ScriptUIPreviewEditorPreviewPanel( _compositeClient, false );
}
/**
 * Returns the primary control associated with this viewer.
 */
public Control getControl() {
	return _previewPanel;
}
public ISelection getSelection() {
	return StructuredSelection.EMPTY;
}
/**
 * Refreshes this viewer completely, getting new contents
 * for the input from the content provider.
 */
public void refresh() {}
/**
 * A script runner is created, and is used to run the script, or
 * render the user interface.
 */
protected void run()
{
//	Trace.functionEntry( this, "run" );

	_previewPanel.setPresentationFilename( _strPresentationFilename );
	_previewPanel.render();
	
//	Trace.functionExit( this, "run" );
}
/**
 * setInitializationData method comment.
 */
protected void setInitializationData( IConfigurationElement configElement,
								   String                strName,
								   Object                objData)
{
//	Trace.functionEntry( this, "setInitializationData" );

	_scriptRunner = new ScriptRunnerExtended( configElement, objData );
	_scriptRunner.setBuildTime( true );	

//	Trace.functionExit( this, "setInitializationData" );
}
/**
 */
public void setInput( Object object ) 
{
//	Trace.functionEntry( this, "setInput" );

	if( object instanceof FileEditorInput )
	{
		String strFilename = ((FileEditorInput)object).getFile().getLocation().toOSString();

		_strPresentationFilename = strFilename;
	}

//	Trace.functionExit( this, "setInput" );
}
public void setSelection(ISelection selection, boolean reveal) {}
}
