package org.eclipse.scripting.internal.views;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 *
 */
import org.eclipse.scripting.internal.ScriptPlugin;
import org.eclipse.scripting.internal.ScriptStrings;
import org.eclipse.scripting.internal.uirenderer.ScriptViewport;
import org.eclipse.scripting.internal.uirenderer.XGR;
import org.eclipse.scripting.internal.uirenderer.XGRPresentationModelElement;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.BusyIndicator;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.swt.widgets.Widget;

public class ScriptUIPreviewEditorPreviewPanel extends Composite implements SelectionListener
{
	protected int                     _iPresentationType       = XGR.WIZARD;
	protected Button                  _buttonAutoRefresh       = null;
	protected String                  _strPresentationFilename = null;
	protected String                  _strPresentationString   = null;
	protected ScriptViewport          _viewport                = null;
	protected ToolItem                _toolItemRefresh         = null;
	protected XGR                     _xgr                     = null;
	protected XGRPresentationModelElement _elementHighlight    = null;
	protected boolean                 _bRenderPending          = true;
	protected static Color _colorHighlight = null;
/**
 */
public ScriptUIPreviewEditorPreviewPanel(Composite compositeParent, boolean bRefreshControls) {
	
	super(compositeParent, SWT.BORDER);

	_colorHighlight = new Color(getDisplay(), 255, 255, 0);

	GridLayout layout = new GridLayout();
	layout.numColumns = 1;
	layout.marginHeight = 0;
	layout.marginWidth = 0;
	setLayout(layout);

	GridData gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.verticalAlignment = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.grabExcessVerticalSpace = true;
	setLayoutData(gridData);

	Composite compositeControlPanel = new Composite(this, SWT.NULL);

	layout = new GridLayout();
	layout.numColumns = 3;
	layout.marginHeight = 0;
	layout.marginWidth = 0;
	compositeControlPanel.setLayout(layout);

	gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.verticalAlignment = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.widthHint = 170;
	compositeControlPanel.setLayoutData(gridData);

	// Label
	//------
	Label labelPresentationType = new Label(compositeControlPanel, SWT.NULL);
	labelPresentationType.setText(ScriptStrings.getString("S_Presentation_Type") + ": ");

	gridData = new GridData();
	gridData.horizontalIndent = 8;
	labelPresentationType.setLayoutData(gridData);

	// Combo box
	//----------
	Combo comboPresentationType = new Combo(compositeControlPanel, SWT.READ_ONLY);

	gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.horizontalSpan = 2;
	comboPresentationType.setLayoutData(gridData);

	comboPresentationType.add(ScriptStrings.getString("S_wizard"));
	comboPresentationType.add(ScriptStrings.getString("S_preferences"));
	comboPresentationType.add(ScriptStrings.getString("S_panel"));
//	comboPresentationType.add(ScriptStrings.getString("S_property_sheet"));

	comboPresentationType.setText(ScriptStrings.getString("S_wizard"));

	// Add combo listener
	//-------------------
	comboPresentationType.addSelectionListener(this);

	if (bRefreshControls == true) {

		// Add auto-refresh checkbox
		//--------------------------
		_buttonAutoRefresh = new Button(compositeControlPanel, SWT.CHECK);
		_buttonAutoRefresh.setText(ScriptStrings.getString("S_AutoRefresh"));
		_buttonAutoRefresh.setSelection(true);

		gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 2;
		gridData.horizontalIndent = 8;
		_buttonAutoRefresh.setLayoutData(gridData);

		// Add button listener
		//-------------------	
		_buttonAutoRefresh.addSelectionListener(this);

		// Add Refresh button
		//-------------------
/*		
		_buttonRefresh = new Button(compositeControlPanel, SWT.PUSH);
		_buttonRefresh.setText(ScriptStrings.getString("S_Refresh"));
		_buttonRefresh.setToolTipText(ScriptStrings.getString("S_Refresh"));
		gridData = new GridData();
		gridData.horizontalAlignment = GridData.END;
		_buttonRefresh.setLayoutData(gridData);

		// Set button icon
		//----------------
		ImageDescriptor imageDescriptor = ScriptPlugin.getImageDescriptor("icons/full/clcl16/refresh_edit.gif");
		_buttonRefresh.setImage(imageDescriptor.createImage());

		// Add button listener
		//-------------------	
		_buttonRefresh.addSelectionListener(this);
*/
		ToolBar toolbar = new ToolBar( compositeControlPanel, SWT.FLAT );
		gridData = new GridData();
		gridData.horizontalAlignment = GridData.END;
		toolbar.setLayoutData(gridData);
		_toolItemRefresh = new ToolItem(toolbar, SWT.PUSH);
		_toolItemRefresh.addSelectionListener(this);
		_toolItemRefresh.setImage(ScriptPlugin.getImageDescriptor("icons/full/elcl16/refresh_edit.gif").createImage());
		_toolItemRefresh.setHotImage(ScriptPlugin.getImageDescriptor("icons/full/clcl16/refresh_edit.gif").createImage());
		_toolItemRefresh.setDisabledImage(ScriptPlugin.getImageDescriptor("icons/full/dlcl16/refresh_edit.gif").createImage());
		_toolItemRefresh.setToolTipText( ScriptStrings.getString( ScriptStrings.getString("S_Refresh") ));
	}

	// Build the separator line
	//-------------------------
	Label separator = new Label(compositeControlPanel, SWT.HORIZONTAL | SWT.SEPARATOR);

	gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.horizontalSpan = 3;
	separator.setLayoutData(gridData);

	compositeControlPanel.layout(true);

	// Composite with scroll bars for containing rendered UI
	//------------------------------------------------------
	gridData = new GridData();
	gridData.horizontalAlignment = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	gridData.verticalAlignment = GridData.FILL;
	gridData.grabExcessVerticalSpace = true;

	_viewport = new ScriptViewport(this, SWT.H_SCROLL | SWT.V_SCROLL, gridData);
}
/**
 */
public XGRPresentationModelElement getRootModelElement()
{
	if( _xgr != null )
	{
		return _xgr.getRootModelElement();
	}
	return null;
}
/**
 */
public void highlightUIElement(Object objElement)
{
	// Unhighlight the current highlighted element
	//--------------------------------------------
	if( _elementHighlight != null )
	{
		if( _elementHighlight._elementUI != null )
		{
			_elementHighlight._elementUI.highlight( false, null );
		}
	}

	// Highlight the new element
	//--------------------------
	if( objElement instanceof XGRPresentationModelElement )
	{
		XGRPresentationModelElement element = (XGRPresentationModelElement)objElement;
		if( element._elementUI != null )
		{
			element._elementUI.highlight( true, _colorHighlight );
			_elementHighlight = element;
		}
	}
}
/**
 */
protected void render() {

	if (_toolItemRefresh != null) {
		_toolItemRefresh.setEnabled(false);
	}

	// Dispose the current content
	//----------------------------
	_viewport.setRedraw(false);

	if (_xgr != null) {
		Object objWidget = _xgr.getWidget();

		if (objWidget != null && objWidget instanceof Composite) {
			if (!((Composite) objWidget).isDisposed()) {
				((Composite) objWidget).dispose();
			}
		}

		_xgr = null;
	}

	// Build the editor content
	//-------------------------
	run();

	// Layout the container
	//---------------------
	_viewport.layout();
	_viewport.setRedraw(true);

	if (_toolItemRefresh != null) {
		_toolItemRefresh.setEnabled(true);
	}
}
/**
 */
protected void renderWhenVisible() {

	if (isVisible() == true && _buttonAutoRefresh.getSelection() == true) {
	    _bRenderPending = false;
		render();
	}

	else {
		_bRenderPending = true;
	}
}
/**
 */
public void run() {
	//	Trace.functionEntry( this, "run" );
	
	BusyIndicator.showWhile( getDisplay(), new Runnable() {
		public void run() {

			try {
				_xgr = new XGR();
				_xgr.setBuildTime(true);

				if (_strPresentationFilename != null) {
					_xgr.setPresentationFilename(_strPresentationFilename);
				}
				else {
					_xgr.setPresentationString(_strPresentationString);
				}
				_xgr.setPresentationType(_iPresentationType);
				_xgr.execute(_viewport, true);
				_viewport.layout(true);
				_viewport.updateScrollbars();
			}
			catch (Exception ex) {
//				Trace.string(ex.toString());
				ex.printStackTrace();
			}
		}
	});

	//	Trace.functionExit( this, "run" );
}
/**
 */
public void setPresentationFilename(String strPresentationFilename)
{
	_strPresentationFilename = strPresentationFilename;
}
/**
 * The UI definition is in a string instead of being in a file.
 */
public void setPresentationString( String strPresentationString ) {
	_strPresentationString = strPresentationString;
}
/**
 */
public void setVisible(boolean bVisible) {

	if (_buttonAutoRefresh != null) {
		if (bVisible == true && _bRenderPending == true && _buttonAutoRefresh.getSelection() == true) {
			render();
			_bRenderPending = false;
		}
	}
	else
	{
	    render();
	}
	super.setVisible(bVisible);
}
/**
 */
public void treeChanged( XGRPresentationModelElement element )
{
	// The xml hierarchy/widget hierarchy has changed
	// Update the rendered UI if auto refresh is set
	//-----------------------------------------------
	if( _buttonAutoRefresh.getSelection() == true )
	{
		updatePreview( element );
		_bRenderPending = false;
	}
}
/**
 */
private void updatePreview(final XGRPresentationModelElement element) {

	if (isVisible() == true) {
		setRedraw(false);

		if (_viewport.isDisposed() == false) {
			_viewport.dispose();
		}

		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;

		_viewport = new ScriptViewport(this, SWT.H_SCROLL | SWT.V_SCROLL, gridData);

		BusyIndicator.showWhile(getDisplay(), new Runnable() {
			public void run() {
				_xgr.setPresentationType(_iPresentationType);
				_xgr.treeChanged(_viewport, element);

				layout(true);

				_viewport.updateScrollbars();
			}
		});
		
		setRedraw(true);
	}
}
/**
 */
public void widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent e) {}
/**
 */
public void widgetSelected(org.eclipse.swt.events.SelectionEvent sEvent) {
	
	Widget obj = sEvent.widget;

	// Combobox item was selected
	//---------------------------	
	if (obj instanceof Combo && ((Combo) obj).getSelectionIndex() > -1) {
		
		// Switch on the index, not the translated string
		//-----------------------------------------------
		int iPresentationType = XGR.WIZARD;

		switch (((Combo) obj).getSelectionIndex()) {
			case 0 :
				iPresentationType = XGR.WIZARD;
				break;
			case 1 :
				iPresentationType = XGR.PREFERENCES;
				break;
			case 2 :
				iPresentationType = XGR.PANEL;
				break;
			case 3 :
				iPresentationType = XGR.PROPERTYSHEET;
				break;
		}

		if (iPresentationType != _iPresentationType) {
			_iPresentationType = iPresentationType;
			updatePreview(null);
			_bRenderPending = false;
		}
	}

	// Auto Refresh checkbox
	//----------------------
	else if (obj == _buttonAutoRefresh) {
		if (_buttonAutoRefresh.getSelection() == true && _bRenderPending == true) {
			render();
			_bRenderPending = false;
		}
	}

	// Refresh button was pressed
	//---------------------------
	else if (obj == _toolItemRefresh) {
		render();
		_bRenderPending = false;
	}
}
}
