package org.eclipse.vcm.internal.core.ccvs;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.util.ArrayList;
import java.util.List;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.vcm.internal.core.VCMException;
import org.eclipse.vcm.internal.core.adapter.IRepositoryAdapter;
import org.eclipse.vcm.internal.core.adapter.IRepositoryAdapterFactory;
import org.eclipse.vcm.internal.core.base.IRepositoryLocation;
import org.eclipse.vcm.internal.core.ccvs.base.IConnectionMethod;
import org.eclipse.vcm.internal.core.ccvs.client.PServerConnectionMethod;
import org.eclipse.vcm.internal.core.ccvs.CVSPlugin;

public class CVSAdapterFactory implements IRepositoryAdapterFactory {
private static IConnectionMethod[] pluggedInConnectionMethods = null;

public CVSAdapterFactory() {
}
/** 
 * Returns a repository adapter for the given location.
 * @throw VCMException if this method fails. Reasons include:
 * the location is not valid
 */
public IRepositoryAdapter getAdapter(IRepositoryLocation location) throws VCMException {
	return new CVSAdapter(location);
}
public static IConnectionMethod[] getPluggedInConnectionMethods() {
	if(pluggedInConnectionMethods==null) {
		List connectionMethods = new ArrayList();
		connectionMethods.add(new PServerConnectionMethod());
		IExtension[] extensions = Platform.getPluginRegistry().getExtensionPoint(CVSPlugin.PI_CVS, CVSPlugin.PT_CONNECTIONMETHODS).getExtensions();
		for(int i=0; i<extensions.length; i++) {
			IExtension extension = extensions[i];
			IConfigurationElement[] configs = extension.getConfigurationElements();
			if (configs.length == 0) {
				CVSPlugin.log(new CVSStatus(ICVSStatus.ERROR, null, CVSPlugin.getResourceString("CVSAdapterFactory.execProblem"), null));
				continue;
			}
			try {
				IConfigurationElement config = configs[0];
				connectionMethods.add(config.createExecutableExtension("run"));
			} catch (CoreException ex) {
				CVSPlugin.log(new CVSStatus(ICVSStatus.ERROR, null, CVSPlugin.getResourceString("CVSAdapterFactory.execProblem"), ex));
			}
		}
		pluggedInConnectionMethods = (IConnectionMethod[])connectionMethods.toArray(new IConnectionMethod[0]);
	}
	return pluggedInConnectionMethods;
}
public String getType() {
	return "CVS";
}
public IStatus validateRepositoryLocation(String location) {
	if (location == null)
		return new CVSStatus(CVSStatus.ERROR, null, CVSPlugin.getResourceString("CVSAdapterFactory.nullLocation"));
	if (location.equals(""))
		return new CVSStatus(CVSStatus.ERROR, null, CVSPlugin.getResourceString("CVSAdapterFactory.emptyLocation"));
	if (location.endsWith(" ") || location.endsWith("\t"))
		return new CVSStatus(CVSStatus.ERROR, null, CVSPlugin.getResourceString("CVSAdapterFactory.endWhitespace"));
	if (!location.startsWith(":") || location.indexOf(':', 1)==-1)
		return new CVSStatus(CVSStatus.ERROR, null, CVSPlugin.getResourceString("CVSAdapterFactory.startOfLocation"));
	int secondColon = location.indexOf(':', 1);
	String method = location.substring(1,secondColon);
	IConnectionMethod[] methods = getPluggedInConnectionMethods();
	boolean methodSupported = false;
	StringBuffer methodNames = new StringBuffer();
	for(int i=0; i<methods.length; i++) {
		String name = methods[i].getName();
		if (i>0)
			methodNames.append(", ");
		methodNames.append(name);
		if(name.equals(method))
			methodSupported = true;
	}
	if(!methodSupported)
		return new CVSStatus(CVSStatus.ERROR, null, CVSPlugin.getResourceString("CVSAdapterFactory.methods", new Object[] {methodNames}));

	String formatError = CVSPlugin.getResourceString("CVSAdapterFactory.locationForm");
	int at = location.indexOf('@');
	if (at==-1)
		return new CVSStatus(CVSStatus.ERROR, null, formatError);
	String user = location.substring(secondColon + 1, at);
	if (user.equals(""))
		return new CVSStatus(CVSStatus.ERROR, null, formatError);
	int colon = location.indexOf(':', at);
	if (colon==-1)
		return new CVSStatus(CVSStatus.ERROR, null, formatError);
	String host = location.substring(at + 1, colon);
	if (host.equals(""))
		return new CVSStatus(CVSStatus.ERROR, null, formatError);
	String path = location.substring(colon + 1, location.length());
	if (path.equals(""))
		return new CVSStatus(CVSStatus.ERROR, null, formatError);
	return new CVSStatus(ICVSStatus.OK, "ok");
}
}
