package org.eclipse.vcm.internal.core.ccvs;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.vcm.internal.core.ccvs.utils.*;
import org.eclipse.vcm.internal.core.ccvs.requests.*;
import org.eclipse.vcm.internal.core.ccvs.requests.ILogListener;
import org.eclipse.vcm.internal.core.ccvs.client.*;
import org.eclipse.vcm.internal.core.StateInfo;
import org.eclipse.core.runtime.*;
import org.eclipse.core.resources.*;
import org.eclipse.vcm.internal.core.base.*;
import org.eclipse.vcm.internal.core.adapter.*;
import org.eclipse.vcm.internal.core.VCMException;
import java.util.*;
import java.io.*;

public class CVSResource implements IServerResource {
	protected int type;
	protected IPath path;
	protected Context context;
	protected String stateId;
/**
 * @deprecated. ServerResources don't have paths anymore.
 */
CVSResource(Context context, IPath path, int type, String stateId) {
	this.context = context;
	this.path = path;
	this.type = type;
	this.stateId = stateId;
}
public InputStream fetchContent(IProgressMonitor monitor) throws VCMException {
	Client client = new Client(context.adapter, null);
	final ByteArrayOutputStream bos = new ByteArrayOutputStream();
	final List errors = new ArrayList();
	Request request = new Update(context.getPathPrefix(), path, new String[] { "-r" + stateId }, null, false, new DirectoryEntryIterator(new IPath[] { path.removeLastSegments(1)}), null, new IUpdatedListener() {
		public void fileInformation(IPath contextRelativeDirectory, String entryLine, InputStream content, int bytesToRead) {
			try {
				StreamUtil.transfer(content, bos, bytesToRead);
			} catch (IOException ex) {
				errors.add(new CVSStatus(ICVSStatus.ERROR, context.getPathPrefix().append(path), CVSPlugin.getResourceString("CVSResource.ioError"), ex));
			}
		}
	});
	client.execute(request);
	client.disconnect();
	if (errors.size() > 0) {
		MultiStatus multi = new MultiStatus(CVSPlugin.PI_CVS, ICVSStatus.SERVER_ERROR, (IStatus[]) errors.toArray(new IStatus[errors.size()]), CVSPlugin.getResourceString("CVSResource.ioLoad"), null);
		throw new CVSException(multi);
	}
	return new ByteArrayInputStream(bos.toByteArray());
}
public IStateInfo fetchStateInfo(IProgressMonitor monitor) throws VCMException {
	if (getType() == IResource.FOLDER || getType() == IResource.PROJECT) {
		String versionName = getVersionName();
		if(versionName==null
			|| versionName.equals("HEAD")
			|| Util.arrayIncludes(CVSPlugin.getInstance().getTeamStreamNames(context.getPathPrefix().lastSegment()), versionName)) {
				String na = CVSPlugin.getResourceString("na");
				return new StateInfo(getStateId(), new Date(), na, na, CVSAdapter.FOLDER_REVISION);
		}
		IStateInfo result = context.adapter.getTagInfoEntry(context.getPathPrefix().lastSegment(), getVersionName(), monitor);
		if (result == null)
			throw new CVSException(ICVSStatus.SERVER_ERROR, context.getPathPrefix().append(path), CVSPlugin.getResourceString("CVSResource.tagNotFound"), null);
		return result;
	}
	// Handle files
	final String[] infos = new String[4];
	Request log = new Log(context.getPathPrefix().append(path.removeLastSegments(1)), new Path(path.lastSegment()), stateId, new ILogListener() {
		public void log(String revision, String author, String date, String comment, String state, String[] tags) {
			infos[0] = author;
			infos[1] = date;
			infos[2] = comment;
			infos[3] = tagsToString(tags);
		}
	});
	Client client = new Client(context.adapter, null);
	client.execute(log);
	client.disconnect();

	return new StateInfo(new StateId(stateId), DateUtil.convertFromLogTime(infos[1]), infos[0], infos[2], getVersionName(), infos[3]);
}
public String getName() {
	return path.lastSegment();
}
String getRevisionBranchPart() {
	return stateId.substring(0, stateId.lastIndexOf('.'));
}
int getRevisionNumberPart() {
	return Integer.valueOf(stateId.substring(stateId.lastIndexOf('.') + 1)).intValue();
}
public IServerWorkspace getServerWorkspace() {
	// Not sure that we should be exposing the context, just not sure how to implement fetchFileHistory() without it.
	return context;
}
public String getShallowVersionName() {
	return stateId;
}
public IStateId getStateId() {
	return new StateId(stateId);
}
public int getType() {
	return type;
}
public String getVersionName() {
	return stateId;
}
public static String tagsToString(String[] tags) {
	StringBuffer tagBuffer = new StringBuffer();
	for(int i=0; i<tags.length; i++) {
		if(i>0) tagBuffer.append(", ");
		tagBuffer.append(tags[i]);
	}
	return tagBuffer.toString();
}
public String toString() {
	return path + " in CVS context " + context.getPathPrefix();
}
}
