package org.eclipse.vcm.internal.core.ccvs;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.core.runtime.*;
import org.eclipse.vcm.internal.core.ccvs.requests.*;
import org.eclipse.core.resources.IResource;

import org.eclipse.vcm.internal.core.Policy;

public class CVSResourceTreeBuilder implements IStatusListener {
	CVSResourceTree tree;
	Context context;
	IPath projectRelativeRootPath;
public CVSResourceTreeBuilder(Context context, IPath projectRelativeRootPath) {
	this.context = context;
	this.projectRelativeRootPath = projectRelativeRootPath;
	tree = new CVSResourceTree(null);
}
void addNode(CVSResourceTree tree, IPath relativePath, int segmentIndex, String revision) {
	// segmentIndex iterates over the segments of relativePath
	if (segmentIndex >= relativePath.segmentCount()) {
		// we are at the leaf node we want to add
		int type;
		if (revision.equals(CVSAdapter.FOLDER_REVISION)) {
			type = IResource.FOLDER;
		} else {
			type = IResource.FILE;
		}
		tree.resource = new CVSResource(context, projectRelativeRootPath.append(relativePath), type, revision);
	} else {
		// we are at a parent, create folder resource if necessary
		if (tree.getResource() == null) {
			if (projectRelativeRootPath.isEmpty() && segmentIndex == 0) {
				tree.resource = new CVSModule(context, CVSAdapter.FOLDER_REVISION);
			} else {
				tree.resource = new CVSResource(context, projectRelativeRootPath.append(relativePath.uptoSegment(segmentIndex)), IResource.FOLDER, CVSAdapter.FOLDER_REVISION);
			}
		}
		String currentSegment = relativePath.segment(segmentIndex);
		CVSResourceTree childTree = (CVSResourceTree) tree.getChildTree(currentSegment);
		tree.getChildren().put(currentSegment, childTree);
		addNode(childTree, relativePath, segmentIndex + 1, revision);
	}
}
public void fileStatus(IPath projectRelativePath, String remoteRevision) {
	addNode(tree, projectRelativePath.removeFirstSegments(projectRelativeRootPath.segmentCount()), 0, remoteRevision);
}
public CVSResourceTree getBuiltTree() {
	return tree;
}
}
