package org.eclipse.vcm.internal.core.ccvs;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.vcm.internal.core.base.ISubscriber;
import org.eclipse.vcm.internal.core.base.ISyncInfo;
import org.eclipse.core.runtime.*;
import org.eclipse.core.resources.*;
import org.eclipse.vcm.internal.core.*;
import java.util.*;
import org.eclipse.vcm.internal.core.ccvs.requests.*;
import org.eclipse.vcm.internal.core.base.IResourceEdition;

public class CatchupEntryIterator implements IEntryIterator {
	SortedMap memberMap;
	Iterator parentIterator;
	Iterator memberIterator;
	ISubscriber subscriber;
public CatchupEntryIterator(SortedSet resources, ISubscriber subscriber) {
	memberMap = new TreeMap(new ResourceComparator());
	for (Iterator it = resources.iterator(); it.hasNext();) {
		IResource resource = (IResource) it.next();
		IContainer parent = resource.getParent();
		// Consider parents that are projects or folders. Ignore the
		// workspace root.
		if( parent.getFullPath().segmentCount() > 0 ) {
			List members = (List) memberMap.get(parent);
			if (members == null)
				members = new ArrayList(7);
			if (resource.getType() == IResource.FILE)
				members.add(resource);
			memberMap.put(parent, members);
		}
	}
	this.memberIterator = null;
	this.parentIterator = memberMap.keySet().iterator();
	this.subscriber = subscriber;
	
}
/**
 * Returns the new directory path, or null.
 */
public IPath nextDirectory() {
	if (parentIterator.hasNext()) {
		IContainer currentParent = (IContainer) parentIterator.next();
		memberIterator = ((Collection)memberMap.get(currentParent)).iterator();
		return currentParent.getProjectRelativePath();
	} else
		return null;
}
/**
 *  Returns the next entry line for the current directory, or null.
 */
public String nextEntryLine() {
	IResource resource = null;
	IResourceEdition remote = null;
	// remote may be null if the resource has already been deleted
	// (this happens if there is an incoming delete for one of its parents)
	while (remote == null) {
		if (memberIterator == null || !memberIterator.hasNext())
			return null;
		resource = (IResource) memberIterator.next();
		ISyncInfo info = subscriber.getSyncInfo(resource);
		remote = info == null ? null : info.getRemote();
	}
	String remoteRevision = ((StateId) remote.getStateId()).getId();
	String entryLine = "/" + resource.getName() + "/1.1///T" + remoteRevision;
	return entryLine;
}
}
