package org.eclipse.vcm.internal.core.ccvs;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.io.*;
import org.eclipse.core.runtime.*;
import java.util.*;
import org.eclipse.core.resources.*;
import org.eclipse.vcm.internal.core.ccvs.requests.*;
import org.eclipse.vcm.internal.core.base.*;

public class DiffEntryIterator implements IEntryContentIterator {
	ISharingManager sharingManager;
	Iterator resourceIterator;
	IResource currentResource;
	boolean entrySent = false;
	String tagName;

public DiffEntryIterator(SortedSet resources, ISharingManager sharingManager, String tagName) {
	this.sharingManager = sharingManager;
	this.resourceIterator = resources.iterator();
	this.tagName = tagName;
}
public IPath nextDirectory() {
	if (currentResource == null || currentResource.getType() == IResource.FILE) {
		do {
			if (!resourceIterator.hasNext())
				return null;
			currentResource = (IResource) resourceIterator.next();
		} while (currentResource.getType() == IResource.FILE);
	}
	IPath result = currentResource.getProjectRelativePath();
	currentResource = null;
	return result;
}
public String nextEntryLine() {
	if (currentResource == null) {
		if (!resourceIterator.hasNext())
			return null;
		currentResource = (IResource) resourceIterator.next();
		entrySent = false;
	}
	if (currentResource.getType() == IResource.FILE && !entrySent) {
		ISyncInfo syncInfo = sharingManager.getSyncInfo(currentResource);
		String filename = currentResource.getName();
		String entryLine = "/" + filename + "/";
		if (syncInfo != null && syncInfo.getCommon() != null) {
			StateId stateId = (StateId) syncInfo.getCommon().getStateId();
			entryLine += stateId.getId();
		} else {
			// if we don't know about our comon then make it up :)
			entryLine += "1.1";
		}
		// treat all files as binary for now
		entryLine += "/random/-kb/";

		// Send tag name
		if (!tagName.equals("HEAD")) {
			// append 'T' for Tag.
			entryLine += "T" + tagName;
		}

		entrySent = true;
		return entryLine;
	} else {
		return null;
	}
}
public IFileContent nextFileContent() {
	if (currentResource == null || currentResource.getType() != IResource.FILE) {
		currentResource = null;
		return null;
	}
	if (!currentResource.exists()) {
		currentResource = null;
		return null;
	} else {
		final File file = new File(currentResource.getLocation().toOSString());
		currentResource = null;
		return new IFileContent() {
			public long getSize() {
				return file.length();
			}
			public InputStream getInputStream() throws IOException {
				return new FileInputStream(file);
			}
		};
	}
}
}
