package org.eclipse.vcm.internal.core.ccvs;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.io.*;
import org.eclipse.core.runtime.*;
import java.util.*;
import org.eclipse.core.resources.*;
import org.eclipse.vcm.internal.core.ccvs.client.CVSException;
import org.eclipse.vcm.internal.core.ccvs.requests.*;
import org.eclipse.vcm.internal.core.ccvs.requests.IFileContent;
import org.eclipse.vcm.internal.core.base.*;

public class SyncInfoEntryIterator implements IEntryContentIterator {
	ISharingManager sharingManager;
	Iterator resourceIterator;
	IResource currentResource;
	boolean entrySent = false;
	String versionTag;
	
/**
 * Iterator that returns entry lines for a sorted set of released resources. 
 */	
public SyncInfoEntryIterator(SortedSet resources, ISharingManager sharingManager, String versionTag) {
	this.sharingManager = sharingManager;
	this.resourceIterator = resources.iterator();
	this.versionTag = versionTag;
}
/**
 * Returns the project relative path of the next directory in the resource list, and 
 * null if there are not directories left.
 */
public IPath nextDirectory() {
	if (currentResource == null || currentResource.getType() == IResource.FILE) {
		do {
			if (!resourceIterator.hasNext())
				return null;
			currentResource = (IResource) resourceIterator.next();
		} while (currentResource.getType() == IResource.FILE);
	}
	IPath result = currentResource.getProjectRelativePath();
	currentResource = null;
	return result;
}
/**
 * Returns the entry lines for the files contained in the last directory returned
 * by <code>nextDirectory</code>. Throws an exception if the given resource does
 * not have a base version.. 
 */
public String nextEntryLine() throws CVSException {
	IResourceEdition baseEdition = null;
	if (currentResource == null) {		
		do {
			if (!resourceIterator.hasNext())
				return null;
			currentResource = (IResource) resourceIterator.next();
			baseEdition = sharingManager.getBaseVersion(currentResource);
			entrySent = false;
			
			// TODO: add status messages about resources that where not included in version
			
		} while (currentResource.getType() == IResource.FILE && baseEdition == null);		
	}
	if (currentResource.getType() == IResource.FILE && !entrySent) {
		String filename = currentResource.getName();
		String entryLine = "/" + filename + "/";
		
		StateId stateId = (StateId) baseEdition.getStateId();
		String revision = stateId.getId();
		entryLine += revision;

		// treat all files as binary for now
		entryLine += "//-kb/";

		// Add sticky tag if context is a version or branch
		if (versionTag != null) {
			entryLine += "T" + versionTag;
		}
		entrySent = true;
		currentResource = null;
		return entryLine;
	} else {
		return null;
	}
}
public IFileContent nextFileContent() {
	// Sync info interator doesn't provide content, only entries.
	return null;
}
}
