package org.eclipse.vcm.internal.core.ccvs.client;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
 
import java.io.PrintStream;
import java.io.PrintWriter;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.vcm.internal.core.IVCMStatus;
import org.eclipse.vcm.internal.core.VCMException;
import org.eclipse.vcm.internal.core.ccvs.CVSStatus;

public class CVSException extends VCMException {

	private IPath fPath;
	private Throwable fThrowable;	
	
/**
 * @see CoreExcpetion
 */
public CVSException(int code, IPath path, String message, Throwable exception) {
	super(new CVSStatus(code, path, message, exception));
}
/**
 * Create a new CVS exception.
 */
public CVSException(String message) {
	super(IVCMStatus.ERROR, null, message, null);
}
	/**
	 * Create a new CVS exception.
	 */
	public CVSException(String message, IPath path) {
		this(message, path, null);
	}
/**
 * Create a new CVS exception.
 */
public CVSException(String message, IPath path, Throwable throwable) {
	super(IVCMStatus.ERROR, path, message, throwable);
	fPath = path;
	fThrowable = throwable;
}
public CVSException(org.eclipse.core.runtime.IStatus status) {
	super(status);
}
	/**
	 * Returns the detail message. If the detail message of this exception is
	 * <code>null</code> the detailed message of the wrapped exception is 
	 * returned.
	 */
	public String getMessage() {
		String result= super.getMessage();
		if (result == null && fThrowable != null)
			return fThrowable.getMessage();
			
		return result;
	}
	/**
	 * Prints the stack trace from either the wrapped exceptions if one exists,
	 * or the stack trace of this exception to standard error.
	 * 
	 * @see Throwable#printStackTrace
	 */
	public void printStackTrace() {
		if (fThrowable != null) {
			fThrowable.printStackTrace();
		} else {
			super.printStackTrace();
		}	
	}
	/**
	 * Prints the stack trace from either the wrapped exceptions if one exists,
	 * or the stack trace of this exception to the given <code>PrintStream</code>.
	 * 
	 * @see Throwable#printStackTrace(java.io.PrintStream)
	 */
	public void printStackTrace(java.io.PrintStream s) {
		if (fThrowable != null) {
			fThrowable.printStackTrace(s);
		} else {
			super.printStackTrace(s);
		}	
	}
	/**
	 * Prints the stack trace from either the wrapped exceptions if one exists,
	 * or the stack trace of this exception to the given <code>PrintWriter</code>.
	 * 
	 * @see Throwable#printStackTrace(java.io.PrintWriter)
	 */
	public void printStackTrace(java.io.PrintWriter s) {
		if (fThrowable != null) {
			fThrowable.printStackTrace(s);
		} else {
			super.printStackTrace(s);
		}	
	}
public String toString() {

	StringBuffer buffer = new StringBuffer();

	buffer.append(getStatus().getMessage());
	IStatus children[] = getStatus().getChildren();
	for (int i = 0; i < children.length; i++) {
		buffer.append("\n" + children[i].getMessage());
	}

	return buffer.toString();
}
}
