package org.eclipse.vcm.internal.core.ccvs.client;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
 
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.vcm.internal.core.ccvs.requests.IFileContent;

public interface IRequestContext extends ICVSContext {

	/**
	 * Expand the given module name.
	 */
	public String expandModules(String name) throws CVSException;
	/**
	 * Returns the progress monitor used to report progress.
	 */
	public IProgressMonitor getProgressMonitor();
	/**
	 * Removes the response handler managed under the given key from
	 * this client. The key used to manage a response handler is
	 * <code>ResponeHandler.getName()</code>.
	 */
	public ResponseHandler removeResponseHandler(String key);
	/**
	 * Sends the given argument to the server.
	 */
	public void sendArgument(String arg) throws CVSException;
	/**
	 * Sends the given directories to the server directory.
	 */
	public void sendDirectory(IPath local, IPath rep) throws CVSException;
	/**
	 * Sends the given entry line to the server.
	 */
	public void sendEntry(String entryLine) throws CVSException;
	/**
	 * Sends the given global option to the server.
	 */
	public void sendGlobalOption(String option) throws CVSException;
	/**
	* Sends the given file as modified to the server.
	*/
	public void sendIsModified(String filename) throws CVSException;
	/**
	 * Sends the given file as modified to the server.
	 */
	public void sendModified(String filename, IFileContent fileContent)
		throws CVSException;
	/**
	* Sends the given file as modified to the server.
	*/
	public void sendSticky(String tagName) throws CVSException;
	/**
	 * Sends the given file as unchanged to the server.
	 */
	public void sendUnchanged(String filename) throws CVSException;
	/**
	 * Set a new response handler. If the client already manages a
	 * response handler of that type, the old response handler is
	 * returned.
	 */
	public ResponseHandler setResponseHandler(ResponseHandler handler);
}
