package org.eclipse.vcm.internal.core.ccvs.client;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
 
import java.io.InputStream;

import org.eclipse.core.runtime.IProgressMonitor;

/**
 * A response context can be used to read data from the cvs
 * server. A <code>IResponseContext</code> is passed to
 * the response handlers <code>handle</code> method.
 *
 * @see ResponseHandler#handle(IResponseContext)
 */
public interface IResponseContext extends ICVSContext {

	/**
	 * Add the error message provided by a "E" response.
	 */
	public void addError(String error);
	/**
	 * Returns the value stored under the given key.
	 * @see remember(Object, Object)
	 */
	public Object get(Object key);
	/**
	 * Returns the progress monitor used to report progress.
	 */
	public IProgressMonitor getProgressMonitor();
	/**
	 * Returns the <code>InputStream</code> used to read responses from
	 * the server.
	 */
	public InputStream getResponseStream() throws CVSException;
	/**
	 * Reads a line from the server.
	 */
	public String readLine() throws CVSException;
	/**
	 * Read a token from the server. Tokens are separated by
	 * white space and new line.
	 */
	public String readToken() throws CVSException;
	/**
	 * Remembers the given value under the given key.
	 * Normally a request or response uses this value in 
	 * a further processing.
	 */
	public void remember(Object key, Object value);
}
