package org.eclipse.vcm.internal.core.ccvs.client;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
 
public abstract class Request {

	private String[] fGlobalOptions;
	private String[] fArguments;

	/**
	 * Creates a new <code>Request</code> object with the given
	 * global options and arguments.
	 */
	protected Request(String[] globalOptions, String[] args) {
		fGlobalOptions= globalOptions;
		fArguments= args;
	}
	/**
	 * The request has been finished.
	 */
	protected abstract void finished(IRequestContext context, boolean successful) throws CVSException;
	/**
	 * Returns the request's name.
	 */
	protected abstract String getName();
	/**
	 * Sends any arguments to the server.
	 */
	protected void sendArguments(IRequestContext context) throws CVSException {
		if (fArguments == null)
			return;
		for (int i= 0; i < fArguments.length; i++) {
			context.sendArgument(fArguments[i]);
		}
	}
	//---- Private helpers -------------------------------------------------
	
	/**
	 * Sends the global options to the server.
	 */
	protected void sendGlobalOptions(IRequestContext context) throws CVSException {
		if (fGlobalOptions == null)
			return;
		for (int i= 0; i < fGlobalOptions.length; i++) {
			context.sendGlobalOption(fGlobalOptions[i]);
		}
	}
	/**
	 * The request is about to be started. Set up all necessary information.
	 * For example send entry lines, directories, etc.
	 */
	protected abstract void setUp(IRequestContext context) throws CVSException;
}
