package org.eclipse.vcm.internal.core.ccvs.client;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
 
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;

/**
 * A class that can keep the client-side administrative CVS subdirectories 
 * update to date.
 */
public class WorkingCopyFileUpdater {
	
	/**
	 * Root location on the local file system where the cvs working files
	 * will be placed.
	 */
	String localRoot;
	
	/**
	 * Repository root
	 */
	String repository;
	
	/**
	 * Name of cvs subdirectory, should never change :)
	 */
	private static final String CVS_DIR_NAME = "CVS";
	private static final String CVS_ROOT = "Root";
	private static final String CVS_REPOSITORY = "Repository";
		
	WorkingCopyFileUpdater(String localRootPath, String repository)
	{
		this.localRoot = localRootPath;
		this.repository = repository;
	}
	void ensureCVSExists(String dir) throws IOException {
		IPath localRootPath = new Path(localRoot);
		File cvsDir = new File(localRootPath.append(dir).toOSString(), 
							   CVS_DIR_NAME);
		if (!cvsDir.exists()) {
			cvsDir.mkdir();
			updateRoot(cvsDir, dir);		
			updateRepository(cvsDir);
		}
	}
	void update(ResponseDetail response) throws IOException {
		if( response.getType() == ResponseDetail.ENTRY )
		{
			ensureCVSExists( response.getDirectory() );						
		}
	}
	void updateRepository(File localCVSDir) throws IOException {
		File repositoryFile = new File(localCVSDir, CVS_REPOSITORY);
		if( ! repositoryFile.exists() ) {
			OutputStream os = new FileOutputStream(repositoryFile);
			os.write(repository.getBytes());
			os.close();
		}
	}
	void updateRoot(File localCVSDir, String root) throws IOException {
		File rootFile = new File(localCVSDir, CVS_ROOT);
		if( ! rootFile.exists() ) {
			OutputStream os = new FileOutputStream(rootFile);
			os.write(root.getBytes());
			os.close();
		}
	}
}
