package org.eclipse.vcm.internal.core.ccvs.requests;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
 
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.vcm.internal.core.Assert;
import org.eclipse.vcm.internal.core.ccvs.client.CVSException;
import org.eclipse.vcm.internal.core.ccvs.client.IRequestContext;
import org.eclipse.vcm.internal.core.ccvs.client.IRequestNames;
import org.eclipse.vcm.internal.core.ccvs.client.Request;
import org.eclipse.vcm.internal.core.ccvs.client.ResponseHandler;
import org.eclipse.vcm.internal.core.ccvs.utils.PathUtil;

public class Checkout extends Request {

	private IPath contextPath;
	private ResponseHandler createdHandler;
	IEntryIterator entryIterator;
	private ResponseHandler errorHandler;
	private ResponseHandler messageHandler;
	private ResponseHandler oldErrorHandler;
	private ResponseHandler oldMessageHandler;
	private ResponseHandler oldUpdatedHandler;
	private String tagName;
	private IPath targetPath;
	IUpdatedListener updatedListener;
	IUpdateMessageListener updateMessageListener;

public Checkout(IPath repositoryRelativeContext, IPath contextRelativeTarget, String tagName, IEntryIterator entryIterator, IUpdateMessageListener updateMessageListener, IUpdatedListener updatedListener) {
	super(null, new String[] { "-N" });
	this.contextPath = repositoryRelativeContext;
	this.targetPath = contextRelativeTarget;
	this.tagName = tagName;
	this.updateMessageListener = updateMessageListener;
	this.updatedListener = updatedListener;
	this.entryIterator = entryIterator;
	Assert.isNotNull(contextPath);
}
protected void finished(IRequestContext context, boolean failed) {
	if (oldErrorHandler == null)
		context.removeResponseHandler(errorHandler.getName());
	else
		context.setResponseHandler(oldErrorHandler);
	if (oldMessageHandler == null)
		context.removeResponseHandler(messageHandler.getName());
	else
		context.setResponseHandler(oldMessageHandler);
	if (updatedListener != null) {
		if (oldUpdatedHandler == null)
			context.removeResponseHandler(createdHandler.getName());
		else
			context.setResponseHandler(oldUpdatedHandler);
	}
}
protected String getName() {
	return IRequestNames.co;
}
protected void setUp(IRequestContext context) throws CVSException {
	sendGlobalOptions(context);
	sendArguments(context);
	if (tagName != null && !tagName.equals("HEAD")) {
		context.sendArgument("-r");
		context.sendArgument(tagName);
	}

	IPath rootDir = new Path(context.getRootDirectory());
	IPath dir;
	if (entryIterator != null)
		while ((dir = entryIterator.nextDirectory()) != null) {
			context.sendDirectory(dir, contextPath.append(dir));
			String entryLine;
			while ((entryLine = entryIterator.nextEntryLine()) != null) {
				context.sendEntry(entryLine);
			}
		}
	context.sendDirectory(Path.EMPTY, contextPath);

	// Send argument only if needed
	if (targetPath != null)
		context.sendArgument((contextPath.append(targetPath)).toString());

	messageHandler = new UpdateMessageHandler(updateMessageListener);
	oldMessageHandler = context.setResponseHandler(messageHandler);
	errorHandler = new UpdateErrorHandler(updateMessageListener);
	oldErrorHandler = context.setResponseHandler(errorHandler);
	if (updatedListener != null) {
		createdHandler = new UpdatedHandler(updatedListener);
		oldUpdatedHandler = context.setResponseHandler(createdHandler);
	}
}
}
