package org.eclipse.vcm.internal.core.ccvs.requests;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
 
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.vcm.internal.core.ccvs.client.CVSException;
import org.eclipse.vcm.internal.core.ccvs.client.IRequestContext;
import org.eclipse.vcm.internal.core.ccvs.client.IRequestNames;
import org.eclipse.vcm.internal.core.ccvs.client.Request;
import org.eclipse.vcm.internal.core.ccvs.client.ResponseHandler;
import org.eclipse.vcm.internal.core.ccvs.utils.PathUtil;

public class Commit extends Request {

	private ResponseHandler checkedInHandler;
	ICheckedInListener checkedInListener;
	private IPath contextPath;
	IEntryContentIterator entryContentIterator;
	private ResponseHandler oldCheckedInHandler;
	private ResponseHandler oldRemoveEntryHandler;
	private ResponseHandler removeEntryHandler;
	private ResponseHandler errorHandler;
	private ResponseHandler oldErrorHandler;

	String tagName;

public Commit(IPath repositoryRelativeContext, String logMessage, String tagName, IEntryContentIterator entryContentIterator, ICheckedInListener checkedInListener) {
	super(null, new String[] { "-m", logMessage });
	this.contextPath = repositoryRelativeContext;
	this.checkedInListener = checkedInListener;
	this.entryContentIterator = entryContentIterator;
	this.tagName = tagName;
}
protected void finished(IRequestContext context, boolean failed) {
	if (oldErrorHandler == null)
		context.removeResponseHandler(errorHandler.getName());
	else
		context.setResponseHandler(oldErrorHandler);
	
	if (checkedInListener != null) {
		if (oldCheckedInHandler == null)
			context.removeResponseHandler(checkedInHandler.getName());
		else
			context.setResponseHandler(oldCheckedInHandler);
		if (oldRemoveEntryHandler == null)
			context.removeResponseHandler(removeEntryHandler.getName());
		else
			context.setResponseHandler(oldRemoveEntryHandler);
	}
}
protected String getName() {
	return IRequestNames.ci;
}
protected void setUp(IRequestContext context) throws CVSException {
	sendGlobalOptions(context);
	sendArguments(context);
	IPath dir;
	if (entryContentIterator != null)
		while ((dir = entryContentIterator.nextDirectory()) != null) {
			context.sendDirectory(dir, contextPath.append(dir));

			// first character is a single letter indicating the type of tag: T, N, or D, 
			// for branch tag, nonbranch tag, or date respectively. The rest of the line 
			// is the tag or date itself.
			// Our current cvs client implementation does not differentiate between
			// version and branch contexts (server workspaces). We will always assume
			// that the tag name is a branch.
			if (tagName != null && !tagName.equals("HEAD")) {
				context.sendSticky("T" + tagName);
			}

			String entryLine;
			while ((entryLine = entryContentIterator.nextEntryLine()) != null) {
				// if the entry line's revision starts with '0', it is an addition
				// if it starts with '-', it is a deletion and nextFileContent() should return null
				// otherwise, it is a change
				context.sendEntry(entryLine);
				IFileContent fileContent = entryContentIterator.nextFileContent();
				if (fileContent != null) {
					// outgoing addition or change
					context.sendModified(entryLine.substring(1, entryLine.indexOf('/', 1)), fileContent);
				}
			}
		}
	context.sendDirectory(Path.EMPTY, contextPath);
	
	errorHandler = new CheckedInErrorHandler();
	oldErrorHandler = context.setResponseHandler(errorHandler);
	
	if (checkedInListener != null) {
		checkedInHandler = new CheckedInHandler(checkedInListener);
		oldCheckedInHandler = context.setResponseHandler(checkedInHandler);
		removeEntryHandler = new RemoveEntryHandler(checkedInListener);
		oldRemoveEntryHandler = context.setResponseHandler(removeEntryHandler);
	}
}
}
