package org.eclipse.vcm.internal.core.ccvs.requests;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
 
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.vcm.internal.core.ccvs.client.CVSException;
import org.eclipse.vcm.internal.core.ccvs.client.IRequestContext;
import org.eclipse.vcm.internal.core.ccvs.client.IRequestNames;
import org.eclipse.vcm.internal.core.ccvs.client.Request;
import org.eclipse.vcm.internal.core.ccvs.client.ResponseHandler;
import org.eclipse.vcm.internal.core.ccvs.utils.PathUtil;

public class History extends Request {

	IEntryIterator entryIterator;
	IHistoryListener historyListener;
	ResponseHandler messageHandler;
	String moduleName;
	ResponseHandler oldMessageHandler;

public History(String moduleName, IEntryIterator entryIterator, IHistoryListener historyListener) {
	// VCM TODO: will have to do the right thing with keyword substitution flag :)
	super(null, new String[] { "-T", // show tags
		"-a" // show for all users 
	});

	this.entryIterator = entryIterator;
	this.historyListener = historyListener;
	this.moduleName = moduleName;
}
protected void finished(IRequestContext context, boolean failed) {
	if (oldMessageHandler == null)
		context.removeResponseHandler(messageHandler.getName());
	else
		context.setResponseHandler(oldMessageHandler);
}
protected String getName() {
	return IRequestNames.history;
}
protected void setUp(IRequestContext context) throws CVSException {
	sendGlobalOptions(context);
	sendArguments(context);
	IPath dir;
	if (entryIterator != null)
		while ((dir = entryIterator.nextDirectory()) != null) {
			context.sendDirectory(dir, dir);
			String entryLine;
			while ((entryLine = entryIterator.nextEntryLine()) != null) {
				context.sendArgument(entryLine.substring(1, entryLine.indexOf('/', 1)));
			}
		}

	if (moduleName != null) {
		context.sendArgument("-n");
		context.sendArgument(moduleName);
	}

	messageHandler = new HistoryMessageHandler(historyListener);
	oldMessageHandler = context.setResponseHandler(messageHandler);
}
}
