package org.eclipse.vcm.internal.core.ccvs.requests;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
 
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.vcm.internal.core.ccvs.client.CVSException;
import org.eclipse.vcm.internal.core.ccvs.client.IRequestContext;
import org.eclipse.vcm.internal.core.ccvs.client.IRequestNames;
import org.eclipse.vcm.internal.core.ccvs.client.Request;
import org.eclipse.vcm.internal.core.ccvs.client.ResponseHandler;

public class Log extends Request {

	IPath contextPath;
	ILogListener logListener;
	ResponseHandler messageHandler;
	ResponseHandler oldMessageHandler;
	String tagOrRevision;
	IPath targetPath;

public Log(IPath repositoryRelativeContext, IPath contextRelativeTarget, String tagOrRevision, ILogListener logListener) {
	super(null, null);

	this.contextPath = repositoryRelativeContext;
	this.targetPath = contextRelativeTarget;
	this.tagOrRevision = tagOrRevision;
	this.logListener = logListener;
}
protected void finished(IRequestContext context, boolean failed) {
	if (oldMessageHandler == null)
		context.removeResponseHandler(messageHandler.getName());
	else
		context.setResponseHandler(oldMessageHandler);
}
protected String getName() {
	return IRequestNames.log;
}
protected void setUp(IRequestContext context) throws CVSException {
	sendGlobalOptions(context);
	sendArguments(context);

	context.sendDirectory(Path.EMPTY, contextPath);

	if (tagOrRevision != null) {
		context.sendArgument("-r" + tagOrRevision);
	}
	context.sendArgument(targetPath.toOSString());

	messageHandler = new LogHandler(logListener);
	oldMessageHandler = context.setResponseHandler(messageHandler);
}
}
