package org.eclipse.vcm.internal.core.ccvs.requests;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
 
import java.io.IOException;

import org.eclipse.vcm.internal.core.ccvs.utils.PathUtil;
import org.eclipse.vcm.internal.core.Assert;
import org.eclipse.core.runtime.*;
import org.eclipse.vcm.internal.core.ccvs.client.*;

public class Status extends Request {

	private IPath[] dirPaths;
	IEntryIterator entryIterator;
	private ResponseHandler errorHandler;
	private IPath[] filePaths;
	private ResponseHandler messageHandler;
	private ResponseHandler oldErrorHandler;
	private ResponseHandler oldMessageHandler;
	IStatusListener statusListener;
	private String tagName;

public Status(IPath[] dirPaths, IPath[] filePaths, String[] arguments, String tagName, IEntryIterator entryIterator, IStatusListener statusListener) {
	super(null, arguments);
	this.dirPaths = dirPaths;
	this.filePaths = filePaths;
	this.statusListener = statusListener;
	this.entryIterator = entryIterator;
	this.tagName = tagName;
}
protected void finished(IRequestContext context, boolean failed) {
	if (oldErrorHandler == null)
		context.removeResponseHandler(errorHandler.getName());
	else
		context.setResponseHandler(oldErrorHandler);
	if (oldMessageHandler == null)
		context.removeResponseHandler(messageHandler.getName());
	else
		context.setResponseHandler(oldMessageHandler);
}
protected String getName() {
	return IRequestNames.status;
}
protected void setUp(IRequestContext context) throws CVSException {
	sendGlobalOptions(context);
	if (dirPaths != null) {
		for (int i = 0; i < dirPaths.length; i++) {
			context.sendDirectory(dirPaths[i], dirPaths[i]);
			// first character is a single letter indicating the type of tag: T, N, or D, 
			// for branch tag, nonbranch tag, or date respectively. The rest of the line 
			// is the tag or date itself.
			// Our current cvs client implementation does not differentiate between
			// version and branch contexts (server workspaces). We will always assume
			// that the tag name is a branch.
			if (tagName != null && !tagName.equals("HEAD")) {
				context.sendSticky("T" + tagName);
			}

		}
	}
	sendArguments(context);
	if (filePaths != null) {
		for (int i = 0; i < filePaths.length; i++) {
			context.sendArgument(PathUtil.toString(filePaths[i]));
		}
	}
	IPath dir;
	if (entryIterator != null) {
		while ((dir = entryIterator.nextDirectory()) != null) {
			context.sendDirectory(dir, dir);
			String entryLine;
			while ((entryLine = entryIterator.nextEntryLine()) != null) {
				context.sendEntry(entryLine);
			}
		}
	}
	context.sendDirectory(Path.EMPTY, Path.EMPTY);
	messageHandler = new StatusMessageHandler(statusListener);
	oldMessageHandler = context.setResponseHandler(messageHandler);
	errorHandler = new StatusErrorHandler(statusListener);
	oldErrorHandler = context.setResponseHandler(errorHandler);
}
}
