package org.eclipse.vcm.internal.core.ccvs.requests;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
 
import java.io.IOException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.vcm.internal.core.Assert;
import org.eclipse.vcm.internal.core.ccvs.client.*;
import org.eclipse.vcm.internal.core.ccvs.utils.PathUtil;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;


public class Tag extends Request {

	private IPath contextPath;
	private IEntryContentIterator entryContentIterator;
	private ITagListener tagListener;
	private ResponseHandler handler;
	private ResponseHandler oldHandler;
	private ResponseHandler messageHandler;
	private ResponseHandler oldMessageHandler;
	
public Tag(IPath repositoryRelativeContext, String versionTag, IEntryContentIterator entryContentIterator, ITagListener tagListener) {
	super(null, new String[] { versionTag });
	this.tagListener = tagListener;
	this.contextPath = repositoryRelativeContext;
	this.entryContentIterator = entryContentIterator;
}
	protected void finished(IRequestContext context, boolean failed) {
		if (oldHandler == null)
			context.removeResponseHandler(handler.getName());
		else
			context.setResponseHandler(oldHandler);		
		if (oldMessageHandler == null)
			context.removeResponseHandler(messageHandler.getName());
		else
			context.setResponseHandler(oldMessageHandler);		
}
	protected String getName() {
		return IRequestNames.tag;
	}
	protected void setUp(IRequestContext context) throws CVSException {
		sendGlobalOptions(context);
		sendArguments(context);
		IPath dir;
		if (entryContentIterator != null)
			while ((dir = entryContentIterator.nextDirectory()) != null) {
				context.sendDirectory(dir, contextPath.append(dir));
				String entryLine;
				while ((entryLine = entryContentIterator.nextEntryLine()) != null) {
					context.sendEntry(entryLine);
				}
			}
		
		context.sendDirectory(Path.EMPTY, contextPath);
	
	handler = new RTagErrorHandler();
	oldHandler = context.setResponseHandler(handler);	
	messageHandler = new TagMessageHandler(tagListener);
	oldMessageHandler = context.setResponseHandler(messageHandler);
	}
}
