package org.eclipse.vcm.internal.core.ccvs.requests;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
 
import org.eclipse.core.runtime.Path;
import org.eclipse.vcm.internal.core.ccvs.client.*;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.vcm.internal.core.ccvs.client.ResponseHandler;

class TagMessageHandler extends ResponseHandler {

	/** The response key */
	public static final String NAME = "M";
	
	ITagListener tagListener;

public TagMessageHandler(ITagListener tagListener) {
	this.tagListener = tagListener;
}
	/**
	 * @see ResponseHandler#getName()
	 */
	public String getName() {
		return NAME;
	}
/**
 * Handle the response. This response sends the message to the
 * progress monitor using <code>IProgressMonitor.subTask(Strin)
 * </code>.
 */
protected Object handle(IResponseContext context) throws CVSException {
	String line = context.readLine();
	
	// The tag command has warning messages embedded in the regular
	// messages. These should be shown to the user because they 
	// indicate that the command had problems.
	if( line.charAt(0) == 'W' ) {
		context.addError(line);
		return null;
	}
	
	if (tagListener == null)
		return null;
	
	String path = line.substring(2);
	
	if( tagListener != null )
		tagListener.tagged(new Path(path));
	
	return null;
}
}
