package org.eclipse.vcm.internal.core.ccvs.requests;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
 
import java.io.IOException;
import java.io.InputStream;

import org.eclipse.core.runtime.Path;
import org.eclipse.vcm.internal.core.Assert;
import org.eclipse.vcm.internal.core.ccvs.client.CVSException;
import org.eclipse.vcm.internal.core.ccvs.client.CVSFileException;
import org.eclipse.vcm.internal.core.ccvs.client.IResponseContext;
import org.eclipse.vcm.internal.core.ccvs.client.ResponseHandler;

/**
 * Handles a valid request from the server.
 */
class UpdatedHandler extends ResponseHandler {
	public static final String NAME = "Updated";
	
	private static final int BUFFER_SIZE = 4096;
	private IUpdatedListener updatedListener;

public UpdatedHandler(IUpdatedListener updatedListener) {
	this.updatedListener = updatedListener;
	Assert.isNotNull(updatedListener);
}
	public String getName() {
		return NAME;
	}
protected Object handle(IResponseContext context) throws CVSException {

	String localDirectory = context.readLine();
	String repositoryFilename = context.readLine();

	// Read the entry line.
	String entry = context.readLine();

	// Skip the user rights. No way to set them in Java.
	context.readLine();

	// Read the number of bytes
	String line = context.readLine();
	int size = Integer.parseInt(line);

	InputStream in = context.getResponseStream();

	if (updatedListener != null) {
		updatedListener.fileInformation(new Path(localDirectory), entry, in, size);
	} else {
		// Read file content.
		try {
			byte[] buffer = new byte[BUFFER_SIZE];
			int read = 0;
			int totalRead = 0;
			int toRead = Math.min(BUFFER_SIZE, size - totalRead);
			while ((totalRead < size) && (read = in.read(buffer, 0, toRead)) != -1) {
				totalRead += read;
				toRead = Math.min(BUFFER_SIZE, size - totalRead);
			}
		} catch (IOException e) {
			throw new CVSFileException(null, e);
		}
	}

	// Set modification time if supplied.
	//String modTime = (String) context.get(ModTimeHandler.NAME);
	//if (modTime != null) {
	//Date d = TimeUtil.convertFromModTime(modTime);
	//if (d != null) {
	//file.setLastModified(d.getTime());
	//}
	//}

	// Retrieve the real modification time from the file because the
	// file system must not honor the exact time.
	//modTime = Long.toString(file.lastModified());
	//entry = entry + '/' + modTime;

	return null;
}
}
