package org.eclipse.vcm.internal.core.ccvs.ssh;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.eclipse.vcm.internal.core.ccvs.CVSPlugin;
import org.eclipse.vcm.internal.core.ccvs.base.IServerConnection;
import org.eclipse.vcm.internal.core.ccvs.client.CVSAuthenticationException;
import org.eclipse.vcm.internal.core.ccvs.client.CVSCommunicationException;
import org.eclipse.vcm.internal.core.ccvs.client.CVSException;

public class SSHServerConnection implements IServerConnection {
	
	// command to start remote cvs in server mode
	private static final String INVOKE_SVR_CMD = "cvs server";
	
	// cvs format for the repository (e.g. :extssh:user@host:/home/cvs/repo)
	private String location;
	
	// password for user specified in repository location string
	private String password;
	
	// incoming from remote host
	InputStream inputStream;
	
	// outgoing to remote host
	OutputStream outputStream;

	protected SSHServerConnection(String location, String password) {
		this.location = location;
		this.password = password;
	}
	public void close() throws IOException {
		inputStream.close();
		outputStream.close();
	}
	/**
	 * Extracts the host from the cvsroot.
	 */
	private String getHost() {		
		int start = location.indexOf('@');
		if( start == -1 ) return "";
		int end = location.indexOf(':', start);
		return location.substring(start + 1, end);
	}
	/**
	 * Returns the <code>InputStream</code> used to read data from the
	 * server.
	 */
	public InputStream getInputStream() {
		return inputStream;
	}
	/**
	 * Returns the <code>OutputStream</code> used to send data to the
	 * server.
	 */
	public OutputStream getOutputStream() {
		return outputStream;
	}
	/**
	 * Extracts the user from the cvsroot.
	 */
	private String getUser() {
		int start = location.indexOf(':', 1);
		if( start == -1 ) return "";
		int end = location.indexOf('@');
		if( end == -1 ) return "";
		return location.substring(start + 1, end);
	}
	/**
	 * Opens the connection and invokes cvs in server mode.
	 *
	 * @see Connection.open()
	 */
	public void open() throws CVSException {
		String hostname = "";
		try {
			String username = getUser();
			hostname = getHost();
		
			// create the connection using host, username, and password
			Client client = new Client(hostname, 22, username, password, INVOKE_SVR_CMD);		
			client.connect();
			inputStream = client.getInputStream();
			outputStream = client.getOutputStream();
		} catch (IOException ex) {
			try {
				close();
			} finally {
				throw new CVSCommunicationException(CVSPlugin.getResourceString("PServerConnection.socket", new Object[] {hostname} ), ex);
			}
			
		} catch (RuntimeException e) {
			try {
				close();
			} finally {
				throw e;
			}
		}
	}
}
