package org.eclipse.vcm.internal.core.ccvs.utils;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import java.io.*;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.vcm.internal.core.ccvs.CVSPlugin;

public class StreamUtil {
	private static final byte[] BUFFER = new byte[4096];

public static void transfer(InputStream in, OutputStream out, long size) throws IOException {
	int read = 0;
	long totalRead = 0;
	// buffer size is smaller than MAXINT...
	int toRead = (int) Math.min(BUFFER.length, size);
	synchronized (BUFFER) {
		while ((totalRead < size) && (read = in.read(BUFFER, 0, toRead)) != -1) {
			totalRead += read;
			out.write(BUFFER, 0, read);
			toRead = (int) Math.min(BUFFER.length, size - totalRead);
		}
	}
}
public static void transferWithProgress(InputStream in, OutputStream out, long size, IProgressMonitor monitor, String title) throws IOException {
	// This special transfer utility will show progress to
	// the monitor for files that are bigger than 25K
	boolean progress = size > 25000;
	int read = 0;
	long totalRead = 0;
	long ksize = size / 1024;
	// buffer size is smaller than MAXINT...
	int toRead = (int) Math.min(BUFFER.length, size);
	synchronized (BUFFER) {
		while ((totalRead < size) && (read = in.read(BUFFER, 0, toRead)) != -1) {
			if (progress && totalRead > 0) {
				monitor.subTask(CVSPlugin.getResourceString("StreamUtil.monitor",
														new Object[] {title, new Long(totalRead/1024), new Long(ksize)}));
				monitor.worked(read);
			}
			totalRead += read;
			out.write(BUFFER, 0, read);
			toRead = (int) Math.min(BUFFER.length, size - totalRead);
		}
	}
}
}
