package org.eclipse.vcm.internal.ui.ccvs;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.vcm.internal.core.base.*;
import org.eclipse.vcm.internal.ui.adapter.*;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import java.util.HashMap;
import java.util.Map;

/**
 * The ClearCase adapter that supplies CC-specific repository
 * extensions to the generic VCM UI.
 */
public class CVSUIAdapter implements IRepositoryUIAdapter {
	static String previousComment = "";
/**
 * Creates a new CCUIAdapter.
 */
public CVSUIAdapter() {
	super();
}
/** (Non-javadoc)
 * Method declared on IRepositoryUIAdapter.
 */
public IChangeSet[] fetchChangeSets(IRepository repository, IResource[] resources, final Shell shell, IProgressMonitor monitor)	throws CoreException {
	try {
		monitor.beginTask("", 1);
		monitor.subTask(WorkbenchCVSPlugin.getResourceString("CVSUIAdapter.gather"));
		final Display display = Display.getDefault();
		final InputDialog dialog = new InputDialog(shell, WorkbenchCVSPlugin.getResourceString("CVSUIAdapter.release"), WorkbenchCVSPlugin.getResourceString("CVSUIAdapter.comment", new Object[] {repository.getRepositoryLocation().getLocation()}), previousComment, null);
		display.syncExec(new Runnable() {
			public void run() {
				dialog.open();
			}
		});
		monitor.worked(1);
		if (dialog.getReturnCode() != InputDialog.OK) {
			//user canceled
			monitor.setCanceled(true);
			return null;
		}
		previousComment = dialog.getValue();
		return new IChangeSet[] {new ChangeSet(repository, resources, dialog.getValue())};
	} finally {
		if (!monitor.isCanceled())
			monitor.done();
	}

}
}
