package org.eclipse.vcm.internal.ui.ccvs;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
import java.util.StringTokenizer;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.help.DialogPageContextComputer;
import org.eclipse.ui.help.WorkbenchHelp;
import org.eclipse.vcm.internal.core.base.IRepository;
import org.eclipse.vcm.internal.core.base.IRepositoryLocation;
import org.eclipse.vcm.internal.core.base.IVCMProvider;
import org.eclipse.vcm.internal.core.base.VCMPlugin;
import org.eclipse.vcm.internal.core.ccvs.CVSAdapter;
import org.eclipse.vcm.internal.core.ccvs.CVSAdapterFactory;
import org.eclipse.vcm.internal.core.ccvs.base.IConnectionMethod;
import org.eclipse.vcm.internal.core.ccvs.client.Client;
import org.eclipse.vcm.internal.ui.IVCMHelpContextIds;
import org.eclipse.vcm.internal.ui.IWorkbenchVCMConstants;
import org.eclipse.vcm.internal.ui.RepositoryView;
import org.eclipse.vcm.internal.ui.WorkbenchVCMPlugin;
import org.eclipse.vcm.internal.ui.wizards.VCMWizardPage;

/**
 * First wizard page for creating a new repository.
 */
public class NewRepositoryMainPage extends VCMWizardPage {
	private IVCMProvider provider = VCMPlugin.getProvider();
	
	private Combo protocolTypeField;
	private Text usernameField;
	private Text hostnameField;
	private Text repositoryPathField;
	
	private Text nameSoFar;
	private Button validate;

	//the created repository
	private IRepository result;
	
	private boolean updatingText = false;
	
/**
 * NewRepositoryMainPage constructor comment.
 * @param pageName java.lang.String
 * @param title java.lang.String
 * @param titleImage org.eclipse.jface.resource.ImageDescriptor
 */
public NewRepositoryMainPage(String pageName, String title, ImageDescriptor titleImage) {
	super(pageName, title, titleImage);
}
/**
 * Creates the UI part of the page. Subclasses must reimplement 
 * this method to provide their own graphical page content.
 */
public void createControl(Composite parent) {
	Composite composite = createComposite(parent, 2);
	//set F1 help
	WorkbenchHelp.setHelp(composite, new DialogPageContextComputer (this, IVCMHelpContextIds.REPO_CONNECTION_MAIN_PAGE));
	
	createLabel(composite, WorkbenchCVSPlugin.getResourceString("NewRepositoryMainPage.connection"));
	protocolTypeField = createCombo(composite);
	createLabel(composite, WorkbenchCVSPlugin.getResourceString("NewRepositoryMainPage.name"));
	usernameField = createTextField(composite);	
	createLabel(composite, WorkbenchCVSPlugin.getResourceString("NewRepositoryMainPage.host"));
	hostnameField = createTextField(composite);	
	createLabel(composite, WorkbenchCVSPlugin.getResourceString("NewRepositoryMainPage.path"));
	repositoryPathField = createTextField(composite);
	
	createLabel(composite, "");
	createLabel(composite, "");
	
	//create result widget by hand because we don't want a listener installed
	createLabel(composite, WorkbenchCVSPlugin.getResourceString("NewRepositoryMainPage.location"));
	nameSoFar = new Text(composite, SWT.SINGLE | SWT.BORDER);
	nameSoFar.addListener(SWT.Modify, new Listener() {
		public void handleEvent(Event e) {
			textFieldChanged();
		}
	});
	GridData data = new GridData(GridData.FILL_HORIZONTAL);
	data.verticalAlignment = GridData.CENTER;
	data.grabExcessVerticalSpace = false;
	data.widthHint = IDialogConstants.ENTRY_FIELD_WIDTH;
	nameSoFar.setLayoutData(data);

	createLabel(composite, "");
	createLabel(composite, "");
	validate = createCheckBox(composite, WorkbenchCVSPlugin.getResourceString("NewRepositoryMainPage.validateButton"));
	IPreferenceStore store = WorkbenchVCMPlugin.getDefault().getPreferenceStore();
	validate.setSelection(store.getBoolean(IWorkbenchVCMConstants.PREF_VALIDATE_LOCATION));

	initializeValues();
	updateWidgetEnablements();
	usernameField.setFocus();
	
	setControl(composite);
}
/**
 * @see WizardPage#finish
 */
public boolean finish() {
	IRepositoryLocation location;
	try {
		location = provider.newRepositoryLocation("CVS", nameSoFar.getText());
	} catch (CoreException e) {
		ErrorDialog.openError(getContainer().getShell(), WorkbenchCVSPlugin.getResourceString("NewRepositoryMainPage.repository"), null, e.getStatus());
		WorkbenchVCMPlugin.log(e.getStatus());
		return true;
	}
	
	if (validate.getSelection()) {
		try {
			Client client = new Client(new CVSAdapter(location), null);
			client.disconnect();
			// Success
		} catch (CoreException e) {
			boolean keep = MessageDialog.openQuestion(getShell(),
				WorkbenchCVSPlugin.getResourceString("NewRepositoryMainPage.validationFailedTitle"),
				WorkbenchCVSPlugin.getResourceString("NewRepositoryMainPage.validationFailedText"));
			if (!keep) return false;
		}
	}
	
	try {
		result = provider.newRepository(location);
	} catch (CoreException e) {
		ErrorDialog.openError(getContainer().getShell(), WorkbenchCVSPlugin.getResourceString("NewRepositoryMainPage.repository"), null, e.getStatus());
		WorkbenchVCMPlugin.log(e.getStatus());
	}
	IPreferenceStore store = WorkbenchVCMPlugin.getDefault().getPreferenceStore();
	store.setValue(IWorkbenchVCMConstants.PREF_VALIDATE_LOCATION, validate.getSelection());
	
	// Switch to managing perspective and displays Repository view
	RepositoryView view = (RepositoryView)WorkbenchVCMPlugin.getActivePage().findView(RepositoryView.VIEW_ID);
	if (view == null) {
		WorkbenchVCMPlugin plugin = WorkbenchVCMPlugin.getDefault();
		plugin.switchToManagingPerspective(plugin.getWorkbench().getActiveWorkbenchWindow());
		view = RepositoryView.findInActivePerspective();
	}
	if (view != null) {
		try {
			WorkbenchVCMPlugin.getActivePage().showView(RepositoryView.VIEW_ID);
		} catch (PartInitException e) {
			WorkbenchVCMPlugin.log(e.getStatus());
		}
	}
	return true;
}
/**
 * Returns the created repository, or null if no repository was created.
 */
public IRepository getResult() {
	return result;
}
/**
 * Initializes states of the controls.
 */
private void initializeValues() {
	IConnectionMethod[] methods = CVSAdapterFactory.getPluggedInConnectionMethods();
	for(int i=0; i<methods.length; i++) {
		protocolTypeField.add(methods[i].getName());
	}
	protocolTypeField.select(0);
}
/**
 * The user has typed in the lower text field. Update
 * the other text fields and update the widget enablements.
 */
private void textFieldChanged() {
	if (updatingText) return;
	String location = nameSoFar.getText();
	IStatus validation = provider.validateRepositoryLocation("CVS", location);
	if (!validation.isOK()) {
		setErrorMessage(validation.getMessage());
		setPageComplete(false);
		return;
	}
	updatingText = true;
	/* Update the other text widgets */
	try {
		StringTokenizer tokenizer = new StringTokenizer(location, ":");
		/* First element is pserver */
		tokenizer.nextToken();
		String token = tokenizer.nextToken();
		if (token != null) {
			int at = token.indexOf('@');
			String user = "";
			if (at > 0) user = token.substring(0, at);
			String host = "";
			if (at + 1 < token.length()) host = token.substring(at + 1, token.length());
			usernameField.setText(user);
			hostnameField.setText(host);
		}
		if (tokenizer.hasMoreElements()) {
			String path = tokenizer.nextToken("");
			path = path.substring(1, path.length());
			repositoryPathField.setText(path);
		} else {
			repositoryPathField.setText("");
		}
	} finally {
		updatingText = false;
	}
	validateFields();
}
/**
 * Updates widget enablements and sets error message if appropriate.
 */
protected void updateWidgetEnablements() {
	if (updatingText) return;
	//build the server string:
	StringBuffer buf = new StringBuffer(":");
	buf.append(protocolTypeField.getItem(protocolTypeField.getSelectionIndex()));
	buf.append(":");
	buf.append(usernameField.getText());
	buf.append("@");
	buf.append(hostnameField.getText());
	buf.append(":");
	buf.append(repositoryPathField.getText());
	
	String location = buf.toString();
	updatingText = true;
	nameSoFar.setText(location);
	updatingText = false;
	validateFields();
}
private void validateFields() {
	String user = usernameField.getText();
	if (user.length() == 0) {
		setErrorMessage(null);
		setPageComplete(false);
		return;
	}
	if ((user.indexOf('@') != -1) || (user.indexOf(':') != -1)) {
		setErrorMessage(WorkbenchCVSPlugin.getResourceString("NewRepositoryMainPage.invalidUserName"));
		setPageComplete(false);
		return;
	}
	String host = hostnameField.getText();
	if (host.length() == 0) {
		setErrorMessage(null);
		setPageComplete(false);
		return;
	}
	if (host.indexOf(':') != -1) {
		setErrorMessage(WorkbenchCVSPlugin.getResourceString("NewRepositoryMainPage.invalidHostName"));
		setPageComplete(false);
		return;
	}
	if (repositoryPathField.getText().length() == 0) {
		setErrorMessage(null);
		setPageComplete(false);
		return;
	}		
	IStatus validation = provider.validateRepositoryLocation("CVS", nameSoFar.getText());
	if (!validation.isOK()) {
		setErrorMessage(validation.getMessage());
		setPageComplete(false);
		return;
	}
	setErrorMessage(null);
	setPageComplete(true);
}
}
