package org.eclipse.vcm.internal.ui.ccvs;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
import java.lang.reflect.InvocationTargetException;
import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
/**
 * A wizard for creating new repositories.  The appropriate
 * server (depending on the repository format) must already be running
 * on the server, this wizard merely connects to it, and creates a
 * local representation of the repository.
 */
public class NewRepositoryWizard extends Wizard implements INewWizard {
	private NewRepositoryMainPage mainPage;
/**
 * NewRepositoryWizard constructor comment.
 */
public NewRepositoryWizard() {
	super();
	setWindowTitle(WorkbenchCVSPlugin.getResourceString("new"));
	initializeDefaultPageImageDescriptor();
}
/**
 *	Create the wizard pages
 */
public void addPages() {
	mainPage = new NewRepositoryMainPage("repositoryPage1", WorkbenchCVSPlugin.getResourceString("NewRepositoryWizard.title"), null);
	mainPage.setDescription(WorkbenchCVSPlugin.getResourceString("NewRepositoryWizard.description"));
	addPage(mainPage);
}
/**
 * Initializes this creation wizard using the passed workbench and
 * object selection.
 *
 * @param workbench the current workbench
 * @param selection the current object selection
 */
public void init(IWorkbench workbench, IStructuredSelection selection) {
}
/**
 * Declares the wizard banner iamge descriptor
 */
protected void initializeDefaultPageImageDescriptor() {
	String iconPath = "icons/full/";
//	if(Display.getCurrent().getIconDepth() > 4)
//		iconPath = "icons/full/";//$NON-NLS-1$
//	else
//		iconPath = "icons/basic/";//$NON-NLS-1$
		
	try {
		URL installURL = WorkbenchCVSPlugin.getDefault().getDescriptor().getInstallURL();
		URL url = new URL(installURL, iconPath + "wizban/newconnect_wizban.gif");//$NON-NLS-1$
		ImageDescriptor desc = ImageDescriptor.createFromURL(url);
		setDefaultPageImageDescriptor(desc);
	} catch (MalformedURLException e) {
		// Should not happen.  Ignore.
	}
}
/* (Non-javadoc)
 * Method declared on IWizard.
 */
public boolean needsProgressMonitor() {
	return true;
}
/**
 * Completes processing of the wizard. If this method returns <code>
 * true</code>, the wizard will close; otherwise, it will stay active.
 */
public boolean performFinish() {
	final boolean[] result = new boolean[] {false};
	try {
		getContainer().run(false, false, new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) {
				try {
					monitor.beginTask(WorkbenchCVSPlugin.getResourceString("NewRepositoryWizard.creating"), IProgressMonitor.UNKNOWN);
					result[0] = mainPage.finish();
				} finally {
					monitor.done();
				}
			}
		});
	} catch (InterruptedException e) {
		return true;
	} catch (InvocationTargetException e) {
		//no exceptions are explicitly thrown by the above runnable
		//pass through runtime exceptions to catch coding errors
		Throwable target = e.getTargetException();
		if (target instanceof RuntimeException) {
			throw (RuntimeException)target;
		}
		if (target instanceof Error) {
			throw (Error)target;
		}
	}
	return result[0];
}
}
