package org.eclipse.vcm.internal.ui.ccvs;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.vcm.internal.core.base.ITeamStream;
import org.eclipse.vcm.internal.core.base.VCMPlugin;
import org.eclipse.vcm.internal.core.ccvs.CVSAdapter;
import org.eclipse.vcm.internal.ui.actions.VCMAction;
import org.eclipse.jface.action.IAction;
/**
 * Temporary action for generating Diff report on Stdout.  A
 * self-hosting recovery tool.  To enable this action, add
 * the folling to plugin.xml for org.eclipse.vcm.ui.cvs plugin:
 * 
 * 	<objectContribution
 * 	   id="org.eclipse.ui.vcm.cvs.ProjectContributions"
 *     objectClass="org.eclipse.core.resources.IProject">
 *		<action 
 *			id="org.eclipse.ui.vcm.cvs.Version"
 * 			label="&amp;Selfhost - Diff report"
 *			tooltip="Prints a report on differences between local and team stream to stdout"
 *			menubarPath="vcm.managing/manageGroup"
 *			class="org.eclipse.vcm.internal.ui.ccvs.SelfHostDiffAction">
 *		</action>
 *	</objectContribution>
 */
public class SelfHostDiffAction extends VCMAction {
/**
 * Method declared on IActionDelegate
 */
public void run(IAction action) {
	IProject[] projects = getSelectedProjects();
	if (projects.length != 1) {
		return;
	}
	IProject project = projects[0];
	try {
		ITeamStream ts = VCMPlugin.getProvider().getSharingManager().getSharing(project);
		if (ts == null) {
			// error, there is no stream to compare with
		} else {
			new CVSAdapter(ts.getRepository().getRepositoryLocation()).diffReport(project, ts.getName(), null);
		}
	} catch (CoreException e) {
		// ignore these, the server might report an error after generating the diff output
	}

}
}
