package org.eclipse.vcm.internal.ui.ccvs;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
import org.eclipse.core.runtime.*;
import org.eclipse.vcm.internal.core.ccvs.IUserInfo;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.help.WorkbenchHelp;
import org.eclipse.vcm.internal.ui.IVCMHelpContextIds;

/**
 * Dialog for changing the workspace user.
 */
public class UserValidationDialog extends Dialog {
	protected Text usernameField;
	protected Text passwordField;

	protected String domain;
	protected String defaultUsername;
	protected String password = null;
	protected boolean isUsernameMutable = true;
	protected String username = null;
	protected String message = null;

/**
 * Creates a new UserValidationDialog.
 */
public UserValidationDialog(Shell parentShell, String location, String defaultName, String message) {
	super(parentShell);
	this.defaultUsername = defaultName;
	this.domain = location;
	this.message = message;
}
/**
 * @see Window#configureShell
 */
protected void configureShell(Shell newShell) {
	super.configureShell(newShell);
	newShell.setText(WorkbenchCVSPlugin.getResourceString("UserValidationDialog.required"));
	// set F1 help
	WorkbenchHelp.setHelp(newShell, new Object[] {IVCMHelpContextIds.USER_VALIDATION_DIALOG});	
}
/**
 * @see Window@create
 */
public void create() {
	super.create();
	//add some default values
	usernameField.setText(defaultUsername);

	if (isUsernameMutable) {
		//give focus to username field
		usernameField.selectAll();
		usernameField.setFocus();
	} else {
		usernameField.setEditable(false);
		passwordField.setFocus();
	}
}
/**
 * @see Dialog#createDialogArea
 */
protected Control createDialogArea(Composite parent) {
	Composite main = new Composite(parent, SWT.NONE);
	GridLayout layout = new GridLayout();
	layout.numColumns = 3;
	main.setLayout(layout);
	main.setLayoutData(new GridData(GridData.FILL_BOTH));

	if (message != null) {
		Label messageLabel = new Label(main, SWT.WRAP);
		messageLabel.setText(message);
		messageLabel.setForeground(messageLabel.getDisplay().getSystemColor(SWT.COLOR_RED));
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 3;
		messageLabel.setLayoutData(data);
	}
	
	Label label = new Label(main, SWT.WRAP);
	if (isUsernameMutable) {
		label.setText(WorkbenchCVSPlugin.getResourceString("UserValidationDialog.labelUser", new Object[] {domain}));
	} else {
		label.setText(WorkbenchCVSPlugin.getResourceString("UserValidationDialog.labelPassword", new Object[] {defaultUsername, domain}));
	}
	GridData data = new GridData(GridData.FILL_HORIZONTAL);
	data.horizontalSpan = 3;
	label.setLayoutData(data);
	
	createUsernameFields(main);
	createPasswordFields(main);

	return main;
}
/**
 * Creates the three widgets that represent the user name entry
 * area.
 */
protected void createPasswordFields(Composite parent) {
	new Label(parent, SWT.NONE).setText(WorkbenchCVSPlugin.getResourceString("UserValidationDialog.password"));
	
	passwordField = new Text(parent, SWT.BORDER);
	GridData data = new GridData(GridData.FILL_HORIZONTAL);
	data.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.ENTRY_FIELD_WIDTH);
	passwordField.setLayoutData(data);
	passwordField.setEchoChar('*');
	
	new Label(parent, SWT.NONE);//spacer
}
/**
 * Creates the three widgets that represent the user name entry
 * area.
 */
protected void createUsernameFields(Composite parent) {
	new Label(parent, SWT.NONE).setText(WorkbenchCVSPlugin.getResourceString("UserValidationDialog.user"));
	
	usernameField = new Text(parent, SWT.BORDER);
	GridData data = new GridData(GridData.FILL_HORIZONTAL);
	data.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.ENTRY_FIELD_WIDTH);
	usernameField.setLayoutData(data);
	
	new Label(parent, SWT.NONE);//spacer
}
/**
 * Returns the password entered by the user, or null
 * if the user canceled.
 */
public String getPassword() {
	return password;
}
/**
 * Returns the username entered by the user, or null
 * if the user canceled.
 */
public String getUsername() {
	return username;
}
/**
 * Notifies that the ok button of this dialog has been pressed.
 * <p>
 * The default implementation of this framework method sets
 * this dialog's return code to <code>Window.OK</code>
 * and closes the dialog. Subclasses may override.
 * </p>
 */
protected void okPressed() {
	password = passwordField.getText();
	username = usernameField.getText();

	super.okPressed();
}
/**
 * Sets whether or not the username field should be mutable.
 * This method must be called before create(), otherwise it
 * will be ignored.
 */
public void setUsernameMutable(boolean value) {
	isUsernameMutable = value;
}
}
