package org.eclipse.vcm.internal.ui.ccvs;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPluginDescriptor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.vcm.internal.ui.WorkbenchVCMPlugin;

import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.text.MessageFormat;

/**
 * Plugin for all VCM desktop elements.
 */
public class WorkbenchCVSPlugin extends WorkbenchVCMPlugin {
	private static WorkbenchCVSPlugin instance;
	private static ResourceBundle resourceBundle;
	public static final String ID = "org.eclipse.vcm.ui.cvs";

/**
 * Creates a new WorkbenchCVSPlugin.
 */
public WorkbenchCVSPlugin(IPluginDescriptor descriptor) {
	super(descriptor, true);
	instance = this;
	try {
		resourceBundle = ResourceBundle.getBundle("org.eclipse.vcm.internal.ui.ccvs.WorkbenchCVSPluginResources");
	} catch (MissingResourceException e) {
		resourceBundle = null;
	}		
}
/**
 * Return the default instance of the receiver. This represents the runtime plugin.
 */
public static WorkbenchVCMPlugin getDefault() {
	return instance;
}
/**
 * Gets a string from the resource bundle.
 * We don't want to crash because of a missing String.
 * Returns the key if not found.
 */
public static String getResourceString(String key) {
	try {
		return resourceBundle.getString(key);
	} catch (MissingResourceException e) {
		return key;
	} catch (NullPointerException e) {
		return "!" + key + "!";
	}			
}
/**
 * Gets a string from the resource bundle and binds it
 * with the given arguments. If the key is not found,
 * return the key.
 */
public static String getResourceString(String key, Object[] args) {
	try {
		return MessageFormat.format(getResourceString(key), args);
	} catch (MissingResourceException e) {
		return key;
	} catch (NullPointerException e) {
		return "!" + key + "!";
	}
}
/** (Non-javadoc)
 * Method declared on Plugin.
 */
public void startup() throws CoreException {
}
}
