package org.eclipse.scripting;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.scripting.ScriptAdapterEventHandler;
import org.eclipse.core.runtime.IExecutableExtension;

/**
 * This interface is used to register a custom event handler
 * for a script adapter.
 * <p>
 * This interface is primarily intended for use 
 * by developers who are exploiting the UI-rendering support in
 * this package as part of their workbench extension.
 * </p>
 * <p>
 * Clients may implement this interface.
 * </p>
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under development and expected to
 * change significantly before reaching stability. It is being made available at this early stage to solicit feedback
 * from pioneering adopters on the understanding that any code that uses this API will almost certainly be broken
 * (repeatedly) as the API evolves.
 * </p>
 *
 * @see Renderer
 * @see ScriptAdapterWorkbenchAction
 * @see ScriptAdapterWorkbenchNewWizard
 * @see ScriptAdapterWorkbenchPreferencePage
 * @see ScriptAdapterWorkbenchResourceEditor
 * @see ScriptAdapterWorkbenchResourceEditorContent
 * @see ScriptAdapterWorkbenchResourceExportWizard
 * @see ScriptAdapterWorkbenchResourceImportWizard
 */
public interface IScriptAdapter extends IExecutableExtension {
/** *
 * Sets the handler that handles user interface events.
 *
 * @param handler event handler
 */
void setScriptAdapterEventHandler(ScriptAdapterEventHandler handler);
}
