package org.eclipse.scripting;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * This is the abstract base class for all script event handlers
 * <p>
 * This class is primarily intended for use 
 * by developers who are exploiting the UI-rendering support in
 * this package as part of their workbench extension.
 * </p>
 * <p>
 * This class is intended to be subclassed by clients.
 * </p>
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under development and expected to
 * change significantly before reaching stability. It is being made available at this early stage to solicit feedback
 * from pioneering adopters on the understanding that any code that uses this API will almost certainly be broken
 * (repeatedly) as the API evolves.
 * </p>
 *
 * @see Renderer
 */
public abstract class ScriptAdapterEventHandler {
	IToolkitUI iToolkitUI = null;
/**
 * Constructs a script event handler. 
 */
public ScriptAdapterEventHandler() {
}
/**
 * Returns the script user interface toolkit.
 *
 * @return user interface toolkit object
 */
public IToolkitUI getToolkit() {
	return iToolkitUI;
}
/**
 * Sets the script user interface toolkit.  This is called only by the script runtime
 * so that subclasses may call the getToolkit member function.
 *
 * @param toolkitUI user interface toolkit
 */
public void setToolkit( IToolkitUI toolkitUI ) {
	
	iToolkitUI = toolkitUI;
}
}
