package org.eclipse.scripting;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.scripting.internal.Script;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

/**
 * This class is a script adapter for workbench menubar and toolbar
 * actions.
 * <p>
 * This class is primarily intended for use 
 * by developers who are exploiting the UI-rendering support in
 * this package as part of their workbench extension.
 * </p>
 * <p>
 * This class is intended to be subclassed by clients.
 * </p>
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under development and expected to
 * change significantly before reaching stability. It is being made available at this early stage to solicit feedback
 * from pioneering adopters on the understanding that any code that uses this API will almost certainly be broken
 * (repeatedly) as the API evolves.
 * </p>
 *
 * @see IScriptAdapter
 * @see IToolkitUI
 * @see Renderer
 */
public class ScriptAdapterWorkbenchAction implements IWorkbenchWindowActionDelegate, IScriptAdapter {
	private Script script = new Script();
/**
 * Constructs an instance of ScriptAdapterWorkbenchAction.
 */
public ScriptAdapterWorkbenchAction() {
	super();
}
/**
 * Disposes this action delegate.  This implementation does nothing.
 */
public void dispose() {}
/**
 * Initializes the action delegate with the workbench window it will work in.
 * This implmentation does nothing.
 *
 * @param window the window that provides the context for this delegate
 */
public void init(IWorkbenchWindow window) {}
/**
 * Runs the textual, or user interface script.
 *
 * @param action action proxy that handles the presentation
 *		portion of the plugin action
 */
public void run(IAction action) {
	this.script.run();
}
/**
 * Remembers the workbench selection.
 *
 * @param action 
 * @param selection 
 */
public void selectionChanged( IAction action, ISelection selection) {
	
	if( selection instanceof IStructuredSelection ) 	{
		
		this.script._structuredSelection = (IStructuredSelection)selection;
	}
}
/**
 * This method is called on a newly constructed extension, passing 
 * it its relevant configuration information.
 *
 * @param config the configuration element used to trigger this execution. 
 *		It can be queried by the executable extension for specific
 *		configuration properties
 * @param propertyName the name of an attribute of the configuration element
 *		used on the <code>createExecutableExtension(String)</code> call. This
 *		argument can be used in the cases where a single configuration element
 *		is used to define multiple executable extensions.
 * @param data adapter data in the form of a <code>String</code>, 
 *		a <code>Hashtable</code>, or <code>null</code>.
 * @exception CoreException if error(s) detected during initialization processing
 * @see IConfigurationElement#createExecutableExtension
 */
public void setInitializationData(IConfigurationElement config, String propertyName, java.lang.Object data) throws CoreException {
	this.script.setInitializationData(config, propertyName, data);
}
/**
 * Associates an event handler with the adaptor
 *
 * @param handler script adaptor event handler
 */
public void setScriptAdapterEventHandler( ScriptAdapterEventHandler handler) {
	
	this.script._scriptRunner.setScriptAdapterEventHandler( handler );
}
}
