package org.eclipse.scripting;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExecutableExtension;
import org.eclipse.scripting.internal.ScriptRunnerExtended;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

/**
 * This class is a script adapter for workbench preference page.
 * <p>
 * This class is primarily intended for use 
 * by developers who are exploiting the UI-rendering support in
 * this package as part of their workbench extension.
 * </p>
 * <p>
 * This class is intended to be subclassed by clients.
 * </p>
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under development and expected to
 * change significantly before reaching stability. It is being made available at this early stage to solicit feedback
 * from pioneering adopters on the understanding that any code that uses this API will almost certainly be broken
 * (repeatedly) as the API evolves.
 * </p>
 *
 * @see IScriptAdapter
 * @see IToolkitUI
 * @see Renderer
 */
public class ScriptAdapterWorkbenchPreferencePage extends ScriptAdapterDelegatingPreferencePage implements IExecutableExtension, IWorkbenchPreferencePage {
	
	private ScriptRunnerExtended scriptRunner = null;
/**
 * Constructs an instance of ScriptAdapterWorkbenchPreferencePage.
 */
public ScriptAdapterWorkbenchPreferencePage() {
	super();
}
/**
 * Initializes and displays the preference page.
 *
 * @param workbench
 */
public void init( IWorkbench workbench ) {
	
	if (this.scriptRunner != null) {
	    
		this.scriptRunner.initialize(null);

		// Override the presentation type
		//-------------------------------
		this.scriptRunner.setPresentationType("preferencepage");

		this.scriptRunner.run(this.page);
	}
}
/**
 * This method is called on a newly constructed extension, passing 
 * it its relevant configuration information.
 *
 * @param config the configuration element used to trigger this execution. 
 *		It can be queried by the executable extension for specific
 *		configuration properties
 * @param propertyName the name of an attribute of the configuration element
 *		used on the <code>createExecutableExtension(String)</code> call. This
 *		argument can be used in the cases where a single configuration element
 *		is used to define multiple executable extensions.
 * @param data adapter data in the form of a <code>String</code>, 
 *		a <code>Hashtable</code>, or <code>null</code>.
 */
public void setInitializationData(
	IConfigurationElement configElement,
	String strName,
	Object objData) {
	this.scriptRunner = new ScriptRunnerExtended(configElement, objData);
}
/**
 * Associates event handler with the adaptor
 *
 * @param handler script adaptor event handler
 */
public void setScriptAdapterEventHandler( ScriptAdapterEventHandler handler ) { 

	this.scriptRunner.setScriptAdapterEventHandler( handler );
}
}
