package org.eclipse.scripting;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.scripting.internal.Script;
import org.eclipse.ui.IExportWizard;
import org.eclipse.ui.IWorkbench;

/**
 * This class is a script adapter for workbench export wizard
 * <p>
 * This class is primarily intended for use 
 * by developers who are exploiting the UI-rendering support in
 * this package as part of their workbench extension.
 * </p>
 * <p>
 * This class is intended to be subclassed by clients.
 * </p>
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under development and expected to
 * change significantly before reaching stability. It is being made available at this early stage to solicit feedback
 * from pioneering adopters on the understanding that any code that uses this API will almost certainly be broken
 * (repeatedly) as the API evolves.
 * </p>
 *
 * @see IScriptAdapter
 * @see IToolkitUI
 * @see Renderer
 */
public class ScriptAdapterWorkbenchResourceExportWizard extends ScriptAdapterDelegatingWizard implements IExportWizard, IScriptAdapter {
	
	private Script script = null;
/**
 * Constructs an instance of ScriptAdapterWorkbenchResourceExportWizard.
 */
public ScriptAdapterWorkbenchResourceExportWizard() {
	
	super();
	
	// Remove the smart guide dialog
	//------------------------------
	this.wizard._wizardDialog = null;

	// Create a work object
	//---------------------
	this.script = new Script();
}
/**
 * Initializes and displays the wizard.  The selection is passed to the script.
 *
 * @param workbench
 * @param structuredSelection
 */
public void init(IWorkbench workbench, IStructuredSelection structuredSelection) {

	// This will be converted to an array in the superclass' run method
	//-----------------------------------------------------------------
	this.script._structuredSelection = structuredSelection;

	// Remove the wizard dialog
	//-------------------------
	this.wizard._wizardDialog = null;

	// Override the presentation type
	//-------------------------------
	this.script._scriptRunner.setPresentationType("wizard");

	this.script.run(this.wizard);

	return;
}
/**
 * This method is called on a newly constructed extension, passing 
 * it its relevant configuration information.
 *
 * @param config the configuration element used to trigger this execution. 
 *		It can be queried by the executable extension for specific
 *		configuration properties
 * @param propertyName the name of an attribute of the configuration element
 *		used on the <code>createExecutableExtension(String)</code> call. This
 *		argument can be used in the cases where a single configuration element
 *		is used to define multiple executable extensions.
 * @param data adapter data in the form of a <code>String</code>, 
 *		a <code>Hashtable</code>, or <code>null</code>.
 */
public void setInitializationData(
	IConfigurationElement configElement,
	String strName,
	Object objData) {

	this.script.setInitializationData(configElement, strName, objData);
}
/**
 * Associates an event handler with the adaptor
 *
 * @param handler script adaptor event handler
 */
public void setScriptAdapterEventHandler( ScriptAdapterEventHandler handler ) { 

	this.script._scriptRunner.setScriptAdapterEventHandler( handler );
}
}
