package org.eclipse.scripting;

/*
 * (c) Copyright IBM Corp. 2000, 2001.
 * All Rights Reserved.
 */

/**
 * This class may be used as an intermediate buffer between a
 * programmer's string/object pairs, and the user interface renderer.
 * <p>
 * This class is primarily intended for use 
 * by workbench users writing script code, or
 * by developers who are exploiting the UI-rendering support in
 * this package as part of their workbench extension.
 * </p>
 * <p>
 * This class is not intended to be subclassed by clients.
 * </p>
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under development and expected to
 * change significantly before reaching stability. It is being made available at this early stage to solicit feedback
 * from pioneering adopters on the understanding that any code that uses this API will almost certainly be broken
 * (repeatedly) as the API evolves.
 * </p>
 */
public class StringDataSingle implements IStringData {
	
	private String _string = null;  // May be modified
	private Object _object = null;  // Will never be modified
/**
 * Constructs a StrinDataSingle instance.
 */
public StringDataSingle() {
	_string = new String();
}
/**
 * Constructs an instance with a string value and an optional object.
 *
 * @param string
 * @param object
 */
public StringDataSingle( String string, Object object ) {
	_string = string;
	_object = object;
}
/**
 * The object of the string/object pair is returned.
 *
 * @return object
 */
public Object getObject() {
	return _object;
}
/**
 * The string of the string/object pair is returned.  Boolean values may be returned from a user interface widget.
 * A true value is represented by the string <code>true</code>, and a false value is represented by the string <code>false</code>.
 *
 * @return string
 */
public String getString() {
	return _string;
}
/**
 * Sets the current object of the string/object pair.
 *
 * @param object
 */
public void setObject( Object object ) {
	_object = object;
}
/**
 * Sets the string of the string/object pair.  Boolean values may be given to a user interface widget.
 * A true value is represented by the string <code>true</code>, and a false value is represented by the string <code>false</code>.
 *
 * @param string
 */
public void setString( String string ) {
	_string = string;
}
}
